import React from 'react';
import PropTypes from 'prop-types';
import { observer } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';

const styles = () => ({
  root: {
    position: 'absolute',
    display: 'table',
    padding: 0,
    width: 100,
    minHeight: 50,
    zIndex: 20,
    fontWeight: 'bold',
    fontSize: 11,
    cursor: 'move',
    background: 'linear-gradient(180deg, #ffffff, #f0f0f0)',
    border: '1px solid silver',
    borderSpacing: 0,
    color: 'black',
    boxSizing: 'border-box',
    boxShadow: 'none',
    '&:hover': {
      boxShadow: '0px 0px 6px silver',
    },
    '& > div': {
      width: 90,
      margin: '4px auto',
      '& > p': {
        display: 'table-cell',
        verticalAlign: 'middle',
        textAlign: 'center',
        padding: '0px 2px',
        breakInside: 'avoid',
      },
    },
  },
  title: {
    height: 40,
    width: 86,
  },
  healthState: {
    '& > p': {
      fontWeight: 'normal',
      width: 86,
      height: 40,
    },
  },
  infection: {
    borderRadius: '70px/35px',
  },
  acute: {
    background: 'linear-gradient(180deg, #ffdc54, #ffcc00)',
    '& $healthState': {
      border: '1px solid #ffe98f',
      '&:hover': {
        backgroundColor: '#ffe98f',
      },
    },
  },
  asymptomatic: {
    background: 'linear-gradient(180deg, #c4dbe7, #a0bfcf)',
  },
  death: {
    background: 'linear-gradient(180deg, #cccccc, #c0c0c0)',
  },
  recovery: {
    borderRadius: 35,
    background: 'white',
  },
});

@withStyles(styles)
@observer
class HealthOutcomeWidget extends React.Component {
  static propTypes = {
    classes: PropTypes.object.isRequired,
    healthOutcome: PropTypes.object.isRequired,
  };

  render() {
    const { classes, healthOutcome } = this.props;

    const localStyles = {
      root: {
        top: healthOutcome.top,
        left: healthOutcome.left,
      },
    };

    let className = `healthOutcome ${classes.root}`;
    const type = healthOutcome.type.toLowerCase();
    if (type === 'infection') {
      className += ` ${classes.infection}`;
    }
    if (type === 'acute') {
      className += ` ${classes.acute}`;
    }
    if (type === 'asymptomatic') {
      className += ` ${classes.asymptomatic}`;
    }
    if (type === 'recovery') {
      className += ` ${classes.recovery}`;
    }
    if (type === 'death') {
      className += ` ${classes.death}`;
    }

    const titleNode = (
      <div id={healthOutcome.id + '_title'}>
        <p className={classes.title}>{healthOutcome.name}</p>
      </div>
    );

    let healthStateNodes;
    const { healthStatesAsArray: healthStates } = healthOutcome;
    if (healthStates.length > 0) {
      healthStateNodes = healthStates.map(healthState => (
        <div
          id={healthState.id}
          key={healthState.id}
          className={`healthState ${classes.healthState}`}
        >
          <p>{healthState.name}</p>
        </div>
      ));
    }

    return (
      <div id={healthOutcome.id} className={className} style={localStyles.root}>
        {titleNode}
        {healthStateNodes}
      </div>
    );
  }
}

export default HealthOutcomeWidget;
