import React from 'react';
import PropTypes from 'prop-types';
import { observer } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import Popover from '@material-ui/core/Popover';
import Divider from '@material-ui/core/Divider';
import ValidationTextField from './ValidationTextField';
import ValidateName from '../utilities/ValidateName';

const validTxt = 'Type a new name';

const styles = () => ({
  root: {
    padding: 10,
    width: 400,
  },
  divider: {
    marginTop: 10,
    marginBottom: 10,
  },
});

const ItemAddWidget = props => {
  const {
    classes,
    valid,
    open,
    anchorEl,
    restrictedNames,
    onNameChange,
    onCloseRequest,
    onAddBtnClick,
  } = props;
  const addBtnDisabled = !valid;

  return (
    <Popover
      anchorEl={anchorEl}
      getContentAnchorEl={null}
      anchorOrigin={{
        vertical: 'bottom',
        horizontal: 'left',
      }}
      open={open}
      onClose={onCloseRequest}
    >
      <form className={classes.root}>
        <Typography variant="h6">Add a new item</Typography>
        <br />
        <ValidationTextField
          label="Item name"
          helperText={validTxt}
          onChange={onNameChange}
          validationFunc={ValidateName(restrictedNames, validTxt)}
          autoFocus
          fullWidth
          required
        />
        <Divider className={classes.divider} />
        <div style={{ textAlign: 'right' }}>
          <Button color="secondary" onClick={onCloseRequest}>
            Cancel
          </Button>
          <Button
            color="primary"
            disabled={addBtnDisabled}
            onClick={onAddBtnClick}
            type="submit"
          >
            OK
          </Button>
        </div>
      </form>
    </Popover>
  );
};

ItemAddWidget.propTypes = {
  classes: PropTypes.object.isRequired,
  valid: PropTypes.bool.isRequired,
  open: PropTypes.bool.isRequired,
  anchorEl: PropTypes.instanceOf(Element),
  restrictedNames: PropTypes.arrayOf(PropTypes.string),
  onNameChange: PropTypes.func,
  onCloseRequest: PropTypes.func,
  onAddBtnClick: PropTypes.func,
};

ItemAddWidget.defaultProps = {
  restrictedNames: [],
  onNameChange: () => {},
  onCloseRequest: () => {},
  onAddBtnClick: () => {},
};

export default observer(withStyles(styles)(ItemAddWidget));
