import React from 'react';
import PropTypes from 'prop-types';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import List from '@material-ui/core/List';
import ListItem from '@material-ui/core/ListItem';
import Typography from '@material-ui/core/Typography';
import IconButton from '@material-ui/core/IconButton';
import Tooltip from '@material-ui/core/Tooltip';
import DeleteOutlinedIcon from '@material-ui/icons/DeleteOutlined';
import EditOutlinedIcon from '@material-ui/icons/EditOutlined';
import SaveOutlinedIcon from '@material-ui/icons/SaveOutlined';
import EmailOutlinedIcon from '@material-ui/icons/EmailOutlined';
import AddBoxOutlinedIcon from '@material-ui/icons/AddBoxOutlined';
import Chip from '@material-ui/core/Chip';
import { ElementType } from '../settings';

const styles = () => ({
  chip: {
    width: 70,
    height: 17,
    fontSize: '0.6rem',
    marginTop: -1,
    background: 'linear-gradient(180deg, #ffffff, #f0f0f0)',
    borderRadius: 0,
  },
  chipLabel: {
    paddingLeft: 7,
    paddingRight: 7,
  },
  infection: {
    borderRadius: '55px/15px',
  },
  acute: {
    background: 'linear-gradient(180deg, #ffdc54, #ffcc00)',
  },
  asymptomatic: {
    background: 'linear-gradient(180deg, #c4dbe7, #a0bfcf)',
  },
  recovery: {
    borderRadius: '10px',
    background: 'white',
  },
  death: {
    background: 'linear-gradient(180deg, #cccccc, #c0c0c0)',
  },
  deleteBtn: {
    position: 'absolute',
    right: 4,
    top: '50%',
    transform: 'translateY(-50%)',
    fontSize: 16,
  },
  modifyBtn: {
    position: 'absolute',
    right: 36,
    top: '50%',
    transform: 'translateY(-50%)',
    fontSize: 16,
  },
  addHealthStateBtn: {
    position: 'absolute',
    right: 68,
    top: '50%',
    transform: 'translateY(-50%)',
    fontSize: 16,
  },
  saveBtn: {
    position: 'absolute',
    right: 36,
    top: '50%',
    transform: 'translateY(-50%)',
    fontSize: 16,
  },
  emailBtn: {
    position: 'absolute',
    right: 68,
    top: '50%',
    transform: 'translateY(-50%)',
    fontSize: 16,
  },
});

@withStyles(styles)
@observer
class ItemsList extends React.Component {
  static propTypes = {
    classes: PropTypes.object.isRequired,
    itemsArray: PropTypes.arrayOf(PropTypes.object),
    onListItemClick: PropTypes.func,
    onListSubItemClick: PropTypes.func,
    onDeleteItemBtnClick: PropTypes.func,
    onModifyItemBtnClick: PropTypes.func,
    onAddSubItemBtnClick: PropTypes.func,
    onModifySubItemBtnClick: PropTypes.func,
    onDeleteSubItemBtnClick: PropTypes.func,
    onSaveItemBtnClick: PropTypes.func,
    onEmailItemBtnClick: PropTypes.func,
    selectedItem: MobxPropTypes.observableObject,
    listType: PropTypes.string,
  };

  static defaultProps = {
    itemsArray: [],
    onListItemClick: () => {},
    onListSubItemClick: () => {},
    onDeleteItemBtnClick: null,
    onModifyItemBtnClick: null,
    onAddSubItemBtnClick: null,
    onModifySubItemBtnClick: null,
    onDeleteSubItemBtnClick: null,
    onEmailItemBtnClick: null,
    selectedItem: null,
    listType: '',
  };

  getModelItem = (
    item,
    onDeleteItemBtnClick,
    onSaveItemBtnClick,
    onEmailItemBtnClick,
    classes,
  ) => {
    const actionBtns = [];

    if (onEmailItemBtnClick) {
      actionBtns.push(
        <Tooltip key="email" title='Email model'>
          <IconButton
            color="primary"
            aria-label="Email"
            className={classes.emailBtn}
            onClick={event => {
              event.stopPropagation();
              onEmailItemBtnClick(event, item);
            }}
          >
            <EmailOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    if (onSaveItemBtnClick) {
      actionBtns.push(
        <Tooltip key="save" title='Save model to file'>
          <IconButton
            color="primary"
            aria-label="Save"
            className={classes.saveBtn}
            onClick={event => {
              event.stopPropagation();
              onSaveItemBtnClick(event, item);
            }}
          >
            <SaveOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    if (onDeleteItemBtnClick) {
      actionBtns.push(
        <Tooltip key="delete" title='Delete model'>
          <IconButton
            color="primary"
            aria-label="Delete"
            className={classes.deleteBtn}
            onClick={event => {
              event.stopPropagation();
              onDeleteItemBtnClick(event, item);
            }}
          >
            <DeleteOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    return {
      actionBtns,
    };
  };

  getRepoFileNameItem = item => {
    const actionBtns = [];

    return {
      actionBtns,
    };
  };

  getOutcomeTreeItem = (item, onDeleteItemBtnClick, classes) => {
    const actionBtns = [];

    if (onDeleteItemBtnClick) {
      actionBtns.push(
        <Tooltip key="delete" title='Delete outcome tree'>
          <IconButton
            color="primary"
            aria-label="Delete"
            className={classes.deleteBtn}
            onClick={event => {
              event.stopPropagation();
              onDeleteItemBtnClick(event, item);
            }}
          >
            <DeleteOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    return {
      actionBtns,
    };
  };

  getHealthOutcomeItem = (
    item,
    onModifyItemBtnClick,
    onDeleteItemBtnClick,
    onAddSubItemBtnClick,
    classes,
  ) => {
    let className = `${classes.chip}`;
    const itemType = item.type.toUpperCase();
    if (itemType === 'INFECTION') {
      className += ` ${classes.infection}`;
    }
    if (itemType === 'ACUTE') {
      className += ` ${classes.acute}`;
    }
    if (itemType === 'ASYMPTOMATIC') {
      className += ` ${classes.asymptomatic}`;
    }
    if (itemType === 'RECOVERY') {
      className += ` ${classes.recovery}`;
    }
    if (itemType === 'DEATH') {
      className += ` ${classes.death}`;
    }

    const typeChip = (
      <React.Fragment>
        <Chip
          className={className}
          label={item.type}
          variant="outlined"
          size="small"
          classes={{
            label: classes.chipLabel
          }}
        />
        &nbsp;
      </React.Fragment>
    );

    const actionBtns = [];

    if (onAddSubItemBtnClick && (itemType === 'ACUTE' || itemType === 'SEQUELA')) {
      actionBtns.push(
        <Tooltip key="addHealthState" title='Add health state'>
          <IconButton
            color="primary"
            aria-label="Add Health State"
            className={classes.addHealthStateBtn}
            onClick={event => {
              event.stopPropagation();
              onAddSubItemBtnClick(event, item);
            }}
          >
            <AddBoxOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    if (onModifyItemBtnClick) {
      actionBtns.push(
        <Tooltip key="modify" title='Modify health outcome'>
          <IconButton
            color="primary"
            aria-label="Modify"
            className={classes.modifyBtn}
            onClick={event => {
              event.stopPropagation();
              onModifyItemBtnClick(event, item);
            }}
          >
            <EditOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    if (onDeleteItemBtnClick) {
      actionBtns.push(
        <Tooltip key="delete" title='Delete health outcome'>
          <IconButton
            color="primary"
            aria-label="Delete"
            className={classes.deleteBtn}
            onClick={event => {
              event.stopPropagation();
              onDeleteItemBtnClick(event, item);
            }}
          >
            <DeleteOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    return {
      typeChip,
      actionBtns,
    };
  };

  getTransitionItem = (
    item,
    onModifyItemBtnClick,
    onDeleteItemBtnClick,
    classes,
  ) => {
    const actionBtns = [];
    if (onModifyItemBtnClick) {
      actionBtns.push(
        <Tooltip key="modify" title='Modify transition'>
          <IconButton
            color="primary"
            aria-label="Modify"
            key="modify"
            className={classes.modifyBtn}
            onClick={event => {
              event.stopPropagation();
              onModifyItemBtnClick(event, item);
            }}
          >
            <EditOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    if (onDeleteItemBtnClick) {
      actionBtns.push(
        <Tooltip key="delete" title='Delete transition'>
          <IconButton
            color="primary"
            aria-label="Delete"
            key="delete"
            className={classes.deleteBtn}
            onClick={event => {
              event.stopPropagation();
              onDeleteItemBtnClick(event, item);
            }}
          >
            <DeleteOutlinedIcon fontSize="inherit" />
          </IconButton>
        </Tooltip>,
      );
    }

    return {
      actionBtns,
    };
  };

  render() {
    const {
      classes,
      itemsArray,
      selectedItem,
      onListItemClick,
      onListSubItemClick,
      onDeleteItemBtnClick,
      onModifyItemBtnClick,
      onAddSubItemBtnClick,
      onModifySubItemBtnClick,
      onDeleteSubItemBtnClick,
      onSaveItemBtnClick,
      onEmailItemBtnClick,
      listType,
    } = this.props;

    const itemsList = itemsArray.map((item, idx) => {
      let subItemsList;
      let typeChip;
      let actionBtns;
      if (listType === 'MODEL') {
        const itemElems = this.getModelItem(
          item,
          onDeleteItemBtnClick,
          onSaveItemBtnClick,
          onEmailItemBtnClick,
          classes,
        );
        if (itemElems.actionBtns.length > 0) {
          actionBtns = <div>{itemElems.actionBtns}</div>;
        }
      } else if (listType === 'REPO') {
        const itemElems = this.getRepoFileNameItem(item);
        if (itemElems.actionBtns.length > 0) {
          actionBtns = <div>{itemElems.actionBtns}</div>;
        }
      } else if (listType === 'OUTCOME_TREE') {
        const itemElems = this.getOutcomeTreeItem(
          item,
          onDeleteItemBtnClick,
          classes,
        );
        typeChip = itemElems.typeChip; // eslint-disable-line prefer-destructuring
        if (itemElems.actionBtns.length > 0) {
          actionBtns = <div>{itemElems.actionBtns}</div>;
        }
      } else if (listType === ElementType.HEALTH_OUTCOME) {
        const itemElems = this.getHealthOutcomeItem(
          item,
          onModifyItemBtnClick,
          onDeleteItemBtnClick,
          onAddSubItemBtnClick,
          classes,
        );
        typeChip = itemElems.typeChip; // eslint-disable-line prefer-destructuring
        if (itemElems.actionBtns.length > 0) {
          actionBtns = <div>{itemElems.actionBtns}</div>;
        }
        if (item.healthStatesAsArray.length > 0) {
          const subItems = item.healthStatesAsArray.map((healthState, i) => {
            const actionBtnsSubArray = [];
            if (onModifySubItemBtnClick) {
              actionBtnsSubArray.push(
                <Tooltip key="modify" title='Modify health state'>
                  <IconButton
                    color="primary"
                    aria-label="Modify"
                    className={classes.modifyBtn}
                    onClick={event => onModifySubItemBtnClick(event, healthState)}
                  >
                    <EditOutlinedIcon fontSize="inherit" />
                  </IconButton>
                </Tooltip>,
              );
            }

            if (onDeleteSubItemBtnClick) {
              actionBtnsSubArray.push(
                <Tooltip key="delete" title='Delete health state'>
                  <IconButton
                    color="primary"
                    aria-label="Delete"
                    className={classes.deleteBtn}
                    onClick={event => onDeleteSubItemBtnClick(event, healthState)}
                  >
                    <DeleteOutlinedIcon fontSize="inherit" />
                  </IconButton>
                </Tooltip>,
              );
            }
            return (
              <ListItem
                button
                dense
                key={healthState.id}
                onClick={event => onListSubItemClick(event, healthState)}
                selected={selectedItem === healthState}
              >
                <Typography>
                  {idx + 1}.{i + 1}
                  .&nbsp;
                </Typography>
                {healthState.name}
                <div>{actionBtnsSubArray}</div>
              </ListItem>
            );
          });
          subItemsList = (
            <List component="div" dense disablePadding style={{ marginLeft: 10 }}>
              {subItems}
            </List>
          );
        }
      } else if (listType === ElementType.TRANSITION) {
        const itemElems = this.getTransitionItem(
          item,
          onModifyItemBtnClick,
          onDeleteItemBtnClick,
          classes,
        );
        typeChip = itemElems.typeChip; // eslint-disable-line prefer-destructuring
        if (itemElems.actionBtns.length > 0) {
          actionBtns = <div>{itemElems.actionBtns}</div>;
        }
      }

      return (
        <React.Fragment key={item.id}>
          <ListItem
            button
            dense
            onClick={event => onListItemClick(event, item)}
            selected={selectedItem === item}
          >
            <Typography>
              {idx + 1}
              .&nbsp;
            </Typography>
            {typeChip}
            <Typography>{item.name}</Typography>
            {actionBtns}
          </ListItem>
          {subItemsList}
        </React.Fragment>
      );
    });

    return (
      <List component="nav" dense>
        {itemsList}
      </List>
    );
  }
}

export default ItemsList;
