import React from 'react';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import Button from '@material-ui/core/Button';
import Menu from '@material-ui/core/Menu';
import MenuItem from '@material-ui/core/MenuItem';
import ItemAddWidget from './ItemAddWidget';
// import RepoExplorer from './RepoExplorer';
import LoadLocalJSONFile from '../utilities/LoadLocalJSONFile';
import LoadRemoteJSONFile from '../utilities/LoadRemoteJSONFile';

@observer
class ModelManageWidget extends React.Component {
  static propTypes = {
    modelsStore: MobxPropTypes.observableObject.isRequired,
  };

  state = {
    addMenuOpen: false,
    newWidgetOpen: false,
    anchorEl: null,
    modelName: '',
    modelNameValid: false,
    repoExplOpen: false,
    repoExpOkEnabled: false,
    repoMeta: [],
    selectedRepoFileSpec: null,
  };

  handleOpenAddMenu = event => {
    this.setState({
      anchorEl: event.currentTarget,
      addMenuOpen: true,
    });
  };

  handleCloseAddMenu = () => {
    this.setState({
      addMenuOpen: false,
    });
  };

  handleNewBtnClick = () => {
    this.setState({
      addMenuOpen: false,
      newWidgetOpen: true,
    });
  };

  handleLoadFromFileBtnClick = async event => {
    event.preventDefault();
    this.handleCloseAddMenu();

    const fileName = event.nativeEvent.target.files[0];
    if (fileName) {
      const { modelsStore } = this.props;
      const modelSpec = await LoadLocalJSONFile(fileName);
      modelsStore.addModelFromSpec(modelSpec);
    }
  };

  handleLoadFromRepoBtnClick = async event => {
    event.preventDefault();
    this.handleCloseAddMenu();
    const repoMeta = await LoadRemoteJSONFile('meta.json');
    this.setState({ repoExplOpen: true, repoMeta });
  };

  handleModelNameChange = (name, valid) => {
    this.setState({
      modelName: name,
      modelNameValid: valid,
    });
  };

  handleCloseNewWidget = () => {
    this.setState({
      newWidgetOpen: false,
    });
  };

  handleCreateNewModel = () => {
    const { modelName, modelNameValid } = this.state;

    this.handleCloseNewWidget();
    if (modelNameValid) {
      const { modelsStore } = this.props;
      modelsStore.createModel(modelName);

      this.setState({
        modelName: '',
        modelNameValid: false,
      });
    }
  };

  handleRepoExplCloseRequest = () => {
    this.setState({ repoExplOpen: false });
  };

  /*
  handleRepoFileSpecClick = (event, repoFileSpec) => {
    this.setState({ selectedRepoFileSpec: repoFileSpec });
  };
  */

  handleRepoFileLoad = async () => {
    const { selectedRepoFileSpec } = this.state;
    const fileName = selectedRepoFileSpec.name;
    const { modelsStore } = this.props;
    const modelSpec = await LoadRemoteJSONFile(fileName);
    console.log(modelSpec);
    modelsStore.addModelFromSpec(modelSpec);
  };

  render() {
    const {
      modelsStore: { modelsNames },
    } = this.props;
    const {
      anchorEl,
      newWidgetOpen,
      addMenuOpen,
      modelNameValid,
      //repoExplOpen,
      //repoMeta,
      //selectedRepoFileSpec,
    } = this.state;

    return (
      <div>
        <Button
          onClick={this.handleOpenAddMenu}
          color="primary"
          aria-haspopup="true"
          aria-owns={null}
        >
          Add
        </Button>
        <Menu
          anchorEl={anchorEl}
          getContentAnchorEl={null}
          anchorOrigin={{
            vertical: 'bottom',
            horizontal: 'left',
          }}
          open={addMenuOpen}
          onClose={this.handleCloseAddMenu}
          disableRestoreFocus
        >
          <MenuItem
            onClick={this.handleNewBtnClick}
            style={{ minHeight: 0 }}
            dense
          >
            Create new
          </MenuItem>
          <MenuItem
            component="label"
            style={{ width: 150, minHeight: 0 }}
            dense
          >
            Load from file
            <input
              type="file"
              style={{ visibility: 'hidden' }}
              onChange={this.handleLoadFromFileBtnClick}
            />
          </MenuItem>
          {/*
          <MenuItem
            onClick={this.handleLoadFromRepoBtnClick}
            style={{ minHeight: 0 }}
            dense
          >
            Load from repository
          </MenuItem>
          */}
        </Menu>
        <ItemAddWidget
          valid={modelNameValid}
          open={newWidgetOpen}
          anchorEl={anchorEl}
          restrictedNames={modelsNames}
          onNameChange={this.handleModelNameChange}
          onCloseRequest={this.handleCloseNewWidget}
          onAddBtnClick={this.handleCreateNewModel}
        />
        {/*
        <RepoExplorer
          open={repoExplOpen}
          repoMeta={repoMeta}
          onCloseRequested={this.handleRepoExplCloseRequest}
          onItemClick={this.handleRepoFileSpecClick}
          okEnabled={!!selectedRepoFileSpec}
          onOkRequested={this.handleRepoFileLoad}
          selectedItem={selectedRepoFileSpec}
        />
        */}
      </div>
    );
  }
}

export default ModelManageWidget;
