import React from 'react';
import PropTypes from 'prop-types';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import ListTitle from './ListTitle';
import DescriptionText from './DescriptionText';
import OutcomeTreesOverview from './OutcomeTreesOverview';
import OutcomeTreeRootWidget from './OutcomeTreeRootWidget';
import MappingTable from './MappingTable';

const styles = () => ({
  root: {
    flex: 1,
    display: 'flex',
    flexDirection: 'row',
    overflow: 'hidden',
  },
  leftPanel: {
    padding: 10,
    width: 285,
    minWidth: 285,
    overflow: 'auto',
  },
  rightPanel: {
    overflow: 'auto',
    width: '100%',
    padding: 10,
    flex: 1,
    display: 'flex',
    flexDirection: 'column',
  },
});

const ModelTabOutcomeTrees = ({ classes, model }) => (
  <div className={classes.root}>
    <div className={classes.leftPanel}>
      <ListTitle title="Outcome Trees list" />
      <DescriptionText text="A disease model can have multiple outcome trees." />
      <OutcomeTreesOverview model={model} />
      <br />
      <ListTitle title="Outcome Trees mapping" />
      <DescriptionText text="Each sex/age category must be assigned an outcome tree." />
      <MappingTable model={model} />
    </div>
    <div className={classes.rightPanel}>
      <OutcomeTreeRootWidget model={model} />
    </div>
  </div>
);

ModelTabOutcomeTrees.propTypes = {
  classes: PropTypes.object.isRequired,
  model: MobxPropTypes.observableObject.isRequired,
};

export default observer(withStyles(styles)(ModelTabOutcomeTrees));
