import React from 'react';
import PropTypes from 'prop-types';
import { observer, inject, PropTypes as MobxPropTypes } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import Typography from '@material-ui/core/Typography';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import ModelTabMeta from './ModelTabMeta';
import ModelTabIncidence from './ModelTabIncidence';
import ModelTabOutcomeTrees from './ModelTabOutcomeTrees';
import { INITIAL_TAB } from '../settings';

function TabContainer(props) {
  const { children } = props;

  return (
    <Typography component="div" style={{ padding: 10 }}>
      {children}
    </Typography>
  );
}

TabContainer.propTypes = {
  children: PropTypes.node.isRequired,
};

const styles = theme => ({
  root: {
    flexGrow: 1,
    backgroundColor: theme.palette.background.paper,
  },
  noModel: {
    textAlign: 'center',
    padding: 20,
  },
});

@withStyles(styles)
@inject('modelsStore')
@observer
class ModelTabs extends React.Component {
  static propTypes = {
    classes: PropTypes.object.isRequired,
    modelsStore: MobxPropTypes.observableObject.isRequired,
  };

  state = {
    value: INITIAL_TAB,
  };

  handleChange = (event, value) => {
    this.setState({ value });
  };

  render() {
    const {
      classes,
      modelsStore: { selectedModel: model, nonSelectedModelsNames },
    } = this.props;
    const { value } = this.state;

    let content;
    if (model) {
      content = (
        <React.Fragment>
          <Tabs
            value={value}
            onChange={this.handleChange}
            indicatorColor="primary"
            textColor="primary"
          >
            <Tab label="Meta information" />
            <Tab label="Incidence data" />
            <Tab label="Outcome trees" />
          </Tabs>
          {value === 0 && (
            <TabContainer>
              <ModelTabMeta
                model={model}
                nonSelectedModelsNames={nonSelectedModelsNames}
              />
            </TabContainer>
          )}
          {value === 1 &&
            <TabContainer>
              <ModelTabIncidence model={model}/>
            </TabContainer>
          }
          {value === 2 && (
            <TabContainer>
              <ModelTabOutcomeTrees model={model} />
            </TabContainer>
          )}
        </React.Fragment>
      );
    } else {
      content = (
        <Typography variant="subtitle1" className={classes.noModel}>
          No disease models have been created yet.
          <br />
          Create a new model in the menu on the left.
        </Typography>
      );
    }

    return <div className={classes.root}>{content}</div>;
  }
}

export default ModelTabs;
