import React from 'react';
import PropTypes from 'prop-types';
import { observer, inject, PropTypes as MobxPropTypes } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import ListTitle from './ListTitle';
import DescriptionText from './DescriptionText';
import ModelManageWidget from './ModelManageWidget';
import ItemsList from './ItemsList';

const styles = () => ({
  root: {
    padding: 10,
  },
});

@withStyles(styles)
@inject('modelsStore')
@observer
class ModelsOverview extends React.Component {
  static propTypes = {
    classes: PropTypes.object.isRequired,
    modelsStore: MobxPropTypes.observableObject.isRequired,
  };

  handleListItemClick = (event, model) => {
    const { modelsStore } = this.props;
    const { id, name } = model;
    modelsStore.selectModel(id, name);
  };

  handleModelDeleteBtnClick = (event, model) => {
    const { modelsStore } = this.props;
    const { id, name } = model;
    modelsStore.deleteModel(id, name);
  };

  handleModelSaveBtnClick = (event, model) => {
    const { modelsStore } = this.props;
    const { id, name } = model;
    modelsStore.saveModel(id, name);
  };

  /*
  handleModelEmailBtnClick = () => {
    if (window.modelCreateWebGUI) {
      return (event, model) => {
        const { modelsStore } = this.props;
        const { id, name } = model;
        modelsStore.emailModel(id, name);
      };
    }
    return null;
  };
  */

  render() {
    const { classes, modelsStore } = this.props;

    return (
      <div className={classes.root}>
        <ListTitle title="Disease Models list" />
        <DescriptionText text="Multiple disease models can be loaded at the same time." />
        <ModelManageWidget modelsStore={modelsStore} />
        <ItemsList
          listType="MODEL"
          itemsArray={modelsStore.modelsAsArray}
          selectedItem={modelsStore.selectedModel}
          onListItemClick={this.handleListItemClick}
          onDeleteItemBtnClick={this.handleModelDeleteBtnClick}
          onSaveItemBtnClick={this.handleModelSaveBtnClick}
          //onEmailItemBtnClick={this.handleModelEmailBtnClick()}
        />
      </div>
    );
  }
}

export default ModelsOverview;
