import React from 'react';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import Button from '@material-ui/core/Button';
import ItemAddWidget from './ItemAddWidget';

@observer
class OutcomeTreeAddWidget extends React.Component {
  static propTypes = {
    model: MobxPropTypes.observableObject.isRequired,
  };

  state = {
    newWidgetOpen: false,
    anchorEl: null,
    outcomeTreeName: '',
    outcomeTreeNameValid: false,
  };

  handleOpenNewWidget = event => {
    this.setState({
      anchorEl: event.currentTarget,
      newWidgetOpen: true,
    });
  };

  handleCloseNewWidget = () => {
    this.setState({
      newWidgetOpen: false,
    });
  };

  handleOutcomeTreeNameChange = (name, valid) => {
    this.setState({
      outcomeTreeName: name,
      outcomeTreeNameValid: valid,
    });
  };

  handleCreateNewOutcomeTree = () => {
    const { outcomeTreeName, outcomeTreeNameValid } = this.state;

    this.handleCloseNewWidget();
    if (outcomeTreeNameValid) {
      const { model } = this.props;
      model.addOutcomeTree(outcomeTreeName);

      this.setState({
        outcomeTreeName: '',
        outcomeTreeNameValid: false,
      });
    }
  };

  render() {
    const {
      model: { outcomeTreeNames },
    } = this.props;
    const { anchorEl, newWidgetOpen, outcomeTreeNameValid } = this.state;

    return (
      <div>
        <Button
          onClick={this.handleOpenNewWidget}
          color="primary"
          aria-haspopup="true"
          aria-owns={null}
        >
          Add
        </Button>
        <ItemAddWidget
          valid={outcomeTreeNameValid}
          open={newWidgetOpen}
          anchorEl={anchorEl}
          restrictedNames={outcomeTreeNames}
          onNameChange={this.handleOutcomeTreeNameChange}
          onCloseRequest={this.handleCloseNewWidget}
          onAddBtnClick={this.handleCreateNewOutcomeTree}
        />
      </div>
    );
  }
}

export default OutcomeTreeAddWidget;
