import React from 'react';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import Grid from '@material-ui/core/Grid';
import Button from '@material-ui/core/Button';
import ItemsList from './ItemsList';
import ListTitle from './ListTitle';
import HealthOutcomeAddWidget from './HealthOutcomeAddWidget';
import HealthOutcomeModifyWidget from './HealthOutcomeModifyWidget';
import HealthStateAddWidget from './HealthStateAddWidget';
import HealthStateModifyWidget from './HealthStateModifyWidget';
import TransitionModifyWidget from './TransitionModifyWidget';
import OutcomeTreeRenderWidget from './OutcomeTreeRenderWidget';
import { ElementType } from '../settings';

@observer
class OutcomeTreeRootWidget extends React.Component {
  static propTypes = {
    model: MobxPropTypes.observableObject.isRequired,
  };

  state = {
    action: '',
    modifiedObject: null,
  };

  handleDeleteHealthOutcomeBtnClick = (event, healthOutcome) => {
    event.preventDefault();
    const { id, name, outcomeTree } = healthOutcome;
    if (outcomeTree) {
      outcomeTree.deleteHealthOutcome(id, name);
    }
  };

  handleDeleteHealthStateBtnClick = (event, healthState) => {
    event.preventDefault();
    const { id, name, healthOutcome } = healthState;
    if (healthOutcome) {
      healthOutcome.deleteHealthState(id, name);
    }
  };

  handleModifyHealthOutcomeBtnClick = (event, healthOutcome) => {
    event.preventDefault();
    this.setState({
      action: 'MODIFY_HEALTH_OUTCOME',
      modifiedObject: healthOutcome
    });
    const { outcomeTree } = healthOutcome;
    outcomeTree.selectHealthOutcome(healthOutcome.id);
  };

  handleModifyHealthOutcomeWidgetClose = (event, healthOutcome) => {
    event.preventDefault();
    this.setState({
      action: '',
      modifiedObject: null
    });
    const {outcomeTree} = healthOutcome;
    outcomeTree.deselectHealthOutcome();
  };

  handleAddHealthStateBtnClick = (event, healthOutcome) => {
    event.preventDefault();
    this.setState({
      action: 'ADD_HEALTH_STATE',
      modifiedObject: healthOutcome,
    });
  };

  handleAddHealthStateWidgetClose = () => {
    this.setState({
      action: '',
      modifiedObject: null,
    });
  };

  handleModifyHealthStateBtnClick = (event, healthState) => {
    event.preventDefault();
    this.setState({
      action: 'MODIFY_HEALTH_STATE',
      modifiedObject: healthState,
    });
  };

  handleModifyHealthStateWidgetClose = () => {
    this.setState({
      action: '',
      modifiedObject: null,
    });
  };

  handleAddTransitionBtnClick = event => {
    event.preventDefault();
    this.setState({
      action: 'ADD_TRANSITION',
      modifiedObject: null
    });
  };

  handleDeleteTransitionBtnClick = (event, transition) => {
    event.preventDefault();
    const { id, name, outcomeTree } = transition;
    if (outcomeTree) {
      outcomeTree.deleteTransition(id, name);
    }
  };

  handleModifyTransitionBtnClick = (event, transition) => {
    event.preventDefault();
    this.setState({
      action: 'MODIFY_TRANSITION',
      modifiedObject: null
    });
    const { outcomeTree } = transition;
    outcomeTree.selectTransition(transition.id);
  };

  handleModifyTransitionWidgetClose = (event, outcomeTree) => {
    event.preventDefault();
    this.setState({
      action: '',
      modifiedObject: null
    });
    outcomeTree.deselectTransition();
  };

  render() {
    const { model: {selectedOutcomeTree: outcomeTree} } = this.props;

    if (!outcomeTree) {
      return null;
    }

    const {
      action,
      modifiedObject,
    } = this.state;

    const {
      runOrderedHealthOutcomesAsArray: healthOutcomes,
      transitionsAsArray: transitions,
    } = outcomeTree;

    const elementsNode = (
      <Grid container>
        <Grid item xs={6}>
          <ListTitle title="Health outcomes list" />
          <HealthOutcomeAddWidget outcomeTree={outcomeTree} />
          <HealthOutcomeModifyWidget
            open={action === 'MODIFY_HEALTH_OUTCOME'}
            healthOutcome={modifiedObject}
            onClose={this.handleModifyHealthOutcomeWidgetClose}
          />
          <HealthStateAddWidget
            open={action === 'ADD_HEALTH_STATE'}
            healthOutcome={modifiedObject}
            onClose={this.handleAddHealthStateWidgetClose}
          />
          <HealthStateModifyWidget
            open={action === 'MODIFY_HEALTH_STATE'}
            healthState={modifiedObject}
            onClose={this.handleModifyHealthStateWidgetClose}
          />
          <ItemsList
            itemsArray={healthOutcomes}
            listType={ElementType.HEALTH_OUTCOME}
            onDeleteItemBtnClick={this.handleDeleteHealthOutcomeBtnClick}
            onModifyItemBtnClick={this.handleModifyHealthOutcomeBtnClick}
            onAddSubItemBtnClick={this.handleAddHealthStateBtnClick}
            onModifySubItemBtnClick={this.handleModifyHealthStateBtnClick}
            onDeleteSubItemBtnClick={this.handleDeleteHealthStateBtnClick}
          />
        </Grid>
        <Grid item xs={6}>
          <ListTitle title="Transitions list" />
          <div>
            <Button
              onClick={this.handleAddTransitionBtnClick}
              color="primary"
              aria-haspopup="true"
              aria-owns={null}
            >
              Add
            </Button>
            <TransitionModifyWidget
              action={action}
              outcomeTree={outcomeTree}
              onClose={this.handleModifyTransitionWidgetClose}
            />
          </div>
          <ItemsList
            itemsArray={transitions}
            listType={ElementType.TRANSITION}
            onDeleteItemBtnClick={this.handleDeleteTransitionBtnClick}
            onModifyItemBtnClick={this.handleModifyTransitionBtnClick}
          />
        </Grid>
      </Grid>
    );

    return (
      <div>
        {elementsNode}
        <OutcomeTreeRenderWidget
          outcomeTree={outcomeTree}
        />
      </div>
    );
  }
}

export default OutcomeTreeRootWidget;
