import React from 'react';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import OutcomeTreeAddWidget from './OutcomeTreeAddWidget';
import ItemsList from './ItemsList';

@observer
class OutcomeTreesOverview extends React.Component {
  static propTypes = {
    model: MobxPropTypes.observableObject.isRequired,
  };

  handleListItemClick = (event, outcomeTree) => {
    event.stopPropagation();
    const { model } = this.props;
    const { id, name } = outcomeTree;
    model.selectOutcomeTree(id, name);
  };

  handleOutcomeTreeDeleteBtnClick = (event, outcomeTree) => {
    event.stopPropagation();
    const { model } = this.props;
    const { id, name } = outcomeTree;
    model.deleteOutcomeTree(id, name);
  };

  render() {
    const { model } = this.props;

    return (
      <div>
        <OutcomeTreeAddWidget model={model} />
        <ItemsList
          listType="OUTCOME_TREE"
          itemsArray={model.outcomeTreesAsArray}
          selectedItem={model.selectedOutcomeTree}
          onListItemClick={this.handleListItemClick}
          onDeleteItemBtnClick={this.handleOutcomeTreeDeleteBtnClick}
        />
      </div>
    );
  }
}

export default OutcomeTreesOverview;
