import React from 'react';
import PropTypes from 'prop-types';
import { observer } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';
import Dialog from '@material-ui/core/Dialog';
import Divider from '@material-ui/core/Divider';
import ItemsList from './ItemsList';

const styles = () => ({
  root: {
    padding: 10,
    width: 670,
  },
  divider: {
    marginTop: 10,
    marginBottom: 10,
  },
  descrText: {
    marginRight: 10,
  },
  complDescrText: {
    marginTop: 15,
  },
});

const RepoExplorer = props => {
  const {
    classes,
    open,
    onCloseRequested,
    onItemClick,
    onOkRequested,
    okEnabled,
    repoMeta,
  } = props;

  return (
    <Dialog open={open} onClose={onCloseRequested} maxWidth={false}>
      <form className={classes.root}>
        <Typography variant="h6">Repository explorer</Typography>
        <Grid container spacing={2}>
          <Grid item xs={12}>
            <ItemsList
              listType="REPO"
              itemsArray={repoMeta}
              onListItemClick={onItemClick}
            />
          </Grid>
        </Grid>
        <Divider className={classes.divider} />
        <div style={{ textAlign: 'right' }}>
          <Button color="secondary" onClick={onCloseRequested}>
            Cancel
          </Button>
          <Button
            color="primary"
            disabled={!okEnabled}
            onClick={onOkRequested}
            type="submit"
          >
            OK
          </Button>
        </div>
      </form>
    </Dialog>
  );
};

RepoExplorer.propTypes = {
  classes: PropTypes.object.isRequired,
  open: PropTypes.bool,
  onCloseRequested: PropTypes.func,
  onOkRequested: PropTypes.func,
  onItemClick: PropTypes.func,
  okEnabled: PropTypes.bool,
  repoMeta: PropTypes.arrayOf(PropTypes.object),
};

RepoExplorer.defaultProps = {
  open: false,
  onCloseRequested: () => {},
  onOkRequested: () => {},
  onItemClick: () => {},
  okEnabled: false,
  repoMeta: [],
};

export default observer(withStyles(styles)(RepoExplorer));
