import React from 'react';
import PropTypes from 'prop-types';
import { observer, PropTypes as MobxPropTypes } from 'mobx-react';
import { withStyles } from '@material-ui/core/styles';
import Button from '@material-ui/core/Button';
import Typography from '@material-ui/core/Typography';
import Grid from '@material-ui/core/Grid';
import Dialog from '@material-ui/core/Dialog';
import Divider from '@material-ui/core/Divider';
import Tabs from '@material-ui/core/Tabs';
import Tab from '@material-ui/core/Tab';
import TextField from '@material-ui/core/TextField';
import Autocomplete from '@material-ui/lab/Autocomplete';
import FormControlLabel from '@material-ui/core/FormControlLabel';
import Switch from '@material-ui/core/Switch';
import SwipeableViews from 'react-swipeable-views';
import ValidationTextField from './ValidationTextField';
import ValidateName from '../utilities/ValidateName';
import TabContainer from './TabContainer';
import ItemsList from './ItemsList';
import DataTable from './DataTable';
import { ElementType, GreekLetters } from '../settings';
import CompareArrays from '../utilities/CompareArrays';

const validTxt = 'Type a new name';

const styles = () => ({
  root: {
    padding: 10,
    width: 670,
  },
  divider: {
    marginTop: 10,
    marginBottom: 10,
  },
  descrText: {
    marginRight: 10,
  },
  complDiv: {
    paddingRight: 10,
  },
});

@withStyles(styles)
@observer
class TransitionModifyWidget extends React.Component {
  static propTypes = {
    classes: PropTypes.object.isRequired,
    action: PropTypes.string,
    outcomeTree: MobxPropTypes.observableObject.isRequired,
    onClose: PropTypes.func,
  };

  static defaultProps = {
    outcomeTree: null,
    onClose: null,
  };

  state = {
    tabIdx: 0,
    open: false,
    title: '',
    outcomeTree: null,
    transition: null,
    name: '',
    nameValid: false,
    nameValidFunc: null,
    transitionNames: [],
    source: null,
    target: null,
    annualType: false,
    annualDuration: -1.0,
    redistributeType: false,
    probabilityArray: null,
    redistributeArray: null,
    complementaryType: false,
    complementaryTransition: null,
  };

  static getDerivedStateFromProps(props, state) {
    const { action, outcomeTree } = props;
    const { selectedTransition: transition } = outcomeTree;

    let newState = null;
    if (action === 'ADD_TRANSITION' && outcomeTree && !transition && outcomeTree !== state.outcomeTree) {
      // New transition
      const nameValidFunc = ValidateName(outcomeTree.transitionNames, validTxt);
      newState = {
        tabIdx: 0,
        open: true,
        title: 'Add transition',
        outcomeTree,
        transition: null,
        name: '',
        nameValid: false,
        nameValidFunc,
        transitionNames: outcomeTree.transitionNames,
        source: null,
        target: null,
        annualType: false,
        annualDuration: -1.0,
        redistributeType: false,
        probabilityArray: outcomeTree.model.createDataArray(),
        redistributeArray: outcomeTree.model.createDataArray(),
        complementaryType: false,
        complementaryTransition: null,
      };
    } else if (action === 'MODIFY_TRANSITION' && outcomeTree && transition && transition !== state.transition) {
      // Edit transition
      const nameValidFunc = ValidateName(
        outcomeTree.nonSelectedTransitionNames,
        validTxt,
      );
      newState = {
        tabIdx: 0,
        open: true,
        title: 'Modify transition',
        outcomeTree,
        transition,
        name: transition.name,
        nameValid: nameValidFunc(transition.name) === validTxt,
        nameValidFunc,
        transitionNames: outcomeTree.nonSelectedTransitionNames,
        source: transition.source,
        target: transition.target,
        annualType: transition.annualType,
        annualDuration: transition.annualDuration,
        redistributeType: transition.redistributeType,
        probabilityArray: transition.probabilityArray,
        redistributeArray: transition.redistributeArray,
        complementaryType: transition.complementaryType,
        complementaryTransition: transition.complementaryTransition,
      };
    } else if (
      outcomeTree && outcomeTree === state.outcomeTree && !transition &&
      !CompareArrays(outcomeTree.transitionNames, state.transitionNames, true, true)
    ) {
      console.log('TransitionModifyWidget:getDerivedStateFromProps');
      const nameValidFunc = ValidateName(outcomeTree.transitionNames, validTxt);
      newState = {
        nameValidFunc,
        transitionNames: outcomeTree.transitionNames,
      };
    }
    return newState;
  }

  handleTabChange = (event, tabIdx) => {
    this.setState({ tabIdx });
  };

  handleSwipeableChange = tabIdx => {
    this.setState({ tabIdx });
  };

  handleCloseRequest = event => {
    event.preventDefault();
    this.setState({
      tabIdx: 0,
      open: false,
      title: '',
      outcomeTree: null,
      transition: null,
    });
    const { onClose, outcomeTree } = this.props;
    if (onClose) {
      onClose(event, outcomeTree);
    }
  };

  handleNameChange = (name, valid) => {
    this.setState({
      name,
      nameValid: valid,
    });
  };

  handleAnnualDurationChange = event => {
    const { value: annualDuration } = event.target;
    console.log(annualDuration);
    this.setState({
      annualDuration,
    });
  };

  handleProbabilityArrayChange = probabilityArray => {
    this.setState({
      probabilityArray,
    });
  };

  handleRedistributeArrayChange = redistributeArray => {
    this.setState({
      redistributeArray,
    });
  };

  handleSourceItemClick = (event, transition) => {
    event.preventDefault();
    this.setState({ source: transition });
  };

  handleTargetItemClick = (event, transition) => {
    event.preventDefault();
    this.setState({ target: transition });
  };

  handleSwitchChange = name => event => {
    this.setState({ [name]: event.target.checked });
  };

  handleComplementaryTransitionChange = (event, transition) => {
    const {
      complementaryTransition: currentComplementaryTransition,
    } = this.state;
    if (transition !== currentComplementaryTransition) {
      this.setState({ complementaryTransition: transition });
    } else {
      this.setState({ complementaryTransition: null });
    }
  };

  handleAddTransition = (
    outcomeTree,
    transition,
    name,
    nameValid,
    source,
    target,
    annualType,
    annualDuration,
    redistributeType,
    probabilityArray,
    redistributeArray,
    complementaryType,
    complementaryTransition,
  ) => {
    if (outcomeTree && !transition && nameValid && source && target) {
      if (outcomeTree) {
        outcomeTree.addTransition(
          name,
          source,
          target,
          annualType,
          annualDuration,
          redistributeType,
          probabilityArray,
          redistributeArray,
          complementaryType,
          complementaryTransition,
        );
      }
    }
  };

  handleModifyTransition = (
    outcomeTree,
    transition,
    name,
    nameValid,
    source,
    target,
    annualType,
    annualDuration,
    redistributeType,
    probabilityArray,
    redistributeArray,
    complementaryType,
    complementaryTransition,
  ) => {
    if (outcomeTree && transition && nameValid && source && target) {
      outcomeTree.modifyTransition(
        transition.id,
        name,
        source,
        target,
        annualType,
        annualDuration,
        redistributeType,
        probabilityArray,
        redistributeArray,
        complementaryType,
        complementaryTransition,
      );
    }
  };

  handleOKBtnClick = event => {
    event.preventDefault();

    const {
      transition,
      name,
      nameValid,
      source,
      target,
      annualType,
      annualDuration,
      redistributeType,
      probabilityArray,
      redistributeArray,
      complementaryType,
      complementaryTransition,
    } = this.state;

    const { action, outcomeTree } = this.props;

    let onOK = null;
    if (action === 'ADD_TRANSITION') {
      onOK = this.handleAddTransition;
    } else if (action === 'MODIFY_TRANSITION') {
      onOK = this.handleModifyTransition;
    }

    if (onOK) {
      onOK(
        outcomeTree,
        transition,
        name,
        nameValid,
        source,
        target,
        annualType,
        annualDuration,
        redistributeType,
        probabilityArray,
        redistributeArray,
        complementaryType,
        complementaryTransition,
      );
    }

    this.handleCloseRequest(event);
  };

  render() {
    const { classes, outcomeTree } = this.props;

    if (!outcomeTree) return null;

    const {
      tabIdx,
      open,
      title,
      name,
      nameValid,
      nameValidFunc,
      source,
      target,
      annualType,
      annualDuration,
      redistributeType,
      probabilityArray,
      redistributeArray,
      complementaryType,
      complementaryTransition,
    } = this.state;

    const healthOutcomes = outcomeTree.runOrderedHealthOutcomesAsArray;
    const complTransCandidates = outcomeTree.complTransCandidatesAsArray;
    const addBtnDisabled = !(nameValid && source && target);

    const annualTypeSwitch = (
      <Switch
        checked={annualType}
        value="annualType"
        color="primary"
      />
    );
    const redistributeTypeSwitch = (
      <Switch
        checked={redistributeType}
        value="redistributeType"
        color="primary"
      />
    );
    const complementaryTypeSwitch = (
      <Switch
        checked={complementaryType}
        onChange={this.handleSwitchChange('complementaryType')}
        disabled={complTransCandidates.length === 0}
      />
    );

    let complementaryTransitionItemList = null;
    if (complementaryType) {
      complementaryTransitionItemList = (
        <div className={classes.complDiv}>
          <Typography variant="overline">
            Source of complementary transition
          </Typography>
          <ItemsList
            itemsArray={complTransCandidates}
            listType={ElementType.TRANSITION}
            onListItemClick={this.handleComplementaryTransitionChange}
            selectedItem={complementaryTransition}
          />
        </div>
      );
    }

    return (
      <Dialog
        open={open}
        onClose={this.handleCloseRequest}
        maxWidth={false}
      >
        <form className={classes.root}>
          <Typography variant="h6">{title}</Typography>
          <Tabs
            value={tabIdx}
            indicatorColor="primary"
            textColor="primary"
            onChange={this.handleTabChange}
          >
            <Tab label="General settings" />
            <Tab label="Source : Target" />
            <Tab label="Probability table" />
            <Tab label="Redistribution table" />
          </Tabs>
          <SwipeableViews
            axis="x"
            index={tabIdx}
            onChangeIndex={this.handleSwipeableChange}
          >
            <TabContainer>
              <Grid container spacing={2}>
                <Grid item xs={6}>
                  <Autocomplete
                    options={GreekLetters}
                    onChange={(event, name) => this.handleNameChange(name, true)}
                    autoComplete
                    autoHighlight
                    disableOpenOnFocus
                    closeIcon={null}
                    freeSolo
                    value={name}
                    renderInput={params => (
                      <ValidationTextField
                        {...params}
                        label="Item name"
                        helperText={validTxt}
                        onChange={this.handleNameChange}
                        validationFunc={nameValidFunc}
                        autoFocus
                        fullWidth
                        required
                        variant="filled"
                      />
                    )}
                  />
                </Grid>
                <Grid item container xs={12}>
                  <Grid item xs={6}>
                    <FormControlLabel
                      control={complementaryTypeSwitch}
                      label="Complementary type"
                    />
                    {complementaryTransitionItemList}
                  </Grid>
                  <Grid item xs={6}>
                    <Typography
                      variant="body2"
                    >
                      <b>Complementary transition</b> represents a transition
                      whose probability of transitioning is computed as 1 minus
                      probability of transitioning its complementary source.
                      <br />
                      Typically this type of transition is used for
                      transitioning from infection to asymptomatic health
                      outcome, where its complementary source is transition from
                      infection to symptomatic health outcome.
                    </Typography>
                  </Grid>
                </Grid>
                <Grid item container xs={12}>
                  <Grid item xs={6}>
                    <FormControlLabel
                      control={annualTypeSwitch}
                      onChange={this.handleSwitchChange('annualType')}
                      label="Annual transition probability"
                    />
                    <TextField
                      style={{ paddingRight: 10 }}
                      label="Duration"
                      value={annualDuration}
                      variant="filled"
                      margin="normal"
                      onChange={this.handleAnnualDurationChange}
                      fullWidth
                      disabled={!annualType}
                    />
                  </Grid>
                  <Grid item xs={6}>
                    <Typography variant="body2" className={classes.descrText}>
                      Parameter <b>Annual transition probability</b> indicates
                      that the transition is of type &#34;Annual
                      transition probability (ATP)&#34; with parameter &#34;Duration&#34;
                      applicable. Specify &#34;-1&#34; in &#34;Duration&#34; to
                      specify remaining life expectancy.
                      <br />
                      Otherwise, it is assumed that the transition is of type
                      &#34;Lifetime transition probability (LTP)&#34;.
                    </Typography>
                  </Grid>
                </Grid>
              </Grid>
            </TabContainer>
            <TabContainer>
              <Grid container>
                <Grid item xs={6}>
                  <Typography variant="overline">Source</Typography>
                  <ItemsList
                    itemsArray={healthOutcomes}
                    listType={ElementType.HEALTH_OUTCOME}
                    onListItemClick={this.handleSourceItemClick}
                    onListSubItemClick={this.handleSourceItemClick}
                    selectedItem={source}
                  />
                </Grid>
                <Grid item xs={6}>
                  <Typography variant="overline">Target</Typography>
                  <ItemsList
                    itemsArray={healthOutcomes}
                    listType={ElementType.HEALTH_OUTCOME}
                    onListItemClick={this.handleTargetItemClick}
                    onListSubItemClick={this.handleTargetItemClick}
                    selectedItem={target}
                  />
                </Grid>
              </Grid>
            </TabContainer>
            <TabContainer>
              <Grid container>
                <Grid item xs={6}>
                  <Typography variant="body2" className={classes.descrText}>
                    <b>Probability table</b> represents the probability of
                    transitioning to the next sequelae.
                    <br />
                    It can be specified as a constant or a function.
                  </Typography>
                </Grid>
                <Grid item xs={6}>
                  <DataTable
                    dataArray={probabilityArray}
                    onChange={this.handleProbabilityArrayChange}
                    disabled={complementaryType}
                  />
                </Grid>
              </Grid>
            </TabContainer>
            <TabContainer>
              <Grid container>
                <Grid item xs={6}>
                  <FormControlLabel
                    control={redistributeTypeSwitch}
                    onChange={this.handleSwitchChange('redistributeType')}
                    label="Age-group redistribution"
                  />
                  <Typography variant="body2" className={classes.descrText}>
                    Parameter <b>Age-group redistribution</b> indicates whether
                    the cases should be redistributed across age categories or
                    not.
                    <br />
                    Redistribution array can be specified in the table to the
                    right.
                  </Typography>
                </Grid>
                <Grid item xs={6}>
                  <DataTable
                    dataArray={redistributeArray}
                    onChange={this.handleRedistributeArrayChange}
                    disabled={!redistributeType}
                  />
                </Grid>
              </Grid>
            </TabContainer>
          </SwipeableViews>
          <Divider className={classes.divider} />
          <div style={{ textAlign: 'right' }}>
            <Button color="secondary" onClick={this.handleCloseRequest}>
              Cancel
            </Button>
            <Button
              color="primary"
              disabled={addBtnDisabled}
              onClick={this.handleOKBtnClick}
              type="submit"
            >
              OK
            </Button>
          </div>
        </form>
      </Dialog>
    );
  }
}

export default TransitionModifyWidget;
