import React from 'react';
import PropTypes from 'prop-types';
import { observer } from 'mobx-react';
import TextField from '@material-ui/core/TextField';

@observer
class ValidationTextField extends React.Component {
  static propTypes = {
    onChange: PropTypes.func.isRequired,
    validationFunc: PropTypes.func,
    helperText: PropTypes.string,
  };

  static defaultProps = {
    validationFunc: () => '',
    helperText: '',
  };

  state = {
    error: false,
    errorText: '',
  };

  static getDerivedStateFromProps(props) {
    if (props.value) {
      const errorText = props.validationFunc(props.value);
      return {
        error: errorText !== props.helperText,
        errorText,
      };
    }
    return null;
  }

  handleValueChange = event => {
    const { value } = event.target;
    const {
      onChange,
      validationFunc,
      helperText: propsHelperText,
    } = this.props;

    const errorText = validationFunc(value);
    const error = errorText !== propsHelperText;
    const valid = !error;

    this.setState({ error, errorText });

    if (onChange) {
      onChange(value, valid);
    }
  };

  render() {
    const {
      onChange,
      validationFunc,
      helperText: propsHelperText,
      ...restProps
    } = this.props;
    const { error, errorText } = this.state;

    const helperText = errorText || propsHelperText;

    return (
      <TextField
        {...restProps}
        error={error}
        helperText={helperText}
        onChange={this.handleValueChange}
      />
    );
  }
}

export default ValidationTextField;
