/*
 * Application settings
 */

import { version } from '../package.json';

// DEBUG?
export const DEBUG = false;
export const INITIAL_TAB = 0;

// Application name
export const NAME = 'BCoDE Model Create';

// Application version
export const VERSION = version;

// Supported model specification file versions
export const SupportedSpecVersions = Object.freeze([1]);

// Duration of the notification displayed at the bottom of the screen [in milliseconds]
export const NOTIFICATION_DURATION = 5000;

// Web repository API address
export const WEB_REPOSITORY_ADDRESS =
  'https://nextpagesoft.net/bcode/model-create/repo/v1/';

// Element types
const [HEALTH_OUTCOME, HEALTH_STATE, TRANSITION] = [
  'HEALTH_OUTCOME',
  'HEALTH_STATE',
  'TRANSITION',
];
export const ElementType = Object.freeze({
  HEALTH_OUTCOME,
  HEALTH_STATE,
  TRANSITION,
});

// Health outcome types
const [INFECTION, ASYMPTOMATIC, ACUTE, SEQUELA, RECOVERY, DEATH] = [
  'INFECTION',
  'ASYMPTOMATIC',
  'ACUTE',
  'SEQUELA',
  'RECOVERY',
  'DEATH',
];
export const HealthOutcomeType = Object.freeze({
  INFECTION,
  ASYMPTOMATIC,
  ACUTE,
  SEQUELA,
  RECOVERY,
  DEATH,
});

// Sex categories
export const Sex = Object.freeze(['Female', 'Male']);

// Age categories
export const Age = Object.freeze([
  '0', '1-4', '5-9', '10-14', '15-19', '20-24', '25-29', '30-34', '35-39', '40-44',
  '45-49', '50-54', '55-59', '60-64', '65-69', '70-74', '75-79', '80-84', '85+',
]);

// Greek letters
export const GreekLetters = Object.freeze([
  'alpha', 'beta', 'gamma', 'delta', 'epsilon', 'zeta', 'eta', 'theta', 'iota', 'kappa',
  'lambda', 'mu', 'nu', 'xi', 'omicron', 'pi', 'rho', 'sigma', 'tau', 'upsilon', 'phi',
  'chi', 'psi', 'omega',
]);

export const HealthOutcomeAllowedOrder = [
  INFECTION,
  ASYMPTOMATIC,
  ACUTE,
  SEQUELA,
  RECOVERY,
  DEATH,
];

// Health outcomes allowed source priorities
export const HealthOutcomeAllowedSources = Object.freeze({
  INFECTION: [],
  ASYMPTOMATIC: [INFECTION, ACUTE],
  ACUTE: [INFECTION, ASYMPTOMATIC],
  SEQUELA: [ASYMPTOMATIC, ACUTE, SEQUELA],
  RECOVERY: [ASYMPTOMATIC, ACUTE, SEQUELA],
  DEATH: [ASYMPTOMATIC, ACUTE, SEQUELA],
});

// Health outcomes allowed target priorities
export const HealthOutcomeAllowedTargets = Object.freeze({
  INFECTION: [ASYMPTOMATIC, ACUTE],
  ASYMPTOMATIC: [ACUTE, SEQUELA, RECOVERY, DEATH],
  ACUTE: [ASYMPTOMATIC, SEQUELA, RECOVERY, DEATH],
  SEQUELA: [SEQUELA, RECOVERY, DEATH],
  RECOVERY: [],
  DEATH: [],
});
