import { HealthOutcomeAllowedOrder } from '../settings';

// Kahn's algorithm with custom sorting to ensure health outcome types order
// https://en.wikipedia.org/wiki/Topological_sorting
export default graph => {
  let { edges } = graph;
  const { nodes } = graph;

  // Catch information locally
  const nodesArray = Object.values(nodes);
  const transInCounts = nodesArray.reduce((counts, node) => {
    // eslint-disable-next-line no-param-reassign
    counts[node.id] = edges.filter(edge => edge.targetId === node.id).length;
    return counts;
  }, {});

  // Set of all nodes with no incoming edge
  const S = nodesArray
    .filter(node => transInCounts[node.id] === 0)
    // Sort ascendingly
    .sort(
      (nodeA, nodeB) =>
        HealthOutcomeAllowedOrder.indexOf(nodeA.type) -
        HealthOutcomeAllowedOrder.indexOf(nodeB.type),
    )
    .map(node => node.id);
  // Empty list that will contain the sorted elements
  const L = [];
  while (S.length > 0) {
    // Remove a node n from S
    const n = S.shift();
    // Add n to tail of L
    L.push(n);
    // For each node m with an edge e from n to m do...
    const ms = edges
      .filter(edge => edge.sourceId === n)
      // Sort descendingly
      .sort(
        (edgeA, edgeB) =>
          HealthOutcomeAllowedOrder.indexOf(nodes[edgeB.targetId].type) -
          HealthOutcomeAllowedOrder.indexOf(nodes[edgeA.targetId].type),
      );
    for (let i = 0; i < ms.length; i += 1) {
      const m = ms[i].targetId;
      // ...remove edge from n to m
      edges = edges.filter(
        edge => !(edge.sourceId === n && edge.targetId === m),
      );
      // If m has no other incoming edges then insert m into S
      if (edges.every(edge => edge.targetId !== m)) {
        S.unshift(m);
      }
    }
  }

  const result = {
    order: L,
    success: edges.length === 0,
  };

  return result;
};
