/*
Copyright (c) 2014-2015, University of Applied Sciences Augsburg
All rights reserved.

Redistribution and use in source and binary forms, with or without
modification, are permitted provided that the following conditions are met:
    * Redistributions of source code must retain the above copyright
      notice, this list of conditions and the following disclaimer.
    * Redistributions in binary form must reproduce the above copyright
      notice, this list of conditions and the following disclaimer in the
      documentation and/or other materials provided with the distribution.
    * Neither the name of the University of Applied Sciences Augsburg nor the
      names of its contributors may be used to endorse or promote products
      derived from this software without specific prior written permission.

THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND
ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS
BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE
OODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
UT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
*/

#ifndef _ganalytics_
#define _ganalytics_

#include <QObject>
#include <QVariantMap>

class QNetworkAccessManager;

class GAnalytics : public QObject
{
    Q_OBJECT
    Q_ENUMS(LogLevel)
    Q_PROPERTY(LogLevel logLevel READ logLevel WRITE setLogLevel NOTIFY logLevelChanged)
    Q_PROPERTY(QString viewportSize READ viewportSize WRITE setViewportSize NOTIFY viewportSizeChanged)
    Q_PROPERTY(QString language READ language WRITE setLanguage NOTIFY languageChanged)
    Q_PROPERTY(QString trackingID READ trackingID WRITE setTrackingID NOTIFY trackingIDChanged)
    Q_PROPERTY(QString userID READ userID WRITE setUserID NOTIFY userIDChanged)
    Q_PROPERTY(int sendInterval READ sendInterval WRITE setSendInterval NOTIFY sendIntervalChanged)
    Q_PROPERTY(bool isSending READ isSending NOTIFY isSendingChanged)

public:
    explicit GAnalytics(QObject *parent = nullptr);
    explicit GAnalytics(const QString &trackingID, QObject *parent = nullptr);
    ~GAnalytics();

public:
    enum LogLevel
    {
        Debug,
        Info,
        Error,
        None
    };

    void setLogLevel(LogLevel logLevel);
    LogLevel logLevel() const;

    // Getter and Setters
    void setViewportSize(const QString& viewportSize);
    QString viewportSize() const;

    void setLanguage(const QString& language);
    QString language() const;

    void setTrackingID(const QString& trackingID);
    QString trackingID() const;

    void setUserID(const QString& userID);
    QString userID() const;

    void setSendInterval(int milliseconds);
    int sendInterval() const;

    void startSending();
    bool isSending() const;

    /// Get or set the network access manager. If none is set, the class creates its own on the first request
    void setNetworkAccessManager(QNetworkAccessManager* networkAccessManager);
    QNetworkAccessManager* networkAccessManager() const;

public slots:
    void sendScreenView(const QString& screenName,
                        const QVariantMap& customValues = QVariantMap());
    void sendEvent(const QString& category,
                   const QString& action,
                   const QString& label = QString(),
                   const QVariant& value = QVariant(),
                   const QVariantMap& customValues = QVariantMap());
    void sendException(const QString& exceptionDescription,
                       bool exceptionFatal = true,
                       const QVariantMap& customValues = QVariantMap());
    void startSession();
    void endSession();

signals:
    void logLevelChanged();
    void viewportSizeChanged();
    void languageChanged();
    void trackingIDChanged();
    void userIDChanged();
    void sendIntervalChanged();
    void isSendingChanged(bool isSending);

private:
    class Private;
    Private* d;

    friend QDataStream& operator<<(QDataStream& outStream, const GAnalytics& analytics);
    friend QDataStream& operator>>(QDataStream& inStream, GAnalytics& analytics);
};

QDataStream& operator<<(QDataStream& outStream, const GAnalytics& analytics);
QDataStream& operator>>(QDataStream& inStream, GAnalytics& analytics);

#endif // _ganalytics_
