//==========================================================================================================================
//  'ArrayHelpers'
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_ArrayHelpers_
#define _fn_ArrayHelpers_

#include "fn/globals.h"
#include "fn/CustomDataTypes.h"
#include "fn/InputFactor.h"
#include "fn/DataArray.h"

namespace fn {

// Append meta data to variant map
//
void AppendMetaDataToVariantMap(QVariantMap& variantMap, const fn::StringToStringMap& metaData);

// Convert a formula (string) array to Variant Map
//
QVariantMap FormulaArrayToVariantMap(const fn::StringBaseArray& formulaArray, const fn::ConstInputFactorPtrList& inputFactorList);

// Convert a numeric array to variant
//
QVariantMap NumericArrayToVariantMap(const fn::NumericBaseArray& numericArray, const fn::ConstInputFactorPtrList& inputFactorList);

// Convert a boolean array to variant
//
QVariantMap BoolArrayToVariantMap(const fn::BoolBaseArray& numericArray, const fn::ConstInputFactorPtrList& inputFactorList);

// Convert a variant to formula (string) array
//
fn::StringBaseArray VariantMapToFormulaArray(const QVariantMap& variantArray, const fn::ConstInputFactorPtrList& inputFactorList);

// Convert a variant to meta data map
//
fn::StringToStringMap VariantMapToMetaData(const QVariantMap& variantArray);

// Convert data array to Variant List
QVariantList DataArrayToVariantList(const fn::DataArray& array);

// Convert meta data array to Variant List
QVariantList MetaDataToVariantList(const fn::StringToStringMap& metaData);

// Convert a formula (string) array to Variant List
//
QVariantList FormulaArrayToVariantList(const fn::StringBaseArray& formulaArray,
                                       const fn::ConstInputFactorPtrList& inputFactorList);

// New methods, use these, instead of the ones above -----------------------------------------------------------------------

// Convert a data array to variant map
//
// 'dataArray' - [in] Data array to export
// 'dataArrayType' - [in] Array type to export (Formula or Numeric)
//
QVariantMap DataArrayToVariantMap(const fn::DataArray& dataArray,
                                  const fn::DataArray::ArrayType& dataArrayType,
                                  const fn::DataArray::FormulaType& formulaType);

// Convert a variant map to data array
//
fn::DataArray VariantMapToDataArray(const QVariantMap& variantArray, const fn::ConstInputFactorPtrList& inputFactorList);

}  // namespace fn

#endif  // _fn_ArrayHelpers_


