//==========================================================================================================================
//  Contains custom data types based on core Qt data types
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_CustomDataTypes_
#define _fn_CustomDataTypes_

#include <QList>
#include <QStringList>
#include <QMap>

#include "fn/TransitionProbabilityComplement.h"

namespace fn {

class OutcomeTreeTemplate;
class DataArray;
class InputFactor;
class HealthOutcome;
class HealthState;
class TransitionProbability;
class DiseaseModel;

// Vector of boolean
typedef QVector<bool> BooleanVector;

// List of models
typedef QList<fn::DiseaseModel> DiseaseModelList;

// Country => DataArray
typedef QMap<fn::InputType::Country, fn::DataArray> CountryToInputDataMap;
// Disease => DataArray
typedef QMap<fn::InputType::Disease, fn::DataArray> DiseaseToInputDataMap;
// Country => Disease => DataArray
typedef QMap<fn::InputType::Country, DiseaseToInputDataMap> CountryToDiseaseToInputDataMap;

// List of input factors.
typedef QList<fn::InputFactor> InputFactorList;

//+ Map of input factor lists - keyed on disease.
typedef QMap<fn::InputType::Disease, InputFactorList> DiseaseToInputFactorListMap;

// List of health outcomes
typedef QList<fn::HealthOutcome> HealthOutcomeList;

typedef QMap<QString, fn::HealthOutcome> StringToHealthOutcomeMap;
typedef QMap<QString, fn::HealthState> StringToHealthStateMap;

// List of health states
typedef QList<fn::HealthState> HealthStateList;

// List of transition probabilities
typedef QList<fn::TransitionProbability> TransitionProbabilityList;
typedef QList<fn::TransitionProbabilityComplement> TransitionProbabilityComplementList;

// String to string map
typedef QMap<QString, QString> StringToStringMap;

// String to string list map
typedef QMap<QString, QStringList> StringToStringListMap;

// String to outcome tree template
typedef QMap<QString, fn::OutcomeTreeTemplate> StringToOutcomeTreeTemplateMap;

typedef QMap<fn::InputType::Disease, StringToStringMap> DiseaseToStringToStringMap;
typedef QMap<fn::InputType::Country, DiseaseToStringToStringMap> CountryToDiseaseToStringToStringMap;

typedef QMap<fn::InputType::Country, QMap<fn::InputType::Disease, QMap<QString, StringToStringListMap > > > DistrInfoType;
typedef QMap<fn::InputType::Country, QMap<fn::InputType::Disease, QMap<QString, QMap<QString, StringToStringListMap > > > > DistrInfoTypeForTransProb;

// String => DataArray
typedef QMap<QString, fn::DataArray> StringTableType;
// Disease => String => DataArray
typedef QMap<fn::InputType::Disease, StringTableType> DiseaseTableType;
// Country => Disease => String => DataArray
typedef QMap<fn::InputType::Country, DiseaseTableType> TableType;

// String => String => DataArray
typedef QMap<QString, StringTableType> StringToStringTableType;
// Disease => String => String => DataArray
typedef QMap<fn::InputType::Disease, StringToStringTableType> DiseaseToStringToStringTableType;
// Country => Disease => String => String => DataArray
typedef QMap<fn::InputType::Country, DiseaseToStringToStringTableType> CountryToDiseaseToStringToStringTableType;

}  // namespace fn

#endif  // _fn_CustomDataTypes_
