//==========================================================================================================================
//  Implementation file for "DataArray.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "DataValue.h"

#include "fn/formulaParserHelper.h"

namespace fn {

DataValue::DataValue(const QString& formula):
    m_formula(formula),
    m_value(0),
    m_parser()
{
    initializeParser();
}

DataValue::DataValue():
    m_formula("0"),
    m_value(0),
    m_parser()
{
    initializeParser();
}

DataValue::~DataValue()
{

}

void DataValue::setFormula(const QString& formula)
{
    m_formula = formula;
    initializeParser();
}

const fn::SampleVal& DataValue::getValue()
{
    evaluateFormula();
    return m_value;
}

void DataValue::initializeParser()
{
    m_parser.SetArgSep(',');
    m_parser.SetDecSep('.');
    fn::defineAdditionalFunctions(&m_parser);

    try {
        m_parser.SetExpr(m_formula.toStdWString());
    } catch (mu::Parser::exception_type&) {
        m_formula.replace(QRegExp("*"), "0");
        m_parser.SetExpr(m_formula.toStdWString());
        UT_THROW(QString("Formula cannot be parsed.\nInstead it has been set to evaluate to 0."))
    }
}

void DataValue::evaluateFormula()
{
    m_value = m_parser.Eval();
}

}  // namespace fn
