//==========================================================================================================================
//  'DiseaseTemplateMgr' class
//			This class implements manager of disease templates
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_DiseaseTemplateMgr_
#define _fn_DiseaseTemplateMgr_

#include "fn/DiseaseTemplate.h"

namespace fn {

class DiseaseTemplateMgr
{
public:
    // Constructor.
    //
    DiseaseTemplateMgr();

    // Destructor.
    //
    virtual ~DiseaseTemplateMgr();

    // Add disease template
    void addDiseaseTemplate(const fn::InputType::Disease disease, const fn::DiseaseTemplate& diseaseTemplate);

    // Clear internal data
    //
    void clearData();

    // Get disease templates map
    //
    const fn::DiseaseToDiseaseTemplateMap& getDiseaseTemplates() const;

    // Get disease template for a given disease
    //
    // 'disease' - [in] Disease
    //
    // Return - disease template
    fn::DiseaseTemplate& getDiseaseTemplate(const fn::InputType::Disease& disease);

    // Set disease templates map
    //
    // 'diseaseTemplatesMap' - [in] Disease templates map keyed on disease
    //
    void setDiseaseTemplates(const fn::DiseaseToDiseaseTemplateMap& diseaseTemplatesMap);

    // Check if a given disease exists
    //
    // 'disease' - [in] Diseas to be checked
    //
    // Return - TRUE only if disease exists
    bool diseaseExists(const fn::InputType::Disease& disease) const;

private:

    fn::DiseaseToDiseaseTemplateMap m_diseaseTemplates;
};


}  // namespace fn

#endif  // _fn_DiseaseTemplateMgr_

