//==========================================================================================================================
//  Implementation file for "FormulaChecker.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2013 NextPage Software
//==========================================================================================================================

#include "FormulaChecker.h"

#include "fn/formulaParserHelper.h"

namespace fn {

bool FormulaChecker::checkFormula(const QString& formula)
{
    bool allFine = true;

    FormulaChecker checker;
    checker.m_parser.SetExpr(formula.toStdWString());
    try {
        checker.m_parser.Eval();
    } catch (mu::Parser::exception_type&) {
        allFine = false;
    }

    return allFine;
}

bool FormulaChecker::checkFormulas(const fn::StringBaseArray& formulaArray, fn::BoolBaseArray& formulaCheck)
{

    formulaCheck.clear();
    bool allFine = true;
    FormulaChecker checker;
    fn::StringBaseArray::const_iterator it = formulaArray.constBegin();
    for (; it != formulaArray.end(); ++it) {
        const QString formula = *it;

        // Check 1 - see if the parser can understand the formula
        checker.m_parser.SetExpr(formula.toStdWString());
        int numFormulas;
        try {
            checker.m_parser.Eval(numFormulas);
        } catch (mu::Parser::exception_type&) {
            formulaCheck.push_back(false);
            allFine = false;
            continue;
        }

        // Check 2 - see if the parser does not understand the formula as it, in fact, contains multiple formulas concatenated
        if (numFormulas > 1) {
            formulaCheck.push_back(false);
            allFine = false;
            continue;
        }

        // If we got till here, then the checks are fine
        formulaCheck.push_back(true);
    }

    return allFine;

}

FormulaChecker::FormulaChecker() :
    m_parser()
{
    m_parser.SetArgSep(',');
    m_parser.SetDecSep('.');
    fn::defineAdditionalFunctions(&m_parser);
}


FormulaChecker::~FormulaChecker()
{
    
}

}  // namespace fn

