//==========================================================================================================================
//  Implementation file for "ImportExportHelpers.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "fn/ImportExportHelpers.h"

#include <QJsonObject>
#include <QJsonArray>

#include "fn/ArrayHelpers.h"
#include "fn/DataArray.h"
#include "fn/InputFactorMgr.h"

namespace fn {

fn::StringToStringMap readStringMapData(ut::Excel& excelApp, const int startCellRow, int& lastRow)
{
    fn::StringToStringMap result;

    lastRow = startCellRow;
    QString key = excelApp.getStringCellValue(lastRow, 1).toLower();
    // Parse rows
    while (!key.isEmpty()) {
        QString value = excelApp.getStringCellValue(lastRow, 2);
        result.insert(key, value);
        key = excelApp.getStringCellValue(++lastRow, 1).toLower();
    }
    lastRow--;
    return result;
}

fn::DataArray readInputDataFromExcel(QAxObject* headerCell, ut::Excel& excelApp, int& lastRow)
{
    const QString kMissingValue = "0";

    QString name = ut::Excel::getStringCellValue(headerCell);

    // Set the row presumably holding the meta data ("Source")
    int metaDataStartRow = ut::Excel::getCellRow(headerCell) + 2;
    QString firstString = excelApp.getCellValue(metaDataStartRow, 1).toString();
    fn::StringToStringMap metaData;
    if (firstString.toLower() != "sex") {
        metaData = readStringMapData(excelApp, metaDataStartRow, lastRow);
    } else {
        lastRow = metaDataStartRow - 2;
    }

    // Get input table range
    lastRow += 2;
    QAxObject* formulaRange = excelApp.getRange(lastRow, 1);
    QVariantList formulaVariantArray = excelApp.getRangeValues(formulaRange).toList();

    int numCols = 0;
    int numRows = formulaVariantArray.length();
    if (numRows > 0) {
        numCols = formulaVariantArray.at(0).toList().length();
    }

    // Process input factor 1
    QVariantList variantRow = formulaVariantArray.at(0).toList();
    QString inputFactorName1 = variantRow.at(0).toString();
    fn::StringBaseArray inputFactorValues1;
    for (int j = 1; j < numCols; ++j) {
        QString inputFactorValue = variantRow.at(j).toString();
        if (!inputFactorValue.isEmpty()) {
            inputFactorValues1.append(inputFactorValue);
        }
    }
    fn::InputFactor inputFactor1(inputFactorName1, inputFactorValues1);

    // Process input factor 2 and input data
    fn::StringBaseArray formulaArray;
    QString inputFactorName2 = formulaVariantArray.at(1).toList().at(0).toString();
    fn::StringBaseArray inputFactorValues2;
    for (int i = 2; i < numRows; ++i) {
        QVariantList variantRow = formulaVariantArray.at(i).toList();
        inputFactorValues2.append(variantRow.at(0).toString());
        for (int j = 1; j < numCols; ++j) {
            QString formula = variantRow.at(j).toString();
            if (formula.isEmpty()) {
                formula = kMissingValue;
            }
            formulaArray.append(formula);
        }
    }
    fn::InputFactor inputFactor2(inputFactorName2, inputFactorValues2);

    fn::InputFactorList inputFactorList;
    // Store in reverse order!
    inputFactorList.append(inputFactor2);
    inputFactorList.append(inputFactor1);

    fn::Dims dims;
    dims.append(inputFactor1.getNumCategories());
    dims.append(inputFactor2.getNumCategories());

    lastRow += static_cast<int>(numRows) - 1;

    fn::DataArray result = fn::DataArray(inputFactorList, name, metaData, formulaArray);

    return result;
}


fn::DataArray readInputDataFromJSON(const fn::InputFactorList& inputFactorList, const QJsonObject& arrayJson, const QString& name)
{
    fn::StringToStringMap metaData;
    fn::StringBaseArray formulaArray;
    QJsonArray valsArrayJson = arrayJson.value(QStringLiteral("vals")).toArray();
    for (int i = 0; i < valsArrayJson.size(); ++i) {
        QJsonArray valsRowJson = valsArrayJson.at(i).toArray();
        for (int j = 0; j < valsRowJson.size(); ++j) {
            QString val = valsRowJson.at(j).toString();
            if (val == "") {
                val = "0";
            }
            formulaArray.append(val);
        }
    }

    fn::DataArray result = fn::DataArray(inputFactorList, name, metaData, formulaArray);

    return result;
}

void writeInputDataToExcel(QAxObject* headerCell, ut::Excel& excelApp, const fn::DataArray& array, int& lastRow)
{
    QVariantList variantArray = fn::DataArrayToVariantList(array);

    int dataStartRow = ut::Excel::getCellRow(headerCell);
    excelApp.setRangeValues(QPoint(dataStartRow, 1), variantArray, 3, dataStartRow);

    lastRow = dataStartRow;
}

}  // namespace fn
