//==========================================================================================================================
//  'ImportExportHelpers' utilities
//
//  Written by Daniel Lewandowski
//  Copyright @ 2012 NextPage Software
//==========================================================================================================================

#ifndef _fn_ImportExportHelpers_
#define _fn_ImportExportHelpers_

#include "ut/Excel.h"
#include "fn/globals.h"
#include "fn/CustomDataTypes.h"
#include "fn/InputFactor.h"

namespace fn {

class DataArray;
class InputFactorMgr;

// Read string data
//
// 'excelApp' - [in] Instance of Excel holding the data
// 'startCellRow' - [in] Starting row of the map data (column 1 assumed)
// 'lastRow' - [out] Index of the last row of the meta data in the spreadsheet
//
// Return - string data
fn::StringToStringMap readStringMapData(ut::Excel& excelApp, const int startCellRow, int& lastRow);

// Reads input data table.
//
// 'headerCell' - [in] Header cell of the data
// 'excelApp' - [in] Instance of Excel holding the data
// 'lastRow' - [out] Index of the last row of the input table in the spreadsheet
//
fn::DataArray readInputDataFromExcel(QAxObject* headerCell, ut::Excel& excelApp, int& lastRow);

// Reads input data table.
//
// 'inputFactorList' - [in] Input factor list
// 'arrayJson' - [in] JSON object with the data array content
// 'name' - [in] Name
//
fn::DataArray readInputDataFromJSON(const fn::InputFactorList& inputFactorList, const QJsonObject& arrayJson, const QString& name = "");

// Writes input data table
//
// 'headerCell' - [in] Header cell of the data (starting cell)
// 'excelApp' - [in] Instance of Excel holding the data
// 'array' - [in] Data array to export
//
void writeInputDataToExcel(QAxObject* headerCell, ut::Excel& excelApp, const fn::DataArray& array, int& lastRow);

}  // namespace fn

#endif  // _fn_ImportExportHelpers_


