//==========================================================================================================================
//  'Infection' Infection
//			This class represents the top event in the outcome tree - infection
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_Infection_
#define _fn_Infection_

#include "fn/TransitionProbabilityBase.h"
#include "fn/CustomDataTypes.h"
#include "fn/SequelaBase.h"

namespace fn {

class DiseaseModel;

class Infection : public fn::SequelaBase
{
public:
    // Constructor.
    //
    Infection(const fn::OutcomeTreeNode::ID id, const QString& caption, const fn::Dims& dims, const fn::HealthOutcome& parent);

    // Destructor.
    //
    virtual ~Infection();

    // Run calculations
    //
    void runCalculation(const fn::NumericBaseArray& coverage, const double timeDiscountRate, const fn::NumericBaseArray& underestimation);

    // Get raw incidence
    //
    // Return - array of raw incidence
    const fn::DataArray& getRawCasesCount() const;

    // Set raw incidence
    //
    // 'rawCasesCount' - [in] Uncorrected cases count
    //
    void setRawCasesCount(const fn::DataArray& rawCasesCount);

    // Get life expectancy
    //
    // Return - array of life expectancy
    const fn::DataArray& getLifeExpectancy() const;

    // Set lif expectancy
    //
    // 'lifeExp' - [in] Life expectancy
    //
    void setLifeExpectancy(const fn::DataArray& lifeExp);

    // Initialize sample containers to the specified size
    //
    // 'sampleDims' - [in] Dimensions of the sample container
    //
    void initializeSampling(const fn::Dims& sampleDims);

    // Clear sample
    //
    void clearSample();

    // Formula parser
    void createAllFormulaParsers();
    void destroyAllFormulaParsers();
    void evaluateAllFormulas();

    // Find alpha transition
    void determineAlpha();

    // Get alpha
    const fn::TransitionProbabilityBase* getAlpha() const;

private:

    // Compute corrected case count
    //
    // 'alpha' - [in] Alpha transition probability
    // 'coverage' - [in] Coverage
    // 'underestimation' - [in] Underestimation
    //
    // Return - corrected total incidence
    fn::NumericBaseArray computeCorrectedCasesCount(const fn::NumericBaseArray& alpha, const fn::NumericBaseArray& coverage,
                                                    const fn::NumericBaseArray& underestimation);

    // ----------------------------------------------
    // Reference data

    // Life expectancy
    fn::DataArray m_lifeExp;

    // Input raw case count (symptomatic, before corrections)
    fn::DataArray m_rawCasesCount;

    // ----------------------------------------------
    // Sample storage
    fn::NumericBaseArray m_sampleCoverage;
    fn::NumericBaseArray m_sampleUnderestimation;

    fn::TransitionProbabilityBase* m_alpha;
};

}  // namespace fn

#endif  // _fn_Infection_
