//==========================================================================================================================
//  'InputFactor' class
//      This class implements an input factor class
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_InputFactor_
#define _fn_InputFactor_

#include "fn/globals.h"

namespace fn {

class InputFactor
{

public:

    // Default constructor
    InputFactor();

    // Constructor
    //
    // 'name' - [in] Name
    // 'categories' - [in] List of categories
    //
    InputFactor(const QString& name, const QStringList& categories);

    // Destructor
    //
    virtual ~InputFactor();

    // Get the name.
    //
    // Return - name
    const QString getName() const;

    // Set the name.
    //
    // 'name' - [in] Name
    //
    void setName(const QString name);

    // Get the number of categories
    //
    // Return - Number of categories
    size_t getNumCategories() const;

    // Get categories list
    //
    // Return - list of categories
    const QStringList& getCategoryList() const;

    // Set categories list
    //
    // 'categories' - [in] List of categories
    //
    void setCategoryList(const QStringList& categories);

private:

    // Name
    QString m_name;

    // Categories
    QStringList m_categories;

};

// List of constant pointers to input factors.
typedef QList<const fn::InputFactor*> ConstInputFactorPtrList;

} // namespace fn

#endif // _fn_InputFactor_
