//==========================================================================================================================
//  Implementation file for "InputFactorMgr.h"
//  Copyright @ 2011 Daniel Lewandowski, NextPage Software
//==========================================================================================================================

#include <QMap>
#include <QList>

#include "InputFactorMgr.h"

namespace fn {

InputFactorMgr::InputFactorMgr():
    m_inputFactors()
{

}

InputFactorMgr::~InputFactorMgr()
{

}

void InputFactorMgr::addInputFactor(const InputType::Disease& disease, const QString& inputFactorName,
                                    const QStringList& inputFactorCategories)
{
    fn::InputFactor newInputFactor(inputFactorName, inputFactorCategories);
    m_inputFactors[disease].append(newInputFactor);
}

const fn::InputFactor* InputFactorMgr::getInputFactor(const InputType::Disease& disease, const QString& name) const
{
    const fn::InputFactor* result = nullptr;
    if (m_inputFactors.contains(disease)) {
        const InputFactorList& inputFactorList = m_inputFactors[disease];
        QListIterator<fn::InputFactor> inputFactorIter(inputFactorList);
        while (inputFactorIter.hasNext()) {
            const fn::InputFactor& inputFactor = inputFactorIter.next();
            if (inputFactor.getName() == name) {
                result = &inputFactor;
                break;
            }
        }
    }
    return result;
}

const fn::ConstInputFactorPtrList InputFactorMgr::getInputFactorList(const InputType::Disease& disease)
{
    fn::ConstInputFactorPtrList inputFactors;
    if (m_inputFactors.contains(disease)) {
        const InputFactorList& list = m_inputFactors[disease];
        QListIterator<fn::InputFactor> inputFactorIter(list);
        inputFactorIter.toFront();
        while (inputFactorIter.hasNext()) {
            inputFactors.append(&(inputFactorIter.next()));
        }
    }
    return inputFactors;
}

void InputFactorMgr::setInputFactors(const fn::DiseaseToInputFactorListMap& inputFactors)
{
    m_inputFactors = inputFactors;
}

const fn::DiseaseToInputFactorListMap& InputFactorMgr::getInputFactors() const
{
    return m_inputFactors;
}

void InputFactorMgr::clearData()
{
    m_inputFactors.clear();
}

} // namespace fn
