//==========================================================================================================================
//  'InputFactorMgr' class
//      This class manages input factors.
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_InputFactorMgr_
#define _fn_InputFactorMgr_

#include "fn/globals.h"
#include "fn/CustomDataTypes.h"
#include "fn/InputFactor.h"

namespace fn {

class InputFactorMgr
{

public:

    // Constructor
    //
    explicit InputFactorMgr();

    // Destructor
    //
    virtual ~InputFactorMgr();

    // Add input factor
    //
    // 'disease' - [in] Disease for which this input factor is specified
    // 'inputFactorName' - [in] Input factor name.
    // 'inputFactorCategories' - [in] Input factor categories.
    //
    void addInputFactor(const fn::InputType::Disease& disease,
                        const QString& inputFactorName,
                        const QStringList& inputFactorCategories);

    // Get input factor
    //
    // 'disease' - [in] Disease to which the input factor belongs
    // 'inputFactorName' - [in] Input factor name
    //
    // Return - Pointer to input factor, NULL if not found
    const fn::InputFactor* getInputFactor(const fn::InputType::Disease& disease,
                                          const QString& name) const;

    // Get input factor list for a specified disease as a final set for calculations
    //
    // 'disease' - [in] Disease
    //
    // Return - List of input factor pointers.
    const fn::ConstInputFactorPtrList getInputFactorList(const fn::InputType::Disease& disease);

    // Set input factors map
    //
    // 'inputFactors' - [in] Input factors map keyed on country
    //
    void setInputFactors(const fn::DiseaseToInputFactorListMap& inputFactors);

    // Get a list of all input factors
    //
    // Return - Reference to the input factors list
    const fn::DiseaseToInputFactorListMap& getInputFactors() const;

    // Clear internal data
    //
    void clearData();

private:

    // Local map of input factor maps keyed on country.
    fn::DiseaseToInputFactorListMap m_inputFactors;
};

} // namespace fn

#endif // _fn_InputFactorMgr_
