//==========================================================================================================================
//  'JSONDiseaseTemplate' class
//
//  Written by Daniel Lewandowski
//  Copyright @ 2019 NextPage Software
//==========================================================================================================================

#ifndef _fn_JSONDiseaseTemplate_
#define _fn_JSONDiseaseTemplate_

#include <QMap>
#include <QJsonObject>

#include "fn/globals.h"
#include "fn/CustomDataTypes.h"
#include "fn/DataArray.h"
#include "fn/OutcomeTreeTemplate.h"

namespace fn {

class JSONDiseaseTemplate
{
public:
    enum ParamType {Severity, Duration, Proportion, Probability, Redistribution};

    // Constructor.
    //
    JSONDiseaseTemplate();

    // Destructor.
    //
    virtual ~JSONDiseaseTemplate();

    // Get/set JSON object
    void setJsonObject(const QJsonObject& jsonObj);
    const QJsonObject& getJsonObject() const;

    // Get disease
    //
    fn::InputType::Disease getDisease() const;

    // Get caption
    //
    QString getCaption() const;

    // Get indicator that this is congenital
    //
    bool getIsCongenital() const;

    // Get the total number of outcome tree IDs
    //
    int getNumOutcomeTreeTemplates() const;

    // Get outcome tree ID
    //
    const QString getOutcomeTreeTemplateID(const int index) const;

    // Get outcome tree template
    //
    const fn::OutcomeTreeTemplate getOutcomeTreeTemplate(const QString& id) const;

    // Get parameter
    //
    // 'outcomeTreeId' - [in] Id of the outcome tree
    // 'healthStateId' - [in] Id of the health state
    // 'paramType' - [in] Parameter type (Severity, Duration, Proportion)
    //
    fn::DataArray getParameter(const QString& outcomeTreeId, const QString& elementId, const ParamType& paramType) const;

    // Get input factor list
    //
    // Return - List of input factor pointers
    const fn::ConstInputFactorPtrList getConstInputFactorPtrList() const;

    // Get input factor list
    //
    // Return - List of input factor
    const fn::InputFactorList& getInputFactorList() const;

    // Get outcome tree mapping
    //
    // Return - outcome tree mapping DataArray
    const fn::DataArray& getOutcomeTreeMapping() const;

    // Get raw cases count
    //
    // Return - raw cases count DataArray
    const fn::DataArray& getRawCasesCount() const;

private:

    void parseJsonObject();

    QJsonObject m_jsonObj;

    StringToOutcomeTreeTemplateMap m_outcomeTreeTemplateMap;

    QStringList m_outcomeTreeIdList;

    // Local map of Severity parameters
    fn::StringToStringTableType m_severityParameters;

    // Local map of Duration parameters
    fn::StringToStringTableType m_durationParameters;

    // Local map of Proportion parameters
    fn::StringToStringTableType m_proportionParameters;

    // Local map of Probability parameters
    fn::StringToStringTableType m_probabilityParameters;

    // Local map of Redistribution parameters
    fn::StringToStringTableType m_redistributionParameters;

    // Input factor list
    fn::InputFactorList m_inputFactorList;

    // Outcome tree mapping
    fn::DataArray m_outcomeTreeMapping;

    // Average number of cases
    fn::DataArray m_rawCasesCount;
};

typedef QMap<QString, fn::JSONDiseaseTemplate> DiseaseIdToJSONDiseaseTemplateMap;

}  // namespace fn

#endif  // _fn_JSONDiseaseTemplate_


