//==========================================================================================================================
//  Implementation file for "InputDataMgr2.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2019 NextPage Software
//==========================================================================================================================

#include "JSONInputDataMgr.h"
#include <QDir>
#include <QJsonDocument>
#include <QJsonObject>
#include <QtDebug>

namespace fn {

#if (defined(FINAL))
    const QString kModelsDataPath = "./models/";
#else
    const QString kModelsDataPath = "./../../../../../_models/";
#endif

JSONInputDataMgr::JSONInputDataMgr() :
    m_appPath(""),
    m_modelFileInfos(),
    m_diseaseTemplates()
{

}

JSONInputDataMgr::~JSONInputDataMgr()
{

}

void JSONInputDataMgr::setDirPath(const QString& dirPath)
{
    m_appPath = dirPath;
    importReferenceData();
}

void JSONInputDataMgr::importReferenceData()
{
    QDir dir(m_appPath + kModelsDataPath);
    dir.setFilter(QDir::Files | QDir::Readable | QDir::NoSymLinks);
    dir.setSorting(QDir::Name);
    m_modelFileInfos = dir.entryInfoList();

    readData();
}

void JSONInputDataMgr::setFilePaths(const QFileInfoList& filePaths)
{
    m_modelFileInfos = filePaths;

    readData();
}

QStringList JSONInputDataMgr::getDiseaseIds() const
{
    return m_diseaseTemplates.keys();
}

void JSONInputDataMgr::readData()
{
    for (int i = 0; i < m_modelFileInfos.length(); ++i) {
        const QFileInfo& modelDataFileInfo = m_modelFileInfos.at(i);

        if (modelDataFileInfo.exists()) {
            QFile file(modelDataFileInfo.filePath());
            file.open(QIODevice::ReadOnly | QIODevice::Text);
            QString val = file.readAll();
            file.close();
            QJsonDocument jsonDoc = QJsonDocument::fromJson(val.toUtf8());
            QJsonObject jsonObj = jsonDoc.object();
            JSONDiseaseTemplate diseaseTemplate;
            diseaseTemplate.setJsonObject(jsonObj);
            fn::InputType::Disease disease = diseaseTemplate.getDisease();
            m_diseaseTemplates.insert(disease, diseaseTemplate);
        }
    }
}

QStringList JSONInputDataMgr::getDiseaseNames() const
{
    QStringList result;
    fn::DiseaseIdToJSONDiseaseTemplateMap::const_iterator iter;
    for (iter = m_diseaseTemplates.constBegin(); iter != m_diseaseTemplates.constEnd(); ++iter) {
        result.append(iter.value().getCaption());
    }

    return(result);
}

QVariantList JSONInputDataMgr::getDiseaseCaptions() const
{
    QVariantList result;
    fn::DiseaseIdToJSONDiseaseTemplateMap::const_iterator iter;
    for (iter = m_diseaseTemplates.constBegin(); iter != m_diseaseTemplates.constEnd(); ++iter) {
        fn::JSONDiseaseTemplate tmpl = iter.value();
        const QString diseaseId = tmpl.getDisease();
        const QString diseaseName = tmpl.getCaption();
        QVariantMap diseaseMap;
        diseaseMap.insert("id", diseaseId);
        diseaseMap.insert("disease", diseaseName);
        result.append(diseaseMap);
    }

    return(result);
}

fn::JSONDiseaseTemplate& JSONInputDataMgr::getDiseaseTemplate(const fn::InputType::Disease& disease)
{
    return m_diseaseTemplates[disease];
}

bool JSONInputDataMgr::diseaseDataExist(const InputType::Disease& disease) const
{
    return m_diseaseTemplates.contains(disease);
}

}
