//==========================================================================================================================
//  'JSONInputDataMgr' class
//			This class manages input data as JSON objects
//
//  Written by Daniel Lewandowski
//  Copyright @ 2019 NextPage Software
//==========================================================================================================================

#ifndef _fn_JSONInputDataMgr_
#define _fn_JSONInputDataMgr_

#include<QObject>
#include<QFileInfoList>
#include "JSONDiseaseTemplate.h"
#include "InputFactorMgr.h"

namespace fn {

class JSONInputDataMgr : public QObject
{

    Q_OBJECT

public:

    // Constructor
    //
    explicit JSONInputDataMgr();

    // Destructor.
    //
    virtual ~JSONInputDataMgr();

    // Import reference data from JSON file
    //
    void importReferenceData();

public slots:

    // Set path to models files folder
    //
    void setDirPath(const QString& dirPath);

    // Set paths to models files
    //
    void setFilePaths(const QFileInfoList& filePaths);

    // Get ids of disease templates
    //
    QStringList getDiseaseIds() const;

    // Get names of disease templates
    //
    QStringList getDiseaseNames() const;

    // Get list of ids and names of disease templates
    //
    QVariantList getDiseaseCaptions() const;

    // Get disease template
    //
    fn::JSONDiseaseTemplate& getDiseaseTemplate(const fn::InputType::Disease& disease);

    // Check if disease data exist in the database
    //
    // 'disease' - [in] Disease
    //
    // Return - TRUE only if exists
    bool diseaseDataExist(const InputType::Disease& disease) const;

private:

    // Main work horse of data loading
    //
    void readData();

    // Reads factor data.
    //
    // 'disease' - [in] Disease for which input factors will be stored
    //
    // void readFactors(const fn::InputType::Disease& disease);

    // Application path
    QString m_appPath;

    // Models input file infos
    QFileInfoList m_modelFileInfos;

    // Map of disease templates
    fn::DiseaseIdToJSONDiseaseTemplateMap m_diseaseTemplates;
};

}

#endif  // _fn_InputDataMgr2_
