//==========================================================================================================================
//  Implementation file for "Class.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "fn/ModelRunWorker.h"

#include "fn/DiseaseModel.h"
#include "fn/formulaParserHelper.h"

namespace fn {

ModelRunWorker::ModelRunWorker(const int numSamples, const int randomSeed, const double timeDiscountRate, fn::DiseaseModelList& diseaseModelList) :
    m_numSamples(numSamples),
    m_randomSeed(randomSeed),
    m_timeDiscountRate(timeDiscountRate),
    m_numOutcomeTrees(0),
    m_currentOutcomeTree(0),
    m_numModels(diseaseModelList.size()),
    m_currentModel(0),
    m_modelCaption(""),
    m_diseaseModelList(diseaseModelList)
{

}

ModelRunWorker::~ModelRunWorker()
{

}

void ModelRunWorker::process()
{
    emit started();
    for (m_currentModel = 0; m_currentModel < m_numModels; ++m_currentModel) {
        fn::DiseaseModel& diseaseModel = m_diseaseModelList[m_currentModel];
        m_numOutcomeTrees = diseaseModel.getOutcomeTreeCount();
        m_modelCaption = diseaseModel.getCaption();
        m_currentOutcomeTree = 0;
        updateProgress();
        fn::resetRandomNumberGenerator(m_randomSeed);
        diseaseModel.runModel(m_numSamples, m_timeDiscountRate, m_currentOutcomeTree);
    }
    updateProgress();
    emit finished();
}

void ModelRunWorker::updateProgress()
{
    float modelCompletedPerc = (float)m_currentModel/(float)m_numModels;
    float percProgress = modelCompletedPerc;

    emit progressInfo(percProgress, m_currentModel, m_numModels, m_modelCaption);
}

}  // namespace fn
