//==========================================================================================================================
//  Implementation file for "OutcomeTreeNode.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2012 NextPage Software
//==========================================================================================================================

#include "OutcomeTreeNode.h"

#include "Overloads.h"

namespace fn {

OutcomeTreeNode::OutcomeTreeNode(ID id, const QString& caption, NodeType nodeType, const fn::Dims& dims) :
    m_nodeType(nodeType),
    m_id(id),
    m_caption(caption),
    m_parentTransProbs(),
    m_childTransProbs(),
    m_sumProbs(),
    m_dims(dims)
{

}

OutcomeTreeNode::~OutcomeTreeNode()
{

}

const fn::OutcomeTreeNode::ID fn::OutcomeTreeNode::getId() const
{
    return m_id;
}

void OutcomeTreeNode::addTransitionFromParent(TransitionProbabilityBase* trans)
{
    m_parentTransProbs.append(trans);
}

void OutcomeTreeNode::addTransitionToChild(TransitionProbabilityBase* trans)
{
    m_childTransProbs.append(trans);
}

const int OutcomeTreeNode::getCountTransitionFromParent() const
{
    return m_parentTransProbs.length();
}

const int OutcomeTreeNode::getCountTransitionToChild() const
{
    return m_childTransProbs.length();
}

const fn::NumericBaseArray OutcomeTreeNode::getInputRemLifeExp() const
{
    fn::NumericBaseArray result = fn::NumericBaseArray(m_dims.begin(), m_dims.end(), 0);
    if (getCountTransitionFromParent() > 0) {
        result = m_parentTransProbs[0]->getOutputRemLifeExp();
    }
    return result;
}

const NumericBaseArray OutcomeTreeNode::getInputCumulativeDuration() const
{
    fn::NumericBaseArray result = fn::NumericBaseArray(m_dims.begin(), m_dims.end(), 0);
    if (getCountTransitionFromParent() > 0) {
        result = m_parentTransProbs[0]->getOutputCumulativeDuration();
    }
    return result;
}

const fn::NumericBaseArray OutcomeTreeNode::getInputCasesCount() const
{
    fn::NumericBaseArray result = fn::NumericBaseArray(m_dims.begin(), m_dims.end(), 0);
    for (int i = 0; i < m_parentTransProbs.length(); ++i) {
        result += m_parentTransProbs[i]->getOutputCasesCount();
    }
    return result;
}

const QString& OutcomeTreeNode::getCaption() const
{
    return m_caption;
}

const OutcomeTreeNode::NodeType OutcomeTreeNode::getNodeType() const
{
    return m_nodeType;
}

void OutcomeTreeNode::computeSumProbsToChildren(const fn::Dims& runDims)
{
    if (m_childTransProbs.length() > 0) {
        m_sumProbs = fn::NumericBaseArray(runDims.begin(), runDims.end(), 0);
        for (int i = 0; i < m_childTransProbs.length(); ++i) {
            m_sumProbs += m_childTransProbs[i]->getIterProbability();
        }
    }
}

const fn::NumericBaseArray& OutcomeTreeNode::getSumProbsToChildren() const
{
    return m_sumProbs;
}

}  // namespace fn
