//==========================================================================================================================
//  Implementation file for "OutcomeTreeTemplate.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "OutcomeTreeTemplate.h"

#include <QStringList>

namespace fn {

OutcomeTreeTemplate::OutcomeTreeTemplate() :
    m_id(),
    m_name(),
    m_healthOutcomeRunOrder(),
    m_healthOutcomeTemplates(),
    m_healthStateTemplates(),
    m_transProbTemplates()
{

}

OutcomeTreeTemplate::~OutcomeTreeTemplate()
{

}

void OutcomeTreeTemplate::setId(const QString& id)
{
    m_id = id;
}

QString OutcomeTreeTemplate::getId() const
{
    return m_id;
}

void OutcomeTreeTemplate::setName(const QString& name)
{
    m_name = name;
}

QString OutcomeTreeTemplate::getName() const
{
    return m_name;
}

void OutcomeTreeTemplate::addHealthOutcome(const QString& id, const QString& caption, const QString& sequelaType, const QString& visible)
{
    QStringList list;
    list.append(id);
    list.append(caption);
    list.append(sequelaType);
    list.append(visible);

    m_healthOutcomeTemplates.insert(id, list);
    m_healthOutcomeRunOrder.append(id);
}

bool OutcomeTreeTemplate::hasHealthOutcome(const QString& id) const
{
    return m_healthOutcomeTemplates.contains(id);
}

const StringToStringListMap& OutcomeTreeTemplate::getHealthOutcomeMap() const
{
    return m_healthOutcomeTemplates;
}

void OutcomeTreeTemplate::setHealthOutcomeMap(const StringToStringListMap& healthOutcomeMap)
{
    m_healthOutcomeTemplates = healthOutcomeMap;
}

void OutcomeTreeTemplate::addHealthState(const QString& id, const QString& caption, const QString& parentHealthOutcomeID, const QString& type)
{
    QStringList list;
    list.append(id);
    list.append(caption);
    list.append(parentHealthOutcomeID);
    list.append(type);

    m_healthStateTemplates.insert(id, list);
}

bool OutcomeTreeTemplate::hasHealthState(const QString &id) const
{
    return m_healthStateTemplates.contains(id);
}

void OutcomeTreeTemplate::addTransProb(const QString& id, const QString& caption, const QString& parentID,
                                   const QString& childID, const QString& sourceTransProbID)
{
    QStringList list;
    list.append(id);
    list.append(caption);
    list.append(parentID);
    list.append(childID);
    list.append(sourceTransProbID);

    m_transProbTemplates.insert(id, list);
}

const StringToStringListMap& OutcomeTreeTemplate::getTransProbMap() const
{
    return m_transProbTemplates;
}

void OutcomeTreeTemplate::setTransProbMap(const StringToStringListMap& transProbMap)
{
    m_transProbTemplates = transProbMap;
}

const QString& OutcomeTreeTemplate::getHealthOutcomeID(const int index) const
{
    return m_healthOutcomeRunOrder[index];
}

int OutcomeTreeTemplate::getNumHealthOutcomes() const
{
    return m_healthOutcomeRunOrder.length();
}

const QStringList& OutcomeTreeTemplate::getHealthOutcomeRunOrder() const
{
    return m_healthOutcomeRunOrder;
}

void OutcomeTreeTemplate::setHealthOutcomeRunOrder(const QStringList& healthOutcomeRunOrder)
{
    m_healthOutcomeRunOrder = healthOutcomeRunOrder;
}

const StringToStringListMap& OutcomeTreeTemplate::getHealthStateMap() const
{
    return m_healthStateTemplates;
}

void OutcomeTreeTemplate::setHealthStateMap(const StringToStringListMap& healthStateMap)
{
    m_healthStateTemplates = healthStateMap;
}

}  // namespace fn
