//==========================================================================================================================
//  'OutcomeTreeTemplate' OutcomeTreeTemplate
//			This class implements template for an outcome tree
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_OutcomeTreeTemplate_
#define _fn_OutcomeTreeTemplate_

#include "fn/CustomDataTypes.h"

namespace fn {

class OutcomeTreeTemplate
{
public:
    // Constructor.
    //
    OutcomeTreeTemplate();

    // Destructor.
    //
    virtual ~OutcomeTreeTemplate();

    // Set id
    //
    void setId(const QString& id);

    // Get id
    //
    QString getId() const;

    // Set name
    //
    void setName(const QString& name);

    // Get name
    //
    QString getName() const;

    // Add a health outcome
    //
    // 'id' - [in] ID
    // 'caption' - [in] Caption
    // 'visible' - [in] Visible
    //
    void addHealthOutcome(const QString& id, const QString& caption, const QString& sequelaType, const QString& visible);

    // Has a health outcome
    //
    // 'id' - [in] ID
    //
    bool hasHealthOutcome(const QString& id) const;

    // Get health outcome map
    //
    const fn::StringToStringListMap& getHealthOutcomeMap() const;

    // Set health outcome map
    //
    void setHealthOutcomeMap(const fn::StringToStringListMap& healthOutcomeMap);

    // Add a health state
    //
    // 'id' - [in] ID
    // 'caption' - [in] Caption
    // 'parentHealthOutcomeID' - [in] Parent health outcome ID
    // 'type' - [in] Type
    //
    void addHealthState(const QString& id, const QString& caption, const QString& parentHealthOutcomeID, const QString& type);

    // Has a health state
    //
    // 'id' - [in] ID
    //
    bool hasHealthState(const QString& id) const;

    // Get health state map
    //
    const fn::StringToStringListMap& getHealthStateMap() const;

    // Set health state map
    //
    void setHealthStateMap(const fn::StringToStringListMap& healthStateMap);

    // Add a transition probability
    //
    // 'id' - [in] ID
    // 'caption' - [in] Caption
    // 'parentID' - [in] Parent health state ID
    // 'childID' - [in] Child health outcome ID
    // 'sourceTransProbID' - [in] Source transition probability ID
    //
    void addTransProb(const QString& id, const QString& caption, const QString& parentID, const QString& childID, const QString& sourceTransProbID);

    // Get transition probability map
    //
    const fn::StringToStringListMap& getTransProbMap() const;

    // Set transition probability map
    //
    void setTransProbMap(const fn::StringToStringListMap& transProbMap);

    // Get i-th health outcome name
    //
    // 'index' - [in] Index of the health outcome
    //
    // Return - Name of the health outcome
    const QString& getHealthOutcomeID(const int index) const;

    // Get the number of health outcomes
    //
    // Return - Number of health outcomes
    int getNumHealthOutcomes() const;

    // Get list of health outcome id's in run order
    //
    // Return - List of names of health outcomes in run order
    const QStringList& getHealthOutcomeRunOrder() const;

    // Set list of health outcome id's in run order
    //
    // healthOutcomeRunOrder - [in] List of names of health outcomes in run order
    //
    void setHealthOutcomeRunOrder(const QStringList& healthOutcomeRunOrder);

private:

    QString m_id;
    QString m_name;
    QStringList m_healthOutcomeRunOrder;
    fn::StringToStringListMap m_healthOutcomeTemplates;
    fn::StringToStringListMap m_healthStateTemplates;
    fn::StringToStringListMap m_transProbTemplates;
};

}  // namespace fn

#endif  // _fn_OutcomeTreeTemplate_

