//==========================================================================================================================
//  Contains set of overloaded operators
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_Overloads_
#define _fn_Overloads_

#include "fn/globals.h"
#include "fn/ParamsMgr.h"
#include "fn/TransitionParamsMgr.h"
#include "fn/InputFactorMgr.h"
#include "fn/InputFactor.h"
#include "fn/DataArray.h"
#include "fn/OutcomeTreeTemplate.h"
#include "fn/DiseaseModel.h"

namespace fn {

class DiseaseTemplate;
class DiseaseTemplateMgr;

// Disability weight manager
QDataStream& operator << (QDataStream& out, const ParamsMgr& paramsMgr);
QDataStream& operator >> (QDataStream& in, ParamsMgr& paramsMgr);

// Transitions manager
QDataStream& operator << (QDataStream& out, const TransitionParamsMgr& transProbInfoMgr);
QDataStream& operator >> (QDataStream& in, TransitionParamsMgr& transProbInfoMgr);

// Input factor manager
QDataStream& operator << (QDataStream& out, const InputFactorMgr& inputFactorMgr);
QDataStream& operator >> (QDataStream& in, InputFactorMgr& inputFactorMgr);

// Input factor
QDataStream& operator << (QDataStream& out, const InputFactor& inputFactor);
QDataStream& operator >> (QDataStream& in, InputFactor& inputFactor);

// Outcome tree template
QDataStream& operator << (QDataStream& out, const OutcomeTreeTemplate& outcomeTreeTemplate);
QDataStream& operator >> (QDataStream& in, OutcomeTreeTemplate& outcomeTreeTemplate);

// Disease template
QDataStream& operator << (QDataStream& out, const DiseaseTemplate& diseaseTemplate);
QDataStream& operator >> (QDataStream& in, DiseaseTemplate& diseaseTemplate);

// Disease template manager
QDataStream& operator << (QDataStream& out, const DiseaseTemplateMgr& diseaseTemplateMgr);
QDataStream& operator >> (QDataStream& in, DiseaseTemplateMgr& diseaseTemplateMgr);

// Data array
QDataStream& operator << (QDataStream& out, const DataArray& dataArray);
QDataStream& operator >> (QDataStream& in, DataArray& dataArray);

//fn::NumericBaseArray operator - (const fn::SampleVal& lhs, const fn::NumericBaseArray& rhs);

// Mathematical functions

fn::NumericBaseArray pow(const fn::NumericBaseArray& lhs, const fn::NumericBaseArray& rhs);
fn::NumericBaseArray max(const fn::NumericBaseArray& lhs, const fn::NumericBaseArray& rhs);
fn::NumericBaseArray max(const fn::NumericBaseArray& lhs, const fn::SampleVal& rhs);
fn::NumericBaseArray max(const fn::NumericBaseArray& lhs, const fn::NumericBaseArray& rhs);
fn::NumericBaseArray min(const fn::NumericBaseArray& lhs, const fn::NumericBaseArray& rhs);
fn::NumericBaseArray min(const fn::NumericBaseArray& lhs, const fn::SampleVal& rhs);

// Utilities
fn::DataArray copyFilteredDataArray(const fn::DataArray& source, const fn::DataArray& filter, const fn::SampleVal condition);
fn::NumericBaseArray copyFilteredNumericBaseArray(const fn::NumericBaseArray& source, const fn::NumericBaseArray& filter, const fn::SampleVal condition);
void filterNumericBaseArray(fn::NumericBaseArray& source, const fn::NumericBaseArray& filter, const fn::SampleVal condition);

// Apply time discount rate
//
// 'd' - [in] Time discount rate to be applied
// 'data' - [in/out] (Duration) data to which time discount rate will be applied
//
void applyTimeDiscountRate(const double d, fn::NumericBaseArray& duration);
void applyTimeDiscountRate(const double d, const fn::NumericBaseArray& start, fn::NumericBaseArray& duration);

// Statistics

// Compute sample mean per category
fn::NumericBaseArray getSampleMean(const fn::NumericBaseArray& data);

//
SampleVal getSum2D(const fn::NumericBaseArray& data);
fn::NumericBaseArray getColumnSum2D(const fn::NumericBaseArray& data);
fn::NumericBaseArray getAggregatedSample(const fn::NumericBaseArray& data);

// Store sample
void storeSample(const fn::NumericBaseArray& sample, fn::NumericBaseArray& sampleContainer, size_t iteration);

fn::NumericBaseArray replicate(const fn::NumericBaseArray& data, int numReplicates);
QVariantMap getStatistics(const fn::NumericBaseArray& data, const fn::ConstInputFactorPtrList& inputFactors);
QVariantMap getAggregatedStatistics(const fn::NumericBaseArray& data);

// Get dimensions of array
fn::Dims getDims(const fn::NumericBaseArray& array);

}  // namespace fn

#endif  // _fn_Overloads_

