//==========================================================================================================================
//  Implementation file for "DisabilityWeightsMgr.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "ParamsMgr.h"

namespace fn {

ParamsMgr::ParamsMgr():
    m_severityParameters(),
    m_durationParameters(),
    m_timeToDeathParameters()
{

}

ParamsMgr::~ParamsMgr()
{

}

void ParamsMgr::addParameter(const fn::InputType::Country& country, const InputType::Disease& disease,
                             const QString& outcomeTreeID, const QString& healthStateID, const ParamType& paramType, const fn::DataArray& inputData)
{
    switch (paramType)
    {
        case Severity:
            m_severityParameters[country][disease][outcomeTreeID][healthStateID] = inputData;
            break;
        case Duration:
            m_durationParameters[country][disease][outcomeTreeID][healthStateID] = inputData;
            break;
        case TimeToDeath:
            m_timeToDeathParameters[country][disease][outcomeTreeID][healthStateID] = inputData;
            break;
        case Proportion:
            m_proportionParameters[country][disease][outcomeTreeID][healthStateID] = inputData;
            break;
    }
}

const fn::DataArray ParamsMgr::getParameter(const InputType::Country& country,
                                            const InputType::Disease& disease,
                                            const QString& outcomeTreeID,
                                            const QString& healthStateID,
                                            const ParamsMgr::ParamType& paramType,
                                            const fn::ConstInputFactorPtrList& defaultInputFactors)
{
    fn::CountryToDiseaseToStringToStringTableType* map;
    QString name;
    QString initFormula;
    fn::StringToStringMap metaData;
    switch (paramType)
    {
        case Severity:
            map = &m_severityParameters;
            initFormula = "0";
            break;
        case Duration:
            map = &m_durationParameters;
            initFormula = "0";
            break;
        case TimeToDeath:
            map = &m_timeToDeathParameters;
            initFormula = "0";
            break;
        case Proportion:
            map = &m_proportionParameters;
            initFormula = "1";
            break;
    }

    fn::InputType::Country finalCountry = country;
    if (!map->contains(country)) {
        finalCountry = "Generic";
    }

    if (map->contains(finalCountry)) {
        const fn::DiseaseToStringToStringTableType& diseaseMap = (*map)[finalCountry];
        if (diseaseMap.contains(disease)) {
            const fn::StringToStringTableType& outcomeTreeMap = diseaseMap[disease];
            if (outcomeTreeMap.contains(outcomeTreeID)) {
                const fn::StringTableType& healthStateMap = outcomeTreeMap[outcomeTreeID];
                if (healthStateMap.contains(healthStateID)) {
                    return(healthStateMap.value(healthStateID));
                }
            }
        }
    }

    fn::DataArray defaultResult = fn::DataArray(defaultInputFactors, initFormula);

    return defaultResult;
}

const fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getSeverityParameters() const
{
    return m_severityParameters;
}

fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getSeverityParameters()
{
    return m_severityParameters;
}

void ParamsMgr::setSeverityParameters(const fn::CountryToDiseaseToStringToStringTableType& severityParameters)
{
    m_severityParameters = severityParameters;
}

const fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getDurationParameters() const
{
    return m_durationParameters;
}

fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getDurationParameters()
{
    return m_durationParameters;
}

void ParamsMgr::setDurationParameters(const fn::CountryToDiseaseToStringToStringTableType& durationParameters)
{
    m_durationParameters = durationParameters;
}

const fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getTimeToDeathParameters() const
{
    return m_timeToDeathParameters;
}

fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getTimeToDeathParameters()
{
    return m_timeToDeathParameters;
}

void ParamsMgr::setTimeToDeathParameters(const fn::CountryToDiseaseToStringToStringTableType& timeToDeathParameters)
{
    m_timeToDeathParameters = timeToDeathParameters;
}

const fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getProportionParameters() const
{
    return m_proportionParameters;
}

fn::CountryToDiseaseToStringToStringTableType& ParamsMgr::getProportionParameters()
{
    return m_proportionParameters;
}

void ParamsMgr::setProportionParameters(const fn::CountryToDiseaseToStringToStringTableType& proportionParameters)
{
    m_proportionParameters = proportionParameters;
}

void ParamsMgr::clearData()
{
    m_severityParameters.clear();
    m_durationParameters.clear();
    m_timeToDeathParameters.clear();
    m_proportionParameters.clear();
}

}  // namespace fn
