//==========================================================================================================================
//  'ResultsComputer' class
//			This class calculates results
//
//  Written by Daniel Lewandowski
//  Copyright @ 2015 NextPage Software
//==========================================================================================================================

#ifndef _fn_ResultsComputer_
#define _fn_ResultsComputer_

#include "globals.h"
#include "CustomDataTypes.h"
#include "InputFactor.h"

namespace fn {

class ResultsComputer
{

public:
	// Constructor.
	//
    ResultsComputer(const fn::DiseaseModelList& diseaseModelList);

	// Destructor.
	//
    virtual ~ResultsComputer();

    // Initialize the results tables - mainly dimensions
    //
    void initializeSample(const int numSamples);

    // Compute total categorized results
    //
    void computeAggregatedResults();

    // Add YLD sample
    //
    void addYLDSample(const fn::NumericBaseArray& sample);

    // Add YLL sample
    //
    void addYLLSample(const fn::NumericBaseArray& sample);

    // Add age distribution sample
    //
    void addAgeDistrSample(const fn::NumericBaseArray& sample);

    // Add number of symptomatic cases count sample
    //
    void addNumSymptomaticCasesCountSample(const fn::NumericBaseArray& sample);

    // Add number of death cases count sample
    //
    void addDeathCasesCountSample(const fn::NumericBaseArray& sample);

    // Add total population
    //
    void addTotalPopulation(const fn::SampleVal& totalPopulation);

    // Get results
    //
    // Return - aggregated page results as VariantMap
    QVariantMap getAggregatedResults() const;

private:	

    // Clear the sample tables
    //
    void clearSample();

    // Clear the results tables
    //
    void clearResults();

    // Sample dimensions
    fn::Dims m_sampleDims;

    // Total population
    fn::SampleVal m_totalPopulation;

    // Number of congenital models
    int m_numCongenitalModels;

    // Input factors list
    fn::ConstInputFactorPtrList m_runFactorList;

    // Sample containers
    fn::NumericBaseArray m_sampleYLD;
    fn::NumericBaseArray m_sampleYLL;
    fn::NumericBaseArray m_sampleAgeDistr;
    fn::NumericBaseArray m_sampleNumSymptomaticCasesCount;
    fn::NumericBaseArray m_sampleDeathCasesCount;

    // Reference to the model list
    const fn::DiseaseModelList& m_diseaseModelList;

    // CATEGORIZED RESULTS (PER SEX AND AGE)
    QVariantMap m_categorizedResults;

    // Total for aggregated results table
    QVariantMap m_aggregatedPageResults;
};

}  // namespace fn

#endif  // _fn_ResultsComputer_

