//==========================================================================================================================
//  Implementation file for "SequelaBase.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "fn/SequelaBase.h"

#include "fn/Overloads.h"

namespace fn {

SequelaBase::SequelaBase(fn::OutcomeTreeNode::ID id, const QString& caption, const fn::InputType::HealthState& type,
                         const fn::Dims& dims, const fn::HealthOutcome& parent) :
    fn::OutcomeTreeNode(id, caption, fn::OutcomeTreeNode::HealthState, dims),
    m_type(type),
    m_casesCount(),
    m_remLifeExp(),
    m_cumulativeDuration(dims.begin(), dims.end(), 0),
    m_currentIteration(0),
    m_parent(parent),
    m_sampleCasesCount(),
    m_sampleMeanCasesCount(),
    m_sampleAggregatedCasesCount(),
    m_sampleRemLifeExp(),
    m_sampleMeanRemLifeExp()
{

}

SequelaBase::~SequelaBase()
{

}

const HealthOutcome& SequelaBase::getParentHealthOutcome() const
{
    return m_parent;
}

const InputType::HealthState& SequelaBase::getType() const
{
    return m_type;
}

const fn::NumericBaseArray& SequelaBase::getSampleMeanCasesCount() const
{
    return m_sampleMeanCasesCount;
}

const fn::NumericBaseArray& SequelaBase::getSampleCasesCount() const
{
    return m_sampleCasesCount;
}

const fn::NumericBaseArray& SequelaBase::getSampleMeanRemLifeExp() const
{
    return m_sampleMeanRemLifeExp;
}

const fn::NumericBaseArray SequelaBase::getOutputCasesCount() const
{
    return m_casesCount;
}

const fn::NumericBaseArray SequelaBase::getOutputRemLifeExp() const
{
    return m_remLifeExp;
}

const fn::NumericBaseArray SequelaBase::getOutputCumulativeDuration() const
{
    return m_cumulativeDuration;
}

void SequelaBase::initializeSampling(const fn::Dims& sampleDims)
{
    m_currentIteration = 0;
    m_sampleCasesCount = fn::NumericBaseArray(sampleDims.begin(), sampleDims.end(), 0);
    m_sampleRemLifeExp = fn::NumericBaseArray(sampleDims.begin(), sampleDims.end(), 0);
}

void SequelaBase::clearSample()
{
    m_sampleCasesCount = fn::NumericBaseArray();
    m_sampleRemLifeExp = fn::NumericBaseArray();
}

void SequelaBase::computeSampleMeanResults()
{
    m_sampleMeanCasesCount = fn::getSampleMean(m_sampleCasesCount);
    m_sampleMeanRemLifeExp = fn::getSampleMean(m_sampleRemLifeExp);
}

void SequelaBase::computeAggregatedResults()
{
    m_sampleAggregatedCasesCount = fn::getAggregatedSample(m_sampleCasesCount);
}

const fn::NumericBaseArray& SequelaBase::getAggregatedCasesCount() const
{
    return m_sampleAggregatedCasesCount;
}

}  // namespace fn
