//==========================================================================================================================
//  'SequelaBase' SeuqelaBase
//			This class implements a base class for all sequela boxes.
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_SequelaBase_
#define _fn_SequelaBase_

#include "fn/OutcomeTreeNode.h"
#include "fn/TransitionProbability.h"

namespace fn {

class HealthOutcome;

class SequelaBase : public fn::OutcomeTreeNode
{
public:

	// Constructor.
	//
    SequelaBase(fn::OutcomeTreeNode::ID id, const QString& caption, const fn::InputType::HealthState& type, const fn::Dims& dims, const fn::HealthOutcome& parent);

	// Destructor.
	//
    virtual ~SequelaBase();

    // Get type (YLL, YLD, Infection, Asymptomatic)
    //
    const fn::InputType::HealthState& getType() const;

    // Get parent health outcome
    //
    const fn::HealthOutcome& getParentHealthOutcome() const;

    // Get cases count
    //
    virtual const fn::NumericBaseArray getOutputCasesCount() const;

    // Get remaining life expectancy
    //
    virtual const fn::NumericBaseArray getOutputRemLifeExp() const;

    // Get cumulative duration
    //
    const fn::NumericBaseArray getOutputCumulativeDuration() const;

    // Compute mean sample results
    //
    void computeSampleMeanResults();

    // Compute aggregated results
    //
    void computeAggregatedResults();

    // Get sample mean cases count
    //
    const fn::NumericBaseArray& getSampleMeanCasesCount() const;

    // Get sample cases count
    //
    const fn::NumericBaseArray& getSampleCasesCount() const;

    // Get sample mean remaining life expectancy
    //
    const fn::NumericBaseArray& getSampleMeanRemLifeExp() const;

    // Get aggregated cases count
    //
    const fn::NumericBaseArray& getAggregatedCasesCount() const;

protected:

    // Initialize sample containers to the specified size
    //
    // 'sampleDims' - [in] Dimensions
    //
    void initializeSampling(const fn::Dims& sampleDims);

    // Clear sample
    //
    void clearSample();

    // Reference to the parent health outcome
    const fn::HealthOutcome& m_parent;

    // ----------------------------------------------
    // Iteration output incidence
    fn::NumericBaseArray m_casesCount;

    // Remaining life expectancy
    fn::NumericBaseArray m_remLifeExp;

    // Duration up till sequela
    fn::NumericBaseArray m_cumulativeDuration;

    // ----------------------------------------------
    // Sample storage
    fn::NumericBaseArray m_sampleCasesCount;
    fn::NumericBaseArray m_sampleRemLifeExp;

    // ----------------------------------------------
    // Sample mean output
    fn::NumericBaseArray m_sampleMeanCasesCount;
    fn::NumericBaseArray m_sampleMeanRemLifeExp;

    // ----------------------------------------------
    // Aggregated cases count
    fn::NumericBaseArray m_sampleAggregatedCasesCount;

    // Type
    fn::InputType::HealthState m_type;

    // Current iteration
    int m_currentIteration;
};

}  // namespace fn

#endif  // _fn_SeuqelaBase_
