//==========================================================================================================================
//  Implementation file for "Settings.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "fn/Settings.h"

#include <QCoreApplication>
#include "fn/globals.h"

namespace fn {

Settings::Settings() :
    m_settings(qApp->applicationName() + ".ini", QSettings::IniFormat)
{

}

Settings::~Settings()
{

}

int Settings::getNumSamples()
{
    return m_settings.value("RunSettings/DefaultNumberOfSamples", fn::kDefaultNumSamples).toInt();
}

void Settings::setNumSamples(int numSamples)
{
    m_settings.beginGroup("RunSettings");
    m_settings.setValue("DefaultNumberOfSamples", numSamples);
    m_settings.endGroup();
}

int Settings::getRandomSeed()
{
    return m_settings.value("RunSettings/RandomSeed", fn::kDefaultRandomSeed).toInt();
}

void Settings::setRandomSeed(int randomSeed)
{
    m_settings.beginGroup("RunSettings");
    m_settings.setValue("RandomSeed", randomSeed);
    m_settings.endGroup();
}

double Settings::getTimeDiscountRate()
{
    return m_settings.value("RunSettings/TimeDiscountRate", fn::kDefaultTimeDiscountRate).toDouble();
}

void Settings::setTimeDiscountRate(double timeDiscountRate)
{
    m_settings.beginGroup("RunSettings");
    m_settings.setValue("TimeDiscountRate", timeDiscountRate);
    m_settings.endGroup();
}

void Settings::writeSettings()
{
    m_settings.sync();
}

}  // namespace fn

