//==========================================================================================================================
//  Implementation file for "TransProbInfoMgr.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include "TransitionParamsMgr.h"

#include "fn/Overloads.h"

namespace fn {

TransitionParamsMgr::TransitionParamsMgr() :
    m_probabilityParameters(),
    m_redistributionParameters()
{

}

TransitionParamsMgr::~TransitionParamsMgr()
{

}

void TransitionParamsMgr::addParameter(const fn::InputType::Country& country, const fn::InputType::Disease& disease,
                                       const QString& outcomeTreeID, const QString& transProbID, const ParamType& paramType,
                                       const fn::DataArray& inputData)
{
    switch (paramType)
    {
        case Probability:
            m_probabilityParameters[country][disease][outcomeTreeID][transProbID] = inputData;
            break;
        case Redistribution:
            m_redistributionParameters[country][disease][outcomeTreeID][transProbID] = inputData;
            break;
    }
}

fn::DataArray TransitionParamsMgr::getParameter(const InputType::Country& country, const InputType::Disease& disease,
                                                const QString& outcomeTreeID, const QString& transProbID, const ParamType& paramType,
                                                const fn::ConstInputFactorPtrList& defaultInputFactors)
{
    fn::DataArray result;

    fn::CountryToDiseaseToStringToStringTableType* map;
    fn::StringToStringMap metaData;
    switch (paramType)
    {
        case Probability:
            map = &m_probabilityParameters;
            result = fn::DataArray(defaultInputFactors, "1");
            result.setName("Probability");
            break;
        case Redistribution:
            map = &m_redistributionParameters;
            result = fn::DataArray(defaultInputFactors, "0.052632");
            result.setName("Redistribution");
            break;
    }
    result.setMetaData(metaData);

    fn::InputType::Country finalCountry = country;
    if (!map->contains(country)) {
        finalCountry = "Generic";
    }

    if (map->contains(finalCountry)) {
        const fn::DiseaseToStringToStringTableType& diseaseMap = (*map)[finalCountry];
        if (diseaseMap.contains(disease)) {
            const fn::StringToStringTableType& outcomeTreeMap = diseaseMap[disease];
            if (outcomeTreeMap.contains(outcomeTreeID)) {
                const fn::StringTableType& transProbMap = outcomeTreeMap[outcomeTreeID];
                if (transProbMap.contains(transProbID)) {
                    result = transProbMap.value(transProbID);
                }
            }
        }
    }

    return result;
}

const fn::CountryToDiseaseToStringToStringTableType& TransitionParamsMgr::getProbabilityParameters() const
{
    return m_probabilityParameters;
}

fn::CountryToDiseaseToStringToStringTableType& TransitionParamsMgr::getProbabilityParameters()
{
    return m_probabilityParameters;
}

void TransitionParamsMgr::setProbabilityParameters(const CountryToDiseaseToStringToStringTableType& probabilityParameters)
{
    m_probabilityParameters = probabilityParameters;
}

const fn::CountryToDiseaseToStringToStringTableType& TransitionParamsMgr::getRedistributionParameters() const
{
    return m_redistributionParameters;
}

fn::CountryToDiseaseToStringToStringTableType& TransitionParamsMgr::getRedistributionParameters()
{
    return m_redistributionParameters;
}

void TransitionParamsMgr::setRedistributionParameters(const fn::CountryToDiseaseToStringToStringTableType& redistributionParameters)
{
    m_redistributionParameters = redistributionParameters;
}

void TransitionParamsMgr::clearData()
{
    m_probabilityParameters.clear();
    m_redistributionParameters.clear();
}

}  // namespace fn
