//==========================================================================================================================
//  'TransProbInfoMgr' class
//			This class implements manager of transition probability info
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_TransProbInfoMgr_
#define _fn_TransProbInfoMgr_

#include "fn/InputFactor.h"
#include "fn/CustomDataTypes.h"

namespace fn {

class TransitionParamsMgr
{
public:

    enum ParamType {Probability, Redistribution};

	// Constructor.
	//
    TransitionParamsMgr();

	// Destructor.
	//
    virtual ~TransitionParamsMgr();

    // Add transition probability info
    void addParameter(const fn::InputType::Country& country, const fn::InputType::Disease& disease,
                      const QString& outcomeTreeID, const QString& transProbID, const ParamType& paramType, const fn::DataArray& inputData);

    // Get transition probabability parameter
    fn::DataArray getParameter(const InputType::Country& country, const InputType::Disease& disease,
                               const QString& outcomeTreeID, const QString& transProbID, const ParamType& paramType,
                               const fn::ConstInputFactorPtrList& defaultInputFactors);

    // Get all transition probability parameters
    const fn::CountryToDiseaseToStringToStringTableType& getProbabilityParameters() const;
          fn::CountryToDiseaseToStringToStringTableType& getProbabilityParameters();

    // Set all transition probability parameters
    void setProbabilityParameters(const fn::CountryToDiseaseToStringToStringTableType& probabilityParameters);

    // Get all transition redistribution parameters
    const fn::CountryToDiseaseToStringToStringTableType& getRedistributionParameters() const;
          fn::CountryToDiseaseToStringToStringTableType& getRedistributionParameters();

    // Set all transition redistribution parameters
    void setRedistributionParameters(const fn::CountryToDiseaseToStringToStringTableType& redistributionParameters);

    // Clear all data
    void clearData();

private:	

    // Transition tables
    fn::CountryToDiseaseToStringToStringTableType m_probabilityParameters;

    // Transition tables
    fn::CountryToDiseaseToStringToStringTableType m_redistributionParameters;
};

}  // namespace fn

#endif  // _fn_TransProbInfoMgr_

