//==========================================================================================================================
//  'TransitionProbability' class
//			This class implements transition probability
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _fn_TransitionProbability_
#define _fn_TransitionProbability_

#include "fn/TransitionProbabilityBase.h"
#include "fn/DataArray.h"

namespace fn {

class OutcomeTreeNode;

class TransitionProbability : public TransitionProbabilityBase
{
public:

	// Constructor.
	//
    TransitionProbability(const ID& id, const QString& caption, const fn::OutcomeTreeNode& parent, const fn::OutcomeTreeNode& child,
                          const fn::DataArray& probability, const fn::DataArray& redistribution);

	// Destructor.
	//
    virtual ~TransitionProbability();

    // Return TRUE is transition probability is age dependent
    virtual bool isRedistributeType() const;

    // Return type of transition probability
    virtual bool isDirectType() const;

    // Get input transition probability
    //
    // Return - Reference to transition probability data array
    const fn::DataArray& getProbability() const;
          fn::DataArray& getProbability();

    // Set input transition probability
    //
    // 'probability' - [in] Transition probability array
    //
    void setProbability(const fn::DataArray& probability);

    // Get input redistribution
    //
    // Return - Reference to redistribution data array
    const fn::DataArray& getRedistribution() const;
          fn::DataArray& getRedistribution();

    // Set input redistribution
    //
    // 'redistribution' - [in] Redistribution array
    //
    void setRedistribution(const fn::DataArray& redistribution);

    // Set meta data
    //
    // 'metaData' - [in] Transition probability meta data
    //
    void setMetaData(const fn::StringToStringMap& metaData);

    // Formula parser
    void initializeAllFormulaParsers();
    void deinitializeAllFormulaParsers();
    virtual void evaluateAllFormulas();

private:

    // Make sure meta deta has all fields populated in the transition probability data array
    void completeMetaData();

    // Force update of redistribute flag
    //
    void invalidateRedistributeType();

    // Force update of type
    //
    void invalidateDirectType();

    // Force update of duration
    //
    void invalidateDuration();

    // "Redistribute" type
    bool m_redistributeType;

    // "Direct" type
    bool m_directType;

    // ----------------------------------------------
    // Probability data
    fn::DataArray m_probability;

    // Redistribution data
    fn::DataArray m_redistribution;

};

}  // namespace fn

#endif  // _fn_TransitionProbability_


