//==========================================================================================================================
//  Implementation file for "formulaParserHelper.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2012 NextPage Software
//==========================================================================================================================

// TODO: Consider replacing with exprtk (https://github.com/ArashPartow/exprtk)

#include "fn/formulaParserHelper.h"

#include <boost/random/mersenne_twister.hpp>
#include <boost/random/uniform_01.hpp>
#include <boost/random/gamma_distribution.hpp>
#include <boost/random/lognormal_distribution.hpp>

namespace fn {

namespace {
    boost::random::mt19937 rndGenerator;
}

fn::SampleVal RandUniform(fn::SampleVal a, fn::SampleVal b)
{
    boost::random::uniform_01<fn::SampleVal> rv;
    return a + rv(rndGenerator)*(b - a);
}

fn::SampleVal RandPert(fn::SampleVal a, fn::SampleVal b, fn::SampleVal c)
{
    const fn::SampleVal alpha = 1 + 4*(b - a)/(c - a);
    const fn::SampleVal beta = 1 + 4*(c - b)/(c - a);

    return a + RandBeta(alpha, beta)*(c - a);
}

fn::SampleVal RandBeta(fn::SampleVal a, fn::SampleVal b)
{
    boost::random::gamma_distribution<fn::SampleVal> x(a, 1);
    boost::random::gamma_distribution<fn::SampleVal> y(b, 1);

    const fn::SampleVal rx = x(rndGenerator);
    const fn::SampleVal ry = y(rndGenerator);

    return rx/(rx + ry);
}

fn::SampleVal RandGamma(fn::SampleVal a, fn::SampleVal b)
{
    boost::random::gamma_distribution<fn::SampleVal> x(a, 1/b);

    return x(rndGenerator);
}

fn::SampleVal RandLogNormal(fn::SampleVal a, fn::SampleVal b)
{
    boost::random::lognormal_distribution<fn::SampleVal> x(a, b);

    return x(rndGenerator);
}

fn::SampleVal RandNormal(fn::SampleVal a, fn::SampleVal b)
{
    boost::random::normal_distribution<fn::SampleVal> x(a, b);

    return std::max(x(rndGenerator), 0.0);
}

void defineAdditionalFunctions(fn::FormulaParser* parserPtr)
{
    parserPtr->DefineFun(QStringLiteral("RandUniform").toStdWString(), fn::RandUniform, true);
    parserPtr->DefineFun(QStringLiteral("RandPert").toStdWString(), fn::RandPert, true);
    parserPtr->DefineFun(QStringLiteral("RandBeta").toStdWString(), fn::RandBeta, true);
    parserPtr->DefineFun(QStringLiteral("RandGamma").toStdWString(), fn::RandGamma, true);
    parserPtr->DefineFun(QStringLiteral("RandLogNormal").toStdWString(), fn::RandLogNormal, true);
    parserPtr->DefineFun(QStringLiteral("RandNormal").toStdWString(), fn::RandNormal, true);
}

void setRandomSeed(int seed)
{
    if (seed == 0) {
        rndGenerator.seed(static_cast<uint>(std::time(nullptr)));
    } else {
        rndGenerator.seed(static_cast<uint>(seed));
    }
}

void resetRandomNumberGenerator(int seed)
{
    if (seed != 0) {
        rndGenerator.seed(static_cast<uint>(seed));
    }
}

}  // namespace fn
