//==========================================================================================================================
//  Formula parser additions
//      - extra function definitions
//
//  Written by Daniel Lewandowski
//  Copyright @ 2012 NextPage Software
//==========================================================================================================================

#ifndef _fn_formulaParserHelper_
#define _fn_formulaParserHelper_

#include "fn/globals.h"

namespace fn {

// Set seed for the random number generator
//
// 'seed' - [in] Random seed
//
void resetRandomNumberGenerator(int seed);
void setRandomSeed(int seed);

// Define additional functions available in the parser
//
// 'parserPtr' - [in] Pointer to parser
//
void defineAdditionalFunctions(fn::FormulaParser* parserPtr);

// Sample of a Uniform distribution
//
// 'a' - [in] Lower bound
// 'b' - [in] Lower bound
//
// Return - realization of the distribution
fn::SampleVal RandUniform(fn::SampleVal a, fn::SampleVal b);

// Sample of a Pert distribution
//
// 'a' - [in] Lower bound
// 'b' - [in] Mode
// 'c' - [in] Upper bound
//
// Return - realization of the distribution
fn::SampleVal RandPert(fn::SampleVal a, fn::SampleVal b, fn::SampleVal c);

// Sample of a Beta distribution
//
// 'a' - [in] Shape parameter
// 'b' - [in] Shape parameter
//
// Return - realization of the distribution
fn::SampleVal RandBeta(fn::SampleVal a, fn::SampleVal b);

// Sample of a Gamma distribution
//
// 'a' - [in] Shape parameter
// 'b' - [in] Rate parameter
//
// Return - realization of the distribution
fn::SampleVal RandGamma(fn::SampleVal a, fn::SampleVal b);

// Sample of a Log-normal distribution
//
// 'a' - [in] Log-scale parameter
// 'b' - [in] Shape parameter
//
// Return - realization of the distribution
fn::SampleVal RandLogNormal(fn::SampleVal a, fn::SampleVal b);

}  // namespace fn

#endif  // _fn_formulaParserHelper_

