//==========================================================================================================================
//  Globals for the "fn" library.
//
//  Copyright 2011 Daniel Lewandowski
//==========================================================================================================================

#ifndef _fn_globals_
#define _fn_globals_

#include <QVector>
#include <QMetaEnum>
#include "external/marray.hxx"
#include "external/muParser/muParser.h"

#include "ut/globals.h"

namespace fn {

enum FileOperationType {Export = 0, Import};

// Type definitions -------------------------------------------
// Sampling value type
typedef double SampleVal;

// Short integer
typedef size_t PosInt;

// Data array types
typedef QVector<PosInt> Dims;
typedef QVector<PosInt> Indices;
typedef QVector<SampleVal> Params;
typedef QVector<SampleVal> SampleVector;

// Base class for Multi dimensional string array
typedef QStringList StringBaseArray;
// Base class for Multi dimensional numeric array
typedef marray::Marray<SampleVal> NumericBaseArray;
// Base class for Multi dimensional boolean array
typedef QList<bool> BoolBaseArray;

// Mathematical formula parser type
typedef mu::Parser FormulaParser;

// Constants --------------------------------------------------
const int kDefaultNumSamples = 1000;
const int kDefaultRandomSeed = 0;
const SampleVal kDefaultTimeDiscountRate = 0.0;

// Input type
class InputType : public QObject
{

    Q_OBJECT
    Q_ENUMS(TransProb)
    Q_ENUMS(Distribution)
    Q_ENUMS(HealthState)
    Q_ENUMS(HealthStateParamType)

public:

    typedef QString Country;
    typedef QString Disease;

    static QStringList CountryIds;
    static QStringList BaselineAMRDiseaseIds;

    enum TransProb {Direct = 0, Cumulative};

    enum Distribution {Constant = 0, Uniform, Pert, Beta};

    enum HealthState {INFECTION = 0, ASYMPTOMATIC, YLD, YLL};

    enum HealthStateParamType {Severity = 0, Duration, Proportion};

};

inline InputType::TransProb getTransProbTypeByName(const QString& transProbString)
{
    int index = fn::InputType::staticMetaObject.indexOfEnumerator("TransProb");
    QMetaEnum metaEnum = fn::InputType::staticMetaObject.enumerator(index);
    fn::InputType::TransProb transProb = static_cast<fn::InputType::TransProb>(metaEnum.keyToValue(transProbString.toUtf8()));
    return transProb;
}

inline InputType::Distribution getDistrTypeByName(const QString& distrString)
{
    int index = fn::InputType::staticMetaObject.indexOfEnumerator("Distribution");
    QMetaEnum metaEnum = fn::InputType::staticMetaObject.enumerator(index);
    fn::InputType::Distribution distr = static_cast<fn::InputType::Distribution>(metaEnum.keyToValue(distrString.toUtf8()));
    return distr;
}

inline QString getDistrStringByEnum(const InputType::Distribution& distribution)
{
    int index = fn::InputType::staticMetaObject.indexOfEnumerator("Distribution");
    QMetaEnum metaEnum = fn::InputType::staticMetaObject.enumerator(index);
    QString distrName = metaEnum.valueToKey(distribution);
    return distrName;
}


inline InputType::HealthState getHealthStateTypeByName(const QString& healthStateString)
{
    int index = fn::InputType::staticMetaObject.indexOfEnumerator("HealthState");
    QMetaEnum metaEnum = fn::InputType::staticMetaObject.enumerator(index);
    fn::InputType::HealthState healthState = static_cast<fn::InputType::HealthState>(metaEnum.keyToValue(healthStateString.toUtf8()));
    return healthState;
}

} // namespace fn

#endif // _fn_globals_
