/*global
    window, setTimeout, $, console
*/

/*
Tabs
 */

var finalizeModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars
  "use strict";

  // Page tabs
  var pageTabs = $("#tabs").tabs("#panes > .layout", {
    effect: "fade",
    current: "active",
    fadeOutSpeed: null,
    fadeInSpeed: 150,
    initialIndex: 0,
    onBeforeClick: function (ignore, tabIndex) {
      if (tabIndex === 4) { // "Run" tab
        this.click(this.getIndex());
        bcode.initialize.launchWindow("#runDialog", 203, 150);
        webGUI.pageOpened(tabIndex, "Run models");
        return false;
      }
    },
    onClick: function (ignore, tabIndex) {
      bcode.editModelData.diseasePageState.flowChartVisible = false;
      if (tabIndex === 2) { // "Create models" tab
        bcode.createModels.countryGrid.scrollRowIntoView(-1);
        bcode.createModels.diseaseGrid.scrollRowIntoView(-1);
      } else if ((tabIndex === 3) && ($("#flowChartTab").is(":visible"))) { // "Edit model data" tab
        $.event.trigger("flowChartShown");
      } else if ((tabIndex === 6) && (!$.isEmptyObject(bcode.showAggregatedResults.getAggrResults1DiseaseGrid()))) { // "Aggregated results" tab
        $("#aggrResults1DiseaseSelectTable").height($("#aggrResults1ChartContainer").height());
        $("#aggrResults2DiseaseSelectTable").height($("#aggrResults2ChartContainer").height());
        bcode.showAggregatedResults.getAggrResults1DiseaseGrid().resizeCanvas();
        bcode.showAggregatedResults.getAggrResults2DiseaseGrid().resizeCanvas();
      }

      let pageName = "Main Page";
      switch (tabIndex) {
        case 1:
          pageName = "Tutorial";
          break;
        case 2:
          pageName = "Create models";
          break;
        case 3:
          pageName = "Edit model data";
          break;
        case 4:
          pageName = "Run models";
          break;
        case 5:
          pageName = "View detailed results";
          break;
        case 6:
          pageName = "View aggregated results";
          break;
      }

      webGUI.pageOpened(tabIndex, pageName);

      return true;
    }
  });

  // Model tabs (within "Edit model data")
  var modelTabs = $("#modelTabs").tabs("#modelTabsContainer > div", {
    effect: "fade",
    fadeOutSpeed: null,
    fadeInSpeed: 150,
    current: "active",
    onClick: function (ignore, tabIndex) {
      if (tabIndex === 2) { // "Outcome tree" tab
        $.event.trigger("flowChartShown");
      } else {
        bcode.editModelData.diseasePageState.flowChartVisible = false;
      }
      return true;
    }
  });

  var pageTabsApi = pageTabs.data("tabs");
  var modelTabsApi = modelTabs.data("tabs");

  // Guided tour
  bcode.tutorial.initialize(pageTabsApi, modelTabsApi);

  $(".pdf").on("click", function () {
    webGUI.exportPageToPdf();
  });

  function onProgramReinitialized() {
    console.log("onProgramReinitialized");
    bcode.createModels.modelsPageState.reinitialize();
    bcode.editModelData.diseasePageState.reinitialize();
    bcode.showDetailedResults.detailedResultsPageState.reinitialize();
    bcode.showAggregatedResults.aggregatedResultsPageState.reinitialize();
  }

  // Show/hide pages
  function onNewModelsCreated(newModelsInfo) {
    console.log("onNewModelsCreated");
    if (newModelsInfo.length > 0) {
      $("#applyModelParamChagesBtn").show();
      $("#editModelData > .contentRow").css("display", "block");
    }
  }

  function onModelsRun() {
    console.log("onModelsRun");
    bcode.showAggregatedResults.aggregatedResultsPageState.reinitialize();
    $("#viewDetailedResults > .contentRow").css("display", "block");
    $("#viewAggregatedResults > .contentRow").css("display", "block");
  }

  // Layout
  var layout = $(window);
  function setLayoutSize() {
    var layoutHeight = layout.height();

    // Create models page
    bcode.createModels.countryTable.height(layoutHeight - 371);
    bcode.createModels.countryGrid.resizeCanvas();

    bcode.createModels.diseaseTable.height(layoutHeight - 71);
    bcode.createModels.diseaseGrid.resizeCanvas();

    bcode.createModels.modelTable.height(layoutHeight - 99);
    bcode.createModels.modelGrid.resizeCanvas();

    // Edit model data page
    bcode.editModelData.flowChartContainer.height(layoutHeight - 87);
  }
  setLayoutSize();
  layout.resize(setLayoutSize);

  /*
   * Hook up C++ engine with JavaScript
   */
  (function(test) {
    if (!test) {
      webGUI.programReinitialized.connect(onProgramReinitialized);
      webGUI.modelsCreated.connect(onNewModelsCreated);
      webGUI.showMessage.connect(bcode.messageSystem.showNotification);
      webGUI.createUpdatableMessage.connect(bcode.messageSystem.showUpdatableNotification);
      webGUI.updateMessage.connect(bcode.messageSystem.showUpdatableNotification);
      webGUI.destroyUpdatableMessage.connect(bcode.messageSystem.hideUpdatableNotification);
      webGUI.runFinished.connect(onModelsRun);
    }
  }(bcode.settings.development));
};
