/* global
    $, Slick, tooltipContentFormatter, copyObject, dispFormula, dispFloat, toFloat, bcode
*/

var gridsModule = function (webGUI) { // eslint-disable-line no-unused-vars

  "use strict";

  // Utilities
  var _getOutputRow = function (metaData, colIds, sourceData, colMapping) {
    colMapping = (typeof (colMapping) === "undefined") ? colIds : colMapping;

    var dataTypes = ["Mean", "Quantile0.025", "Quantile0.5", "Quantile0.975"];
    var result = metaData;

    if (typeof (colIds) !== "undefined") {
      colIds.forEach(function (colId, index) {
        var colSourceData = sourceData[colMapping[index]];
        dataTypes.forEach(function (inDataType) {
          var outDataType;
          if (inDataType === "Mean") {
            outDataType = "";
          } else {
            outDataType = inDataType;
          }
          result[colId + outDataType] = colSourceData[inDataType];
        });
      });
    }

    return result;
  };

  var _tooltipHandles = {
    handleMouseEnter: function (e, args) {
      var position = { x: e.pageX, y: e.pageY };
      var _grid = args.grid;
      var cell = _grid.getCellFromEvent(e);

      if (cell.cell > 0) {
        var node = $(_grid.getCellNode(cell.row, cell.cell));
        if (node) {
          var column = _grid.getColumns()[cell.cell];
          var colId = column.id;
          var item = _grid.getDataItem(cell.row);
          if (item[colId] || (item[colId] === 0)) {
            var contId = $(args.grid.getCanvasNode()).parent().parent().attr("id");
            var label = item.model;
            if (contId !== "aggrResultsTable") {
              label = item.illness;
              if (item.parent) {
                label = _grid.getDataItem(item.parent).illness + "<br />" + label;
              }
            }

            label += "<br />" + column.name;
            // Setup new content
            var contentObject = {
              label: label, // get label from parent
              entries: [
                { label: "Mean", value: item[colId] },
                { label: "2.5%", value: item[colId + "Quantile0.025"] },
                { label: "50%", value: item[colId + "Quantile0.5"] },
                { label: "97.5%", value: item[colId + "Quantile0.975"] },
              ]
            };

            $(node).tooltip({ automatic: false });
            var tooltip = $(node).data("tooltip");
            tooltip.updateContent(tooltipContentFormatter(contentObject));
            $(".tooltip").remove();
            tooltip.show(position);
          }
        }
      }
    },

    handleMouseLeave: function (e, args) {
      var _grid = args.grid;
      var cell = _grid.getCellFromEvent(e);
      var node = $(_grid.getCellNode(cell.row, cell.cell));
      if (node) {
        var tooltip = $(node).data("tooltip");
        if (tooltip) {
          tooltip.hide();
        }
      }
    }
  };

  // Formatters
  var _gridFormatterDefs = {
    formula: function (row, cell, value) {
      if (value == -1) {
        return "Rem. life exp.";
      } else if (value == -2) {
        return "Rem. life exp. - 10";
      } else if (value == -3) {
        return "Remaining cases";
      } else {
        return dispFormula(value);
      }
    },
    numeric: function (row, cell, value) {
      if (value == -1) {
        return "Rem. life exp.";
      } else if (value == -2) {
        return "Rem. life exp. - 10";
      } else if (value == -3) {
        return "Remaining cases";
      } else {
        return dispFloat(value);
      }
    },
    weblink: function (row, cell, value) {
      return "<a href='" + value + "'>" + value + "</a>";
    },
    textCapitalized: function (row, cell, value) {
      return value.replace(/\w\S*/g, function (txt) { return txt.charAt(0).toUpperCase() + txt.substr(1).toLowerCase(); });
    },
    checkbox: function (row, cell, value, columnDef, dataContext) {
      return "<input type='checkbox' " + (value.toLowerCase() === "yes" ? "checked='checked'" : "") + " name='" + dataContext.id + "'>";
    }
  };

  // Editors
  var _gridEditorDefs = {
    textEditor: function (args) {
      var $input;
      var defaultValue;

      this.init = function () {
        $input = $("<input type=text class='editor-text' />")
          .appendTo(args.container)
          .on("keydown.nav", function (e) { if (e.keyCode === 37 || e.keyCode === 39) { e.stopImmediatePropagation(); } })
          .focus()
          .select();
      };

      this.destroy = function () {
        $input.remove();
      };

      this.focus = function () {
        $input.focus();
      };

      this.getValue = function () {
        return $input.val();
      };

      this.setValue = function (val) {
        $input.val(val);
      };

      this.loadValue = function (item) {
        defaultValue = item[args.column.field] || "";
        var numericValue = toFloat(defaultValue);
        if (!isNaN(numericValue)) {
          defaultValue = numericValue;
        }

        $input.val(defaultValue);
        $input[0].defaultValue = defaultValue;
        $input.select();
      };

      this.serializeValue = function () {
        return $input.val();
      };

      this.applyValue = function (item, state) {
        item[args.column.field] = state;
      };

      this.isValueChanged = function () {
        return (!($input.val() == "" && defaultValue == null)) && ($input.val() != defaultValue);
      };

      this.validate = function () {
        return {
          valid: true,
          msg: null
        };
      };

      this.init();
    },

    numericEditor: function (args) {
      var $input;
      var defaultValue;

      this.init = function () {
        $input = $("<input type=text class='editor-text' />");
        $input.appendTo(args.container);
        $input.focus().select();
      };

      this.destroy = function () {
        $input.remove();
      };

      this.focus = function () {
        $input.focus();
      };

      this.loadValue = function (item) {
        defaultValue = item[args.column.field];
        $input.val(defaultValue);
        $input[0].defaultValue = defaultValue;
        $input.select();
      };

      this.serializeValue = function () {
        return parseFloat($input.val()) || 0;
      };

      this.applyValue = function (item, state) {
        item[args.column.field] = state;
        //_updateFooterRow(this.gridStratified, this._options.stratified.aggregatorType);
      };

      this.isValueChanged = function () {
        return (!($input.val() == "" && defaultValue == null)) && ($input.val() != defaultValue);
      };

      this.validate = function () {
        if (isNaN($input.val())) {
          return {
            valid: false,
            msg: "Please enter a valid integer"
          };
        }

        return {
          valid: true,
          msg: null
        };
      };

      this.init();
    },

    urlEditor: function (args) {
      var $input;
      var defaultValue;

      this.init = function () {
        $input = $("<input type=text class='editor-text' />")
          .appendTo(args.container)
          .on("keydown.nav", function (e) { if (e.keyCode === 37 || e.keyCode === 39) { e.stopImmediatePropagation(); } })
          .focus()
          .select();
      };

      this.destroy = function () {
        $input.remove();
      };

      this.focus = function () {
        $input.focus();
      };

      this.getValue = function () {
        return $input.val();
      };

      this.setValue = function (val) {
        $input.val(val);
      };

      this.loadValue = function (item) {
        defaultValue = item[args.column.field] || "";
        $input.val(defaultValue);
        $input[0].defaultValue = defaultValue;
        $input.select();
      };

      this.serializeValue = function () {
        var link = $input.val();
        if (link.search(/^(ht|f)tp(s?):\/\//) == -1) {
          link = "http://" + link;
        }

        return link;
      };

      this.applyValue = function (item, state) {
        item[args.column.field] = state;
      };

      this.isValueChanged = function () {
        return (!($input.val() == "" && defaultValue == null)) && ($input.val() != defaultValue);
      };

      this.validate = function () {
        return {
          valid: true,
          msg: null
        };
      };

      this.init();
    },

    longTextEditor: function (args) {
      var $input, $wrapper;
      var defaultValue;
      var scope = this;

      this.init = function () {
        var $container = $("body");
        var textAreaWidth = args.column.width - 20;
        $wrapper = $("<div class='shadow' style='z-index:10000;position:absolute;background:white;padding:5px;border: 1px solid gray;'/>").appendTo($container);
        $input = $("<textarea hidefocus style='backround:white;width:" + textAreaWidth + "px;height:80px;border:1px solid silver'>").appendTo($wrapper);
        $("<div style='text-align:right'><button>Save</button><button>Cancel</button></div>").appendTo($wrapper);

        var buttonWidth = Math.min(textAreaWidth / 2 + 4, 100);
        $wrapper.find("button:first").css("width", buttonWidth).on("click", this.save);
        $wrapper.find("button:last").css("width", buttonWidth).on("click", this.cancel);
        $input.on("keydown", this.handleKeyDown);

        scope.position(args.position);
        $input.focus().select();
      };

      this.handleKeyDown = function (e) {
        if (e.which == 13 && e.ctrlKey) {// ENTER
          scope.save();
        } else if (e.which == 27) { // ESCAPE
          e.preventDefault();
          scope.cancel();
        } else if (e.which == 9 && e.shiftKey) { // TAB
          e.preventDefault();
          args.grid.navigatePrev();
        } else if (e.which == 9) { // TAB
          e.preventDefault();
          args.grid.navigateNext();
        }
      };

      this.save = function () {
        args.commitChanges();
      };

      this.cancel = function () {
        $input.val(defaultValue);
        args.cancelChanges();
      };

      this.hide = function () {
        $wrapper.hide();
      };

      this.show = function () {
        $wrapper.show();
      };

      this.position = function (position) {
        $wrapper.css("top", position.top).css("left", position.left);
      };

      this.destroy = function () {
        $wrapper.remove();
      };

      this.focus = function () {
        $input.focus();
      };

      this.loadValue = function (item) {
        $input.val(defaultValue = item[args.column.field]);
        $input.select();
      };

      this.serializeValue = function () {
        var text = $input.val().split("\n").filter(function (elem) { return elem != ""; });
        text.forEach(function (elem, index, array) { array[index] = elem.trim(); });
        return text.join("\n");
      };

      this.applyValue = function (item, state) {
        item[args.column.field] = state;
      };

      this.isValueChanged = function () {
        return (!($input.val() == "" && defaultValue == null)) && ($input.val() != defaultValue);
      };

      this.validate = function () {
        return {
          valid: true,
          msg: null
        };
      };

      this.init();
    }
  };


  // Columns definitions
  var _gridColumnDefs = {
    formulaNonEditColumns: [
      { id: "Age", name: "Age", field: "Age", cssClass: "alignCenter", headerCssClass: "alignCenter", selectable: false, focusable: false, width: 40 },
      { id: "Female", name: "Female", field: "Female", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.formula, selectable: false, focusable: false },
      { id: "Male", name: "Male", field: "Male", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.formula, selectable: false, focusable: false }
    ],
    formulaEditColumns: [
      { id: "Age", name: "Age", field: "Age", cssClass: "alignCenter", headerCssClass: "alignCenter", selectable: false, focusable: false, width: 40 },
      { id: "Female", name: "Female", field: "Female", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.formula, editor: _gridEditorDefs.textEditor },
      { id: "Male", name: "Male", field: "Male", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.formula, editor: _gridEditorDefs.textEditor }
    ],
    numericNonEditColumns: [
      { id: "Age", name: "Age", field: "Age", cssClass: "alignCenter", headerCssClass: "alignCenter", selectable: false, focusable: false, width: 40 },
      { id: "Female", name: "Female", field: "Female", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false },
      { id: "Male", name: "Male", field: "Male", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false }
    ],
    numericEditColumns: [
      { id: "Age", name: "Age", field: "Age", cssClass: "alignCenter", headerCssClass: "alignCenter", selectable: false, focusable: false, width: 40 },
      { id: "Female", name: "Female", field: "Female", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, editor: _gridEditorDefs.numericEditor },
      { id: "Male", name: "Male", field: "Male", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, editor: _gridEditorDefs.numericEditor }
    ],
    statsNumericColumns: [
      { id: "Age", name: "Age", field: "Age", cssClass: "alignCenter", headerCssClass: "alignCenter", selectable: false, focusable: false, width: 60 },
      { id: "Quantile0.025", name: "2.5%", field: "Quantile0.025", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false },
      { id: "Quantile0.5", name: "50%", field: "Quantile0.5", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false },
      { id: "Quantile0.975", name: "97.5%", field: "Quantile0.975", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false },
      { id: "Mean", name: "Mean", field: "Mean", cssClass: "alignRight", headerCssClass: "alignRight", formatter: _gridFormatterDefs.numeric, selectable: false, focusable: false }
    ],
    metaColumns: [
      { id: "DisplayString", name: "Parameter", field: "DisplayString", cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false, width: 150 },
      { id: "Value", name: "Value", field: "Value", cssClass: "alignLeft", headerCssClass: "alignLeft", editor: _gridEditorDefs.textEditor }
    ]
  };

  var _gridTooltipHandles = {
    handleMouseEnter: function (e, args) {
      var position = { x: e.pageX, y: e.pageY };

      var node = {};
      if (args) {
        var _grid = args.grid;
        var cell = _grid.getCellFromEvent(e);
        if (cell) {
          node = $(_grid.getCellNode(cell.row, cell.cell));
        }
      } else {
        node = $(e.target);
      }

      if (node) {
        if (node.innerWidth() < node[0].scrollWidth) {
          var text = $.trim(node.text());
          if (text != "") {
            $(node).tooltip({ automatic: false });
            var tooltip = $(node).data("tooltip");
            tooltip.updateContent(text);
            tooltip.show(position);
          }
        }
      }
    },

    handleMouseLeave: function (e, args) {
      if (args) {
        var _grid = args.grid;
        var cell = _grid.getCellFromEvent(e);
        var node = $(_grid.getCellNode(cell.row, cell.cell));
        if (node) {
          var tooltip = $(node).data("tooltip");
          if (tooltip) {
            tooltip.hide();
          }
        }
      }
    }
  };

  // Aggregation
  var _sumAggregator = function (grid) {
    var data = grid.getData();
    var columns = grid.getColumns();
    var i = 0;

    var result = [];
    if (typeof (data) != "undefined") {
      if (!$.isArray(data)) {
        data = data.getItems();
      }

      result = Array.dim(columns.length - 1, 0);
      for (i = 0; i < data.length; ++i) {
        for (var j = 1; j < columns.length; ++j) {
          result[j - 1] += toFloat(data[i][columns[j].id]);
        }
      }
    }

    for (i = 0; i < columns.length; ++i) {
      var footer = grid.getFooterRowColumn(columns[i].id);
      if (i === 0) {
        $(footer).text("Total");
      } else {
        $(footer).text(dispFloat(result[i - 1]));
      }
    }
  };

  var _averageAggregator = function (grid) {
    var data = grid.getData();
    var columns = grid.getColumns();
    var i = 0;

    var result = [];
    if (typeof (data) !== "undefined") {
      if (!$.isArray(data)) {
        data = data.getItems();
      }
      result = Array.dim(columns.length - 1, 0);
      for (i = 0; i < data.length; ++i) {
        for (var j = 1; j < columns.length; ++j) {
          result[j - 1] += toFloat(data[i][columns[j].id]) / data.length;
        }
      }
    }

    for (i = 0; i < columns.length; ++i) {
      var footer = grid.getFooterRowColumn(columns[i].id);
      if (i === 0) {
        $(footer).text("Average");
      } else {
        $(footer).text(dispFloat(result[i - 1]));
      }
    }

    return result;
  };

  // Drop down
  var _showValidationMsg = function (validationMsgElem, validationCheck) {
    validationCheck.success ? validationMsgElem.removeClass("error").addClass("success") : validationMsgElem.removeClass("success").addClass("error");
    validationMsgElem.text(validationCheck.msg);
  };

  var _showStatisticValues = function (distrDef, statsMsgElem, values) {
    statsMsgElem[0].html(dispFloat(distrDef.mean(values)));
    statsMsgElem[1].html(dispFloat(distrDef.variance(values)));
  };

  var _setSimplifiedData = function (inputTable, distrType, values) {
    var vals = inputTable._options.simplified.valueType === "percentage" ? values.map(function (elem) { return elem / 100; }) : values;
    inputTable._simplifiedData = {
      distrType: distrType,
      values: vals,
      string: distrType !== "Constant" ? "Rand" + distrType + "(" + vals.join(", ") + ")" : vals[0]
    };
  };

  // Inputs
  var distrDefs = {
    Constant: {
      htmlString: "<label>Value</label><input type='text' value='1' />",
      weblink: "",
      validate: function () {
        return { success: true, msg: "Valid value" };
      },
      mean: function (values) { return values[0]; },
      variance: function () { return 0; }
    },
    Uniform: {
      htmlString: "<label>Lower bound</label><input type='text' value='0' /><br />" +
        "<label>Upper bound</label><input type='text' value='1' />",
      weblink: "http://en.wikipedia.org/wiki/Uniform_distribution_(continuous)",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[0] > values[1]) {
          validationCheck.msg = "Lower bound cannot be greater than upper bound";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) { return (values[0] + values[1]) / 2; },
      variance: function (values) { return 1 / 12 * Math.pow(values[1] - values[0], 2); }
    },
    Pert: {
      htmlString: "<label>Lower bound</label><input type='text' value='0' /><br />" +
        "<label>Most likely</label><input type='text' value='0.5' /><br />" +
        "<label>Upper bound</label><input type='text' value='1' />",
      weblink: "",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[0] > values[1]) {
          validationCheck.msg = "Lower bound cannot be greater than upper bound";
        } else if (values[1] > values[2]) {
          validationCheck.msg = "Most likely value cannot be greater than upper bound";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) {
        var a = values[0];
        var b = values[1];
        var c = values[2];
        var alpha = 1 + 4 * (b - a) / (c - a);
        var beta = 1 + 4 * (c - b) / (c - a);
        return a + distrDefs.Beta.mean([alpha, beta]) * (c - a);
      },
      variance: function (values) {
        var a = values[0];
        var b = values[1];
        var c = values[2];
        var alpha = 1 + 4 * (b - a) / (c - a);
        var beta = 1 + 4 * (c - b) / (c - a);
        return distrDefs.Beta.variance([alpha, beta]) * Math.pow(c - a, 2);
      },
    },
    Beta: {
      htmlString: "<label>Shape α</label><input type='text' value='1' /><br />" +
        "<label>Shape β</label><input type='text' value='1' /><br />",
      weblink: "http://en.wikipedia.org/wiki/Beta_distribution",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[0] <= 0) {
          validationCheck.msg = "Shape parameter α cannot be lower than 0";
        } else if (values[1] <= 0) {
          validationCheck.msg = "Shape parameter β cannot be lower than 0";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) {
        var alpha = values[0];
        var beta = values[1];
        return alpha / (alpha + beta);
      },
      variance: function (values) {
        var alpha = values[0];
        var beta = values[1];
        return alpha * beta / (Math.pow(alpha + beta, 2) * (alpha + beta + 1));
      }
    },
    Gamma: {
      htmlString: "<label>Shape α</label><input type='text' value='1' /><br />" +
        "<label>Rate β</label><input type='text' value='1' /><br />",
      weblink: "http://en.wikipedia.org/wiki/Gamma_distribution",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[0] <= 0) {
          validationCheck.msg = "Shape parameter α cannot be lower than 0";
        } else if (values[1] <= 0) {
          validationCheck.msg = "Rate parameter β cannot be lower than 0";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) {
        var alpha = values[0];
        var beta = values[1];
        return alpha / beta;
      },
      variance: function (values) {
        var alpha = values[0];
        var beta = values[1];
        return alpha / Math.pow(beta, 2);
      }
    },
    LogNormal: {
      htmlString: "<label>Log-scale μ</label><input type='text' value='1' /><br />" +
        "<label>Shape σ</label><input type='text' value='1' /><br />",
      weblink: "http://en.wikipedia.org/wiki/Log-normal_distribution",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[1] <= 0) {
          validationCheck.msg = "Shape parameter σ cannot be non-positive";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) {
        var m = values[0];
        var s = values[1];
        return Math.pow(Math.E, m + Math.pow(s, 2) / 2);
      },
      variance: function (values) {
        var m = values[0];
        var s = values[1];
        var e = Math.E;
        var s2 = Math.pow(s, 2);
        return (Math.pow(e, s2) - 1) * Math.pow(e, 2 * m + s2);
      }
    },
    Normal: {
      htmlString: "<label>Scale μ</label><input type='text' value='0' /><br />" +
        "<label>Std. dev. σ</label><input type='text' value='1' /><br />",
      weblink: "https://en.wikipedia.org/wiki/Normal_distribution",
      validate: function (values) {
        var validationCheck = { success: false };
        if (values[1] < 0) {
          validationCheck.msg = "Standard deviation parameter σ cannot be negative";
        } else {
          validationCheck.msg = "Valid values";
          validationCheck.success = true;
        }
        return validationCheck;
      },
      mean: function (values) {
        var m = values[0];
        return m;
      },
      variance: function (values) {
        var s = values[1];
        var s2 = Math.pow(s, 2);
        return s2;
      }
    },
  };

  // Private utilities
  var _updateFooterRow = function (grid, aggregatorType) {
    var data = grid.getData();
    if ((["sum", "average"].indexOf(aggregatorType) != -1) && data && ((data.length > 0) || (data.getItems().length > 0))) {
      var aggregatorFunction;
      switch (aggregatorType) {
        case "sum":
          aggregatorFunction = _sumAggregator;
          break;
        case "average":
          aggregatorFunction = _averageAggregator;
          break;
      }
      grid.showFooterRowColumns();
      aggregatorFunction(grid);
    }
  };

  var _setErrorStatus = function (inputTable, error) {
    if (error) {
      $("span.moduleCaption", inputTable.container).addClass("red");
    } else {
      $("span.moduleCaption", inputTable.container).removeClass("red");
    }
  };

  var _switchTableType = function (inputTable, viewType) {
    var speed = 80;
    if (viewType != inputTable._options.viewType) {
      inputTable[inputTable._options.viewType + "Container"].fadeOut(speed, function () { inputTable[viewType + "Container"].fadeIn(speed); });
      $(".gridTrigger", inputTable.container).removeClass("active");
      $("." + viewType + "Trigger", inputTable.container).addClass("active");
      inputTable._options.viewType = viewType;
    }
  };

  var _setDistr = function (inputTable, distrType) {
    var distrDef = distrDefs[distrType];
    $(".distrParams", inputTable.simplifiedContainer).showHtml(distrDef.htmlString);
    if (distrDef.weblink != "") {
      $(".distrWeblink", inputTable.simplifiedContainer).showHtml("<div><label>Weblink</label></div><div style='word-break: break-word;'><a href='" + distrDef.weblink + "'>" + distrDef.weblink + "</a></div>");
    } else {
      $(".distrWeblink", inputTable.simplifiedContainer).showHtml("");
    }
    var inputs = $(".distrParams > input", inputTable.simplifiedContainer);
    var validationMsgElem = $(inputs[0]).parent().next().next().children(".distrValid-msg");
    var statsMsgElem = [];
    statsMsgElem.push(validationMsgElem.parent().next().children(".distrMean-value"));
    statsMsgElem.push(validationMsgElem.parent().next().next().children(".distrVariance-value"));
    var autoNumericOptions = {
      aSep: ",",
      aDec: ".",
      aSign: inputTable._options.simplified.valueType === "percentage" ? "%" : "",
      pSign: "s",
      vMin: inputTable._options.simplified.minimum ? inputTable._options.simplified.minimum : -99999999999,
      vMax: inputTable._options.simplified.maximum ? inputTable._options.simplified.maximum : 99999999999,
      mDec: "7",
      aPad: false,
      wEmpty: "zero",
      lZero: "deny"
    };
    $.each(inputs, function (index, input) {
      $(input).autoNumeric(autoNumericOptions).on("keyup", function () {
        var values = inputs.map(function (index, input) { return parseFloat($(input).autoNumeric("get")); }).toArray();
        if (inputTable._options.simplified.valueType === "percentage") {
          values = values.map(function (elem) { return elem / 100; });
        }
        var validationCheck = distrDef.validate(values);
        _showValidationMsg(validationMsgElem, validationCheck);
        if (validationCheck.success) {
          _setSimplifiedData(inputTable, distrType, values);
          _showStatisticValues(distrDef, statsMsgElem, values);
        }
      });
    });
    // Fire first check
    {
      var values = inputs.map(function (index, input) { return parseFloat($(input).autoNumeric("get")); }).toArray();
      if (inputTable._options.simplified.valueType === "percentage") {
        values = values.map(function (elem) { return elem / 100; });
      }
      var validationCheck = distrDef.validate(values);
      _showValidationMsg(validationMsgElem, validationCheck);
      if (validationCheck.success) {
        _setSimplifiedData(inputTable, distrType, values);
        _showStatisticValues(distrDef, statsMsgElem, values);
      }
    }

    inputTable._options.simplified.distrType = distrType;
  };

  var _setFormulaInGrid = function (inputTable, formula) {
    var finalValue = formula;
    var numericValue = toFloat(formula);
    if (!isNaN(numericValue)) {
      finalValue = numericValue;
    }

    var newData = inputTable.getData();
    newData.stratified = newData.stratified.map(function (elem) {
      var newElem = elem;
      newElem.Female = finalValue;
      newElem.Male = finalValue;
      return newElem;
    });
    inputTable.setData(newData);
    //var oldData = inputTable.getData();
    //var newData = oldData.map(function(elem, index) {var newElem = elem; newElem.Female = finalValue; newElem.Male = finalValue; return newElem;});
    //inputTable.gridStratified.setData(newData);
    //inputTable.gridStratified.invalidate();
  };

  var _exportGridToText = function (inputTable) {
    var grid = inputTable.gridStratified;
    var headerColNames = grid.getColumns().map(function (elem) { return elem.field; });
    var dataString = grid.getData().map(function (elem) { return headerColNames.map(function (el) { return elem[el]; }).join("\t"); }).join("\n");
    return [headerColNames.join("\t"), dataString].join("\n");
  };

  var _setColumnsWidths = function (columns, totalWidth) {
    columns = columns.map(function (elem) { return $.extend(true, {}, elem); });
    var columnWidths = columns.map(function (elem) { return elem.width ? elem.width : 0; });
    var colWithoutWidthCount = columnWidths.filter(function (elem) { return elem === 0; }).length;
    var takenWidth = columnWidths.reduce(function (a, b) { return a + b; });
    var widthPerCol = Math.ceil((totalWidth - takenWidth) / colWithoutWidthCount);
    columns.forEach(function (elem) { if (!elem.width) { elem.width = widthPerCol; } });
    var lastColWidth = totalWidth - takenWidth - widthPerCol * colWithoutWidthCount;
    columns[columns.length - 1].width += lastColWidth;
    return columns;
  };

  var _delegateEvents = function (inputTable) {
    $(".stratifiedTrigger", inputTable.container).on("click", function (event) { event.preventDefault(); inputTable.switchTableType("stratified"); });
    inputTable.gridStratified.onCellChange.subscribe(function () {
      _updateFooterRow(inputTable.gridStratified, inputTable._options.stratified.aggregatorType);
    });

    if (inputTable._options.simplified.enabled) {
      $(".simplifiedTrigger", inputTable.container).on("click", function (event) { event.preventDefault(); inputTable.switchTableType("simplified"); });
      if (inputTable.simplifiedContainer) {
        $("select", inputTable.simplifiedContainer).on("change", function (event) { event.preventDefault(); _setDistr(inputTable, $(this).val()); });
      }
      $(".copyToStratified", inputTable.simplifiedContainer).on("click", function (event) {
        event.preventDefault();
        var formula = inputTable._simplifiedData.string;
        _setFormulaInGrid(inputTable, formula);
        _switchTableType(inputTable, "stratified");
        bcode.messageSystem.showNotification("Simplified to Stratified copying", "Stratified view of " + inputTable._options.caption + " table has been filled in with: \"" + formula + "\"");
      });
    }

    if (inputTable._options.meta.enabled) {
      $(".metaTrigger", inputTable.container).on("click", function (event) { event.preventDefault(); inputTable.switchTableType("meta"); });
    }

    inputTable.container.on("blur", "input.editor-text", function () {
      window.setTimeout(function () {
        if (Slick.GlobalEditorLock.isActive())
          Slick.GlobalEditorLock.commitCurrentEdit();
      });
    });

    if (inputTable._options.meta.enabled) {
      inputTable.metaContainer.on("click", "input[type=\"checkbox\"]", function () {
        var id = $(this).prop("name");
        var item = inputTable.gridDataView.getItemById(id);
        item.Value = $(this).prop("checked") ? "Yes" : "No";
      });
    }
  };

  // HTML templates
  var containerTemplate = "<div>%caption%</div>";
  var stratifiedContainerTemplate = "<div class='stratifiedContainer' style='height: %height%px'></div>";
  var simplifiedContainerTemplate = "<div class='simplifiedContainer' style='height: %height%px'>" +
    "   <fieldset style='height: %heightSimplified%px'>" +
    "       <div>" +
    "           <div><label>Distribution type</label></div>" +
    "           <div><select>" +
    "               <option value='Constant'>Constant</option>" +
    "               <option value='Uniform'>Uniform</option>" +
    "               <option value='Pert'>Pert</option>" +
    "               <option value='Beta'>Beta</option>" +
    "               <option value='Gamma'>Gamma</option>" +
    "               <option value='LogNormal'>LogNormal</option>" +
    "               <option value='Normal'>Normal</option>" +
    "           </select></div>" +
    "       </div>" +
    "       <div class='distrParams'></div>" +
    "       <button class='copyToStratified'>Copy to Stratified</button>" +
    "       <div class='distrValid'><div><label>Validation</label></div><div class='distrValid-msg'></div></div>" +
    "       <div class='distrMean'><div><label>Mean</label></div><div class='distrMean-value'></div></div>" +
    "       <div class='distrVariance'><div><label>Variance</label></div><div class='distrVariance-value'></div></div>" +
    "       <div class='distrWeblink'></div>" +
    "   </fieldset>" +
    "</div>";
  var metaContainerTemplate = "<div class='metaContainer' style='height: %height%px'></div>";

  var defaultInputTableOptions = {
    caption: "",
    tooltip: "",
    viewType: "stratified",
    editable: true,
    append: true, // appendTo if true, prependTo if false
    width: 300,
    height: 525,
    table: { containerId: "", containerClass: "moduleBox floatLeft" },
    stratified: { formulaType: true, aggregatorType: "none", createContexMenu: true },
    simplified: { enabled: true, distrType: "Constant", valueType: "float" }, // valueType can be either 'float' or 'percentage'
    meta: { enabled: true }
  };

  // Grid options
  var defaultGridOptions = {
    asyncEditorLoading: false,
    autoEdit: false,
    enableColumnReorder: false,
    editable: defaultInputTableOptions.editable,
    createFooterRow: true,
    autoHeight: true,
    enableAddRow: false,
    enableCellNavigation: true,
    forceFitColumns: true
  };

  // Constructor
  function InputTable(parentElem, options) {
    // Public properties
    this._options = $.extend(true, {}, defaultInputTableOptions, (typeof (options) == "object") ? options : {});
    this._data = {};

    // Private properties
    var _self = this;
    var _formulaType = this._options.stratified.formulaType;
    var _editable = this._options.editable;

    var height = this._options.stratified.aggregatorType != "none" ? this._options.height + 25 : this._options.height;

    // Initialization
    var locContainerTemplate = "";
    if (this._options.caption !== "") {
      locContainerTemplate = containerTemplate.replace("%caption%",
        "<h2 class='moduleTitle'>%tooltip%<span class='moduleCaption'>" + this._options.caption + "</span>" +
        (this._options.simplified.enabled || this._options.meta.enabled ? "<div class='moduleIcons'><i class='stratifiedTrigger gridTrigger icon-stratified active'></i>" : "") +
        (this._options.simplified.enabled ? "<i class='simplifiedTrigger gridTrigger icon-simplified'></i>" : "") +
        (this._options.meta.enabled ? "<i class='metaTrigger gridTrigger icon-meta'></i>" : "") +
        (this._options.simplified.enabled || this._options.meta.enabled ? "</div>" : "") +
        "</h2>");
      if (this._options.tooltip !== "") {
        locContainerTemplate = locContainerTemplate.replace("%tooltip%", "<span class='help'>?</span><div class='tooltipContent'>" + this._options.tooltip + "</div>");
      } else {
        locContainerTemplate = locContainerTemplate.replace("%tooltip%", "");
      }
    } else {
      locContainerTemplate = containerTemplate.replace("%caption%", "");
    }

    if (this._options.append) {
      this.container = $(locContainerTemplate).appendTo(parentElem);
    } else {
      this.container = $(locContainerTemplate).prependTo(parentElem);
    }

    this.container.addClass(this._options.table.containerClass);
    this.container.css("width", this._options.width);
    if (this._options.table.containerId !== "") {
      this.container.attr("id", this._options.table.containerId);
    }
    // Attach tooltip
    $(".help", this.container).tooltip();

    // Three types of input table UI are available:
    // a) Stratified = grid
    // b) Simplified = form (enabled only if the table is editable)
    // c) Meta = meta information

    // a) Grid container
    var columns = [];
    if (_formulaType && !_editable) {
      columns = _gridColumnDefs.formulaNonEditColumns;
    } else if (_formulaType && _editable) {
      columns = _gridColumnDefs.formulaEditColumns;
    } else if (!_formulaType && !_editable) {
      columns = _gridColumnDefs.numericNonEditColumns;
    } else if (!_formulaType && _editable) {
      columns = _gridColumnDefs.numericEditColumns;
    }

    // Deep copy with methods is required here since width is overwritten later
    columns = _setColumnsWidths(columns, this._options.width);

    var locStratifiedContainerTemplate = stratifiedContainerTemplate.replace("%height%", height);
    this.stratifiedContainer = $(locStratifiedContainerTemplate).appendTo(this.container);
    if (_editable) {
      this.stratifiedContainer.addClass("editableGrid");
    }

    var gridOptions = $.extend({}, defaultGridOptions, { editable: _editable, forceFitColumns: false });
    this.gridStratified = new Slick.Grid(this.stratifiedContainer, [], columns, gridOptions);
    this.gridStratified.onMouseEnter.subscribe(_gridTooltipHandles.handleMouseEnter);
    this.gridStratified.onMouseLeave.subscribe(_gridTooltipHandles.handleMouseLeave);
    this.gridStratified.onDblClick.subscribe(_gridTooltipHandles.handleMouseLeave);
    $(".slick-footerrow-column", this.stratifiedContainer).on("mouseenter", { grid: this.gridStratified }, _gridTooltipHandles.handleMouseEnter);
    $(".slick-footerrow-column", this.stratifiedContainer).on("mouseleave", { grid: this.gridStratified }, _gridTooltipHandles.handleMouseLeave);

    if (this._options.stratified.createContexMenu) {
      var _menu = [
        {
          "Fill with this formula": function () {
            if (_self.gridStratified.getActiveCellNode()) {
              _setFormulaInGrid(_self, $(_self.gridStratified.getActiveCellNode()).text());
            }
          }
        },
        { "Copy": function () { webGUI.copyTextToClipboard(_exportGridToText(_self)); } },
        {
          "Paste as Constant": function () {
            var columns = _self.gridStratified.getColumns();
            var firstColName = columns[0].id;
            var rowNames = _self._data.stratified.map(function (elem) { return elem[firstColName]; });
            var colNames = columns.filter(function (elem, index) { return index > 0; }).map(function (elem) { return elem.id; });
            webGUI.pasteFromClipboardToGrid(rowNames, colNames, "Constant", function (data) {
              _self.setData(data);
            });
          }
        },
        {
          "Paste as Uniform distribution": function () {
            var columns = _self.gridStratified.getColumns();
            var firstColName = columns[0].id;
            var rowNames = _self._data.stratified.map(function (elem) { return elem[firstColName]; });
            var colNames = columns.filter(function (elem, index) { return index > 0; }).map(function (elem) { return elem.id; });
            webGUI.pasteFromClipboardToGrid(rowNames, colNames, "Uniform", function (data) {
              _self.setData(data);
            });
          }
        },
        {
          "Paste as Pert distribution": function () {
            var columns = _self.gridStratified.getColumns();
            var firstColName = columns[0].id;
            var rowNames = _self._data.stratified.map(function (elem) { return elem[firstColName]; });
            var colNames = columns.filter(function (elem, index) { return index > 0; }).map(function (elem) { return elem.id; });
            webGUI.pasteFromClipboardToGrid(rowNames, colNames, "Pert", function (data) {
              _self.setData(data);
            });
          }
        }
      ];
      this.stratifiedContainer.children(".slick-viewport").contextMenu(_menu);
    }

    // b) Drop down container (initially hidden)
    if (this._options.simplified.enabled) {
      var locSimplifiedContainerTemplate = simplifiedContainerTemplate.replace("%height%", height).replace("%heightSimplified%", height - 12);
      this.simplifiedContainer = $(locSimplifiedContainerTemplate).appendTo(this.container).hide();
      _setDistr(this, this._options.simplified.distrType);
    }

    // c) Meta container (initially hidden)
    if (this._options.meta.enabled) {
      columns = [];
      columns = _setColumnsWidths(_gridColumnDefs.metaColumns, this._options.width);
      var locMetaContainerTemplate = metaContainerTemplate.replace("%height%", height);
      this.metaContainer = $(locMetaContainerTemplate).appendTo(this.container).hide();
      this.gridDataView = new Slick.Data.DataView();
      this.gridDataView.getItemMetadata = function (row) {
        var options;
        var paramName = this.getItem(row).ParameterName.toLowerCase();
        switch (paramName) {
          case "description":
            options = { "columns": { "Value": { "editor": _gridEditorDefs.longTextEditor } } };
            break;
          case "weblink":
            options = { "columns": { "Value": { "formatter": _gridFormatterDefs.weblink, "editor": _gridEditorDefs.urlEditor } } };
            break;
          case "annual profile":
          case "remaining cases":
          case "redistribute":
            options = { "columns": { "Value": { "formatter": _gridFormatterDefs.checkbox, "editor": null } } };
            break;
          case "direct type":
            options = { "columns": { "Value": { "formatter": _gridFormatterDefs.checkbox, "editor": null } } };
            break;
          case "duration":
            options = { "columns": { "Value": { "formatter": _gridFormatterDefs.formula, "editor": _gridEditorDefs.textEditor } } };
            break;
          default:
            options = null;
            break;
        }

        return options;
      };
      this.gridMeta = new Slick.Grid(this.metaContainer, this.gridDataView, columns, gridOptions);
      this.gridMeta.onMouseEnter.subscribe(_gridTooltipHandles.handleMouseEnter);
      this.gridMeta.onMouseLeave.subscribe(_gridTooltipHandles.handleMouseLeave);
      this.gridMeta.onDblClick.subscribe(_gridTooltipHandles.handleMouseLeave);
    }

    // Delegate events
    _delegateEvents(this);

  }

  InputTable.prototype = {

    // Public API
    getData: function () {
      return this._data;
    },

    setData: function (data) {
      var dataCopy = copyObject(data);
      $.extend(true, this._data, dataCopy);
      if (dataCopy.hasOwnProperty("stratified"))
        this._data.stratified = dataCopy.stratified;

      if (this._data.hasOwnProperty("stratified") && this._data.stratified.length > 0) {
        this.gridStratified.setData(this._data.stratified);
        _updateFooterRow(this.gridStratified, this._options.stratified.aggregatorType);
        this.gridStratified.invalidate();
      }

      if (this._options.meta.enabled && this._data.hasOwnProperty("meta") && this._data.meta.length > 0) {
        this.gridDataView.setItems(this._data.meta);
        this.gridMeta.invalidate();
      }

      if (this._data.meta) {
        var remCasesObj = $.grep(this._data.meta, function (elem) { return elem.ParameterName === "remaining cases"; })[0];
        if (remCasesObj) {
          if (remCasesObj.Value === "Yes") {
            this.switchTableType("meta");
          }
        }
      }
      $(this).trigger("newDataSet");
    },

    highlightErrorCells: function (highlightData) {
      if (highlightData) {
        this.gridStratified.setCellCssStyles("formulaError", $.extend({}, highlightData.stratified.map(function (elem) {
          return { Female: elem.Female ? "success" : "error", Male: elem.Male ? "success" : "error" };
        })));
        _setErrorStatus(this, true);
      } else {
        this.gridStratified.removeCellCssStyles("formulaError");
        _setErrorStatus(this, false);
      }
      return null;
    },

    switchTableType: function (viewType) {
      if (this._options.simplified.enabled && (viewType === "simplified")) {
        _switchTableType(this, viewType);
      } else if (this._options.meta.enabled && (viewType === "meta")) {
        _switchTableType(this, viewType);
      } else if (viewType === "stratified") {
        _switchTableType(this, viewType);
      }
    }

  };

  var _nonEditGridOptions = $.extend({}, defaultGridOptions, { autoHeight: false });

  return {
    InputTable: InputTable,
    gridColumnDefs: _gridColumnDefs,
    gridFormatterDefs: _gridFormatterDefs,
    gridDefaultOptions: defaultGridOptions,
    nonEditGridOptions: _nonEditGridOptions,
    fillColumnsWidths: _setColumnsWidths,
    updateFooterRow: _updateFooterRow,
    getOutputRow: _getOutputRow,
    resultsTooltipHandles: _tooltipHandles,
    gridTooltipHandles: _gridTooltipHandles
  };

};
