/* global
    $, isNumber, domtoimage
*/

/*
 * Outcome tree connection drawing
 */

var initializeModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars

  "use strict";

  /*
   * Export grid to string in an Excel-friendly format
   */
  function exportGridToExcel(gridId, includeQuantileData) {

    if (typeof (includeQuantileData) === "undefined") {
      includeQuantileData = false;
    }

    var sg = $(gridId).data().grid;

    var columns = sg.getColumns();
    var data = sg.getData();
    if (!$.isArray(sg.getData())) {
      data = data.getItems();
    }

    var headStr = "<html><head>" +
      "<style type='text/css'>" +
      "table {background-color: white; font-family: Tahoma, sans-serif; font-size: 11px; color: #333}" +
      ".header {font-weight: bold; background-color: #E7F0D8}" +
      ".finalTotals, .sequelaTotals, .sectionHeader, .totalInfected {font-weight: bold;} " +
      ".sectionHeader {background-color: #EBEDF2 !important;}" +
      ".healthOutcomeHeader, .sequelaTotals {background-color: whiteSmoke !important;}" +
      ".intervalhealthOutcomeHeader, .intervalsequelaTotals {background-color: whiteSmoke !important}" +
      ".intervalhealthOutcomeHeader, .intervalsequelaTotals, .intervaltotalInfected, .intervaldata, .intervalundefined {color: #C65911}" +
      ".fixedFormat {mso-number-format: '#,##0.00'}" +
      ".scientificFormat {mso-number-format: '0.00E+00'}" +
      ".textFormat {mso-number-format: '@'}" +
      "</style></head><body><table border='1' bordercolor='#B5B5B5'>";

    var footStr = "</table></body></html>";

    var colSpan = includeQuantileData ? 3 : 1;

    var headerStr = "<tr class='header'>" + columns.map(function (column, colIndex) { return "<td" + (colIndex > 0 ? " colspan='" + colSpan + "' align='center'" : "") + ">" + column.name + "</td>"; }).join("") + "</tr>";

    var contentStr = data.map(function (item) {
      var valueRowStr = "<tr class='" + item.type + "'>" +
        columns.map(function (column, colIndex) {
          var value = item[column.id];
          var className = "textFormat";
          if (isNumber(value)) {
            className = ((Math.abs(value) >= 0.01) || (value === 0.0)) ? "fixedFormat" : "scientificFormat";
          }
          var startTag = "<td class='" + className + "' align='center'" + (colIndex > 0 ? " colspan='" + colSpan + "'" : "") + ">";
          return startTag + (typeof (value) === "undefined" || value === null ? "" : value) + "</td>";
        }).join("") +
        "</tr>";

      var intervalRowStr = "";
      if (includeQuantileData) {
        var quantiles = ["Quantile0.025", "Quantile0.5", "Quantile0.975"];
        intervalRowStr = (item.type !== "sectionHeader") ? "<tr class='interval" + item.type + "'>" +
          columns.map(function (column, colIndex) {
            return (colIndex > 0) ? quantiles.map(function (quantile) {
              var value = item[column.id + quantile];
              var className = "textFormat";
              if (isNumber(value)) {
                className = ((Math.abs(value) >= 0.01) || (value === 0.0)) ? "fixedFormat" : "scientificFormat";
              }
              var startTag = "<td class='" + className + "' align='center'>";
              return startTag + (typeof (value) === "undefined" || value === null ? "" : value) + "</td>";
            }).join("") : "<td></td>";
          }).join("") + "</tr>" : "";
      }

      return valueRowStr + intervalRowStr;
    }).join("");

    var totalValues = $(gridId).children(".slick-footerrow").children(".slick-footerrow-columns").children(".slick-footerrow-column");

    var totalStr = "<tr class='finalTotals'>" +
      totalValues.toArray().map(function (column, colIndex) {
        var value = $(column).text().replace(",", "");
        var className = "textFormat";
        if (isNumber(value))
          className = ((Math.abs(value) >= 0.01) || (value === 0.0)) ? "fixedFormat" : "scientificFormat";
        var startTag = "<td class='" + className + "' align='center'" + (colIndex > 0 ? " colspan='" + colSpan + "'" : "") + ">";
        return startTag + (typeof (value) === "undefined" || value === null ? "" : value) + "</td>";
      }).join("") +
      "</tr>";

    return headStr + headerStr + contentStr + totalStr + footStr;
  }

  /*
   * Export grid to string in an Excel-friendly format
   */
  function exportAggrResultsGridToExcel(gridId, includeQuantileData) {

    if (typeof (includeQuantileData) === "undefined") includeQuantileData = false;

    var sg = $(gridId).data().grid;

    var columns = sg.getColumns();
    var data = sg.getData();
    if (!$.isArray(sg.getData())) {
      data = data.getItems();
    }

    var headStr = "<html><head>" +
      "<style type='text/css'>" +
      " table {background-color: white; font-family: Tahoma, sans-serif; font-size: 11px; color: #333}" +
      " .header {font-weight: bold; background-color: #E7F0D8}" +
      " .finalTotals, .sequelaTotals, .sectionHeader, .totalInfected {font-weight: bold;} " +
      " .sectionHeader {background-color: #EBEDF2 !important;}" +
      " .healthOutcomeHeader, .sequelaTotals {background-color: whiteSmoke !important;}" +
      " .intervalhealthOutcomeHeader, .intervalsequelaTotals {background-color: whiteSmoke !important}" +
      " .intervalhealthOutcomeHeader, .intervalsequelaTotals, .intervaltotalInfected, .intervaldata, .intervalundefined {color: #C65911}" +
      " .fixedFormat {mso-number-format: '#,##0.00'}" +
      " .scientificFormat {mso-number-format: '0.00E+00'}" +
      " .textFormat {mso-number-format: '@'}" +
      "</style></head><body><table border='1' bordercolor='#B5B5B5'>";

    var footStr = "</table></body></html>";

    var colSpan = includeQuantileData ? 3 : 1;

    var headerStr = "<tr class='header'>" + columns.map(function (column, colIndex) { return "<td" + (colIndex > 0 ? " colspan='" + colSpan + "' align='center'" : "") + ">" + column.name + "</td>"; }).join("") + "</tr>";

    var contentStr = data.map(function (item) {
      var valueRowStr = "";

      var intervalRowStr = "";
      if (includeQuantileData) {
        var quantiles = ["Quantile0.025", "Quantile0.5", "Quantile0.975"];
        intervalRowStr = (item.type !== "sectionHeader") ? "<tr class='interval" + item.type + "'>" +
          columns.map(function (column, colIndex) {
            return (colIndex > 0) ? quantiles.map(function (quantile) {
              var value = item[column.id + quantile];
              var className = "textFormat";
              if (isNumber(value)) {
                className = ((Math.abs(value) >= 0.01) || (value === 0.0)) ? "fixedFormat" : "scientificFormat";
              }
              var startTag = "<td class='" + className + "' align='center'>";
              return startTag + (typeof (value) === "undefined" || value === null ? "" : value) + "</td>";
            }).join("") : "<td class='textFormat' align='center'>" + item[column.id] + "</td>";
          }).join("") + "</tr>" : "";
      }

      return valueRowStr + intervalRowStr;
    }).join("");

    var totalValues = $(gridId).children(".slick-footerrow").children(".slick-footerrow-columns").children(".slick-footerrow-column");

    var totalStr = "<tr class='finalTotals'>" +
      totalValues.toArray().map(function (column, colIndex) {
        var value = $(column).text().replace(",", "");
        var className = "textFormat";
        if (isNumber(value)) {
          className = ((Math.abs(value) >= 0.01) || (value === 0.0)) ? "fixedFormat" : "scientificFormat";
        }
        var startTag = "<td class='" + className + "' align='center'" + (colIndex > 0 ? " colspan='" + colSpan + "'" : "") + ">";
        return startTag + (typeof (value) === "undefined" || value === null ? "" : value) + "</td>";
      }).join("") + "</tr>";
    return headStr + headerStr + contentStr + totalStr + footStr;
  }

  /*
   * Launch "Run" window
   */
  function launchWindow(id, top, left, showMask) {
    //Get the screen height and width
    var maskHeight = $(document).height();
    var maskWidth = $(window).width();
    if (showMask === undefined)
      showMask = true;

    if (showMask) {
      $("#mask").css({ "width": maskWidth, "height": maskHeight });
    }

    //Set the popup window to center
    $(id).css("top", top);
    $(id).css("left", left);

    if (showMask) {
      $("#mask").fadeIn(100, function () {
        $(id).fadeIn(100);
      });
    } else {
      $(id).fadeIn(100);
    }
  }

  function launchPopup(id) {
    //Get the screen height and width
    var maskHeight = $(document).height();
    var maskWidth = $(window).width();

    $("#mask").css({ "width": maskWidth, "height": maskHeight });

    //Set the popup window to center
    $(id).css({ "top": 10, "left": maskWidth / 2 - $(id).width() / 2 - 20, "margin-bottom": 10 });
    //$(id).css('left', maskWidth/2-$(id).width()/2 - 20);

    $("#mask").fadeIn(100, function () {
      $(id).fadeIn(100);
    });
  }

  /*
   * Events
   */

  $(".popup .close").on("click", function () {
    var popup = $(this).parents(".popup");
    if (!popup.hasClass("trip-exposed")) {
      popup.fadeOut(100, function () {
        $("#mask").fadeOut(100);
      });
    }
    $("#mask").off("click");
  });

  $(".help").tooltip();

  $("body").on("click", "a:not(.tabAnchor)", function (e) {
    e.preventDefault();
    webGUI.webLinkClicked($(this).attr("href"));
  });

  /*
   * Chart context menus
   */
  var resultsTableContextMenu = [
    {
      "Copy table to Excel": function (menuItem, menu) {
        var text = exportGridToExcel("#" + $(this).attr("id"), menu.includeQuantileData);
        webGUI.copyTextToClipboard(text);
      }
    }
  ];
  var aggrResultsTableContextMenu = [
    {
      "Copy table to Excel": function (menuItem, menu) {
        var text = exportAggrResultsGridToExcel("#" + $(this).attr("id"), menu.includeQuantileData);
        webGUI.copyTextToClipboard(text);
      }
    }
  ];
  var chartContextMenu = [
    {
      "Copy to clipboard": function () {
        var elemId = "#" + $(this).attr("id");
        var elem = $(elemId)[0];
        domtoimage
          .toPng(elem, { bgcolor: "#FFF" })
          .then(function (dataUrl) {
            webGUI.copyPngImageToClipboard(dataUrl.substring(22));
          });
      }
    },
    {
      "Save to file": function () {
        var elemId = "#" + $(this).attr("id");
        var elem = $(elemId)[0];
        domtoimage
          .toPng(elem, { bgcolor: "#FFF" })
          .then(function (dataUrl) {
            webGUI.savePngImageToFile(dataUrl.substring(22));
          });
      }
    }
  ];
  $(function () {
    $(".chartContainer").contextMenu(chartContextMenu);
  });

  function setTableWithoutMetaData(grid, arrayData) {
    if (typeof (arrayData) !== "undefined") {
      if (arrayData.length > 0) {
        grid.setData(arrayData);
        grid.invalidate();
      }
    }
  }

  return {
    resultsTableContextMenu: resultsTableContextMenu,
    launchWindow: launchWindow,
    launchPopup: launchPopup,
    chartContextMenu: chartContextMenu,
    aggrResultsTableContextMenu: aggrResultsTableContextMenu,
    setTableWithoutMetaData: setTableWithoutMetaData
  };
};
