/* global
    $, toastr
*/

var messageSystemModule = function () { // eslint-disable-line no-unused-vars

  "use strict";

  var updatableToastClass = "message updatable shadow";

  toastr.options = {
    toastClass: "message shadow",
    containerId: "message-container",
    positionClass: "message-bottom-right",
    iconClass: "message-info",
    iconClasses: {
      error: "error",
      info: "info",
      success: "success",
      warning: "warning"
    },
    titleClass: "message-title",
    messageClass: "message-message",
    timeOut: 5000
  };

  var _showUpdatableNotification = function (titleText, contentText, messageType) {
    messageType = messageType || "info";
    var message = $(".message.updatable");
    if (message.length > 0) {
      _updateNotification(titleText, contentText, messageType);
    } else if (["info", "error", "success", "warning"].indexOf(messageType) != -1) {
      toastr[messageType](contentText, titleText, { toastClass: updatableToastClass, tapToDismiss: false, timeOut: 0, extendedTimeOut: 0 });
    }
  };

  var _updateNotification = function (titleText, contentText, messageType) {
    var message = $(".message.updatable");
    var currentMessageType = message.attr("class").replace(updatableToastClass, "").trim();
    if (currentMessageType !== messageType) {
      message.removeClass(currentMessageType).addClass(messageType);
    }

    message.children(".message-title").html(titleText);
    message.children(".message-message").html(contentText);
  };

  var _hideUpdatableNotification = function () {
    toastr.clear($(".message.updatable"));
  };

  var _showNotification = function (titleText, contentText, messageType) {
    messageType = messageType || "info";
    if (["info", "error", "success", "warning"].indexOf(messageType) != -1) {
      toastr[messageType](contentText, titleText);
    }
  };

  return {
    showNotification: _showNotification,
    showUpdatableNotification: _showUpdatableNotification,
    hideUpdatableNotification: _hideUpdatableNotification
  };
};
