/*global
    setTimeout, $, Trip, console
*/

var tutorialModule = function (bcode) { // eslint-disable-line no-unused-vars

  "use strict";

  var _tour = {};
  var modelTabsApi = {};
  var pageTabsApi = {};

  function _addExtraContentToCurrentTrip(extraContent) {
    var content = _tour.tripData[_tour.tripIndex].content;
    _tour.$tripBlock.children(".trip-content").html(content + "<br /><span class='red'>" + extraContent + "</span>");
  }

  function _removeExtraContentFromCurrentTrip() {
    var content = _tour.tripData[_tour.tripIndex].content;
    _tour.$tripBlock.children(".trip-content").html(content);
  }

  // DYNAMIC TOUR
  var _tourOptions = {
    animation: "fadeIn",
    showCloseBox: true,
    closeBoxLabel: "Close it",
    showHeader: true,
    showNavigation: true,
    header: "Walkthrough: step {{tripIndex}} of {{tripTotal}}",
    delay: -1,
    prevLabel: "Previous",
    nextLabel: "Next",
    finishLabel: "Finish",
    tripTheme: "",
    overlayZindex: 10001,
    onStart: function () {
      pageTabsApi.click(1);
    },
    onEnd: function () {
      $("#mask").insertBefore($("#runDialog"));
    },
    tripBlockHTML: [
      "<div id=\"walkthrough\" class=\"popup trip-block\">",
      "   <h3 class=\"trip-header\"></h3>",
      "   <div class=\"trip-content\">",
      "   </div>",
      "   <div class=\"headerRow\">",
      "       <div class=\"extraHeaderDiv\" style=\"text-align: right\">",
      "           <a href=\"#\" class=\"trip-prev\">Previous</a>",
      "           <a href=\"#\" class=\"trip-next\">Next</a>",
      "           <a href=\"#\" class=\"trip-close\"></a>",
      "       </div>",
      "   </div>",
      "</div>"
    ]
  };

  _tour = new Trip([
    {
      content: "<p>This dynamic tutorial will guide you through the toolkit.</p><p>You will create models, sample them and view the results.</p><p>Click <i>Next</i> to start the walkthrough.</p>",
      position: "screen-center",
      onTripStart: function () {
        pageTabsApi.click(1);
        $("#mask").insertBefore($("#runDialog"));
      },
      onTripEnd: function () {
        pageTabsApi.click(2);
      },
      expose: true
    },
    {
      sel: "#countriesTableContainer",
      content: "<p>This is view \"Create models\".</p><p>Select countries for which you would like to create new models. At least one country must be selected either with a click on the map or in the table.</p>",
      onTripStart: function () {
        pageTabsApi.click(2);
      },
      canGoNext: function () {
        var canGo = bcode.state.selectedCountries.length > 0;
        if (!canGo) {
          _addExtraContentToCurrentTrip("Select at least one country.");
        } else {
          _removeExtraContentFromCurrentTrip();
        }
        return canGo;
      },
      position: "screen-se",
      expose: true
    },
    {
      sel: "#diseaseTableContainer",
      content: "<p>Select diseases for which you would like to create models.<br />At least one disease must be selected.</p>",
      canGoNext: function () {
        var canGo = bcode.state.selectedDiseases.length > 0;
        if (!canGo) {
          _addExtraContentToCurrentTrip("Select at least one disease.");
        } else {
          _removeExtraContentFromCurrentTrip();
        }

        return canGo;
      },
      position: "screen-se",
      expose: true
    },
    {
      sel: "#applyChangesBtn",
      content: "<p>Click here to create the models.</p><p>Models for all combinations of selected countries and diseases will be created.</p>",
      position: "screen-se",
      onTripStart: function () {
        pageTabsApi.click(2);
      },
      onTripEnd: function () {
        pageTabsApi.click(3);
        modelTabsApi.click(0);
      },
      canGoNext: function () {
        var canGo = bcode.state.createdModels.length > 0;
        if (!canGo) {
          _addExtraContentToCurrentTrip("Click \"Create models\".");
        } else {
          _removeExtraContentFromCurrentTrip();
        }
        return canGo;
      },
      expose: true
    },
    {
      sel: "#editModelData_modelSelect",
      content: "<p>This is view \"Edit model data\" presenting inputs of a selected model.</p><p>Select here a model to edit its parameters.</p>",
      onTripStart: function () {
        pageTabsApi.click(3);
        modelTabsApi.click(0);
      },
      position: "screen-se",
      expose: true
    },
    {
      sel: "#caseBeforeCorr",
      content: "<p>This is tab \"Incidence data\".</p><p>Fill in the number of cases before corrections.<br />Cells can be edited directly in the table or copied from Excel.</p><p>There is an option to paste data copied from Excel:</p><ul style=\"text-align: left\"><li>without any conversion (\"Paste as Constant\")</li><li>converted by interpreting copied values as input parameters of a probability distribution. Table will be filled with proper specification of either Uniform distribution (option \"Paste as Uniform distribution\") or Pert distribution (option \"Paste as Pert distribution\").</li></ul>",
      position: "screen-se",
      expose: true
    },
    {
      sel: "#underestimType",
      content: "<p>There are two choices for applying corrections to the average number of cases per year.<br />It is either a single parameter \"Underestimation\" or a product of \"Underreporting\" and \"Underascertainment\".</p><p>Choose the desired parameters set.</p>",
      position: "screen-se",
      expose: true
    },
    {
      sel: "#correctionTables",
      content: "<p>You can edit now the correction multiplication factors.</p>",
      position: "screen-se",
      onTripStart: function () {
        pageTabsApi.click(3);
        modelTabsApi.click(0);
      },
      onTripEnd: function () {
        modelTabsApi.click(1);
      },
      expose: true
    },
    {
      sel: "#ageDistrTable",
      content: "<p>This is tab \"Population data\".</p><p>Review the age distribution table and update if required.</p>",
      position: "screen-se",
      onTripStart: function () {
        modelTabsApi.click(1);
      },
      expose: true
    },
    {
      sel: "#lifeExpectancyTable",
      content: "<p>Review the life expectancy table and update if required.</p>",
      position: "screen-se",
      onTripStart: function () {
        modelTabsApi.click(1);
      },
      expose: true
    },
    {
      sel: "#applyModelParamChagesBtn",
      content: "<p>In order to save the changes made to the parameters click \"Apply changes\".</p>",
      position: "screen-se",
      onTripStart: function () {
        modelTabsApi.click(1);
      },
      onTripEnd: function () {
        modelTabsApi.click(2);
      },
      expose: true
    },
    {
      sel: "#flowChartTab",
      content: "<p>This is tab \"Outcome tree\" giving a visual representation of the progress of the disease through health outcomes.</p><p>Clicking an element of the outcome tree opens a new dialog with this element's specific parameters and results once the model is sampled.</p>",
      position: "screen-se",
      onTripStart: function () {
        modelTabsApi.click(2);
      },
      onTripEnd: function () {
        modelTabsApi.click(3);
      },
      expose: true
    },
    {
      sel: "#diseaseReportTab",
      content: "<p>This is tab \"Disease report\".</p><p>Details of the model can be found here.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#runDialog").fadeOut();
        modelTabsApi.click(3);
      },
      expose: true
    },
    {
      sel: "#runDialog",
      content: "<p>This simulation run dialog can be opened by clicking \"Run models\" button to the left.</p><p>Set run parameters and run the simulation.</p>",
      position: "screen-se",
      onTripStart: function () {
        bcode.initialize.launchWindow("#runDialog", 203, 150, false);
      },
      onTripEnd: function () {
        $("#runDialog").fadeOut();
        pageTabsApi.click(5);
      },
      canGoNext: function () {
        var canGo = bcode.state.sampledModels.length > 0;
        if (!canGo) {
          _addExtraContentToCurrentTrip("Click \"Run simulation\" to sample the created models.<br />Once the simulation is done click \"Next\".");
        } else {
          _removeExtraContentFromCurrentTrip();
        }
        return canGo;
      },
      expose: true
    },
    {
      sel: "#showDetailedResults_modelSelect",
      content: "<p>This is view \"View detailed results\".</p><p>Select model to explore its results.</p>",
      position: "screen-se",
      onTripStart: function () {
        pageTabsApi.click(5);
        $("#viewDetailedResults").children(".headerRow").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#detailedResultsTablesContainer",
      content: "<p>First table presents detailed results in a tabular form.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#detailedResultsTablesContainer").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#detailedResultsChartsContainer",
      content: "<p>Next, results are presented as charts.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#detailedResultsChartsContainer").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#resultsPerAge",
      content: "<p>Finaly, results per age category are shown both as tables and charts.</p><p>You can switch between \"DALY\", \"DALY per case\", \"DALY per 100,000 Stratum Specific Population\" and \"DALY per 100,000 Total Population\" to see the corresponding outputs.</p>",
      position: "screen-se",
      onTripStart: function () {
        pageTabsApi.click(5);
        $("#resultTabs").prevAll("h3").get(0).scrollIntoView();
      },
      onTripEnd: function () {
        pageTabsApi.click(6);
      },
      expose: true
    },
    {
      sel: "#aggrResultsTable",
      content: "<p>This is view \"View aggregated results\".</p><p>It presents results from all sampled disease models.</p><p>First the aggregated results table is presented. The results can be reordered by clicking headers of columns.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#aggrResultsTable").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#aggrResults3ChartContainer",
      content: "<p>The same information can be viewed graphically in the chart showing ranking of diseases according to the calculated burden.</p><p>The table retains order set in the aggregated results view.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#aggrResults3ChartContainer").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#aggrResults1ChartContainer",
      content: "<p>Next two comparison charts are shown: \"Mortality/incidence comparison\" and \"DALY comparison\"</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#aggrResults1ChartContainer").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#aggrResults1DiseaseSelectTable",
      content: "<p>Select models to be included in the chart.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#aggrResults1DiseaseSelectTable").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    },
    {
      sel: "#aggrResultsPerAgeAndSex",
      content: "<p>Finally, results aggregated over all models and split per age and sex category are presented. One should note that the population data for both stratum specific and total population is based on the average of the populations inputted in the models.</p><p>This concludes the walkthrough.</p>",
      position: "screen-se",
      onTripStart: function () {
        $("#aggrResultsPerAgeAndSex").prevAll("h3").get(0).scrollIntoView();
      },
      expose: true
    }
  ], _tourOptions);

  // Workflow
  var _initializeWorkflow = function () {
    var tutorialHelpTexts = {
      t_1: "<h3>A1. Number formats</h3><p>The toolkit requires all number to be given with dot \".\" as decimal separator and comma \",\" as thousands separator. For instance string \"1,000.32\" is a valid number, however \"1.000,32\" is not.</p><img src=\"./images/tutorial/formatNumber.png\"/><p>String \"-1\" has a special meaning in case of the duration table. It is interpreted as the remaining life expectancy.</p><img src=\"./images/tutorial/remainingLifeExpectancy.png\"/>",
      t_2: "<h3>A2. Uncertainty modelling</h3><p>Various parameters can be modelled with uncertainty. The toolkit provides an option to generate samples from the following probability distributions:</p><ul><li>Uniform (two parameters, lower and upper bound),</li><li>Pert (three parameters, lower bound, mode, upper bound),</li><li>Beta (two shape parameters),</li><li>Gamma (two parameters, shape and rate),</li><li>LogNormal (two parameters, log-scale and shape).</li><li>Normal (two parameters, mean and standard deviation).</li></ul><p>They can be specified independently for each gender/age category as string, for instance \"RandUniform(0, 1)\" for Uniform sample on interval [0, 1] or \"RandPert(0, 1, 2)\" for Pert sample on interval [0, 2] with mode equal 1.</p><p>Normal distribution is by definition unbounded, but there is 99.73% probability that the generated sample is in the range [mu - 3*sigma, mu + 3*sigma], where mu is the mean and sigma is the standard deviation. All negative numbers generated from this distirbution are set to 0, thus some skewness may be introduced if parameters are not set carefully.</p><p>Every editable table provides a special view for initializing the stratified view with a spcification of a probability distribution:<br /><img src=\"./images/tutorial/distributionType.png\"/></p>",
      t_3: "<h3>A3. Tables</h3><p>Every input table in the toolkit where values are specified per gender and age is built of three views: Stratified, Simplified and Meta information. The views can be switched by clicking appropriate icon in the upper right corner of the table:<img src=\"./images/tutorial/tableViewSwitch.png\"/></p><ol><li>Stratified (grid) view is the default view with values of formulas given per gender and age category:<img src=\"./images/tutorial/tableStratifiedView.png\"/></li><li>Simplified view allows specifying random values modelled according to a probability distribution selected from drop-down box \"Distribution type\". Once distribution specific parameters are populated you can click \"Copy to Stratified\" in order to apply the selected random distribution for all gender and sex categories:<img src=\"./images/tutorial/tableSimplifiedView.png\"/></li><li>Meta information view allows viewing some description information and weblinks when applicable:<img src=\"./images/tutorial/tableMetaView.png\"/>Any additional parameters relevant to the table are listed here as well. For example, when choosing the distribution of cases within a health state you can specify \"Remaining Cases\" in the \"Proportion of cases covered\":<img src=\"./images/tutorial/remainingCasesOption.png\"/>For \"Transition probability\" additional settings are: \"Age-group redistribution\", \"Lifetime transition probability\", \"Duration\":<img src=\"./images/tutorial/transProbMetaInfo.png\"/><ul><li>Option \"Age-group redistribution\" allows changing distribution of transitioning cases between age categories.</li><li>Option \"Lifetime transition probability\" specifies if the transition is of type \"Lifetime transition probability (LTP)\" or \"Annual transition probability (ATP)\". If transition is \"LTP\" then the number of transitioning cases is the number of cases leaving the parent health state multiplied by the transition probability. If transition is \"ATP\" then the cumulative risk transition formula is used.</li><li>Parameter \"Duration\" specifies duration to be applied in the cumulative risk transition formula if transition is of type \"ATP\" (see option \"Lifetime transition probability\").</li></ul></li></ol>",
      t_4: "<h3>A4. Editing parameters</h3><p>Changes to the input parameters can be done at various levels, but all within view \"Edit model data\".</p><p class=\"bold\">Always remember to click on the \"Apply changes\" button after editing any parameter!!</p><ul><li>Any changes to parameters listed in tabs \"Model parameters\" and \"Population data\" are applied by clicking button \"Apply changes\" at the top of the view next to the drop-down box with name of the currently selected model:<img src=\"./images/tutorial/modelApplyParamChanges.png\"/><br /></li><li>Parameters specific to health state (\"Disability weights\", \"Duration\", \"Proportion of cases covered\") can be edited in a dialog box opened by clicking the health state in the outcome tree. Button \"Apply changes\" is located at the top of the dialog box next to the model name and health state name.<br /><br /></li><li>Similarly parameters specific to transition (\"Transition probability\", \"Redistribution\") can be edited in a dialog box opened by clicking the transition in the outcome tree. Button \"Apply changes\" is located at the top of the dialog box next to the model name and transition name.</li></ul>",
      t_5: "<h3>2. Create models</h3><p>Go to view \"Create models\". Select a country, a set of countries or a custom population where you would like to set your analysis. Also select the disease/s for which you would like to estimate the burden.</p>",
      t_6: "<h3>2a. Select countries</h3><p>Click country outlines on the map or select their names from the list of countries. In order to select multiple countries from the list please click checkbox to the left of the country name rather than the name itself. Otherwise only the clicked country will be selected.</p><img src=\"./images/tutorial/selectCountries.png\"/>",
      t_7: "<h3>2b. Select diseases</h3><p>Select diseases from the list of diseases. In order to select multiple diseases from the list please click checkbox to the left of the disease name rather than the name itself. Otherwise only the clicked disease will be selected.</p><img src=\"./images/tutorial/selectDiseases.png\"/>",
      t_8: "<h3>2c. Generate models</h3><p>Once you've chosen the population and the diseases hit \"Create Models\" and the disease models will be generated in the population you selected.</p><img src=\"./images/tutorial/createModels.png\"/>",
      t_9: "<h3>3. Edit model data</h3><p>Go to view \"Edit model data\". This is the most important interface of the toolkit where you can edit various parameters of the model such as number of cases, adjust for underestimation, change populations, life expectancy and parameters of the outcome tree. Browse from one model to another by navigating through the drop-down box at the top.</p>",
      t_10: "<h3>3a. Incidence data</h3><p>Go to tab \"Incidence data\" and specify the most crucial input of the model - \"Average number of cases per year\". This is the age-group and gender specific number of cases. The table can be filled in by:</p><ol><li>manually typing values in each cell (double-click a cell to enter the edit mode);</li><li>manually typing value in a single cell and applying it to the whole table (right-click and select \"Fill table with this formula\");</li><li>pasting data copied from Excel directly (right-click and select \"Paste as Constant\");</li><li>pasting data copied from Excel by interpreting it as parameters of Uniform distribution (right-click and select \"Paste as Uniform distribution\"). Copied data should be composed of 4 columns of values, two per sex category (lower bound and upper bound);</li><li>pasting data copied from Excel by interpreting it as parameters of Pert distribution (right-click and select \"Paste as Pert distribution\"). Copied data should be composed of 6 columns of values, three per sex category (lower bound, mode, and upper bound);</li><li>using simplified view to specify a probability distribution (switch to the simplified view by clicking second icon \"up/down arrow\" in the upper right corner of the table). Select distribution type, its parameters and click \"Copy to Stratified\".</li></ol><img src=\"./images/tutorial/incidenceData.png\"/><p>The case data can be adjusted for underestimation. This is done through the multiplication factors, which can be specified as a single parameter (underestimation) or a product of two parameters: under-ascertainment AND under-reporting. First select which multiplication factors to set and then specify the multiplication factors themselves. Value of \"1\" means no adjustment.</p><p class=\"bold\">Changes to the parameters specified in this tab must be confirmed by clicking button \"Apply changes\" at the top of the view.</p>",
      t_11: "<h3>3b. Population data</h3><p>In tab \"Population data\" two population related parameters can be adjusted: population distribution (table \"Age distribution data\") and life expectancy (table \"Life expectancy data\"). It is required to adjust the population in case of selecting \"\"Custom Population\") from the list of countries in view \"Create models\".</p><p class=\"bold\">Changes to the parameters specified in this tab must be confirmed by clicking button \"Apply changes\" at the top of the view.</p><img src=\"./images/tutorial/populationData.png\"/>",
      t_12: "<h3>3c. Outcome tree</h3><p>This tab allows investigating outcome tree(s) of the disease model. Outcome tree visualizes the natural history of the disease. It is a graph composed of health outcomes (boxes) and transitions (arrows). A health outcome can be composed of multiple health states represented as rectangles within the health outcome. They are visible only if a health outcome has more than 1 health state:<br /><img src=\"./images/tutorial/outcomeTree.png\"/></p><p>Every element of the outcome tree can be clicked.</p><p>By clicking one of the boxes (Health outcome/Health state) a pop-up dialog appears with element's specific parameters for inputs and outputs (available after model run). Health state's parameters include:</p><ul><li>Disability weights</li><li>Duration</li><li>Proportion of cases covered within the health outcome. This table includes and extra parameter in the Meta information view of the table, \"Remaining Cases\". When this option is checked the proportion table does not need to be filled in. The toolkit will ensure that all cases not covered by the other health states are allocated to this health state.<br /><img src=\"./images/tutorial/remainingCasesOption.png\"/></li></ul><p>By clicking one of the arrows (Transition probabilities) a pop-up dialog appears with element's specific parameters for inputs and outputs (available after model run). Tables you can view and edit are:</p><ul><li>Transition probability</li><li>Redistribution (optional, redistribution table is visible only if option \"Age-group redistribution\" is checked in the Meta information view of the transition probability table).<br /><img src=\"./images/tutorial/redistributionOption.png\"/>Redistribution means that transitioning cases are aggregated per gender and then split between age categories according to the specified proportions given in table \"\"Redistribution\". The total count of transitioning cases does not change, but the allocation to age categories does.</li></ul><p class=\"bold\">Any changes to the parameters of health states/transitions must be confirmed by clicking button \"Apply changes\" in the opened dialog where the parameter were edited.</p>",
      t_13: "<h3>3d. Disease report</h3><p>This tab shows the accompanying disease report if available.</p><img src=\"./images/tutorial/diseaseReport.png\"/>",
      t_14: "<h3>4. Save/load models</h3><p>You can choose to save inputted parameters or load previously saved ones into the toolkit.</p>",
      t_15: "<h3>4a. Save models</h3><p>Created models can be saved for later reference. Go to the top menu and select \"File -> Save disease model\". Specifications of all created models are saved to a single Excel file with each model saved into a separate sheet. You should not edit this file manually in Excel.</p><img src=\"./images/tutorial/saveModels.png\"/>",
      t_16: "<h3>4b. Load models</h3><p>Disease models saved with option \"Save disease models\" can be loaded back to the toolkit by choosing menu \"File -> Load disease models\".</p><img src=\"./images/tutorial/loadModels.png\"/>",
      t_17: "<h3>5. Run models</h3><p>Once the models are created and parameters finalized you can run the burden calculations. Click button \"Run models\" to open the corresponding run dialog. By default 1000 iterations of the calculations are performed, but it can be adjusted in the dialog. This setting is important if the disease model includes parameters with uncertainties around the values.</p><p>Another parameter is the time discounting rate. By default the time discount rate of 3% per year is inputted. Set it to 0 in order to disable time discounting. Default time discount rate can be adjusted in the program options (menu \"Tools -> Options\").</p><p>Click \"Run simulation\" to perform the calculations. Run time per disease model is about 1 second for 1000 iterations. When the calculations are finished as indicated in the progress bar you can close the dialog by clicking \"Close it\". Output results are available now. See tutorial step \"View results\" for information how to explore them.</p><img src=\"./images/tutorial/runDialog.png\"/>",
      t_18: "<h3>6. View results</h3><p>The output results can be viewed per various aggregation levels:</p><ol><li>Most detailed results (per health state/transition) are available in the outcome tree itself.</li><li>Results slightly aggregated, but still per disease model are available in view \"Detailed results\".</li><li>Finally highly aggregated results and comparison between disease models can be explored in view \"Aggregated results\".</li></ol>",
      t_19: "<h3>6a. Outcome tree</h3><p>Click health state or transition in the outcome tree. In the opened dialog go to tab \"Results\".</p><img src=\"./images/tutorial/outcomeTreeResults.png\"/>",
      t_20: "<h3>6b. Detailed results</h3><p>This view is composed of three sections, \"Results table\", \"Results charts\" and \"Results details\".</p><p><span class=\"bold\">a) Section \"Results table\"</span> shows average results by default, but uncertainty bounds can be viewed when hovered over a specific value. Export this table to Excel by right-clicking in the table and selecting \"Copy table to Excel\".</p><img src=\"./images/tutorial/resultsTableCongenital.png\"/><p>Results marked by \"Stratum Specific Population\" mean that only the population specific to the computed item is applied. For instance, \"DALY per 100,000 Stratum Specific Population\" for health outcomes of \"Congenital\" outcome tree of \"Chlamydia\" disease model is computed with only the congenital population as the denomonator. For completeness \"DALY per 100,000 Total Population\" is included in the results table, where the total population is applied as the denominator.</p><p>Diease models with multiple outcome trees include a results table with figures aggregated over all outcome trees: <img src=\"./images/tutorial/resultsTableAll.png\"/></p><p><span class=\"bold\">b) Results chart</span><img src=\"./images/tutorial/resultsChartWomen.png\"/></p><p><span class=\"bold\">c) Section \"Results details\"</span> shows DALY, DALY per case, DALY per 100 000 Stratum Specific Population and DALY per 100,000 Total Population split per gender and age category both in a tabular form and in the chart.</p><img src=\"./images/tutorial/resultsDetails.png\"/>",
      t_21: "<h3>6c. Aggregated results</h3><p>Comparison between disease models is presented in this view. It is composed of five sections: \"Aggregated results table\", \"Ranking of diseases according to burden\", \"Mortality/incidence comparison\" chart, \"DALY comparison\" chart, and \"Detailed results of total DALYs\".</p><p><span class=\"bold\">a) Aggregated results table</span> lists results from all models for easy comparison. Sorting order can be set by clicking headers of columns. The numbers reported here are computed per total population.<img src=\"./images/tutorial/aggrResultsTable.png\"/></p><p><span class=\"bold\">b) Ranking of diseases according to burden</span> gives a visual representation to the aggregated results table. The ordering applied to that table is preserved in this chart. <img src=\"./images/tutorial/aggrRankingChart.png\"/></p><p><span class=\"bold\">c, d) Mortality/incidence and DALY comparison charts</span> provide bubble plots of the results. Models to be included in the chart can be selected in the table next to it.<img src=\"./images/tutorial/aggrMortalityChart.png\"/></p><p><span class=\"bold\">e) Detailed results of total DALYs</span> show results aggregated over all sampled models split per age and sex category. The population data for both stratum specific and total population is based on the average of the populations inputted in the models.<img src=\"./images/tutorial/aggrDALYs.png\"/></p>",
      t_22: "<h3>6d. Exporting results</h3><p>All charts can be exported to a file (\"png\" file, right-click the chart and select \"Save to file\") or to clipboard (select \"Copy to clipboard\").</p><img src=\"./images/tutorial/exportResultsChart.png\"/><p>Similarly, all tables can be exported to Excel through clipboard (right-click in the table area, select \"Copy table to Excel\", paste into an opened blank Excel spreadsheet).</p><img src=\"./images/tutorial/exportResultsTable.png\"/>"
    };

    //var selectedButton = null;
    var tutorialTextContainer = $("#tutorialPopup .contentRow");

    function loadContent(idx) {
      tutorialTextContainer.html(tutorialHelpTexts["t_" + idx]);

      $("#previousTutorialPage").off("click");
      $("#nextTutorialPage").off("click");
      $("#previousTutorialPage").show();
      $("#nextTutorialPage").show();
      $("#previousTutorialPagePlaceholder").hide();
      $("#nextTutorialPagePlaceholder").hide();

      if (idx === 1) {
        $("#previousTutorialPage").hide();
        $("#previousTutorialPagePlaceholder").show();
      } else if (idx === 22) {
        $("#nextTutorialPage").hide();
        $("#nextTutorialPagePlaceholder").show();
      }

      $("#previousTutorialPage").click(function () {
        loadContent(idx - 1);
      });
      $("#nextTutorialPage").click(function () {
        loadContent(idx + 1);
      });
    }

    $(".tutorialSvg .button:not(#t_start)").on("click", function () {
      var id = $(this).attr("id");
      var idx = parseInt(id.split("_")[1]);
      loadContent(idx);
      bcode.initialize.launchPopup("#tutorialPopup");

      $("#mask").on("click", function () {
        $(".popup").fadeOut(200, function () {
          $("#mask").fadeOut("fast");
          $("#mask").off("click");
        });
      });
    });
  };

  var _initialize = function (pageTabsApiExt, modelTabsApiExt) {
    pageTabsApi = pageTabsApiExt;
    modelTabsApi = modelTabsApiExt;
    $("#t_start").on("click", function () {
      _tour.start();
    });
    _initializeWorkflow();
  };

  return {
    initialize: _initialize,
    tour: _tour
  };

};
