/* global
    $, Slick, europeSvgMap
*/

/*
 * CREATE MODELS PAGE
 */

var createModelsModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars

  "use strict";

  /*
   * Settings
   */
  var countryMultiSelect = true;

  /*
   * DOM elements caching
   */

  var countryMap = $("#countryMap");
  var countryTable = $("#countryTable");
  var modelTable = $("#modelTable");
  var diseaseTable = $("#diseaseTable");
  var selCountriesHeader = $("#selectCountriesHeader span");
  var selDiseasesHeader = $("#selectDiseasesHeader span");

  /*
   * Local initialization
   */

  countryMap.height(countryMap.width());

  /*
   * Set up state saving
   */

  var modelTableData = [];
  var modelTableColumns = [
    { id: "Country", name: "Country", field: "Country", cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false, width: 65 },
    { id: "Disease", name: "Disease", field: "Disease", cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false },
    { id: "Model", name: "Model name", field: "Model", cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false }
  ];
  var modelGrid = new Slick.Grid("#modelTable", modelTableData, modelTableColumns, bcode.grids.nonEditGridOptions);
  // Tooltips
  modelGrid.onMouseEnter.subscribe(bcode.grids.gridTooltipHandles.handleMouseEnter);
  modelGrid.onMouseLeave.subscribe(bcode.grids.gridTooltipHandles.handleMouseLeave);
  modelGrid.onClick.subscribe(bcode.grids.gridTooltipHandles.handleMouseLeave);

  var modelsPageState = {
    selectedCountryIds: [],
    selectedDiseaseIds: [],
    enableChangeBtn: function () {
      if (this.selectedCountryIds.length > 0 && this.selectedDiseaseIds.length > 0) {
        $("#applyChangesBtn").prop("disabled", false);
      } else {
        $("#applyChangesBtn").prop("disabled", true);
      }
    },
    setCreatedModelsInfo: function (modelInfo) {
      if (modelInfo.length > 0) {
        $("#deleteModelsBtn").prop("disabled", false);
        $("#runModelsBtn").prop("disabled", false);
      } else {
        $("#deleteModelsBtn").prop("disabled", true);
      }

      $.each(modelInfo, function (index, value) {
        modelTableData.push({
          "id": index,
          "Country": value.countryName,
          "Disease": value.diseaseName,
          "Model": value.modelID
        });
      });
      modelGrid.setData(modelTableData);
      modelGrid.invalidate();
      modelTable.fadeIn("slow");
    },
    reinitialize: function () {
      $("#deleteModelsBtn").prop("disabled", true);
      $("#editModelParamChagesBtn").css({ "display": "none" });
      $("#runModelsBtn").prop("disabled", true);
      modelTableData = [];
      modelGrid.setData(modelTableData);
      modelGrid.invalidate();
      modelTable.fadeOut("slow");
    }
  };

  // Europe map

  var mapSelection = false;
  var defaultEUColor = "#68b022";
  var euCountries = {
    "cp": { selected: false, color: defaultEUColor, id: "CustomPopulation" },
    "eueea": { selected: false, color: defaultEUColor, id: "EUEEA" },
    "at": { selected: false, color: defaultEUColor, id: "Austria" },
    "be": { selected: false, color: defaultEUColor, id: "Belgium" },
    "bg": { selected: false, color: defaultEUColor, id: "Bulgaria" },
    "hr": { selected: false, color: defaultEUColor, id: "Cratia" },
    "cy": { selected: false, color: defaultEUColor, id: "Cyprus" },
    "cz": { selected: false, color: defaultEUColor, id: "CzechRepublic" },
    "dk": { selected: false, color: defaultEUColor, id: "Denmark" },
    "ee": { selected: false, color: defaultEUColor, id: "Estonia" },
    "fi": { selected: false, color: defaultEUColor, id: "Finland" },
    "fr": { selected: false, color: defaultEUColor, id: "France" },
    "de": { selected: false, color: defaultEUColor, id: "Germany" },
    "gr": { selected: false, color: defaultEUColor, id: "Greece" },
    "hu": { selected: false, color: defaultEUColor, id: "Hungary" },
    "is": { selected: false, color: defaultEUColor, id: "Island" },
    "ie": { selected: false, color: defaultEUColor, id: "Ireland" },
    "it": { selected: false, color: defaultEUColor, id: "Italy" },
    "lv": { selected: false, color: defaultEUColor, id: "Latvia" },
    "lt": { selected: false, color: defaultEUColor, id: "Lithuania" },
    "lu": { selected: false, color: defaultEUColor, id: "Luxembourg" },
    "mt": { selected: false, color: defaultEUColor, id: "Malta" },
    "nl": { selected: false, color: defaultEUColor, id: "Netherlands" },
    "no": { selected: false, color: defaultEUColor, id: "Norway" },
    "pl": { selected: false, color: defaultEUColor, id: "Poland" },
    "pt": { selected: false, color: defaultEUColor, id: "Portugal" },
    "ro": { selected: false, color: defaultEUColor, id: "Romania" },
    "sk": { selected: false, color: defaultEUColor, id: "Slovakia" },
    "si": { selected: false, color: defaultEUColor, id: "Slovenia" },
    "es": { selected: false, color: defaultEUColor, id: "Spain" },
    "se": { selected: false, color: defaultEUColor, id: "Sweden" },
    "gb": { selected: false, color: defaultEUColor, id: "UnitedKingdom" }
  };

  countryMap.vectorMap({
    map: "Europe",
    backgroundColor: "#fff",
    color: "#ddd",
    colors: euCountries,
    hoverColor: "#b0d082",
    onRegionOver: function (event, code) {
      if (!(code in euCountries)) {
        event.preventDefault();
      }
    },
    onLabelShow: function (event, label, code) {
      if (!(code in euCountries)) {
        event.preventDefault();
      }
    },
    onRegionClick: function (event, code) {
      if (code in euCountries) {
        // Invert selection
        euCountries[code].selected = !euCountries[code].selected;
        // Add it to the array of selected countries ids, if it's not there yet.
        var countryIdx = countryView.getIdxById(euCountries[code].id);
        var idx = modelsPageState.selectedCountryIds.indexOf(countryIdx);
        var map = countryMap.data("mapObject");
        if (euCountries[code].selected) { // Country has been selected
          if (idx === -1) {
            if (countryMultiSelect) {
              modelsPageState.selectedCountryIds.push(countryIdx);
            } else {
              modelsPageState.selectedCountryIds[0] = countryIdx;
            }
            map.setColors(code, "#495b8e");
            if (!countryMultiSelect) {
              for (var key in euCountries) {
                if (key !== code) {
                  euCountries[key].selected = false;
                  map.setColors(key, defaultEUColor);
                }
              }
            }
          }
        } else { // Country has been deselected
          if (idx !== -1) {
            modelsPageState.selectedCountryIds.splice(idx, 1);
            map.setColors(code, defaultEUColor);
          }
        }
        mapSelection = true;
        countryGrid.setSelectedRows(modelsPageState.selectedCountryIds);
        countryGrid.scrollRowIntoView(modelsPageState.selectedCountryIds[0]);
        selCountriesHeader.html("(" + modelsPageState.selectedCountryIds.length + " selected)");
        modelsPageState.enableChangeBtn();
        let countryIds = modelsPageState.selectedCountryIds.map(function (idx) { return countryData[idx].id });
        webGUI.setSelectedCountries(countryIds);
        if (!countryMultiSelect) {
          webGUI.deleteModels();
        }
        bcode.state.selectedCountries = modelsPageState.selectedCountryIds;
        mapSelection = false;
      }
    }
  });

  // Europe countries list table
  var countryData = [];
  {
    var i = 0;
    for (var key in euCountries) {
      var d = (countryData[i] = {});
      d.key = key;
      d.id = euCountries[key].id;
      d.country = europeSvgMap[key].name;
      ++i;
    }
  }

  var countryColumns = [];
  if (countryMultiSelect) {
    var countryCheckboxSelector = new Slick.CheckboxSelectColumn({ cssClass: "slick-cell-checkboxsel" });
    countryColumns.push(countryCheckboxSelector.getColumnDefinition());
  }
  countryColumns.push({ id: "country", name: "Country", field: "country" });
  var countryView = new Slick.Data.DataView();
  var countryGrid = new Slick.Grid("#countryTable", countryView, countryColumns, $.extend({}, bcode.grids.nonEditGridOptions, { multiSelect: countryMultiSelect }));
  countryGrid.setSelectionModel(new Slick.RowSelectionModel());
  countryView.syncGridSelection(countryGrid, true);

  if (countryMultiSelect) {
    countryGrid.registerPlugin(countryCheckboxSelector);
  }
  countryView.getItemMetadata = function () {
    return { "selectable": true, "focusable": true };
  };

  countryView.setItems(countryData);
  countryGrid.onSelectedRowsChanged.subscribe(function () {
    var map = countryMap.data("mapObject");
    if (!mapSelection) {
      // Deselect all countries
      modelsPageState.selectedCountryIds = [];
      for (var key in euCountries) {
        euCountries[key].selected = false;
        map.setColors(key, defaultEUColor);
      }
      // Select only those selected in the table
      var selRows = countryGrid.getSelectedRows();
      for (var i = 0, l = selRows.length; i < l; ++i) {
        var item = countryView.getItem(selRows[i]);
        euCountries[item.key].selected = true;
        var countryIndex = countryView.getIdxById(item.id);
        modelsPageState.selectedCountryIds.push(countryIndex);
        map.setColors(item.key, "#495b8e");
      }
      selCountriesHeader.html("(" + modelsPageState.selectedCountryIds.length + " selected)");
      modelsPageState.enableChangeBtn();
      let countryIds = modelsPageState.selectedCountryIds.map(function (idx) { return countryData[idx].id });
      console.log(countryIds);
      webGUI.setSelectedCountries(countryIds);
      if (!countryMultiSelect) {
        webGUI.deleteModels();
      }
      bcode.state.selectedCountries = modelsPageState.selectedCountryIds;
    }
  });


  /*
   * Diseases list table
   */

  var diseaseCheckboxSelector = new Slick.CheckboxSelectColumn({ cssClass: "slick-cell-checkboxsel" });

  var diseaseData = [];
  diseaseData.getItemMetadata = function () {
    return { "selectable": true, "focusable": true };
  };

  var diseaseColumns = [];
  diseaseColumns.push(diseaseCheckboxSelector.getColumnDefinition());
  diseaseColumns.push({ id: "disease", name: "Disease", field: "disease" });

  var diseaseGrid = new Slick.Grid("#diseaseTable", diseaseData, diseaseColumns, bcode.grids.nonEditGridOptions);
  diseaseGrid.setSelectionModel(new Slick.RowSelectionModel());
  diseaseGrid.registerPlugin(diseaseCheckboxSelector);
  diseaseGrid.onSelectedRowsChanged.subscribe(function () {
    modelsPageState.selectedDiseaseIds = this.getSelectedRows().map(function (elem) { return diseaseData[elem].id; });
    selDiseasesHeader.html("(" + modelsPageState.selectedDiseaseIds.length + " selected)");
    modelsPageState.enableChangeBtn();
    webGUI.setSelectedDiseases(modelsPageState.selectedDiseaseIds);
    bcode.state.selectedDiseases = modelsPageState.selectedDiseaseIds;
  });

  diseaseGrid.onMouseEnter.subscribe(bcode.grids.gridTooltipHandles.handleMouseEnter);
  diseaseGrid.onMouseLeave.subscribe(bcode.grids.gridTooltipHandles.handleMouseLeave);
  diseaseGrid.onClick.subscribe(bcode.grids.gridTooltipHandles.handleMouseLeave);

  var populateDiseasesListTable = function (diseases) {
    diseaseData = diseases;
    diseaseGrid.setData(diseaseData);
    diseaseGrid.invalidate();
  };

  /*
 * Hook up local events
 */

  $("#applyChangesBtn").click(function () {
    webGUI.createModels();
  });

  $("#deleteModelsBtn").click(function () {
    webGUI.deleteModels();
  });

  /*
 * Hook up JavaScript with C++ engine
 */
  (function (test) {
    if (!test) {
      webGUI.getAvailableDiseases(populateDiseasesListTable);
      webGUI.modelsCreated.connect(modelsPageState.setCreatedModelsInfo);
    }
  })(bcode.settings.development);

  return {
    countryMultiSelect: countryMultiSelect,
    modelsPageState: modelsPageState,
    countryTable: countryTable,
    countryGrid: countryGrid,
    diseaseTable: diseaseTable,
    diseaseGrid: diseaseGrid,
    modelTable: modelTable,
    modelGrid: modelGrid
  };
};
