/*jslint
    this
*/

/*global
    setTimeout, jQuery, $
*/

/*
 * RUN MODEL PAGE
 */

var runModelsModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars
  "use strict";

  /*
   * DOM elements caching
   */

  var runProgressElem = $("#runProgressElem");
  //var progressBarContainer = $('#progress_bar');
  var progressBar = $("#progress_bar .ui-progress");
  var runModelsBtn = $("#runModelsBtn");
  var progressMsg = $("#progress_message");
  var numSamplesInput = $("#numSamplesInput");
  var timeDiscountRateInput = $("#timeDiscountRateInput");

  /*
   * Extra functionality
   */

  (function ($) {
    $.fn.setProgress = function (progress) {
      return this.each(function () {
        $(this).width(progress + "%");
        var labelEl = $(".ui-label", this);
        //var valueEl = $('.value', labelEl);
        if (Math.ceil(progress) < 10 && $(".ui-label", this).is(":visible")) {
          labelEl.hide();
        } else {
          if (labelEl.is(":hidden")) {
            labelEl.fadeIn("slow");
          }
        }

        if (Math.ceil(progress) === 100) {
          labelEl.text("100% done");
        } else {
          labelEl.text(Math.ceil(progress) + "%");
        }
      });
    };
  }(jQuery));

  /*
   * Set up state saving
   */

  var runModelsPageState = {};
  runModelsPageState.setRunProgress = function (percProgress, modelIdx, totalModels, message) {
    if ((percProgress >= 0) && (percProgress <= 1)) {
      progressBar.setProgress(percProgress * 100);
    }

    var msg = "";
    if ((message === "") || (percProgress === 0)) {
      msg = "<p>Status: waiting for starting calculations</p>";
    } else if (percProgress === 1) {
      msg = "<p>Status: simulation finished</p>";
    } else {
      msg = "<p>Status: processing " + message + "</p>";
    }
    if (totalModels >= 0) {
      msg += "<p>Models processed: " + modelIdx + " of " + totalModels + "</p>";
    }
    progressMsg.html(msg);
  };
  runModelsPageState.startProgressMonitor = function () {
    runProgressElem.fadeIn("fast");
    runModelsPageState.setRunProgress(0, 0, 0, "");
    $("#progress_timer span").stopwatch().stopwatch("start");
  };
  runModelsPageState.endProgressMonitor = function () {
    $("#progress_timer span").stopwatch().stopwatch("stop");
    $("#progress_timer span").stopwatch().stopwatch("destroy");
    setTimeout(function () {
      runProgressElem.fadeOut("slow", function () {
        $("#progress_timer span").text("00:00");
      });
    }, 1500);
  };

  /*
   * Initialize
   */

  runModelsBtn.attr("disabled", "disabled");
  numSamplesInput.autoNumeric({ aSep: ",", aDec: ".", aSign: "", pSign: "s", vMin: "0", vMax: "100000", mDec: "0", aPad: false, wEmpty: "zero", lZero: "deny" });
  timeDiscountRateInput.autoNumeric({ aSep: ",", aDec: ".", aSign: "", pSign: "s", vMin: "0", vMax: "1", mDec: "4", aPad: false, wEmpty: "zero", lZero: "deny" });

  /*
   * Hook up C++ engine with JavaScript and vice-versa
   */

  (function (test) {
    if (!test) {
      webGUI.getNumSamples(function (value) {
        numSamplesInput.autoNumeric("set", value);
      });

      webGUI.getTimeDiscountRate(function (value) {
        timeDiscountRateInput.autoNumeric("set", value);
      });

      numSamplesInput.keyup(function () {
        webGUI.setNumSamples(parseInt($(this).autoNumeric("get")));
      });

      timeDiscountRateInput.keyup(function () {
        webGUI.setTimeDiscountRate(parseFloat($(this).autoNumeric("get")));
      });

      runModelsBtn.click(function () {
        webGUI.runModels();
      });

      webGUI.runStarted.connect(runModelsPageState.startProgressMonitor);
      webGUI.runProgressUpdated.connect(runModelsPageState.setRunProgress);
      webGUI.runFinished.connect(runModelsPageState.endProgressMonitor);
    }
  }(bcode.settings.development));

  return {

  };
};
