/* global
    $, Slick, tooltipContentFormatter, copyObject
*/

/*
 * SHOW AGGREGATED RESULTS PAGE
 */

var showAggregatedResultsModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars
  "use strict";

  // DALY data table
  var aggrDalyMaleGrid = new Slick.Grid("#aggrDalyMaleTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var aggrDalyFemaleGrid = new Slick.Grid("#aggrDalyFemaleTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per case data table
  var aggrDalyMalePerCaseGrid = new Slick.Grid("#aggrDalyMalePerCaseTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var aggrDalyFemalePerCaseGrid = new Slick.Grid("#aggrDalyFemalePerCaseTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per 100k data table
  var aggrDalyMalePer100kGrid = new Slick.Grid("#aggrDalyMalePer100kTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var aggrDalyFemalePer100kGrid = new Slick.Grid("#aggrDalyFemalePer100kTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per 100k Total Population data table
  var aggrDalyMalePer100kTotalPopulationGrid = new Slick.Grid("#aggrDalyMalePer100kTotalPopulationTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var aggrDalyFemalePer100kTotalPopulationGrid = new Slick.Grid("#aggrDalyFemalePer100kTotalPopulationTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // Menu
  $("#aggrDalyFemaleTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyMaleTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyFemalePerCaseTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyMalePerCaseTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyFemalePer100kTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyMalePer100kTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyFemalePer100kTotalPopulationTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#aggrDalyMalePer100kTotalPopulationTable").contextMenu(bcode.initialize.resultsTableContextMenu);

  var aggrResults1Chart = {};
  var aggrResults2Chart = {};
  var aggrResultsGrid = {};
  var aggrResultsDataView = {};
  var aggrResults1DiseaseGrid = {};
  var aggrResults2DiseaseGrid = {};

  function nonNegativeTicksGenerator(axis) {
    var ticks = [];
    // spew out all possible ticks
    var start = axis.tickSize * Math.floor(axis.min / axis.tickSize);
    var i = 0;
    var v = Number.NaN;
    var prev = Number.NaN;
    do {
      prev = v;
      v = start + i * axis.tickSize;
      if (v >= 0) {
        ticks.push(v);
      }
      i += 1;
    } while (v < axis.max && v !== prev);
    return ticks;
  }

  var bubblePlotChartOptions = {
    xaxis: {
      ticks: nonNegativeTicksGenerator,
      tickFormatter: bcode.charting.defaultTickFormatter
    },
    yaxis: {
      ticks: nonNegativeTicksGenerator,
      tickFormatter: bcode.charting.defaultTickFormatter
    },
    points: {
      show: true
    },
    series: {
      bubble: {
        show: true
      },
      points: {
        show: true
      }
    },
    grid: {
      markings: [{
        xaxis: {
          to: 0
        },
        color: "#eeeeee"
      },
      {
        yaxis: {
          to: 0
        },
        color: "#eeeeee"
      }
      ],
      margin: {
        top: 0,
        left: 5,
        bottom: 0,
        right: 20
      },
      axisMargin: 0
    },
    legend: {
      show: true,
      noColumns: 3
    }
  };
  bubblePlotChartOptions = $.extend(true, {}, bcode.charting.defaultChartOptions, bubblePlotChartOptions);

  function getBubbleChartTooltip(elem, ignore, coords, item, data, labels) {
    // Grab the API reference
    var tooltip = elem.data("tooltip");

    // If we weren't passed the item object, hide the tooltip and remove cached point data
    if (!item) {
      tooltip.cachedPoint = null;
      tooltip.hide();
      return false;
    }

    if (tooltip.cachedPoint !== item.dataIndex) {
      // Update the cached point data
      tooltip.cachedPoint = item.dataIndex;

      // Setup new content
      var contentObject = {
        label: data[item.seriesIndex].label,
        entries: [{
          label: labels[0],
          value: data[item.seriesIndex].data[item.dataIndex][0]
        },
        {
          label: labels[1],
          value: data[item.seriesIndex].data[item.dataIndex][1]
        },
        {
          label: labels[2],
          value: data[item.seriesIndex].data[item.dataIndex][3]
        }
        ]
      };

      aggrResultsGrid.setActiveCell([]);
      aggrResultsGrid.setSelectedRows(aggrResultsDataView.mapIdsToRows([item.series.id]));

      // Update the tooltip content and show
      tooltip.updateContent(tooltipContentFormatter(contentObject));
      tooltip.show({
        left: coords.pageX,
        top: coords.pageY
      });
    }
  }

  function scaleRadius(data) {
    var maxRadii = 0;
    $.each(data, function (index) {
      var seriesData = data[index].data;
      $.each(seriesData, function (i) {
        seriesData[i][2] = Math.sqrt(seriesData[i][3] / Math.PI);
        maxRadii = Math.max(maxRadii, seriesData[i][2]);
      });
    });
    $.each(data, function (index) {
      var seriesData = data[index].data;
      $.each(seriesData, function (i) {
        var val = seriesData[i][2] / maxRadii * 60;
        seriesData[i][2] = val;
      });
    });
  }

  function plotBubblePlotAccordingToChoices(chartContainerId, choiceGrid, inputDataSet, axisLabels) {
    var finalDataSet = [];
    if (choiceGrid) {
      $.each(choiceGrid.getSelectedRows().sort(), function (ignore, v) {
        finalDataSet.push($.extend({}, inputDataSet[v], {
          id: v
        }));
      });
    }
    // Select all if no choices selected
    if (finalDataSet.length === 0) {
      $.each(inputDataSet, function (key) {
        finalDataSet.push($.extend({}, inputDataSet[key], {
          id: key
        }));
      });

    }
    scaleRadius(finalDataSet);

    var plot = $.plot($(chartContainerId),
      finalDataSet,
      $.extend(true, {}, bubblePlotChartOptions, {
        xaxes: [{
          axisLabel: axisLabels[0],
          position: "bottom"
        }],
        yaxes: [{
          axisLabel: axisLabels[1],
          axisLabelPadding: 20
        }],
        legend: {
          container: chartContainerId + "Legend"
        }
      }));

    $(chartContainerId).tooltip({
      automatic: false
    }).on("plothover", function (event, coords, item) {
      getBubbleChartTooltip($(this), event, coords, item, $(chartContainerId).data("plot").getData(), axisLabels);
    });
    $($(chartContainerId).data("tooltip")).on("afterHide", function () {
      aggrResultsGrid.setSelectedRows([]);
      aggrResultsGrid.setActiveCell([]);
    });
    return plot;
  }

  $("#aggrResultTabs li").click(function () {
    var scrollPosition = $("#aggrResultTabs").offset().top;
    var top = scrollPosition - $("#viewAggregatedResults .headerRow").offset().top + 10;
    $("#aggrResultTabs li").removeClass("active");
    $("#aggrResultTabs li a").removeClass("active");
    $(".aggrResultContentTab").hide();
    var anchor = $(this).children("a");
    var activeTab = anchor.attr("href");
    $(this).addClass("active");
    anchor.addClass("active");
    $(activeTab).fadeIn("slow");
    $("#panes").scrollTop(top - scrollPosition);
    return false;
  });

  /*
   * Set up state saving
   */

  var gridFormatter = bcode.grids.gridFormatterDefs.numeric;
  var aggregatedResultsGridOptions = $.extend(true, {}, bcode.grids.gridDefaultOptions, {
    autoHeight: true,
    editable: false
  });
  var aggrResultsGridSortcol = "dalyPer100k";

  function gridDataComparer(a, b) {
    var x = a[aggrResultsGridSortcol];
    var y = b[aggrResultsGridSortcol];
    return (x === y ? 0 : (x > y ? 1 : -1));
  }

  var aggrResultsGridColumns = [{
    id: "model",
    name: "Model",
    field: "model",
    cssClass: "alignLeft",
    headerCssClass: "alignLeft",
    selectable: true,
    focusable: false,
    sortable: true,
    width: 200
  },
  {
    id: "yld",
    name: "YLD",
    field: "yld",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "yll",
    name: "YLL",
    field: "yll",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "daly",
    name: "DALY",
    field: "daly",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "dalyPerCase",
    name: "DALY per Case",
    field: "dalyPerCase",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "yldPer100k",
    name: "YLD per 100,000",
    field: "yldPer100k",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "yllPer100k",
    name: "YLL per 100,000",
    field: "yllPer100k",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "dalyPer100k",
    name: "DALY per 100,000",
    field: "dalyPer100k",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "cases",
    name: "Cases per year",
    field: "cases",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "incidencePer100k",
    name: "Incidence per 100,000",
    field: "incidencePer100k",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "deaths",
    name: "Deaths per year",
    field: "deaths",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  },
  {
    id: "mortalityPer100k",
    name: "Mortality per 100,000",
    field: "mortalityPer100k",
    cssClass: "alignRight",
    headerCssClass: "alignRight",
    selectable: true,
    focusable: false,
    formatter: gridFormatter,
    sortable: true
  }
  ];

  // Manage the aggregated results page state
  var aggregatedResultsPageState = {
    results: {},
    aggrResults1Dataset: {},
    aggrResults2Dataset: {},
    aggrResults1DiseaseSelectTableData: [],
    aggrResults2DiseaseSelectTableData: [],
    aggrResultsGridDataset: [],
    displayResults: function (resultsC) {
      console.log("aggregatedResultsPageState.displayResults");

      var _this = this;

      function diseaseColorFormatter(row, cell, value) {
        return "<div style='border: 1px solid #ccc; margin-top: 2px; margin-right: 4px; padding: 1px; width: 14px; float: left'><div style='width: 4px; height: 0; border: 5px solid " + _this.aggrResults1DiseaseSelectTableData[row].color + "; overflow: hidden'></div></div>" + value;
      }

      function aggrResultsDiseaseColorFormatter(row, ignore, value) {
        var id = aggrResultsDataView.mapRowsToIds([row])[0];
        var lastRow = aggrResultsDataView.getItems().length;
        if (id < (lastRow - 1)) {
          return "<div style='border: 1px solid #ccc; margin-top: 2px; margin-right: 4px; padding: 1px; width: 14px; float: left'><div style='width: 4px; height: 0; border: 5px solid " + _this.aggrResults1DiseaseSelectTableData[id].color + "; overflow: hidden'></div></div>" + value;
        } else {
          return value;
        }
      }

      this.results = copyObject(resultsC);

      var modelResults = [];
      if (this.results.hasOwnProperty("modelResults")) {
        modelResults = this.results.modelResults;
      }

      if (modelResults.length > 0) {
        this.aggrResults1Dataset = {};
        this.aggrResults2Dataset = {};
        this.aggrResultsGridDataset = [];

        var results1ChartDataLabels = ["Incidence per 100,000", "Mortality per 100,000", "DALY per 100,000"];
        var results2ChartDataLabels = ["Incidence per 100,000", "DALY per Case", "DALY per 100,000"];

        var getOutputRow = bcode.grids.getOutputRow;
        var i = 0;
        var model, countryName, modelName;

        for (i = 0; i < modelResults.length; i += 1) {
          model = modelResults[i];
          countryName = bcode.createModels.countryMultiSelect ? model.countryName + " - " : "";
          modelName = countryName + model.diseaseName + " - " + model.id;

          this.aggrResults1Dataset[i] = {
            label: modelName,
            data: [
              [model.results.incidencePer100k.Mean,
              model.results.mortalityPer100k.Mean,
              model.results.dalyPer100k.Mean,
              model.results.dalyPer100k.Mean]
            ]
          };
          this.aggrResults2Dataset[i] = {
            label: modelName,
            data: [
              [model.results.incidencePer100k.Mean,
              model.results.dalyPerCase.Mean,
              model.results.dalyPer100k.Mean,
              model.results.dalyPer100k.Mean]
            ]
          };

          this.aggrResultsGridDataset.push(
            getOutputRow(
              {
                "id": i,
                "model": modelName
              },
              ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "cases", "incidencePer100k",
                "deaths", "mortalityPer100k"],
              model.results
            )
          );
        }

        // Aggregated view 1 ---------------------
        i = 0;
        $.each(this.aggrResults1Dataset, function (ignore, val) {
          val.color = i;
          i += 1;
        });

        aggrResults1Chart = plotBubblePlotAccordingToChoices("#aggrResults1Chart", null, this.aggrResults1Dataset, results1ChartDataLabels);

        this.aggrResults1DiseaseSelectTableData = [];
        $.each(this.aggrResults1Dataset, function (key) {
          _this.aggrResults1DiseaseSelectTableData.push({
            "id": key,
            "modelLabel": _this.aggrResults1Dataset[key].label,
            "color": aggrResults1Chart.getData()[key].color
          });
        });

        var aggrResults1DiseaseCheckboxSelector = new Slick.CheckboxSelectColumn({
          cssClass: "slick-cell-checkboxsel"
        });
        var aggrResults1DiseaseColumns = [
          aggrResults1DiseaseCheckboxSelector.getColumnDefinition(),
          {
            id: "modelLabel",
            name: "Model",
            field: "modelLabel",
            formatter: diseaseColorFormatter
          }
        ];
        aggrResults1DiseaseGrid = new Slick.Grid("#aggrResults1DiseaseSelectTable", this.aggrResults1DiseaseSelectTableData, aggrResults1DiseaseColumns, bcode.grids.nonEditGridOptions);
        aggrResults1DiseaseGrid.setSelectionModel(new Slick.RowSelectionModel());
        aggrResults1DiseaseGrid.registerPlugin(aggrResults1DiseaseCheckboxSelector);
        aggrResults1DiseaseGrid.onSelectedRowsChanged.subscribe(function () {
          plotBubblePlotAccordingToChoices("#aggrResults1Chart", aggrResults1DiseaseGrid, _this.aggrResults1Dataset, results1ChartDataLabels);
          $("#aggrResults1DiseaseSelectTable").height($("#aggrResults1ChartContainer").height());
          aggrResults1DiseaseGrid.resizeCanvas();
        });

        // Aggregated view 2 ---------------------
        i = 0;
        $.each(this.aggrResults2Dataset, function (ignore, val) {
          val.color = i;
          i += 1;
        });

        aggrResults2Chart = plotBubblePlotAccordingToChoices("#aggrResults2Chart", null, this.aggrResults2Dataset, results2ChartDataLabels);

        this.aggrResults2DiseaseSelectTableData = [];
        $.each(this.aggrResults2Dataset, function (key) {
          _this.aggrResults2DiseaseSelectTableData.push({
            "id": key,
            "modelLabel": _this.aggrResults2Dataset[key].label,
            "color": aggrResults2Chart.getData()[key].color
          });
        });
        var aggrResults2DiseaseCheckboxSelector = new Slick.CheckboxSelectColumn({
          cssClass: "slick-cell-checkboxsel"
        });
        var aggrResults2DiseaseColumns = [
          aggrResults2DiseaseCheckboxSelector.getColumnDefinition(),
          {
            id: "modelLabel",
            name: "Model",
            field: "modelLabel",
            formatter: diseaseColorFormatter
          }
        ];
        aggrResults2DiseaseGrid = new Slick.Grid("#aggrResults2DiseaseSelectTable", this.aggrResults2DiseaseSelectTableData, aggrResults2DiseaseColumns, bcode.grids.nonEditGridOptions);
        aggrResults2DiseaseGrid.setSelectionModel(new Slick.RowSelectionModel());
        aggrResults2DiseaseGrid.registerPlugin(aggrResults2DiseaseCheckboxSelector);
        aggrResults2DiseaseGrid.onSelectedRowsChanged.subscribe(function () {
          plotBubblePlotAccordingToChoices("#aggrResults2Chart", aggrResults2DiseaseGrid, _this.aggrResults2Dataset, results2ChartDataLabels);
          $("#aggrResults2DiseaseSelectTable").height($("#aggrResults2ChartContainer").height());
          aggrResults2DiseaseGrid.resizeCanvas();
        });

        // Aggregated results ----------------------
        if (this.results.hasOwnProperty("totalResults")) {
          this.aggrResultsGridDataset.push(
            getOutputRow(
              {
                "id": modelResults.length,
                "model": "Aggregate",
                "type": "finalTotals"
              },
              ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "cases", "incidencePer100k",
                "deaths", "mortalityPer100k"],
              this.results.totalResults
            )
          );
        }

        aggrResultsDataView = new Slick.Data.DataView();
        aggrResultsDataView.beginUpdate();
        aggrResultsDataView.setItems(this.aggrResultsGridDataset);
        aggrResultsDataView.endUpdate();

        aggrResultsDataView.getItemMetadata = function (rowIdx) {
          var item = this.getItemByIdx(rowIdx);
          var metaData = {};
          $.extend(metaData, { "cssClasses": item.type });
          return metaData;
        };

        aggrResultsGridColumns[0].formatter = aggrResultsDiseaseColorFormatter;
        aggrResultsGrid = new Slick.Grid("#aggrResultsTable", aggrResultsDataView, aggrResultsGridColumns, aggregatedResultsGridOptions);
        aggrResultsGrid.setSelectionModel(new Slick.RowSelectionModel());

        aggrResultsGrid.onSort.subscribe(function (ignore, args) {
          aggrResultsGridSortcol = args.sortCol.field;
          aggrResultsDataView.sort(gridDataComparer, args.sortAsc);
          bcode.charting.plotAggrResults3Chart(modelResults, aggrResultsGridSortcol, "Mean");
        });
        aggrResultsDataView.onRowsChanged.subscribe(function (ignore, args) {
          aggrResultsGrid.invalidateRows(args.rows);
          aggrResultsGrid.render();
        });
        aggrResultsDataView.sort(gridDataComparer, false);
        aggrResultsGrid.setSortColumn(aggrResultsGridSortcol, false);

        // Tooltips
        aggrResultsGrid.onMouseEnter.subscribe(bcode.grids.resultsTooltipHandles.handleMouseEnter);
        aggrResultsGrid.onMouseLeave.subscribe(bcode.grids.resultsTooltipHandles.handleMouseLeave);

        $("#aggrResultsTable").contextMenu(bcode.initialize.aggrResultsTableContextMenu, {
          includeQuantileData: true
        });

        bcode.charting.plotAggrResults3Chart(modelResults, aggrResultsGridSortcol, "Mean");
      }

      if (this.results.hasOwnProperty("allResults")) {
        var allResults = copyObject(this.results.allResults);

        // Adjust congenital
        if (allResults.daly.arrayFemale.length === 20) {
          var congenitalCategoryName = "Congenital";
          var dataNames = ["daly", "dalyPer100k", "dalyPer100kTotalPopulation", "dalyPerCase", "yld", "yldPer100k", "yldPer100kTotalPopulation", "yldPerCase", "yll", "yllPer100k", "yllPer100kTotalPopulation", "yllPerCase"];
          dataNames.forEach(function (elem) {
            allResults[elem].arrayFemale[0].Age = congenitalCategoryName;
            allResults[elem].arrayMale[0].Age = congenitalCategoryName;
          });
        }

        // Show DALY tables
        bcode.initialize.setTableWithoutMetaData(aggrDalyMaleGrid, allResults.daly.arrayMale, false);
        bcode.initialize.setTableWithoutMetaData(aggrDalyFemaleGrid, allResults.daly.arrayFemale, false);

        // Show DALY per case tables
        bcode.initialize.setTableWithoutMetaData(aggrDalyMalePerCaseGrid, allResults.dalyPerCase.arrayMale, false);
        bcode.initialize.setTableWithoutMetaData(aggrDalyFemalePerCaseGrid, allResults.dalyPerCase.arrayFemale, false);

        // Show DALY per 100k tables
        bcode.initialize.setTableWithoutMetaData(aggrDalyMalePer100kGrid, allResults.dalyPer100k.arrayMale, false);
        bcode.initialize.setTableWithoutMetaData(aggrDalyFemalePer100kGrid, allResults.dalyPer100k.arrayFemale, false);

        // Show DALY per 100k Total Population tables
        bcode.initialize.setTableWithoutMetaData(aggrDalyMalePer100kTotalPopulationGrid, allResults.dalyPer100kTotalPopulation.arrayMale, false);
        bcode.initialize.setTableWithoutMetaData(aggrDalyFemalePer100kTotalPopulationGrid, allResults.dalyPer100kTotalPopulation.arrayFemale, false);

        // Plot charts
        var maxXDALYChart = Math.max(bcode.charting.getDALYChartDataMax(allResults, "", "Male"), bcode.charting.getDALYChartDataMax(allResults, "", "Female"));
        var maxXDALYPerCaseChart = Math.max(bcode.charting.getDALYChartDataMax(allResults, "PerCase", "Male"), bcode.charting.getDALYChartDataMax(allResults, "PerCase", "Female"));
        var maxXDALYPer100kChart = Math.max(bcode.charting.getDALYChartDataMax(allResults, "Per100k", "Male"), bcode.charting.getDALYChartDataMax(allResults, "Per100k", "Female"));
        var maxXDALYPer100kTotalPopulationChart = Math.max(bcode.charting.getDALYChartDataMax(allResults, "Per100kTotalPopulation", "Male"), bcode.charting.getDALYChartDataMax(allResults, "Per100kTotalPopulation", "Female"));

        var transFunc = function (v) {
          return -v;
        };

        bcode.charting.plotDALYChart("#aggrMaleDALYChart", "#aggrMaleDALYChartLegend", allResults, "", "Male", {
          xaxis: {
            max: maxXDALYChart * 1.1
          },
          yaxis: {
            position: "right"
          }
        });
        bcode.charting.plotDALYChart("#aggrFemaleDALYChart", "#aggrFemaleDALYChartLegend", allResults, "", "Female", {
          xaxis: {
            max: maxXDALYChart * 1.1,
            transform: transFunc,
            inverseTransform: transFunc
          }
        });
        bcode.charting.plotDALYChart("#aggrMaleDALYPerCaseChart", "#aggrMaleDALYPerCaseChartLegend", allResults, "PerCase", "Male", {
          xaxis: {
            max: maxXDALYPerCaseChart * 1.1
          },
          yaxis: {
            position: "right"
          }
        });
        bcode.charting.plotDALYChart("#aggrFemaleDALYPerCaseChart", "#aggrFemaleDALYPerCaseChartLegend", allResults, "PerCase", "Female", {
          xaxis: {
            max: maxXDALYPerCaseChart * 1.1,
            transform: transFunc,
            inverseTransform: transFunc
          }
        });
        bcode.charting.plotDALYChart("#aggrMaleDALYPer100kChart", "#aggrMaleDALYPer100kChartLegend", allResults, "Per100k", "Male", {
          xaxis: {
            max: maxXDALYPer100kChart * 1.1
          },
          yaxis: {
            position: "right"
          }
        });
        bcode.charting.plotDALYChart("#aggrFemaleDALYPer100kChart", "#aggrFemaleDALYPer100kChartLegend", allResults, "Per100k", "Female", {
          xaxis: {
            max: maxXDALYPer100kChart * 1.1,
            transform: transFunc,
            inverseTransform: transFunc
          }
        });
        bcode.charting.plotDALYChart("#aggrMaleDALYPer100kTotalPopulationChart", "#aggrMaleDALYPer100kTotalPopulationChartLegend", allResults, "Per100kTotalPopulation", "Male", {
          xaxis: {
            max: maxXDALYPer100kTotalPopulationChart * 1.1
          },
          yaxis: {
            position: "right"
          }
        });
        bcode.charting.plotDALYChart("#aggrFemaleDALYPer100kTotalPopulationChart", "#aggrFemaleDALYPer100kTotalPopulationChartLegend", allResults, "Per100kTotalPopulation", "Female", {
          xaxis: {
            max: maxXDALYPer100kTotalPopulationChart * 1.1,
            transform: transFunc,
            inverseTransform: transFunc
          }
        });
      }
    },
    reinitialize: function () {
      console.log("aggregatedResultsPageState.reinitialize");
      this.aggrResults1Dataset = {};
      this.aggrResults2Dataset = {};
      this.aggrResults1DiseaseSelectTableData = [];
      this.aggrResults2DiseaseSelectTableData = [];
      this.aggrResultsGridDataset = [];
      this.aggrResults1FinalDataset = [];
      this.aggrResults2FinalDataset = [];
      this.displayResults({});
      $("#viewAggregatedResults > .contentRow").css("display", "none");
    }
  };

  /*
   * Hook up C++ engine with JavaScript
   */
  (function (test) {
    if (!test) {
      webGUI.aggregatedResultsAvailable.connect(aggregatedResultsPageState.displayResults);
    }
  }(bcode.settings.development));

  return {
    aggregatedResultsPageState: aggregatedResultsPageState,
    getAggrResults1DiseaseGrid: function () { return aggrResults1DiseaseGrid; },
    getAggrResults2DiseaseGrid: function () { return aggrResults2DiseaseGrid; }
  };
};
