/* global
    $, Slick, copyObject, resultsTableContextMenu
*/

/*
 * SHOW DETAILED RESULTS PAGE
 */

var showDetailedResultsModule = function (webGUI, bcode) { // eslint-disable-line no-unused-vars

  "use strict";

  // Custom tabs - includes scrolling to previous position when tab clicked
  $("#resultTabs li").click(function () {
    var scrollPosition = $("#resultTabs").offset().top;
    var top = scrollPosition - $("#showDetailedResults_modelSelect").offset().top + 10;
    $("#resultTabs li").removeClass("active");
    $("#resultTabs li a").removeClass("active");
    $(".resultContentTab").hide();
    var anchor = $(this).children("a");
    var activeTab = anchor.attr("href");
    $(this).addClass("active");
    anchor.addClass("active");
    $(activeTab).fadeIn("slow");
    $("#panes").scrollTop(top - scrollPosition);
    return false;
  });

  var DetailedResultsFormatter = function (row, cell, value, columnDef, dataContext) {
    var spacer = "";
    if (dataContext.indent > 0) {
      spacer = "<span style='display: inline-block; height: 1px; width: " + (9 * (dataContext.indent + 1)) + "px'></span>";
    }
    return spacer + value;
  };

  var gridFormatter = bcode.grids.gridFormatterDefs.numeric;
  var detailedResultsColumns = [
    { id: "illness", name: "Illness", field: "illness", width: 120, formatter: DetailedResultsFormatter, cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false },
    { id: "cases", name: "Cases per Year", field: "cases", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "incidence", name: "Incidence per 100,000<br />Stratum Specific Population", field: "incidence", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yld", name: "YLD per Year", field: "yld", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yll", name: "YLL per Year", field: "yll", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "daly", name: "DALY per Year", field: "daly", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPerCase", name: "DALY per Case", field: "dalyPerCase", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yldPer100k", name: "YLD per 100,000<br />Stratum Specific Population", field: "yldPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yllPer100k", name: "YLL per 100,000<br />Stratum Specific Population", field: "yllPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPer100k", name: "DALY per 100,000<br />Stratum Specific Population", field: "dalyPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPer100kTotalPopulation", name: "DALY per 100,000<br />Total Population", field: "dalyPer100kTotalPopulation", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
  ];
  var detailedAllResultsColumns = [
    { id: "illness", name: "Illness", field: "illness", width: 120, formatter: DetailedResultsFormatter, cssClass: "alignLeft", headerCssClass: "alignLeft", selectable: false, focusable: false },
    { id: "cases", name: "", field: "cases", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "incidence", name: "", field: "incidence", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yld", name: "YLD per Year", field: "yld", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yll", name: "YLL per Year", field: "yll", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "daly", name: "DALY per Year", field: "daly", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPerCase", name: "DALY per Case", field: "dalyPerCase", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yldPer100k", name: "YLD per 100,000", field: "yldPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "yllPer100k", name: "YLL per 100,000", field: "yllPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPer100k", name: "DALY per 100,000", field: "dalyPer100k", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
    { id: "dalyPer100kTotalPopulation", name: "", field: "dalyPer100kTotalPopulation", cssClass: "alignRight", headerCssClass: "alignRight", selectable: false, focusable: false, formatter: gridFormatter },
  ];
  var detailedResultsOptions = $.extend(true, {}, bcode.grids.gridDefaultOptions, { autoHeight: true, editable: false });

  /*
   * DOM elements caching
   */

  var showDetailedResults_modelSelect = $("#showDetailedResults_modelSelect");

  var resultsTableHeaderTemplate = "<div class='resultsTableHeader'></div>";
  var resultsTableTemplate = "<div style='width: 900px;' class='detailedResultsTable gridContainer'></div>";
  var resultsChartTemplate = "<div class='moduleBox width445px floatLeft chartContainer'>" +
    "  <h2 class='moduleTitle'>Burden of disease</h2>" +
    "  <div class='legend'></div>" +
    "  <div style='width: 445px; height: 250px;' class='chart'></div>" +
    "</div>";

  // DALY data table
  var dalyMaleGrid = new Slick.Grid("#dalyMaleTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var dalyFemaleGrid = new Slick.Grid("#dalyFemaleTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per case data table
  var dalyMalePerCaseGrid = new Slick.Grid("#dalyMalePerCaseTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var dalyFemalePerCaseGrid = new Slick.Grid("#dalyFemalePerCaseTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per 100k data table
  var dalyMalePer100kGrid = new Slick.Grid("#dalyMalePer100kTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var dalyFemalePer100kGrid = new Slick.Grid("#dalyFemalePer100kTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // DALY per 100k Total Population data table
  var dalyMalePer100kTotalPopulationGrid = new Slick.Grid("#dalyMalePer100kTotalPopulationTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);
  var dalyFemalePer100kTotalPopulationGrid = new Slick.Grid("#dalyFemalePer100kTotalPopulationTable", [], bcode.grids.gridColumnDefs.statsNumericColumns, bcode.grids.nonEditGridOptions);

  // Menu
  $("#dalyFemaleTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyMaleTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyFemalePerCaseTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyMalePerCaseTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyFemalePer100kTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyMalePer100kTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyFemalePer100kTotalPopulationTable").contextMenu(bcode.initialize.resultsTableContextMenu);
  $("#dalyMalePer100kTotalPopulationTable").contextMenu(bcode.initialize.resultsTableContextMenu);

  /*
   * Set up state saving
   */

  // Manage the results page state
  var detailedResultsPageState = {
    countryID: "",
    diseaseID: "",
    modelID: "",
    results: [],
    setCurrentModelIdentifier: function (value) {
      var values = value.split(",");
      this.countryID = values[0];
      this.diseaseID = values[1];
      this.modelID = values[2];
    },
    getCurrentModelIdentifier: function () {
      return this.countryID + "," + this.diseaseID + "," + this.modelID;
    },
    readSelect: function() {
      console.log("detailedResultsPageState.readSelect");
      var oldCountryID = this.countryID;
      var oldDiseaseID = this.diseaseID;
      var oldModelID = this.modelID;
      this.setCurrentModelIdentifier(showDetailedResults_modelSelect.val());
      if ((oldCountryID !== this.diseaseID) || (oldDiseaseID !== this.diseaseID) || (oldModelID !== this.modelID)) {
        webGUI.showDetailedResults(this.countryID, this.diseaseID, this.modelID);
      }
    },
    setAvailableModels: function(availableModels) {
      showDetailedResults_modelSelect.children("option").remove();
      if (availableModels.length > 0) {
        $.each(availableModels, function (index, value) {
          showDetailedResults_modelSelect
            .append($("<option>", { value: value.countryID + "," + value.diseaseID + "," + value.modelID })
              .text(value.countryName + " - " + value.diseaseName + " - " + value.modelID));
        });
      } else {
        showDetailedResults_modelSelect.append("<option value=\",,\">No models sampled</option>");
      }
      bcode.state.sampledModels = availableModels;
      detailedResultsPageState.readSelect();
    },
    displayResults: function(resultsC) {
      console.log("detailedResultsPageState.displayResults");
      this.results = copyObject(resultsC);
      var models = [];

      $("#detailedResultsTablesContainer").children().remove();
      $("#detailedResultsChartsContainer").children().remove();
      for (var i = 0; i < this.results.detailedResults.length; ++i) {

        var tableDivID = "detailedResults_" + i;

        models[i] = new Slick.Data.DataView();

        models[i].getItemMetadata = function (rowIdx) {
          var item = this.getItemByIdx(rowIdx);
          var metaData = {};
          $.extend(metaData, { "cssClasses": item.type });
          if (item.type === "sectionHeader") {
            $.extend(metaData, { "columns": { 0: { "colspan": "*" } } });
          } else if ((item.type === "healthOutcomeHeader") || (item.type === "finalTotals") || (item.type === "sequelaTotals")) {
            $.extend(metaData, { "columns": { 0: { "colspan": "3" } } });
          }
          return metaData;
        };

        if (this.results.detailedResults.length > 1) {
          $(resultsTableHeaderTemplate).appendTo("#detailedResultsTablesContainer").text(this.results.detailedResults[i].outcomeTreeName);
        }
        $(resultsTableTemplate).appendTo("#detailedResultsTablesContainer").attr("id", tableDivID);
        var table = new Slick.Grid("#" + tableDivID, models[i], detailedResultsColumns, detailedResultsOptions);

        if (this.results.detailedResults.length > 1) {
          $(resultsTableHeaderTemplate).appendTo("#detailedResultsChartsContainer").text(this.results.detailedResults[i].outcomeTreeName);
        }

        var a = $(resultsChartTemplate).appendTo("#detailedResultsChartsContainer").addClass("clear");
        a.attr("id", "detailedResultsBarChartContainer_" + i);
        a.children(".chart").attr("id", "detailedResultsBarChart_" + i);
        a.children(".legend").attr("id", "detailedResultsBarLegend_" + i);

        var b = $(resultsChartTemplate).appendTo("#detailedResultsChartsContainer");
        b.attr("id", "detailedResultsPieChartContainer_" + i);
        b.children(".chart").attr("id", "detailedResultsPieChart_" + i);
        b.children(".legend").attr("id", "detailedResultsPieLegend_" + i);
        $("#detailedResultsChartsContainer").append("<div class=\"clear\">&nbsp;</div>");

        // Display outcome tree results
        detailedResultsPageState.setOutcomeTreeResultsData(models[i], table, i, this.results.detailedResults[i]);

        // Menu
        $(".chartContainer").contextMenu(bcode.initialize.chartContextMenu);
        $("#" + tableDivID).contextMenu(bcode.initialize.resultsTableContextMenu, { includeQuantileData: true });

        table.invalidate();
      }

      // Show results table for All outcome trees (if more than one present in the disease model)
      if (this.results.detailedResults.length > 1) {
        detailedResultsPageState.setDiseaseModelResultsData(this.results.totalSequelaTypeResults);
      }

      // Set results per sex/age category
      detailedResultsPageState.setCategorizedResultsData(this.results.totalCategorizedResults);
    },
    setDiseaseModelResultsData: function(results) {
      var tableDivID = "detailedResults_All";
      var dataView = new Slick.Data.DataView();

      dataView.getItemMetadata = function (rowIdx) {
        var item = this.getItemByIdx(rowIdx);
        var metaData = {};
        $.extend(metaData, { "cssClasses": item.type });
        if (item.type === "sectionHeader") {
          $.extend(metaData, { "columns": { 0: { "colspan": "*" } } });
        } else if ((item.type === "healthOutcomeHeader") || (item.type === "finalTotals") || (item.type === "sequelaTotals")) {
          $.extend(metaData, { "columns": { 0: { "colspan": "3" } } });
        }
        return metaData;
      };

      // Add table header text
      $(resultsTableHeaderTemplate).appendTo("#detailedResultsTablesContainer").text("Total");

      // Add table container
      $(resultsTableTemplate).appendTo("#detailedResultsTablesContainer").attr("id", tableDivID);

      var getOutputRow = bcode.grids.getOutputRow;
      var resultsTableData = [];

      // Acute totals
      resultsTableData.push(getOutputRow({ "id": resultsTableData.length, "indent": 0, parent: null, "illness": "Acute Total", "type": "data" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k"],
        results.acute));

      // Sequelae Total
      resultsTableData.push(getOutputRow({ "id": resultsTableData.length, "indent": 0, parent: null, "illness": "Sequelae Total", "type": "data" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k"],
        results.sequela));

      // All Health Outcomes Total
      resultsTableData.push(getOutputRow({ "id": resultsTableData.length, "indent": 0, parent: null, "illness": "All Health Outcomes Total", "type": "finalTotals", "cssClass": "resultSectionHeader" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k"],
        results.total));

      dataView.setItems(resultsTableData);

      // Create grid table
      var dataGrid = new Slick.Grid("#" + tableDivID, dataView, detailedAllResultsColumns, detailedResultsOptions);

      // Tooltips
      dataGrid.onMouseEnter.subscribe(bcode.grids.resultsTooltipHandles.handleMouseEnter);
      dataGrid.onMouseLeave.subscribe(bcode.grids.resultsTooltipHandles.handleMouseLeave);

      // Add context menu
      $("#" + tableDivID).contextMenu(bcode.initialize.resultsTableContextMenu, { includeQuantileData: true });

      // Show DALY chart
      $(resultsTableHeaderTemplate).appendTo("#detailedResultsChartsContainer").text("Total");
      var barChartContainer = $(resultsChartTemplate).appendTo("#detailedResultsChartsContainer").addClass("clear");
      barChartContainer.attr("id", "detailedResultsBarChartContainer_All");
      barChartContainer.children(".chart").attr("id", "detailedResultsBarChart_All");
      barChartContainer.children(".legend").attr("id", "detailedResultsBarLegend_All");

      bcode.charting.plotBarChart("#detailedResultsBarChart_All", "#detailedResultsBarLegend_All", results);
    },
    setOutcomeTreeResultsData: function(detailedResultsView, detailedResultsGrid, index, results) {
      var sequelaTypeAggregatedResults = results.sequelaTypeAggregatedResults;
      var healthOutcomeAggregatedResults = results.healthOutcomeAggregatedResults;

      var detailedResultsTableData = [];

      var getOutputRow = bcode.grids.getOutputRow;

      // Acute header
      detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 0, parent: null, "illness": "Acute", "type": "sectionHeader" }));

      // Acute details
      var i, parentID;
      for (i = 0; i < healthOutcomeAggregatedResults.length; ++i) {
        if (healthOutcomeAggregatedResults[i].acuteType === "acute") {
          parentID = detailedResultsTableData.length;
          detailedResultsTableData.push(getOutputRow({ "id": parentID, "indent": 0, parent: null, "illness": healthOutcomeAggregatedResults[i].name, "type": "healthOutcomeHeader" },
            ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i]));

          detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 1, parent: parentID, "illness": "cases", "type": "data" },
            ["cases", "incidence", "yld", "daly", "yldPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i],
            ["populationCasesCount", "incidencePer100k", "yld", "yld", "yldPer100k", "yldPer100k", "yldPer100kTotalPopulation"]));

          detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 1, parent: parentID, "illness": "death", "type": "data" },
            ["cases", "incidence", "yll", "daly", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i],
            ["deathCasesCount", "mortalityPer100k", "yll", "yll", "yllPer100k", "yllPer100k", "yllPer100kTotalPopulation"]));
        }
      }

      // Acute totals
      detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 0, parent: null, "illness": "Acute Total", "type": "sequelaTotals", "cssClass": "resultSectionHeader" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
        sequelaTypeAggregatedResults.acute));

      // Sequelae details
      detailedResultsTableData.push({ "id": detailedResultsTableData.length, "indent": 0, parent: null, "illness": "Sequelae", "type": "sectionHeader" });
      for (i = 0; i < healthOutcomeAggregatedResults.length; ++i) {
        if (healthOutcomeAggregatedResults[i].acuteType === "sequela") {
          parentID = detailedResultsTableData.length;
          detailedResultsTableData.push(getOutputRow({ "id": parentID, "indent": 0, parent: null, "illness": healthOutcomeAggregatedResults[i].name, "type": "healthOutcomeHeader" },
            ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i]));

          detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 1, parent: parentID, "illness": "cases", "type": "data" },
            ["cases", "incidence", "yld", "daly", "yldPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i],
            ["populationCasesCount", "incidencePer100k", "yld", "yld", "yldPer100k", "yldPer100k", "yldPer100kTotalPopulation"]));

          detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 1, parent: parentID, "illness": "death", "type": "data" },
            ["cases", "incidence", "yll", "daly", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
            healthOutcomeAggregatedResults[i],
            ["deathCasesCount", "mortalityPer100k", "yll", "yll", "yllPer100k", "yllPer100k", "yllPer100kTotalPopulation"]));
        }
      }

      // Sequelae Total
      detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 0, parent: null, "illness": "Sequelae Total", "type": "sequelaTotals", "cssClass": "resultSectionHeader" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
        sequelaTypeAggregatedResults.sequela));

      // All Health Outcomes Total
      detailedResultsTableData.push(getOutputRow({ "id": detailedResultsTableData.length, "indent": 0, parent: null, "illness": "All Health Outcomes Total", "type": "finalTotals", "cssClass": "resultSectionHeader" },
        ["yld", "yll", "daly", "dalyPerCase", "yldPer100k", "yllPer100k", "dalyPer100k", "dalyPer100kTotalPopulation"],
        sequelaTypeAggregatedResults.total));

      detailedResultsView.beginUpdate();
      detailedResultsView.setItems(detailedResultsTableData);
      detailedResultsView.endUpdate();
      detailedResultsGrid.invalidate();

      // Tooltips
      detailedResultsGrid.onMouseEnter.subscribe(bcode.grids.resultsTooltipHandles.handleMouseEnter);
      detailedResultsGrid.onMouseLeave.subscribe(bcode.grids.resultsTooltipHandles.handleMouseLeave);

      // Show DALY charts
      bcode.charting.plotBarChart("#detailedResultsBarChart_" + index, "#detailedResultsBarLegend_" + index, sequelaTypeAggregatedResults);
      bcode.charting.plotPieChart("#detailedResultsPieChart_" + index, "#detailedResultsPieLegend_" + index, healthOutcomeAggregatedResults);
    },
    setCategorizedResultsData: function(totalCategorizedResults) {
      // Show DALY tables
      bcode.initialize.setTableWithoutMetaData(dalyMaleGrid, totalCategorizedResults.daly.arrayMale, false);
      bcode.initialize.setTableWithoutMetaData(dalyFemaleGrid, totalCategorizedResults.daly.arrayFemale, false);

      // Show DALY per case tables
      bcode.initialize.setTableWithoutMetaData(dalyMalePerCaseGrid, totalCategorizedResults.dalyPerCase.arrayMale, false);
      bcode.initialize.setTableWithoutMetaData(dalyFemalePerCaseGrid, totalCategorizedResults.dalyPerCase.arrayFemale, false);

      // Show DALY per 100k tables
      bcode.initialize.setTableWithoutMetaData(dalyMalePer100kGrid, totalCategorizedResults.dalyPer100k.arrayMale, false);
      bcode.initialize.setTableWithoutMetaData(dalyFemalePer100kGrid, totalCategorizedResults.dalyPer100k.arrayFemale, false);

      // Show DALY per 100k Total Population tables
      bcode.initialize.setTableWithoutMetaData(dalyMalePer100kTotalPopulationGrid, totalCategorizedResults.dalyPer100kTotalPopulation.arrayMale, false);
      bcode.initialize.setTableWithoutMetaData(dalyFemalePer100kTotalPopulationGrid, totalCategorizedResults.dalyPer100kTotalPopulation.arrayFemale, false);

      // Plot charts
      var maxXDALYChart = Math.max(bcode.charting.getDALYChartDataMax(totalCategorizedResults, "", "Male"), bcode.charting.getDALYChartDataMax(totalCategorizedResults, "", "Female"));
      var maxXDALYPerCaseChart = Math.max(bcode.charting.getDALYChartDataMax(totalCategorizedResults, "PerCase", "Male"), bcode.charting.getDALYChartDataMax(totalCategorizedResults, "PerCase", "Female"));
      var maxXDALYPer100kChart = Math.max(bcode.charting.getDALYChartDataMax(totalCategorizedResults, "Per100k", "Male"), bcode.charting.getDALYChartDataMax(totalCategorizedResults, "Per100k", "Female"));
      var maxXDALYPer100kTotalPopulationChart = Math.max(bcode.charting.getDALYChartDataMax(totalCategorizedResults, "Per100kTotalPopulation", "Male"), bcode.charting.getDALYChartDataMax(totalCategorizedResults, "Per100kTotalPopulation", "Female"));

      bcode.charting.plotDALYChart("#maleDALYChart", "#maleDALYChartLegend", totalCategorizedResults, "", "Male", { xaxis: { max: maxXDALYChart * 1.1 }, yaxis: { position: "right" } });
      bcode.charting.plotDALYChart("#femaleDALYChart", "#femaleDALYChartLegend", totalCategorizedResults, "", "Female", { xaxis: { max: maxXDALYChart * 1.1, transform: function (v) { return -v; }, inverseTransform: function (v) { return -v; } } });
      bcode.charting.plotDALYChart("#maleDALYPerCaseChart", "#maleDALYPerCaseChartLegend", totalCategorizedResults, "PerCase", "Male", { xaxis: { max: maxXDALYPerCaseChart * 1.1 }, yaxis: { position: "right" } });
      bcode.charting.plotDALYChart("#femaleDALYPerCaseChart", "#femaleDALYPerCaseChartLegend", totalCategorizedResults, "PerCase", "Female", { xaxis: { max: maxXDALYPerCaseChart * 1.1, transform: function (v) { return -v; }, inverseTransform: function (v) { return -v; } } });
      bcode.charting.plotDALYChart("#maleDALYPer100kChart", "#maleDALYPer100kChartLegend", totalCategorizedResults, "Per100k", "Male", { xaxis: { max: maxXDALYPer100kChart * 1.1 }, yaxis: { position: "right" } });
      bcode.charting.plotDALYChart("#femaleDALYPer100kChart", "#femaleDALYPer100kChartLegend", totalCategorizedResults, "Per100k", "Female", { xaxis: { max: maxXDALYPer100kChart * 1.1, transform: function (v) { return -v; }, inverseTransform: function (v) { return -v; } } });
      bcode.charting.plotDALYChart("#maleDALYPer100kTotalPopulationChart", "#maleDALYPer100kTotalPopulationChartLegend", totalCategorizedResults, "Per100kTotalPopulation", "Male", { xaxis: { max: maxXDALYPer100kTotalPopulationChart * 1.1 }, yaxis: { position: "right" } });
      bcode.charting.plotDALYChart("#femaleDALYPer100kTotalPopulationChart", "#femaleDALYPer100kTotalPopulationChartLegend", totalCategorizedResults, "Per100kTotalPopulation", "Female", { xaxis: { max: maxXDALYPer100kTotalPopulationChart * 1.1, transform: function (v) { return -v; }, inverseTransform: function (v) { return -v; } } });
    },
    reinitialize: function () {
      console.log("detailedResultsPageState.reinitialize");
      this.setAvailableModels([]);
      $("#viewDetailedResults > .contentRow").css("display", "none");
    }
  };

  /*
   * Hook up local events with C++
   */
  showDetailedResults_modelSelect.change(function () {
    detailedResultsPageState.readSelect();
  });

  /*
   * Hook up C++ engine with JavaScript
   */
  (function (test) {
    if (!test) {
      webGUI.runFinished.connect(detailedResultsPageState.setAvailableModels);
      webGUI.detailedResultsAvailable.connect(detailedResultsPageState.displayResults);
    }
  })(bcode.settings.development);

  return {
    detailedResultsPageState: detailedResultsPageState
  };
};
