transProbViewModule = (function (webGUI, webData, bcode) {

  /*
   * Tabs
   */

  $("#tabs").tabs("#tabsContainer > div", {
    effect: 'fade',
    current: 'active'
  });

  /*
   * Transition probability data table
   */

  // Parameters
  var inputRedistributionGrid = new bcode.grids.InputTable('#parametersTab', { editable: true, append: false, caption: "Redistribution", stratified: { aggregatorType: 'sum' }, simplified: { enabled: false }, meta: { enabled: false } });
  inputRedistributionGrid.container.hide();
  var inputProbabilityGrid = new bcode.grids.InputTable('#parametersTab', { editable: true, append: false, caption: "Transition probability", table: { containerClass: 'moduleBox floatLeft firstBox' } });

  $(inputProbabilityGrid).on('newDataSet', function () {
    var ageDepChkb = this.container.children('.metaContainer').find('input[name=2]');
    ageDepChkb.on('change', function () {
      if ($(this).is(':checked')) {
        inputRedistributionGrid.container.show('slide');
        return;
      }
      inputRedistributionGrid.container.hide('slide');
    });
    ageDepChkb.trigger('change');
  });

  // Results (creation order reverted due to prepending)
  var outputIncidenceGrid = new bcode.grids.InputTable('#resultsTab', { editable: false, append: false, width: 250, caption: "Average transitioning cases", stratified: { aggregatorType: 'sum' }, simplified: { enabled: false }, meta: { enabled: false } });
  var outputRedistributionGrid = new bcode.grids.InputTable('#resultsTab', { editable: false, append: false, width: 250, caption: "Average redistribution", stratified: { aggregatorType: 'sum' }, simplified: { enabled: false }, meta: { enabled: false } });
  var outputProbabilityGrid = new bcode.grids.InputTable('#resultsTab', { editable: false, append: false, width: 250, caption: "Average transition probability", table: { containerClass: 'moduleBox floatLeft firstBox' }, simplified: { enabled: false }, meta: { enabled: false } });

  /*
   * SLOTS
   */

  function initializeData(publishedData) {

    modelInfo = publishedData.ElemInfo;
    var caption = publishedData.Caption;
    var resultsAvailable = publishedData.ResultsAvailable;
    var isBaseTrans = publishedData.IsBaseTrans;
    var isRedistributeType = publishedData.IsRedistributeType;
    var inputProbability = publishedData.InputProbability;
    var inputRedistribution = publishedData.InputRedistribution;
    var outputProbabilityData = publishedData.OutputProbabilityData;
    var outputRedistributionData = publishedData.OutputRedistributionData;
    var outputIncidenceData = publishedData.OutputIncidenceData;

    // Set header
    $('#header').html(modelInfo.countryName + " - " + modelInfo.diseaseName + " - " + modelInfo.modelID + ": " + caption);

    var tabsApi = $("ul.tabs").data("tabs");
    if (!isBaseTrans) {
      tabsApi.click(1);
      $("#tabParameters").hide();
    } else if (!resultsAvailable) {
      tabsApi.click(0);
      $("#tabResults").hide();
    } else {
      tabsApi.click(1);
    }

    // Set transition probability data
    inputProbabilityGrid.setData(inputProbability);
    inputRedistributionGrid.setData(inputRedistribution);
    outputProbabilityGrid.setData(outputProbabilityData);
    outputRedistributionGrid.setData(outputRedistributionData);
    if (!isRedistributeType)
      outputRedistributionGrid.container.hide();
    outputIncidenceGrid.setData(outputIncidenceData);

    $('#layout').removeClass('nonvisible');
  }

  var highlightFormulaErrors = function (highlightData) {
    inputProbabilityGrid.highlightErrorCells(highlightData['probability']);
    inputRedistributionGrid.highlightErrorCells(highlightData['redistribution']);
  };

  /*
   * Layout (standard function)
   */

  function layout() {

  }
  $(window).resize(layout);
  layout();

  /*
   * Hook up C++ with JavaScript
   */

  webData.publishData.connect(initializeData);
  webGUI.showMessageInTransPage.connect(bcode.messageSystem.showNotification);
  webGUI.formulaCheckResultsAvailableForTransPage.connect(highlightFormulaErrors);

  /*
   * Hook up local events with C++
   */

  $('#applyModelParamChagesBtn').click(function () {
    webGUI.applyChangesToTransParams(modelInfo, inputProbabilityGrid.getData(), inputRedistributionGrid.getData());
  });
});
