yldHealthOutcomeViewModule = (function (webGUI, webData, bcode) {

  /*
   * Tabs
   */

  $("#tabs").tabs("#tabsContainer > div", {
    effect: 'fade',
    current: 'active'
  });

  // Parameters
  var modelInfo;
  var severityGrid = new bcode.grids.InputTable('#parametersTab', { editable: true, caption: "Disability weights", table: { containerClass: 'moduleBox floatLeft firstBox' } });
  var durationGrid = new bcode.grids.InputTable('#parametersTab', { editable: true, caption: "Duration" });
  var proportionGrid = new bcode.grids.InputTable('#parametersTab', { editable: true, caption: "Proportion of cases covered" });

  // Results (creation order reverted due to prepending)
  var burdenGrid = new bcode.grids.InputTable('#resultsTab', { editable: false, append: false, caption: "Years Lost due to Disability", stratified: { aggregatorType: 'sum' }, simplified: { enabled: false }, meta: { enabled: false } });
  var incidenceGrid = new bcode.grids.InputTable('#resultsTab', { editable: false, append: false, caption: "Symptomatic Cases", table: { containerClass: 'moduleBox floatLeft firstBox' }, stratified: { aggregatorType: 'sum' }, simplified: { enabled: false }, meta: { enabled: false } });

  /*
   * SLOTS
   */

  var initializeData = function (publishedData) {

    modelInfo = publishedData.ElemInfo;
    var caption = publishedData.Caption;
    var resultsAvailable = publishedData.ResultsAvailable;
    var severityData = publishedData.SeverityData;
    var durationData = publishedData.DurationData;
    var proportionData = publishedData.ProportionData;
    var incidenceData = publishedData.IncidenceData;
    var burdenData = publishedData.BurdenData;
    var remLEData = publishedData.RemLEData;

    // Set header
    $('#header').text(modelInfo.countryName + " - " + modelInfo.diseaseName + " - " + modelInfo.modelID + ": " + caption);

    var tabsApi = $("ul.tabs").data("tabs");
    if (!resultsAvailable) {
      tabsApi.click(0);
      $("#tabResults").hide();
    } else {
      tabsApi.click(1);
    }

    // Set severity data
    severityGrid.setData(severityData);
    // Set duration data
    durationGrid.setData(durationData);
    // Set proportion data
    proportionGrid.setData(proportionData);
    // Set incidence data
    incidenceGrid.setData(incidenceData);
    // Set burden data
    burdenGrid.setData(burdenData);

    $('#layout').removeClass('nonvisible');
  };

  var highlightFormulaErrors = function (highlightData) {
    severityGrid.highlightErrorCells(highlightData['severity']);
    durationGrid.highlightErrorCells(highlightData['duration']);
    proportionGrid.highlightErrorCells(highlightData['proportion']);
  };

  /*
   * Layout (standard function)
   */

  function layout() { };
  $(window).resize(layout);
  layout();

  /*
   * Hook up C++ with JavaScript
   */

  webData.publishData.connect(initializeData);
  webGUI.showMessageInYLDPage.connect(bcode.messageSystem.showNotification);
  webGUI.formulaCheckResultsAvailableForYLDPage.connect(highlightFormulaErrors);

  /*
   * Hook up local events with C++
   */

  $('#applyModelParamChagesBtn').click(function () {
    webGUI.applyChangesToYLDParams(modelInfo, severityGrid.getData(), durationGrid.getData(), proportionGrid.getData());
  });
});
