/* global
    jQuery
*/

"use strict";

/* Compute actual size of an element
 *
 *
* ! Copyright 2011, Ben Lin (http://dreamerslab.com/)
* Licensed under the MIT License (LICENSE.txt).
*
* Version: 1.0.6
*
* Requires: jQuery 1.2.3 ~ 1.7.1
*/
(function ($) {
  $.fn.extend({
    actual: function (method, options) {
      var $hidden, $target, configs, css, tmp, actual, fix, restore;

      // check if the jQuery method exist
      if (!this[method]) {
        throw "$.actual => The jQuery method \"" + method + "\" you called does not exist";
      }

      configs = $.extend({
        absolute: false,
        clone: false,
        includeMargin: undefined
      }, options);

      $target = this;

      if (configs.clone === true) {
        fix = function () {
          // this is useful with css3pie
          $target = $target.filter(":first").clone().css({
            position: "absolute",
            top: -1000
          }).appendTo("body");
        };

        restore = function () {
          // remove DOM element after getting the width
          $target.remove();
        };
      } else {
        fix = function () {
          // get all hidden parents
          $hidden = $target.parents().andSelf().filter(":hidden");

          css = configs.absolute === true ?
            { position: "absolute", visibility: "hidden", display: "block" } :
            { visibility: "hidden", display: "block" };

          tmp = [];

          // save the origin style props
          // set the hidden el css to be got the actual value later
          $hidden.each(function () {
            var _tmp = {}, name;
            for (name in css) {
              // save current style
              _tmp[name] = this.style[name];
              // set current style to proper css style
              this.style[name] = css[name];
            }
            tmp.push(_tmp);
          });
        };

        restore = function () {
          // restore origin style values
          $hidden.each(function (i) {
            var _tmp = tmp[i], name;
            for (name in css) {
              this.style[name] = _tmp[name];
            }
          });
        };
      }

      fix();
      // get the actual value with user specific methed
      // it can be 'width', 'height', 'outerWidth', 'innerWidth'... etc
      // configs.includeMargin only works for 'outerWidth' and 'outerHeight'
      actual = /(outer)/g.test(method) ?
        $target[method](configs.includeMargin) :
        $target[method]();

      restore();
      // IMPORTANT, this plugin only return the value of the first element
      return actual;
    }
  });
})(jQuery);

/*
 * Timer
 */
(function ($) {

  function incrementer(ct, increment) {
    return function () { ct += increment; return ct; };
  }

  function pad2(number) {
    return (number < 10 ? "0" : "") + number;
  }

  function formatMilliseconds(millis) {
    var x, seconds, minutes;
    x = millis / 1000;
    seconds = Math.floor(x % 60);
    x /= 60;
    minutes = Math.floor(x % 60);
    return [pad2(minutes), pad2(seconds)].join(":");
  }

  var methods = {
    init: function (options) {
      var settings = {
        updateInterval: 1000,
        startTime: 0,
        formatter: formatMilliseconds
      };

      if (options) { $.extend(settings, options); }

      return this.each(function () {
        var $this = $(this),
          data = $this.data("stopwatch");

        // If the plugin hasn't been initialized yet
        if (!data) {
          // Setup the stopwatch data
          data = settings;
          data.target = $this;
          data.elapsed = settings.startTime;
          // create counter
          data.incrementer = incrementer(data.startTime, data.updateInterval);
          data.tick_function = function () {
            var millis = data.incrementer();
            data.elapsed = millis;
            data.target.trigger("tick.stopwatch", [millis]);
            data.target.stopwatch("render");
          };
          $this.data("stopwatch", data);
        }

      });
    },
    start: function () {
      return this.each(function () {
        var $this = $(this),
          data = $this.data("stopwatch");
        // Mark as active
        data.active = true;
        data.timerID = setInterval(data.tick_function, data.updateInterval);
        $this.data("stopwatch", data);
      });
    },
    stop: function () {
      return this.each(function () {
        var $this = $(this),
          data = $this.data("stopwatch");
        clearInterval(data.timerID);
        data.active = false;
        $this.data("stopwatch", data);
      });
    },
    destroy: function () {
      return this.each(function () {
        var $this = $(this);
        //+ data = $this.data("stopwatch");
        $this.stopwatch("stop").off(".stopwatch").removeData("stopwatch");
      });
    },
    render: function () {
      var $this = $(this),
        data = $this.data("stopwatch");
      $this.html(data.formatter(data.elapsed));
    },
    toggle: function () {
      return this.each(function () {
        var $this = $(this);
        var data = $this.data("stopwatch");
        if (data.active) {
          $this.stopwatch("stop");
        } else {
          $this.stopwatch("start");
        }
      });
    },
    reset: function () {
      return this.each(function () {
        var $this = $(this);
        var data = $this.data("stopwatch");
        data.incrementer = incrementer(data.startTime, data.updateInterval);
        data.elapsed = data.startTime;
        $this.data("stopwatch", data);
      });
    }
  };

  // Define the function
  $.fn.stopwatch = function (method) {
    if (methods[method]) {
      return methods[method].apply(this, Array.prototype.slice.call(arguments, 1));
    } else if (typeof method === "object" || !method) {
      return methods.init.apply(this, arguments);
    } else {
      $.error("Method " + method + " does not exist on jQuery.stopwatch");
    }
  };

})(jQuery);

// Animate content change

(function ($) {
  $.fn.showHtml = function (html) {
    return this.each(function () {
      var el = $(this);
      var finish = { width: this.style.width, height: this.style.height };
      var cur = { width: el.width() + "px", height: el.height() + "px" };
      el.html(html);
      var next = { width: el.width() + "px", height: el.height() + "px" };
      el.css(cur).animate(next, "fast", function () { el.css(finish); });
    });
  };
})(jQuery);

/*
 * Utilities
 */

function toFloat(string) { // eslint-disable-line no-unused-vars
  if (string == null) {
    return 0;
  } else {
    return typeof (string) === "number" ? string : +string.replace(/,/g, "");
  }
}

function copyObject(object) { // eslint-disable-line no-unused-vars
  return JSON.parse(JSON.stringify(object), function (key, value) {
    return (value == null) ? 0 : value;
  });
}

function capitalise(string) {// eslint-disable-line no-unused-vars
  return string.charAt(0).toUpperCase() + string.slice(1);
}

function isNumber(o) {
  return !isNaN(o - 0);
}

Array.prototype.max = function () {
  return Math.max.apply(null, this);
};

Array.prototype.min = function () {
  return Math.min.apply(null, this);
};

Array.prototype.rank = function (func) {
  var that = this;
  var sorted = that.slice().sort(function (a, b) { return func ? func.call(this, a, b) : a - b; });
  return sorted.map(function (v) { return that.indexOf(v); });
};

Array.prototype.order = function (rank) {
  var that = this;
  var orig = this.slice();
  rank.forEach(function (elem, index) { that[index] = orig[elem]; });
  return that;
};

Array.dim = function (dimension, initial) {
  var a = new Array(dimension);
  while (--dimension >= 0) {
    a[dimension] = initial;
  }
  return a;
};

function dispFloat(number) {
  if (number == null) {
    return "";
  }
  if (isNaN(number)) {
    return "-";
  }
  var text = "";
  if (number == 0) {
    return "0.00";
  }
  else if (number == -1) {
    return "Rem. life exp.";
  } else if (number >= 0.01) {
    text = (Math.round(number * 100) / 100).toFixed(2);
  } else {
    return number.toExponential(2);
  }

  var x = text.split(".");
  var x1 = x[0];
  var x2 = x.length > 1 ? "." + x[1] : "";
  var rgx = /(\d+)(\d{3})/;
  while (rgx.test(x1)) {
    x1 = x1.replace(rgx, "$1" + "," + "$2");
  }
  return x1 + x2;
}

function dispFormula(str) { // eslint-disable-line no-unused-vars
  if (str == null) {
    return "";
  }

  // Check if numeric
  if (isNumber(str)) {
    return dispFloat(parseFloat(str));
  } else {
    return str;
  }
}

function tooltipContentFormatter(contentData) { // eslint-disable-line no-unused-vars
  var content = "<span class='tooltipContentLabel'>" + contentData.label + "</span>";

  content += "<table class='tooltipContentTable'>";
  for (var i = 0; i < contentData.entries.length; ++i) {
    content += "<tr><td class='tooltipContentLabelCell'>" + contentData.entries[i].label +
      "</td><td class='tooltipContentSepCell'>:</td><td class='tooltipContentValueCell'>" +
      dispFloat(contentData.entries[i].value) + "</td>";
  }
  content += "</table>";

  return content;
}
