; (function ($, window, document, undefined) {

  var PLUGIN_NS = "tooltip";

  var defaults = {
    offsetX: 10,
    offsetY: 10,
    content: undefined,
    automatic: true
  };

  function Plugin(element, options) {
    this.element = $(element);
    this.options = $.extend({}, defaults, options);
    this._shown = false;
    this._defaults = defaults;
    this._name = PLUGIN_NS;
    this._init();
  }

  Plugin.prototype = {
    // Private methods
    _updateViewportSize: function () {
      var w = $(window);
      this._viewportHeight = w.height() + w.scrollTop();
      this._viewportWidth = w.width() + w.scrollLeft();
    },
    _init: function () {
      this._updateViewportSize();
      if (typeof (this.options.content) === 'undefined') {
        var tooltipElem = this.element.next('.tooltipContent');
        if (typeof (tooltipElem) != 'undefined') {
          this.options.content = tooltipElem.html();
        }
      }
      if (this.options.automatic) {
        this._activate();
      }
    },
    _activate: function () {
      if (typeof (this.options.content) != 'undefined') {
        var that = this;
        /*
        this.element.mouseover(function(e) {
        that.show({left: e.PageX, top: e.PageY});
        });
        this.element.mouseout(function() {
            that.hide();
        });
*/

        this.element.hover(function (e) {
          that.show({ left: e.pageX, top: e.pageY });
        }, function () {
          that.hide();
        });
        $("body").click(function () {
          that.hide()
        });
      }
    },
    _getValidPosition: function (elem, startPos) {
      var topPos = startPos.top + this.options.offsetY;
      if (topPos + elem.outerHeight() > this._viewportHeight) {
        topPos = startPos.top - this.options.offsetY - elem.outerHeight();
      }
      var leftPos = startPos.left + this.options.offsetX;
      if (leftPos + elem.outerWidth() > this._viewportWidth) {
        leftPos = startPos.left - this.options.offsetX - elem.outerWidth();
      }
      return ({ top: topPos, left: leftPos });
    },
    _startTracePosition: function () {
      var that = this;
      var tooltipElem = this.element.data('tooltipElem');
      this.element.mousemove(function (e) {
        tooltipElem.css(that._getValidPosition(tooltipElem, { top: e.pageY, left: e.pageX }));
      });
    },
    _stopTracePosition: function () {
      this.element.off('mousemove');
    },
    // Public methods
    show: function (pos) {
      this.hide();
      this._updateViewportSize();
      var tooltipElem = $('<div class="tooltip shadow">' + this.options.content + '</div>').appendTo('body');
      this.element.data('tooltipElem', tooltipElem);
      tooltipElem.css(this._getValidPosition(tooltipElem, pos));
      tooltipElem.show();
      this._startTracePosition();
      this._shown = true;
    },
    hide: function () {
      if (this._shown) {
        var tooltipElem = this.element.data('tooltipElem');
        if (typeof (tooltipElem) != 'undefined') {
          tooltipElem.remove();
          this.element.removeData('tooltipElem');
          this._stopTracePosition();
        }
        $(this).triggerHandler('afterHide');
      }
      this._shown = false;
    },
    updateContent: function (newContent) {
      this.options.content = newContent;
    }
  };

  $.fn[PLUGIN_NS] = function (methodOrOptions) {
    var args = arguments;
    return this.each(function () {
      var instance = $(this).data(PLUGIN_NS);

      // CASE: action method (public method on PLUGIN class)
      if (instance && instance[methodOrOptions] && $.isFunction(instance[methodOrOptions])) {
        if (methodOrOptions.indexOf('_') != 0) {
          instance[methodOrOptions](Array.prototype.slice.call(args, 1));
        }

        // CASE: argument is options object or empty = initialise
      } else if ($.isPlainObject(methodOrOptions) || !methodOrOptions) {
        $.removeData(this, PLUGIN_NS);
        var instance = new Plugin(this, methodOrOptions);
        $.data(this, PLUGIN_NS, instance);

        // CASE: method called before init
      } else if (!instance) {
        $.error('Plugin must be initialised before using method: ' + methodOrOptions);

        // CASE: invalid method
      } else if (methodOrOptions.indexOf('_') == 0) {
        $.error('Method ' + methodOrOptions + ' is private!');
      } else {
        $.error('Method ' + methodOrOptions + ' does not exist.');
      }
    });
  };

})(jQuery, window, document);
