//==========================================================================================================================
//  'BcodeMainWnd' class, inherited from 'QMainWindow'
//			This is the main application window for Bcode.
//
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#ifndef _ui_BcodeMainWnd_
#define _ui_BcodeMainWnd_

#include <QMainWindow>
#include "ui_BcodeMainWnd.h"
#include "ui/Chief.h"
#include "ui/WebGUI.h"
#include "ui/ModelCreateWebGUI.h"

namespace ui {

class BcodeMainWnd : public QMainWindow, public Ui::BcodeMainWnd
{
    Q_OBJECT

public:

    // Get the one and only instance of the main window of Excalibur.
    static BcodeMainWnd& getInstance();

    // Destructor.
    //
    virtual ~BcodeMainWnd();

    // Set Chief.
    //
    void setChief(ui::Chief* chief);

public slots:

    void showAboutBcodeDialog();
    void showAbout3rdPartyDialog();
    void showWhatsNewDialog();
    void showModelCreateDialog();

signals:

    void modelSaveFileSpecified(const QFileInfo&);
    void modelLoadFileSpecified(const QFileInfo&);

protected:

    // Constructor.
    //
    explicit BcodeMainWnd(QWidget* parent = nullptr);

    void closeEvent(QCloseEvent* event);

private slots:

    void showOptionsDialog();
    void showSaveModelsDialog();
    void showLoadModelsDialog();

private:

    // Set up ui
    void createMenuBar();
    void connectActions();
    bool userReallyWantsToQuit();

    // The one and only MainWindow object
    static BcodeMainWnd* m_mainWnd;

    // Keep reference to the Chief object
    ui::Chief* m_chief;

    // Web-based GUI
    ui::WebGUI m_webGUI;

    // Web-based GUI
    ui::ModelCreateWebGUI m_modelCreateWebGUI;
};

} // namespece ui

#endif // _ui_BcodeMainWnd_
