//==========================================================================================================================
//  Implementation file for "DataPresenter.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include <QVBoxLayout>
#include <QWebChannel>
#include <QWebEngineScript>
#include <QWebEngineScriptCollection>
#include "ui/DataPresenter.h"
#include "ui/WebGUI.h"

namespace ui {

DataPresenter::DataPresenter(const QVariantMap& elemInfo, const QString& caption, const bool resultsAvailable,
                             ui::WebGUI* webGUI, QWidget* parent) :
    QWidget(parent),
    m_webView(this),
    m_webData(),
    m_webGUI(webGUI)
{
    setAttribute(Qt::WA_DeleteOnClose, true);

    // Set ui
    resize(QSize(870, 700));
    QVBoxLayout* layout = new QVBoxLayout(this);
    layout->setMargin(0);
    layout->addWidget(&m_webView);
    setLayout(layout);

    // Remove context menu in production builds
    #if defined(FINAL_GENERAL) || defined(FINAL_ECDC)
        m_webView.setContextMenuPolicy(Qt::PreventContextMenu);
    #else
        m_webView.setContextMenuPolicy(Qt::DefaultContextMenu);
    #endif

    // Store data
    m_webData.addData("Caption", caption);
    m_webData.addData("ResultsAvailable", resultsAvailable);
    m_webData.addData("ElemInfo", elemInfo);

    // Export objects to JavaScript
    QWebChannel* channel = new QWebChannel();
    channel->registerObject(QStringLiteral("webData"), &m_webData);
    channel->registerObject(QStringLiteral("webGUI"), m_webGUI);
    m_webView.page()->setWebChannel(channel);
}

DataPresenter::~DataPresenter()
{
    emit gettingDestroyed(this);
}

const fn::WebData& DataPresenter::getWebData() const
{
    return m_webData;
}

void DataPresenter::keyPressEvent(QKeyEvent* event)
{
    if (event->key() == Qt::Key_Escape) {
        this->close();
    }
}

}  // namespace ui
