//==========================================================================================================================
//  Implementation file for "PresenterMgr.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2013 NextPage Software
//==========================================================================================================================

#include "ui/PresenterMgr.h"

namespace ui {

PresenterMgr::PresenterMgr() :
    m_presenterMap()
{

}

void PresenterMgr::registerPresenter(ui::DataPresenter* presenter)
{
    if (presenter != nullptr) {
        const fn::WebData& webData = presenter->getWebData();
        const fn::DiseaseModel::ID diseaseModelID = webData.getDiseaseModelID();
        const fn::OutcomeTree::ID outcomeTreeID = webData.getOutcomeTreeID();
        const fn::OutcomeTreeNode::ID outcomeTreeNodeID = webData.getOutcomeTreeNodeID();

        if (m_presenterMap.contains(diseaseModelID)) {
            if (m_presenterMap[diseaseModelID].contains(outcomeTreeID)) {
                if (m_presenterMap[diseaseModelID][outcomeTreeID].contains(outcomeTreeNodeID)) {
                    UT_THROW(QString("Presenter for %1, %2, %3 has already been registered").arg(diseaseModelID, outcomeTreeID, outcomeTreeNodeID));
                }
            }
        }

        m_presenterMap[diseaseModelID][outcomeTreeID][outcomeTreeNodeID] = presenter;
    }
}

void PresenterMgr::unregisterPresenter(ui::DataPresenter* presenter)
{
    if (presenter != nullptr) {
        const fn::WebData& webData = presenter->getWebData();
        const fn::DiseaseModel::ID diseaseModelID = webData.getDiseaseModelID();
        const fn::OutcomeTree::ID outcomeTreeID = webData.getOutcomeTreeID();
        const fn::OutcomeTreeNode::ID outcomeTreeNodeID = webData.getOutcomeTreeNodeID();

        bool presenterRemoved = false;
        if (m_presenterMap.contains(diseaseModelID)) {
            if (m_presenterMap[diseaseModelID].contains(outcomeTreeID)) {
                if (m_presenterMap[diseaseModelID][outcomeTreeID].contains(outcomeTreeNodeID)) {
                    m_presenterMap[diseaseModelID][outcomeTreeID].remove(outcomeTreeNodeID);
                    presenterRemoved = true;
                }
            }
        }

        if (!presenterRemoved) {
            UT_THROW(QString("Presenter for %1, %2, %3 has never been registered").arg(diseaseModelID, outcomeTreeID, outcomeTreeNodeID));
        }
    }
}

bool PresenterMgr::presenterExists(const fn::DiseaseModel::ID& diseaseModelID, const fn::OutcomeTree::ID& outcomeTreeID,
                                   const fn::OutcomeTreeNode::ID& outcomeTreeNodeID) const
{
    bool result = false;
    if (m_presenterMap.contains(diseaseModelID)) {
        if (m_presenterMap[diseaseModelID].contains(outcomeTreeID)) {
            if (m_presenterMap[diseaseModelID][outcomeTreeID].contains(outcomeTreeNodeID)) {
                result = true;
            }
        }
    }
    return result;
}

void PresenterMgr::closeAllPresenters()
{
    PresenterMap::const_iterator iter1 = m_presenterMap.begin();
    for (; iter1 != m_presenterMap.end(); ++iter1) {
        const OutcomeTreeMap& outcomeTrees = *iter1;
        OutcomeTreeMap::const_iterator iter2 = outcomeTrees.begin();
        for (; iter2 != outcomeTrees.end(); ++iter2) {
            const OutcomeTreeNodeMap& outcomeTreeNodes = *iter2;
            OutcomeTreeNodeMap::const_iterator iter3 = outcomeTreeNodes.begin();
            for (; iter3 != outcomeTreeNodes.end(); ++iter3) {
                (*iter3)->close();
            }
        }

    }
}

PresenterMgr::~PresenterMgr()
{
    
}

}  // namespace fn

