//==========================================================================================================================
//  'PresenterMgr' class
//			This class implements manager of presenter windows
//
//  Written by Daniel Lewandowski
//  Copyright @ 2013 NextPage Software
//==========================================================================================================================

#ifndef _ui_PresenterMgr_
#define _ui_PresenterMgr_

#include "fn/DiseaseModel.h"
#include "fn/OutcomeTree.h"
#include "fn/OutcomeTreeNode.h"
#include "ui/DataPresenter.h"

namespace ui {

class PresenterMgr : public QObject
{

    Q_OBJECT

public:
    // Constructor.
    //
    PresenterMgr();

    // Register presenter
    //
    // 'presenter' - [in] Pointer to presenter
    //
    void registerPresenter(ui::DataPresenter* presenter);

    // Check if a presenter already exists
    //
    // 'diseaseModelID' - [in] Disease model ID
    // 'outcomeTreeID' - [in] Outcome tree ID
    // 'healthStateID' - [in] Health state ID
    //
    // Return - TRUE only if a corresponding presenter exists
    bool presenterExists(const fn::DiseaseModel::ID& diseaseModelID, const fn::OutcomeTree::ID& outcomeTreeID,
                         const fn::OutcomeTreeNode::ID& healthStateID) const;
    
    // Closes all registered presenters windows (for instance if models are deleted)
    //
    void closeAllPresenters();

    // Destructor.
    //
    virtual ~PresenterMgr();
    
public slots:

    // Unregister presenter
    //
    // 'presenter' - [in] Pointer to presenter
    //
    void unregisterPresenter(ui::DataPresenter* presenter);

private:	

    typedef QMap<fn::OutcomeTreeNode::ID, ui::DataPresenter*> OutcomeTreeNodeMap;
    typedef QMap<fn::OutcomeTree::ID, OutcomeTreeNodeMap> OutcomeTreeMap;
    typedef QMap<fn::DiseaseModel::ID, OutcomeTreeMap> PresenterMap;

    // Map of presenters
    PresenterMap m_presenterMap;
};

}  // namespace ui

#endif  // _ui_PresenterMgr_


