//==========================================================================================================================
//  Implementation file for "SettingsDlg.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2012 NextPage Software
//==========================================================================================================================

#include "ui_SettingsDlg.h"
#include "ui/SettingsDlg.h"
#include "fn/globals.h"

namespace ui {

SettingsDlg::SettingsDlg(fn::Settings& settings, QWidget* parent) :
    QDialog(parent),
    m_settings(settings)
{
    setupUi(this);
    QObject::connect(buttonBox, SIGNAL(clicked(QAbstractButton*)), this, SLOT(dialogButtonClicked(QAbstractButton*)));

    this->setWindowFlags(this->windowFlags() & ~Qt::WindowContextHelpButtonHint);

    lblNumSamples->setBuddy(edtNumSamples);
    lblRandomSeed->setBuddy(edtRandomSeed);
    lblDiscountRate->setBuddy(edtDiscountRate);

    QValidator* numSamplesValidator = new QIntValidator(10, 100000, edtNumSamples);
    edtNumSamples->setValidator(numSamplesValidator);
    edtNumSamples->setText(QString::number(m_settings.getNumSamples()));

    QValidator* randomSeedValidator = new QIntValidator(0, 100000, edtRandomSeed);
    edtRandomSeed->setValidator(randomSeedValidator);
    edtRandomSeed->setText(QString::number(m_settings.getRandomSeed()));

    QValidator* timeDisountRateValidator = new QDoubleValidator(0, 1, 4, edtDiscountRate);
    edtDiscountRate->setValidator(timeDisountRateValidator);
    edtDiscountRate->setText(QString::number(m_settings.getTimeDiscountRate()));
}

SettingsDlg::~SettingsDlg()
{

}

void SettingsDlg::writeSettings()
{
    m_settings.setNumSamples(edtNumSamples->text().toUInt());
    m_settings.setRandomSeed(edtRandomSeed->text().toUInt());
    m_settings.setTimeDiscountRate(edtDiscountRate->text().toDouble());
    m_settings.writeSettings();
}

void SettingsDlg::dialogButtonClicked(QAbstractButton* button)
{
    QDialogButtonBox::StandardButton standardButton = buttonBox->standardButton(button);

    switch(standardButton) {
        case QDialogButtonBox::RestoreDefaults :
            edtNumSamples->setText(QString::number(fn::kDefaultNumSamples));
            edtRandomSeed->setText(QString::number(fn::kDefaultRandomSeed));
            edtDiscountRate->setText(QString::number(fn::kDefaultTimeDiscountRate));
            break;
    }

}

} // namespace ui
