//==========================================================================================================================
//  Implementation file for "WebGUI.h"
//  Written by Daniel Lewandowski
//  Copyright @ 2011 NextPage Software
//==========================================================================================================================

#include <QPrinter>
#include <QMimeData>
#include <QClipboard>
#include <QDesktopServices>
#include <QFileDialog>
#include <QtWebChannel/QtWebChannel>
#include "ui/WebGUI.h"
#include "ut/VariantUtils.h"
#include "fn/ArrayHelpers.h"
#include "fn/OutcomeTreeNode.h"
#include "ui/Chief.h"
#include "ui/BcodeMainWnd.h"
#include "ui/YldHealthOutcomeDataPresenter.h"
#include "ui/YllHealthOutcomeDataPresenter.h"
#include "ui/TransProbDataPresenter.h"

namespace ui {

WebGUI::WebGUI():
    m_chief(nullptr),
    m_page(nullptr),
    m_printer(nullptr)
{

}

WebGUI::~WebGUI()
{

}

void WebGUI::dataRequested()
{
    emit programReinitialized();
}

// Initialization methods

void WebGUI::setChief(Chief* chief)
{
    m_chief = chief;

    // JavaScript to C++
    connect(this, SIGNAL(pageOpenNotified(int, const QString&)), m_chief, SLOT(pageOpened(int, const QString&)));
    connect(this, SIGNAL(countrySelectionChangeRequested(const QStringList&)), m_chief, SLOT(changeCountrySelection(const QStringList&)));
    connect(this, SIGNAL(diseaseSelectionChangeRequested(const QStringList&)), m_chief, SLOT(changeDiseaseSelection(const QStringList&)));
    connect(this, SIGNAL(createModelsRequested()), m_chief, SLOT(createModels()));
    connect(this, SIGNAL(deleteModelsRequested()), m_chief, SLOT(deleteModels()));
    connect(this, SIGNAL(modelSelectionRequested(const QString&, const QString&, const QString&)), m_chief, SLOT(selectModel(const QString&, const QString&, const QString&)));
    connect(this, SIGNAL(showingOutcomeTreeElementDetailsRequested(const fn::OutcomeTree::ID&, const QString&, const fn::OutcomeTreeNode::ID&, const fn::InputType::Country&, const fn::InputType::Disease&, const QString&)), m_chief, SLOT(showOutcomeTreeElementDetails(const fn::OutcomeTree::ID&, const QString&, const fn::OutcomeTreeNode::ID&, const fn::InputType::Country&, const fn::InputType::Disease&, const QString&)));
    connect(this, SIGNAL(applyingChangesToModelParamsRequested(const QVariantMap&, const int, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&)), m_chief, SLOT(applyChangesToModelParams(const QVariantMap&, const int, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&)));
    connect(this, SIGNAL(applyingChangesToYLDParamsRequested(const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&)), m_chief, SLOT(applyChangesToYLDParams(const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&)));
    connect(this, SIGNAL(applyingChangesToYLLParamsRequested(const QVariantMap&, const QVariantMap&)), m_chief, SLOT(applyChangesToYLLParams(const QVariantMap&, const QVariantMap&)));
    connect(this, SIGNAL(applyingChangesToTransParamsRequested(const QVariantMap&, const QVariantMap&, const QVariantMap&)), m_chief, SLOT(applyChangesToTransParams(const QVariantMap&, const QVariantMap&, const QVariantMap&)));
    connect(this, SIGNAL(settingNumSamplesRequested(const int)), m_chief, SLOT(setNumSamples(const int)));
    connect(this, SIGNAL(settingTimeDiscountRateRequested(const double)), m_chief, SLOT(setTimeDiscountRate(const double)));
    connect(this, SIGNAL(showingDetailedResultsRequested(const fn::InputType::Country&, const fn::InputType::Disease&, const QString&)), m_chief, SLOT(showDetailedResults(const fn::InputType::Country&, const fn::InputType::Disease&, const QString&)));

    // C++ to JavaScript
    connect(m_chief, SIGNAL(programReinitialized()), this, SIGNAL(programReinitialized()));
    connect(m_chief, SIGNAL(showMessage(const QString&, const QString&, const QString&)), this, SIGNAL(showMessage(const QString&, const QString&, const QString&)));
    connect(m_chief, SIGNAL(showMessageInYLDPage(const QString&, const QString&, const QString&)), this, SIGNAL(showMessageInYLDPage(const QString&, const QString&, const QString&)));
    connect(m_chief, SIGNAL(showMessageInYLLPage(const QString&, const QString&, const QString&)), this, SIGNAL(showMessageInYLLPage(const QString&, const QString&, const QString&)));
    connect(m_chief, SIGNAL(showMessageInTransPage(const QString&, const QString&, const QString&)), this, SIGNAL(showMessageInTransPage(const QString&, const QString&, const QString&)));
    connect(m_chief, SIGNAL(modelsCreated(const QVariantList&)), this, SIGNAL(modelsCreated(const QVariantList&)));
    connect(m_chief, SIGNAL(modelSelected(const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QString&)), this, SIGNAL(modelSelected(const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QVariantMap&, const QString&)));
    connect(m_chief, SIGNAL(formulaCheckResultsAvailable(const QVariantMap&)), this, SIGNAL(formulaCheckResultsAvailable(const QVariantMap&)));
    connect(m_chief, SIGNAL(formulaCheckResultsAvailableForYLDPage(const QVariantMap&)), this, SIGNAL(formulaCheckResultsAvailableForYLDPage(const QVariantMap&)));
    connect(m_chief, SIGNAL(formulaCheckResultsAvailableForYLLPage(const QVariantMap&)), this, SIGNAL(formulaCheckResultsAvailableForYLLPage(const QVariantMap&)));
    connect(m_chief, SIGNAL(createUpdatableMessage(const QString&, const QString&)), this, SIGNAL(createUpdatableMessage(const QString&, const QString&)));
    connect(m_chief, SIGNAL(updateMessage(const QString&, const QString&)), this, SIGNAL(updateMessage(const QString&, const QString&)));
    connect(m_chief, SIGNAL(destroyUpdatableMessage()), this, SIGNAL(destroyUpdatableMessage()));
    connect(m_chief, SIGNAL(runStarted()), this, SIGNAL(runStarted()));
    connect(m_chief, SIGNAL(runFinished(const QVariantList&)), this, SIGNAL(runFinished(const QVariantList&)));
    connect(m_chief, SIGNAL(runProgressUpdated(const float, const int, const int, const QString&)), this, SIGNAL(runProgressUpdated(const float, const int, const int, const QString&)));
    connect(m_chief, SIGNAL(detailedResultsAvailable(const QVariantMap&)), this, SIGNAL(detailedResultsAvailable(const QVariantMap&)));
    connect(m_chief, SIGNAL(aggregatedResultsAvailable(const QVariantMap&)), this, SIGNAL(aggregatedResultsAvailable(const QVariantMap&)));
}

void WebGUI::setPage(QWebEnginePage* page)
{
    m_page = page;

    QWebChannel* channel = new QWebChannel();
    channel->registerObject(QStringLiteral("webGUI"), this);
    m_page->setWebChannel(channel);
}

// Methods invokable from JavaScript

void WebGUI::webLinkClicked(const QString& webLink)
{
    // If it is an external link then open it in the default browser
    if (webLink.contains("http")) {
        QDesktopServices::openUrl(webLink);
    }
}

void WebGUI::pageOpened(int tabIndex, const QString& pageName)
{
    emit pageOpenNotified(tabIndex, pageName);
}

QVariantList WebGUI::getAvailableDiseases() const
{
    return m_chief->getAvailableDiseases();
}

void WebGUI::setSelectedCountries(const QStringList& countryIDStrs)
{
    emit countrySelectionChangeRequested(countryIDStrs);
}

void WebGUI::setSelectedDiseases(const QStringList& diseaseIDStrs)
{
    emit diseaseSelectionChangeRequested(diseaseIDStrs);
}

void WebGUI::createModels()
{
    emit createModelsRequested();
}

void WebGUI::deleteModels()
{
    emit deleteModelsRequested();
}

void WebGUI::selectModel(const QString& countryId, const QString& diseaseId, const QString& modelId)
{
    emit modelSelectionRequested(countryId, diseaseId, modelId);
}

void WebGUI::showDetailedResults(const QString& countryIdStr, const QString& diseaseIdStr, const QString& modelId)
{
    fn::InputType::Country countryId = static_cast<fn::InputType::Country>(countryIdStr);
    fn::InputType::Disease diseaseId = static_cast<fn::InputType::Disease>(diseaseIdStr);

    emit showingDetailedResultsRequested(countryId, diseaseId, modelId);
}

void WebGUI::showOutcomeTreeElementDetails(const QString& outcomeTreeIdStr, const QString& elementType, const QString& elementIdStr,
                                           const QString& countryIdStr, const QString& diseaseIdStr, const QString& modelId)
{
    fn::OutcomeTree::ID outcomeTreeId = static_cast<fn::OutcomeTree::ID>(outcomeTreeIdStr);
    fn::OutcomeTreeNode::ID elementId = static_cast<fn::OutcomeTreeNode::ID>(elementIdStr);
    fn::InputType::Country countryId = static_cast<fn::InputType::Country>(countryIdStr);
    fn::InputType::Disease diseaseId = static_cast<fn::InputType::Disease>(diseaseIdStr);

    emit showingOutcomeTreeElementDetailsRequested(outcomeTreeId, elementType, elementId,
                                                   countryId, diseaseId, modelId);
}

void WebGUI::applyChangesToModelParams(const QVariantMap& modelInfo, const int estimType, const QVariantMap& underestimationData,
                                       const QVariantMap& underreportingData, const QVariantMap& underascertainmentData,
                                       const QVariantMap& caseBeforeCorrData, const QVariantMap& lifeExpData,
                                       const QVariantMap& ageDistrData)
{
    emit applyingChangesToModelParamsRequested(modelInfo, estimType, underestimationData, underreportingData,
                                               underascertainmentData, caseBeforeCorrData, lifeExpData, ageDistrData);
}

void WebGUI::applyChangesToYLDParams(const QVariantMap& modelInfo, const QVariantMap& severityData,
                                     const QVariantMap& durationData, const QVariantMap& proportionData)
{
    emit applyingChangesToYLDParamsRequested(modelInfo, severityData, durationData, proportionData);
}

void WebGUI::applyChangesToYLLParams(const QVariantMap& modelInfo, const QVariantMap& proportionData)
{
    emit applyingChangesToYLLParamsRequested(modelInfo, proportionData);
}

void WebGUI::applyChangesToTransParams(const QVariantMap& modelInfo, const QVariantMap& probabilityData, const QVariantMap& redistributionData)
{
    emit applyingChangesToTransParamsRequested(modelInfo, probabilityData, redistributionData);
}

int WebGUI::getNumSamples()
{
    return m_chief->getNumSamples();
}

double WebGUI::getTimeDiscountRate()
{
    return m_chief->getTimeDiscountRate();
}

void WebGUI::setNumSamples(const int numSamples)
{
    emit settingNumSamplesRequested(numSamples);
}

void WebGUI::setTimeDiscountRate(const double timeDiscountRate)
{
    emit settingTimeDiscountRateRequested(timeDiscountRate);
}

void WebGUI::runModels()
{
    m_chief->runModels(true);
}

void WebGUI::copyTextToClipboard(const QString& text)
{
    QClipboard* clipboard = QApplication::clipboard();
    clipboard->setText(text);
}

QVariantMap WebGUI::pasteFromClipboardToGrid(const QStringList& rowNames, const QStringList& colNames, const QString& distrType)
{
    QVariantMap result;
    QVariantList array;
    int numParams = 1;
    std::function<QString(const QString&)> paramStrTransformer = [](const QString& paramStr) -> QString{ return paramStr; };
    QString defaultValue = "0";
    if (distrType == "Uniform") {
        numParams = 2;
        defaultValue = "RandUniform(0, 0)";
        paramStrTransformer = [](const QString& paramStr) { return "RandUniform(" + paramStr + ")"; };
    } else if (distrType == "Pert") {
        numParams = 3;
        defaultValue = "RandPert(0, 0, 0)";
        paramStrTransformer = [](const QString& paramStr) { return "RandPert(" + paramStr + ")"; };
    }

    const QClipboard* clipboard = QApplication::clipboard();
    const QMimeData* mimeData = clipboard->mimeData();

    if (mimeData->hasText()) {
        QString str = mimeData->text();
        QStringList rows = str.split('\n');
        QLocale cLocale(QLocale::C);
        int numRows = rows.count();

        for (int i = 0; i < rowNames.length(); ++i) {
            QVariantMap row;
            row["id"] = i;
            row["Age"] = rowNames[i];

            // Extra row is added by Excel
            if (i < numRows - 1) {
                QStringList columns = rows[i].split('\t');
                int numCols = columns.count();
                for (int j = 0; j < colNames.length(); ++j) {
                    QStringList params;
                    for (int k = 0; k < numParams; ++k) {
                        int index = numParams * j + k;
                        if (index < numCols) {
                            // Try converting to number
                            QString stringValue = columns[index];
                            QVariant value;
                            bool ok;
                            double numericValue = cLocale.toDouble(stringValue, &ok);
                            if (ok) {
                                value = numericValue;
                            } else {
                                value = stringValue;
                            }
                            params.append(value.toString());
                        } else {
                            params.append("0");
                        }
                    }
                    row[colNames[j]] = paramStrTransformer(params.join(", "));
                }
            } else {
                row[colNames[0]] = defaultValue;
                row[colNames[1]] = defaultValue;
            }

            array.append(row);
        }
    }

    result.insert("stratified", array);

    return result;
}

void WebGUI::copyPngImageToClipboard(const QString& base64Str)
{
    QImage image = getPngImageFromBase64Str(base64Str);
    QApplication::clipboard()->setImage(image, QClipboard::Clipboard);
}

void WebGUI::savePngImageToFile(const QString& base64Str)
{
    QString filename = QFileDialog::getSaveFileName(nullptr, tr("Save Image"), QDir::currentPath(), tr("PNG files (*.png)") );
    if (!filename.isNull()) {
        QImage image = getPngImageFromBase64Str(base64Str);
        image.save(filename, "PNG");
    }
}

QImage WebGUI::getPngImageFromBase64Str(const QString& base64Str)
{
    QByteArray imgArr = QByteArray::fromBase64(base64Str.toLocal8Bit());
    QImage image;
    image.loadFromData(imgArr, "PNG");

    return image;
}

void WebGUI::printPage()
{

}

void WebGUI::showPrintPreview(QPrinter* printer)
{
    Q_UNUSED(printer);
}

void WebGUI::exportPageToPdf()
{
    const QString fileName = QFileDialog::getSaveFileName(nullptr, tr("Save pdf"), ".", tr("PDF Files (*.pdf)"));
    if (fileName.isEmpty()) {
        return;
    }
    m_page->printToPdf(fileName);
}

}  // namespace ui

