//==========================================================================================================================
//	'Exception' class
//			The main exception class.
//
//  Copyright @ 2003 Daniel Lewandowski
//==========================================================================================================================

#ifndef _ut_Exception_
#define _ut_Exception_

#include "QString"

namespace ut {

class Exception {
public:
	// Constructor.
	//
	// 'errorMessage' - [in] The error message.
	// 'functionName' - [in] The name of the function where the exception was originally thrown.
	//
	Exception(const QString& errorMessage, const QString& functionName);

	// Copy constructor.
	//
	// 'original' - [in] The original object, to be copied into this one.
	//
	Exception(const Exception& original);

	// Destructor.
	//
	virtual ~Exception();

	// Assignment operator.
	//
	// 'original' - [in] The original object, to be copied into this one.
	//
	Exception& operator=(const Exception& original);

	// Get the error message.
	//
	//  Return - The message.
	//
	const QString& getErrMessage() const;

	// Get the name of the function where the exception was originally thrown.
	//
	//  Return - The function name.
	//
	const QString& getFunctionName() const;

private:
	// The error message.
	QString m_errorMessage;

	// The name of the function where the exception was originally thrown.
	QString m_functionName;
};


} // namespace ut

#endif
// _ut_Exception_
