//==========================================================================================================================
//	'SmtpClient' class
//
//  Copyright @ 2019 Daniel Lewandowski
//==========================================================================================================================

#ifndef _ut_SmtpClient_
#define _ut_SmtpClient_

#include <QtNetwork/QAbstractSocket>
#include <QtNetwork/QSslSocket>
#include <QString>
#include <QTextStream>
#include <QDebug>
#include <QtWidgets/QMessageBox>
#include <QByteArray>
#include <QFile>
#include <QFileInfo>

namespace ut {

class SmtpClient : public QObject
{

    Q_OBJECT

public:

    SmtpClient(const QString& user, const QString& pass, const QString& host, int port = 465, int timeout = 30000);

    ~SmtpClient();

    void sendMail(const QString& from, const QString& to, const QString& subject, const QString& body, const QString& fileName, const QByteArray& fileContent);

signals:

    void status(const QString&);

private slots:

    void stateChanged(QAbstractSocket::SocketState socketState);
    void errorReceived(QAbstractSocket::SocketError socketError);
    void disconnected();
    void connected();
    void readyRead();

private:

    int timeout;
    QString message;
    QTextStream* t;
    QSslSocket* socket;
    QString from;
    QString rcpt;
    QString response;
    QString user;
    QString pass;
    QString host;
    int port;
    enum states {Tls, HandShake, Auth, User, Pass, Rcpt, Mail, Data, Init, Body, Quit, Close};
    int state;
};

} // namespace ut

#endif
// _ut_SmtpClient_
