/* global
    $, jsPlumb
*/

"use strict";

var renderDisease = function (flowChartContainer, diseaseSpec) {
  var otSpecs = diseaseSpec.outcomeTrees;

  var otTemplates = "";
  for (let otId in otSpecs) {
    let otSpec = otSpecs[otId];
    let otName = otSpec.name;
    let otMaxTop = 0;

    let hoSpecs = otSpec.healthOutcomes;
    let hoTemplates = "";
    for (let hoId in hoSpecs) {
      let hoSpec = hoSpecs[hoId];

      let hoName = hoSpec.name;
      let hoType = hoSpec.type.toLowerCase();
      let hoTop = hoSpec.top;
      let hoLeft = hoSpec.left;

      otMaxTop = Math.max(otMaxTop, hoTop);

      if (hoType === 'acute') {
        hoType = 'symptomatic';
      }

      if (hoType === 'recovery') {
        hoName = 'R';
      }

      let hsSpecs = hoSpec.healthStates;
      let hoTemplate = "";
      let hsTemplates = "";
      if (Object.keys(hsSpecs).length > 0) {
        let hsTitleTemplate = `<div class='tableRow'><p id='${hoId}_title' class='healthOutcomeTitle'>${hoName}</p></div>`;
        hsTemplates = hsTemplates + hsTitleTemplate;
        for (let hsId in hsSpecs) {
          let hsSpec = hsSpecs[hsId];
          let hsName = hsSpec.name;
          let hsTemplate = `<div class='tableRow'><p id='${hsId}' class='healthState'>${hsName}</p></div>`;
          hsTemplates = hsTemplates + hsTemplate;
        }
        hoTemplate = `<div id=${hoId} class='healthOutcome ${hoType}' style='top: ${hoTop}px; left: ${hoLeft}px'>${hsTemplates}</div>`;
      } else {
        hoTemplate = `<div id='${hoId}' class='healthOutcome ${hoType}' style='top: ${hoTop}px; left: ${hoLeft}px'><p id='${hoId}_hs' class='healthState'>${hoName}</p></div>`;
      }

      hoTemplates = hoTemplates + hoTemplate;
    }

    let otTemplate = `<div id='${otId}' class='outcomeTree' style='margin-bottom: ${otMaxTop+80}px'><div class='outcomeTreeTitle'>${otName}</div>${hoTemplates}</div>`;
    otTemplates = otTemplates + otTemplate;
  }

  flowChartContainer.html(otTemplates);
};

var createConnection = function (source, target, params, nodeID, outcomeTreeID, label) {
  var overlays = [];
  if (arguments.length == 6) {
    overlays = [["Label", { label: label }]];
  }

  var settings = $.extend(true, {}, params, { source: source, target: target, overlays: overlays });

  var connection = jsPlumb.connect(settings);

  if (arguments.length == 6) {
    connection.name = nodeID;
    connection.outcomeTreeID = outcomeTreeID;
  }

  var hoverSourceElem = {};
  var hoverTargetElem = {};

  connection.bind("mouseenter", function (conn) {
    var setHoverElem = function (fireElem) {
      var hoverElem = {
        element: fireElem,
        className: "healthOutcomeHover"
      };
      if (fireElem.hasClass("healthState")) {
        var parent1 = fireElem.parent();
        if (parent1.hasClass("tableRow")) {
          var parent2 = parent1.parent();
          if (parent2.hasClass("symptomatic")) {
            hoverElem.className = "healthStateSymptomaticHover";
          } else if (parent2.hasClass("sequela")) {
            hoverElem.className = "healthStateSequelaHover";
          } else if (parent2.hasClass("death")) {
            hoverElem.className = "healthStateDeathHover";
          }
        } else if (parent1.hasClass("healthOutcome")) {
          hoverElem.element = parent1;
        }
      } else if (fireElem.hasClass("healthOutcomeTitle")) {
        hoverElem.element = fireElem.parent().parent();
      } else if (fireElem.hasClass("healthOutcome")) {
        if (fireElem.hasClass("symptomatic")) {
          hoverElem.className += " healthOutcomeSymptomaticHover";
        } else if (fireElem.hasClass("death")) {
          hoverElem.className += " healthOutcomeDeathHover";
        }
      } else {
        hoverElem.element = fireElem.parent();
      }

      if (!$.isEmptyObject(hoverElem)) {
        hoverElem.element.addClass(hoverElem.className);
      }

      return (hoverElem);
    };

    hoverSourceElem = setHoverElem($(conn.source));
    hoverTargetElem = setHoverElem($(conn.target));

  });

  connection.bind("mouseexit", function () {
    if (!$.isEmptyObject(hoverSourceElem)) {
      hoverSourceElem.element.removeClass(hoverSourceElem.className);
      hoverSourceElem = {};
    }

    if (!$.isEmptyObject(hoverTargetElem)) {
      hoverTargetElem.element.removeClass(hoverTargetElem.className);
      hoverTargetElem = {};
    }

  });
};

var diseaseTemplates = {

  Default: {
    htmlCode: "No outcome tree specification found.",
    jsCode: function () { }
  },

  // Hepatitis B (consistent)
  // 12/09/2014 - updated with Review 2
  // 29/10/2014 - updated with Review 3
  HepatitisB: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 310px">' +
      '<div class="healthOutcome infection" style="top:119px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:0px; left: 200px"><p id="symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="top:260px; left: 200px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="top:119px; left: 200px"><p id="chronic_1" class="healthState">Chronic hepatitis</p></div>' +
      '<div class="healthOutcome sequela" style="top:0px; left: 400px"><p id="fulminant_1" class="healthState">Fulminant liver failure</p></div>' +
      '<div class="healthOutcome sequela" style="top:215px; left: 400px"><p id="compensated_1" class="healthState">Compensated cirrhosis</p></div>' +
      '<div class="healthOutcome sequela" style="top:260px; left: 600px"><p id="decompensated_1" class="healthState">Decompensated cirrhosis</p></div>' +
      '<div class="healthOutcome sequela" style="top:82px; left: 600px">' +
      '   <div class="tableRow"><p id="hepatocellular" class="healthOutcomeTitle">Hepatocellular carcinoma</p></div>' +
      '   <div class="tableRow"><p id="hepatocellular_1" class="healthState">following<br />Chronic hepatitis</p></div>' +
      '   <div class="tableRow"><p id="hepatocellular_2" class="healthState">following<br />Compensated cirrhosis</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:82px; left: 800px">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatalHepatocellular_1" class="healthState">following<br />Chronic hepatitis</p></div>' +
      '   <div class="tableRow"><p id="fatalHepatocellular_2" class="healthState">following<br />Compensated cirrhosis</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:0px; left: 600px"><p id="fatalFulminant_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="top:260px; left: 800px"><p id="fatalDecompensated_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 105px;"><p id="chronic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 240px;"><p id="decompensated_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 283px;"><p id="compensated_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 40px;"><p id="fulminant_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 91px;"><p id="hepatocellular_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "chronic_1", { anchors: ["BottomCenter", "TopCenter"] }, "delta_1", "All", "delta<sub>1</sub>");
      createConnection("asymptomatic_1", "chronic_1", { anchors: ["TopCenter", "BottomCenter"] }, "delta_2", "All", "delta<sub>2</sub>");
      createConnection("chronic_1", "compensated_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("chronic_1", "hepatocellular_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "fulminant_1", {}, "gamma_1", "All", "gamma");
      createConnection("compensated_1", "decompensated_1", {}, "zeta_1", "All", "zeta");
      createConnection("compensated_1", "hepatocellular_2", {}, "theta_1", "All", "theta");
      createConnection("hepatocellular_1", "fatalHepatocellular_1", {}, "kappa_1", "All", "kappa<sub>1</sub>");
      createConnection("hepatocellular_2", "fatalHepatocellular_2", {}, "kappa_2", "All", "kappa<sub>2</sub>");
      createConnection("fulminant_1", "fatalFulminant_1", {}, "beta_1", "All", "beta");
      createConnection("decompensated_1", "fatalDecompensated_1", {}, "lambda_1", "All", "lambda");
      createConnection("chronic_1", "chronic_recovery", {});
      createConnection("hepatocellular", "hepatocellular_recovery", {});
      createConnection("decompensated_1", "decompensated_recovery", {});
      createConnection("compensated_1", "compensated_recovery", {});
      createConnection("fulminant_1", "fulminant_recovery", {});
    }
  },

  // Influenza (consistent)
  // 07/10/2014 - updated with Review 2
  Influenza: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 200px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 45px;"><p id="infection_1" class="healthState">Infections</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 150px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:45px; left: 200px"><p id="acute_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 0px;"><p id="fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 90px;"><p id="ards_long_1" class="healthState">Permanent disability due to ARDS</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 0px; "><p id="acute_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 163px; "><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_1", "ards_long_1", {}, "gamma_1", "All", "gamma");
      createConnection("acute_1", "fatalAcute_1", {}, "beta_1", "All", "beta");
      createConnection("acute_1", "acute_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Measles (consistent)
  // 30/10/2014 - updated with Review 2
  Measles: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 330px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 142px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 280px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="left: 200px; top: 99px;">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 0px;"><p id="otitis_long_1" class="healthState">Permanent disability due to otitis media</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 70px;"><p id="encephalitis_long_1" class="healthState">Permanent disability due to encephalitis</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 140px;"><p id="postinfectious_long_1" class="healthState">Permanent disability due to post-infectious encephalomyelitis</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 210px;"><p id="latency_1" class="healthState">Latency</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 210px;"><p id="sspe_1" class="healthState">SSPE</p></div>' +
      '<div class="healthOutcome death" style="left: 800px; top: 210px;"><p id="fatalSSPE_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 280px;"><p id="fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 50px; "><p id="acute_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 293px; "><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute", "encephalitis_long_1", {}, "zeta_1", "All", "zeta");
      createConnection("acute", "postinfectious_long_1", {}, "theta_1", "All", "theta");
      createConnection("acute", "otitis_long_1", {}, "delta_1", "All", "delta");
      createConnection("acute", "latency_1", {}, "iota_1", "All", "iota");
      createConnection("latency_1", "sspe_1", {}, "omicron_1", "All", "omicron");
      createConnection("acute", "fatalAcute_1", {}, "beta_1", "All", "beta");
      createConnection("sspe_1", "fatalSSPE_1", {}, "nu_1", "All", "nu");
      createConnection("acute", "acute_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Salmonellosis (consistent)
  // 31/10/2014 - updated with Review 2
  Salmonellosis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 280px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 66px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="ge" class="healthOutcome symptomatic" style="left: 200px; top: 0px;">' +
      '   <div class="tableRow"><p id="ge_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="ge_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="ge_2" class="healthState">Complicated (GP)</p></div>' +
      '   <div class="tableRow"><p id="ge_3" class="healthState">Complicated (Hosp)</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 230px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 140px;"><p id="rea_1" class="healthState">Reactive arthritis</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 0px;"><p id="fatal_acute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 80px;"><p id="ge_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 153px;"><p id="rea_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 243px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "ge", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("ge", "rea_1", {}, "delta_1", "All", "delta");
      createConnection("ge", "fatal_acute_1", {}, "beta_1", "All", "beta");
      createConnection("ge", "ge_recovery", {});
      createConnection("rea_1", "rea_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // STEC/VTEC (consistent)
  // 31/10/2014 - updated with Review 2
  STECVTEC: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 515px">' +
      '<div class="healthOutcome infection" style="top:284px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:284px; left: 160px"><p id="gastroenteritis_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="top:465px; left: 160px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="top:284px; left: 330px"><p id="hus_1" class="healthState">Hemolytic Uremic syndrome</p></div>' +
      '<div class="healthOutcome sequela" style="left: 480px; top:150px;"><p id="latency_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome sequela" style="left: 620px; top: 200px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">ESRD</p></div>' +
      '   <div class="tableRow"><p id="esrd_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 910px; top: 240px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Transplanted (remaining of transplantation)</p></div>' +
      '   <div class="tableRow"><p id="transplanted_trans_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="transplanted_trans_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="left: 790px; top: 370px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Transplanted (remaining of dialysis)</p></div>' +
      '   <div class="tableRow"><p id="transplanted_dial_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="transplanted_dial_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="left: 790px; top: 0px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases (transplan- tation)</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_trans_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_trans_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="left: 910px; top: 80px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases (dialysis)</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_dial_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_dial_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="top:465px; left: 510px"><p id="fatalHUS_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="top:465px; left: 330px"><p id="fatalGastroenteritis_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="top:265px; left: 290px"><p id="gastroenteritis_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="top:478px; left: 290px"><p id="asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="top:265px; left: 460px"><p id="hus_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "gastroenteritis_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("gastroenteritis_1", "hus_1", {}, "gamma_1", "All", "gamma");
      createConnection("hus_1", "fatalHUS_1", {}, "delta_1", "All", "delta");
      createConnection("hus_1", "latency_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("latency_1", "esrd_latency", {}, "eta_1", "All", "eta");
      createConnection("hus_1", "esrd_no_latency", {}, "zeta_1", "All", "zeta");

      createConnection("esrd_latency", "esrd_fatal_trans_latency", {}, "kappa_1", "All", "kappa<sub>1</sub>");
      createConnection("esrd_latency", "transplanted_trans_latency", {}, "kappa_3", "All", "1 - kappa<sub>1</sub>");

      createConnection("esrd_no_latency", "esrd_fatal_trans_no_latency", {}, "kappa_2", "All", "kappa<sub>2</sub>");
      createConnection("esrd_no_latency", "transplanted_trans_no_latency", {}, "kappa_4", "All", "1 - kappa<sub>2</sub>");

      createConnection("esrd_latency", "esrd_fatal_dial_latency", {}, "lambda_1", "All", "lambda<sub>1</sub>");
      createConnection("esrd_latency", "transplanted_dial_latency", {}, "lambda_3", "All", "1 - lambda<sub>1</sub>");

      createConnection("esrd_no_latency", "esrd_fatal_dial_no_latency", {}, "lambda_2", "All", "lambda<sub>2</sub>");
      createConnection("esrd_no_latency", "transplanted_dial_no_latency", {}, "lambda_4", "All", "1 - lambda<sub>2</sub>");


      createConnection("gastroenteritis_1", "fatalGastroenteritis_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
      createConnection("gastroenteritis_1", "gastroenteritis_recovery", {});
      createConnection("hus_1", "hus_recovery", {});
    }
  },

  // Campylobacteriosis (consistent)
  // 31/10/2014 - updated with Review 2
  Campylobacteriosis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 350px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 137px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="left: 200px; top: 70px;">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Complicated (GP)</p></div>' +
      '   <div class="tableRow"><p id="acute_3" class="healthState">Complicated (Hosp)</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 300px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 80px;">' +
      '   <div class="tableRow"><p id="gbs_title" class="healthOutcomeTitle">Guillain-Barre syndrome</p></div>' +
      '   <div class="tableRow"><p id="gbs_mild_1" class="healthState">Mild</p></div>' +
      '   <div class="tableRow"><p id="gbs_severe_1" class="healthState">Severe</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 120px;"><p id="gbs_long_term_1" class="healthState">Permanent disability due to GBS</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 240px;"><p id="rea_1" class="healthState">Reactive arthritis</p></div>' +

      '<div class="healthOutcome death" style="left: 400px; top: 0px;"><p id="fatal_acute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="left: 600px; top: 210px;"><p id="fatal_gbs_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 44px;"><p id="ge_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 253px;"><p id="rea_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 313px;"><p id="asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 89px;"><p id="gbs_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
      createConnection("acute", "rea_1", {}, "delta_1", "All", "delta");
      createConnection("acute", "fatal_acute_1", {}, "beta_1", "All", "beta");
      createConnection("acute", "gbs_mild_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("acute", "gbs_severe_1", {}, "theta_1", "All", "theta");
      createConnection("gbs_severe_1", "fatal_gbs_1", {}, "zeta_1", "All", "zeta");
      createConnection("gbs_severe_1", "gbs_long_term_1", {}, "kappa_1", "All", "kappa");
      createConnection("acute", "ge_recovery", {});
      createConnection("rea_1", "rea_recovery", {});
      createConnection("gbs_title", "gbs_recovery", {});
    }
  },

  // Chlamydia (consistent)
  // 10/09/2014 - updated with Review 2
  // 29/10/2014 - updated with Review 3
  // 01/04/2015 - updated with Review 4
  // 31/07/2015 - fixes in the Female OT, it wasn't updated (email from Alessandro 23/07/2015)
  // 01/03/2016 - Women OT: Ectopic pregnancy and Tubal infertility connected to Asymptomatic infection as well
  Chlamydia: {
    htmlCode:
      '<div id="Congenital" class="outcomeTree" style="margin-bottom: 268px"><div class="outcomeTreeTitle">Congenital</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="c_infection_1" class="healthState">Infection</p></div>' +
      '<div id="c_symptomatic" class="healthOutcome symptomatic" style="left: 200px; top:0px">' +
      '   <div class="tableRow"><p id="c_symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="c_symptomatic_1" class="healthState">Ophthalmia neonatorum</p></div>' +
      '   <div class="tableRow"><p id="c_symptomatic_2" class="healthState">Neonatal Pneumonia</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 170px;"><p id="c_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 183px;"><p id="c_asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 56px;"><p id="c_symptomatic_recovery">R</p></div>' +
      '</div>' +

      '<div id="Women" class="outcomeTree" style="margin-bottom: 345px"><div class="outcomeTreeTitle">Women</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 120px;"><p id="w_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 40px;"><p id="w_symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 220px;"><p id="w_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="left: 450px; top: 40px;"><p id="w_pelvic_1" class="healthState">Pelvic Inflammatory Disease</p></div>' +
      '<div class="healthOutcome sequela" style="left: 450px; top: 220px;"><p id="w_tubal_1" class="healthState">Tubal infertility</p></div>' +
      '<div class="healthOutcome sequela" style="left: 450px; top: 130px;"><p id="w_ectopic_1" class="healthState">Ectopic pregnancy</p></div>' +
      '<div class="healthOutcome sequela" style="left: 650px; top: 0px;"><p id="w_chronic_1" class="healthState">Chronic pelvic pain syndrome</p></div>' +
      '<div class="healthOutcome sequela" style="left: 650px; top: 70px;"><p id="w_tubo_1" class="healthState">Tubo-ovarian abscess</p></div>' +
      '<div class="healthOutcome death" style="left: 650px; top: 130px;"><p id="w_fatal_ectopic_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 780px; top: 3px;"><p id="w_chronic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 580px; top: 233px;"><p id="w_tubal_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 13px;"><p id="w_symptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 263px;"><p id="w_asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 580px; top: 3px;"><p id="w_pelvic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 780px; top: 83px;"><p id="w_tubo_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 580px; top: 173px;"><p id="w_ectopic_recovery">R</p>' +
      '</div>' +
      '</div>' +

      '<div id="Men" class="outcomeTree" style="margin-bottom: 295px"><div class="outcomeTreeTitle">Men</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 130px;"><p id="m_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 130px;"><p id="m_symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 264px;"><p id="m_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="m_epididymitis" class="healthOutcome sequela" style="left: 400px; top: 180px">' +
      '   <div class="tableRow"><p id="m_epididymitis_title" class="healthOutcomeTitle">Epididymitis</p></div>' +
      '   <div class="tableRow"><p id="m_epididymitis_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="m_epididymitis_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +
      '<div id="m_reactive" class="healthOutcome sequela" style="left: 400px; top: 0px">' +
      '   <div class="tableRow"><p id="m_reactive_title" class="healthOutcomeTitle">Reactive Arthritis</p></div>' +
      '   <div class="tableRow"><p id="m_reactive_1" class="healthState">Mild</p></div>' +
      '   <div class="tableRow"><p id="m_reactive_2" class="healthState">Severe</p></div>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 236px;"><p id="m_epididymitis_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 240px;"><p id="m_asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 56px;"><p id="m_reactive_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("c_infection_1", "c_symptomatic", {}, "c_alpha_1", "Congenital", "alpha");
      createConnection("c_infection_1", "c_asymptomatic_1", {}, "c_alpha_2", "Congenital", "1 - alpha");
      createConnection("c_asymptomatic_1", "c_asymptomatic_recovery", {});
      createConnection("c_symptomatic", "c_symptomatic_recovery", {});

      createConnection("w_infection_1", "w_symptomatic_1", {}, "w_alpha_1", "Women", "alpha");
      createConnection("w_infection_1", "w_asymptomatic_1", {}, "w_alpha_2", "Women", "1 - alpha");
      createConnection("w_symptomatic_1", "w_pelvic_1", {}, "w_gamma_1", "Women", "gamma<sub>1</sub>");
      createConnection("w_asymptomatic_1", "w_pelvic_1", { overlays: [["Label", { location: 0.4 }]] }, "w_gamma_2", "Women", "gamma<sub>2</sub>");
      createConnection("w_pelvic_1", "w_tubo_1", {}, "w_delta_1", "Women", "delta");
      createConnection("w_pelvic_1", "w_chronic_1", {}, "w_epsilon_1", "Women", "epsilon");
      createConnection("w_symptomatic_1", "w_ectopic_1", {}, "w_zeta_1", "Women", "zeta<sub>1</sub>");
      createConnection("w_asymptomatic_1", "w_ectopic_1", {}, "w_zeta_2", "Women", "zeta<sub>2</sub>");
      createConnection("w_symptomatic_1", "w_tubal_1", { overlays: [["Label", { location: 0.4 }]] }, "w_eta_1", "Women", "eta<sub>1</sub<");
      createConnection("w_asymptomatic_1", "w_tubal_1", {}, "w_eta_2", "Women", "eta<sub>2</sub<");
      createConnection("w_ectopic_1", "w_fatal_ectopic_1", {}, "w_theta_1", "Women", "theta");
      createConnection("w_symptomatic_1", "w_symptomatic_recovery", {});
      createConnection("w_asymptomatic_1", "w_asymptomatic_recovery", {});
      createConnection("w_chronic_1", "w_chronic_recovery", {});
      createConnection("w_tubal_1", "w_tubal_recovery", {});
      createConnection("w_pelvic_1", "w_pelvic_recovery", {});
      createConnection("w_tubo_1", "w_tubo_recovery", {});
      createConnection("w_ectopic_1", "w_ectopic_recovery", {});

      createConnection("m_infection_1", "m_symptomatic_1", {}, "m_alpha_1", "Men", "alpha");
      createConnection("m_infection_1", "m_asymptomatic_1", {}, "m_alpha_2", "Men", "1 - alpha");
      createConnection("m_symptomatic_1", "m_reactive", {}, "m_delta_1", "Men", "delta");
      createConnection("m_symptomatic_1", "m_epididymitis_1", {}, "m_gamma_1", "Men", "gamma");
      createConnection("m_asymptomatic_1", "m_epididymitis_2", {}, "m_zeta_1", "Men", "zeta");
      createConnection("m_asymptomatic_1", "m_asymptomatic_recovery", {});
      createConnection("m_epididymitis", "m_epididymitis_recovery", {});
      createConnection("m_reactive", "m_reactive_recovery", {});
    }
  },

  // Cryptosporidiosis (consistent)
  // 01/11/2014 - updated with Review 2
  Cryptosporidiosis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 240px">' +
      '<div class="healthOutcome infection" style="top:43px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="left: 200px; top:0px">' +
      '   <div class="tableRow"><p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="top:190px; left: 200px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left:400px; top:43px"><p id="fatal_acute_severe" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 203px;"><p id="asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 20px;"><p id="symptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic", "fatal_acute_severe", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
      createConnection("symptomatic", "symptomatic_recovery", {});
    }
  },

  // Giardiasis (consistent)
  // 31/10/2014 - updated with Review 2
  // 20/09/2015 - fatal cases added (email 11/09/2015)
  // 28/09/2015 - fatal cases from all symptomatic cases (email 28/09/2015)
  Giardiasis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 220px">' +
      '<div class="healthOutcome infection" style="top:43px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="left: 200px; top:0px">' +
      '   <div class="tableRow"><p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="top:170px; left: 200px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 183px;"><p id="asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 19px;"><p id="symptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 43px;"><p id="fatalSymptomatic_1" class="healthState">Fatal cases</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
      createConnection("symptomatic", "symptomatic_recovery", {});
    }
  },

  // Hepatitis A (consistent)
  // 31/10/2014 - updated with Review 2
  HepatitisA: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 150px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 0px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 0px;"><p id="symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 100px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 0px;"><p id="fatal_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 40px;"><p id="symptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 113px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatal_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Shigellosis (consistent)
  // 31/10/2014 - updated with Review 2
  Shigellosis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 515px">' +
      '<div class="healthOutcome infection" style="top:285px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="gastroenteritis" class="healthOutcome symptomatic" style="left: 160px; top: 243px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="gastroenteritis_1" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="gastroenteritis_2" class="healthState">Uncomplicated</p></div>' +
      '</div>' +

      '<div class="healthOutcome asymptomatic" style="top:465px; left: 160px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="top:284px; left: 330px"><p id="hus_1" class="healthState">Hemolytic Uremic syndrome</p></div>' +
      '<div class="healthOutcome sequela" style="left: 480px; top:150px;"><p id="latency_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome sequela" style="left: 620px; top: 200px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">ESRD</p></div>' +
      '   <div class="tableRow"><p id="esrd_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="top:80px; left: 330px"><p id="rea_1" class="healthState">Reactive arthritis</p></div>' +

      '<div class="healthOutcome sequela" style="left: 910px; top: 240px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Transplanted (remaining of transplantation)</p></div>' +
      '   <div class="tableRow"><p id="transplanted_trans_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="transplanted_trans_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="left: 790px; top: 370px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Transplanted (remaining of dialysis)</p></div>' +
      '   <div class="tableRow"><p id="transplanted_dial_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="transplanted_dial_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="left: 790px; top: 0px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases (transplan- tation)</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_trans_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_trans_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="left: 910px; top: 80px;">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases (dialysis)</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_dial_latency" class="healthState">20 years latency</p></div>' +
      '   <div class="tableRow"><p id="esrd_fatal_dial_no_latency" class="healthState">No latency</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="top:465px; left: 510px"><p id="fatalHUS_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="top:185px; left: 330px"><p id="fatalGastroenteritis_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="top:335px; left: 290px"><p id="gastroenteritis_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="top:478px; left: 290px"><p id="asymptomatic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="top:265px; left: 460px"><p id="hus_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="top:93px; left: 460px"><p id="rea_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "gastroenteritis", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("gastroenteritis", "hus_1", {}, "gamma_1", "All", "gamma");
      createConnection("hus_1", "fatalHUS_1", {}, "delta_1", "All", "delta");
      createConnection("hus_1", "latency_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("latency_1", "esrd_latency", {}, "eta_1", "All", "eta");
      createConnection("hus_1", "esrd_no_latency", {}, "zeta_1", "All", "zeta");

      createConnection("esrd_latency", "esrd_fatal_trans_latency", {}, "kappa_1", "All", "kappa<sub>1</sub>");
      createConnection("esrd_latency", "transplanted_trans_latency", {}, "kappa_3", "All", "1 - kappa<sub>1</sub>");

      createConnection("esrd_no_latency", "esrd_fatal_trans_no_latency", {}, "kappa_2", "All", "kappa<sub>2</sub>");
      createConnection("esrd_no_latency", "transplanted_trans_no_latency", {}, "kappa_4", "All", "1 - kappa<sub>2</sub>");

      createConnection("esrd_latency", "esrd_fatal_dial_latency", {}, "lambda_1", "All", "lambda<sub>1</sub>");
      createConnection("esrd_latency", "transplanted_dial_latency", {}, "lambda_3", "All", "1 - lambda<sub>1</sub>");

      createConnection("esrd_no_latency", "esrd_fatal_dial_no_latency", {}, "lambda_2", "All", "lambda<sub>2</sub>");
      createConnection("esrd_no_latency", "transplanted_dial_no_latency", {}, "lambda_4", "All", "1 - lambda<sub>2</sub>");

      createConnection("gastroenteritis_1", "rea_1", {}, "theta_1", "All", "theta");

      createConnection("gastroenteritis_1", "fatalGastroenteritis_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
      createConnection("gastroenteritis", "gastroenteritis_recovery", {});
      createConnection("hus_1", "hus_recovery", {});
      createConnection("rea_1", "rea_recovery", {});
    }
  },

  // Tuberculosis (consistent)
  // 02/11/2014 - updated with Review 2
  // 03/04/2015 - updated with Review 3
  Tuberculosis: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 270px">' +
      '<div class="healthOutcome infection" style="top:66px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_1" class="healthState">Active TB</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_2" class="healthState">MDR TB</p></div>' +
      '   <div class="tableRow"><p id="symptomatic_3" class="healthState">XDR TB</p></div>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="top:220px; left: 200px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="top:0px; left: 400px">' +
      '   <div class="tableRow"><p id="fatal_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatal_symptomatic_1" class="healthState">following<br />Active TB</p></div>' +
      '   <div class="tableRow"><p id="fatal_symptomatic_2" class="healthState">following<br />MDR TB</p></div>' +
      '   <div class="tableRow"><p id="fatal_symptomatic_3" class="healthState">following<br />XDR TB</p></div>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="top:9px; left: 330px"><p id="symptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatal_symptomatic_1", {}, "beta_1", "All", "beta<sub>1</sub>");
      createConnection("symptomatic_2", "fatal_symptomatic_2", {}, "beta_2", "All", "beta<sub>2</sub>");
      createConnection("symptomatic_3", "fatal_symptomatic_3", {}, "beta_3", "All", "beta<sub>3</sub>");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
    }
  },

  // Hepatitis C (consistent)
  // 13/09/2014 - updated with Review 2
  // 29/10/2014 - updated with Review 3
  HepatitisC: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 340px">' +
      '<div class="healthOutcome infection" style="top:119px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:0px; left: 200px"><p id="symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="top:290px; left: 200px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="top:119px; left: 200px"><p id="chronic_1" class="healthState">Chronic hepatitis</p></div>' +
      '<div class="healthOutcome sequela" style="top:119px; left: 400px"><p id="compensated_1" class="healthState">Compensated cirrhosis</p></div>' +
      '<div class="healthOutcome sequela" style="top:290px; left: 600px"><p id="decompensated_1" class="healthState">Decompensated cirrhosis</p></div>' +
      '<div class="healthOutcome sequela" style="top:82px; left: 600px">' +
      '   <div class="tableRow"><p id="hepatocellular" class="healthOutcomeTitle">Hepatocellular carcinoma</p></div>' +
      '   <div class="tableRow"><p id="hepatocellular_1" class="healthState">following<br />Compensated cirrhosis</p></div>' +
      '   <div class="tableRow"><p id="hepatocellular_2" class="healthState">following<br />Decompensated cirrhosis</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:82px; left: 800px">' +
      '   <div class="tableRow"><p class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatalHepatocellular_1" class="healthState">following<br />Compensated cirrhosis</p></div>' +
      '   <div class="tableRow"><p id="fatalHepatocellular_2" class="healthState">following<br />Decompensated cirrhosis</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:290px; left: 800px"><p id="fatalDecompensated_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 350px; top: 210px;"><p id="chronic_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 270px;"><p id="decompensated_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 91px;"><p id="hepatocellular_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "chronic_1", { anchors: ["BottomCenter", "TopCenter"] }, "delta_1", "All", "delta<sub>1</sub>");
      createConnection("asymptomatic_1", "chronic_1", { anchors: ["TopCenter", "BottomCenter"] }, "delta_2", "All", "delta<sub>2</sub>");
      createConnection("chronic_1", "compensated_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("compensated_1", "decompensated_1", {}, "zeta_1", "All", "zeta");
      createConnection("compensated_1", "hepatocellular_1", {}, "theta_1", "All", "theta");
      createConnection("decompensated_1", "hepatocellular_2", { anchors: ["TopCenter", "BottomCenter"] }, "eta_1", "All", "eta");
      createConnection("hepatocellular_1", "fatalHepatocellular_1", {}, "kappa_1", "All", "kappa<sub>1</sub>");
      createConnection("hepatocellular_2", "fatalHepatocellular_2", {}, "kappa_2", "All", "kappa<sub>2</sub>");
      createConnection("decompensated_1", "fatalDecompensated_1", {}, "iota_1", "All", "iota");
      createConnection("chronic_1", "chronic_recovery", {}, "mu_1", "All", "mu");
      createConnection("hepatocellular", "hepatocellular_recovery", {});
      createConnection("decompensated_1", "decompensated_recovery", {});
    }
  },

  // HIV (consistent)
  // 15/09/2014 - updated with Review 2
  HIV: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 473px">' +
      '<div class="healthOutcome infection" style="top:95px; left: 0px"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:0px; left: 160px"><p id="symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="top:200px; left: 160px"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +

      '<div id="persistant" class="healthOutcome sequela" style="top:40px; left: 320px">' +
      '   <div class="tableRow"><p id="persistent_title" class="healthOutcomeTitle">Persistent HIV infection</p></div>' +
      '   <div class="tableRow"><p id="persistent_1" class="healthState">Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="persistent_2" class="healthState">Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="persistent_3" class="healthState">Long-term non-progressors</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:40px; left: 500px">' +
      '   <div class="tableRow"><p id="aids_early_title" class="healthOutcomeTitle">AIDS early symptomatic</p></div>' +
      '   <div class="tableRow"><p id="aids_early_1" class="healthState">following<br />Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_early_2" class="healthState">following<br />Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_early_3" class="healthState">following<br />Long-term non-progressors</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:40px; left: 680px">' +
      '   <div class="tableRow"><p id="aids_terminal_title" class="healthOutcomeTitle">AIDS terminal phase</p></div>' +
      '   <div class="tableRow"><p id="aids_terminal_1" class="healthState">following<br />Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_terminal_2" class="healthState">following<br />Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_terminal_3" class="healthState">following<br />Long-term non-progressors</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="top:40px; left: 860px">' +
      '   <div class="tableRow"><p id="aids_fatal_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="aids_fatal_1" class="healthState">following<br />Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_fatal_2" class="healthState">following<br />Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="aids_fatal_3" class="healthState">following<br />Long-term non-progressors</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:290px; left: 500px">' +
      '   <div class="tableRow"><p id="arv_title_1" class="healthOutcomeTitle">Permanent ARV treatment</p></div>' +
      '   <div class="tableRow"><p id="arv_1" class="healthState">following<br />Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="arv_2" class="healthState">following<br />Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="arv_3" class="healthState">following<br />Long-term non-progressors</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:290px; left: 680px">' +
      '   <div class="tableRow"><p id="arv_title_2" class="healthOutcomeTitle">Permanent ARV treatment</p></div>' +
      '   <div class="tableRow"><p id="arv_4" class="healthState">following<br />Rapid progressors</p></div>' +
      '   <div class="tableRow"><p id="arv_5" class="healthState">following<br />Typical progressors</p></div>' +
      '   <div class="tableRow"><p id="arv_6" class="healthState">following<br />Long-term non-progressors</p></div>' +
      '</div>' +

      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "persistant", {}, "gamma_1", "All", "gamma<sub>1</sub>");
      createConnection("asymptomatic_1", "persistant", {}, "gamma_2", "All", "gamma<sub>2</sub>");
      createConnection("persistent_1", "aids_early_1", {}, "delta_1", "All", "delta<sub>1</sub>");
      createConnection("persistent_2", "aids_early_2", {}, "delta_2", "All", "delta<sub>2</sub>");
      createConnection("persistent_3", "aids_early_3", {}, "delta_3", "All", "delta<sub>3</sub>");
      createConnection("aids_early_1", "aids_terminal_1", {}, "epsilon_1", "All", "epsilon<sub>1</sub>");
      createConnection("aids_early_2", "aids_terminal_2", {}, "epsilon_2", "All", "epsilon<sub>2</sub>");
      createConnection("aids_early_3", "aids_terminal_3", {}, "epsilon_3", "All", "epsilon<sub>3</sub>");
      createConnection("aids_terminal_1", "aids_fatal_1", {}, "beta_1", "All", "beta<sub>1</sub>");
      createConnection("aids_terminal_2", "aids_fatal_2", {}, "beta_2", "All", "beta<sub>2</sub>");
      createConnection("aids_terminal_3", "aids_fatal_3", {}, "beta_3", "All", "beta<sub>3</sub>");
      createConnection("persistent_1", "arv_1", {}, "delta_4", "All", "1 - delta<sub>1</sub>");
      createConnection("persistent_2", "arv_2", {}, "delta_5", "All", "1 - delta<sub>2</sub>");
      createConnection("persistent_3", "arv_3", {}, "delta_6", "All", "1 - delta<sub>3</sub>");
      createConnection("aids_early_1", "arv_4", {}, "epsilon_4", "All", "1 - epsilon<sub>1</sub>");
      createConnection("aids_early_2", "arv_5", {}, "epsilon_5", "All", "1 - epsilon<sub>2</sub>");
      createConnection("aids_early_3", "arv_6", {}, "epsilon_6", "All", "1 - epsilon<sub>3</sub>");
    }
  },

  // InvasivePneumococcal (consistent)
  // 11/10/2014 - updated with Review 2
  // 30/10/2014 - updated with Review 3
  InvasivePneumococcal: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 470px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 150px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 420px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 150px;"><p id="invasive_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 420px;"><p id="fatalInvasive_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 0px;"><p id="impairments_1" class="healthState">Multiple impairments</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 60px;"><p id="cognitive_1" class="healthState">Cognitive difficulties</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 120px;"><p id="seizure_1" class="healthState">Seizure disorder</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 180px;"><p id="hearing_1" class="healthState">Hearing loss</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 240px;"><p id="motor_1" class="healthState">Motor deficit</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 300px;"><p id="visual_1" class="healthState">Visual disturbance</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 360px;"><p id="behavioural_1" class="healthState">Behavioural problems</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 420px;"><p id="clinical_1" class="healthState">Clinical impairments</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 110px;"><p id="invasive_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 433px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "invasive_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("invasive_1", "fatalInvasive_1", {}, "beta_1", "All", "beta");
      createConnection("invasive_1", "impairments_1", {}, "gamma_1", "All", "gamma");
      createConnection("invasive_1", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("invasive_1", "seizure_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("invasive_1", "hearing_1", {}, "zeta_1", "All", "zeta");
      createConnection("invasive_1", "motor_1", {}, "eta_1", "All", "eta");
      createConnection("invasive_1", "visual_1", {}, "theta_1", "All", "theta");
      createConnection("invasive_1", "behavioural_1", {}, "iota_1", "All", "iota");
      createConnection("invasive_1", "clinical_1", {}, "kappa_1", "All", "kappa");
      createConnection("invasive_1", "invasive_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }

  },

  // InvasiveMeningococcal (consistent)
  // 11/10/2014 - updated with Review 2
  // 30/10/2014 - updated with Review 3
  InvasiveMeningococcal: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 470px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 150px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 420px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 150px;"><p id="invasive_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 420px;"><p id="fatalInvasive_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 0px;"><p id="impairments_1" class="healthState">Multiple impairments</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 60px;"><p id="cognitive_1" class="healthState">Cognitive difficulties</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 120px;"><p id="seizure_1" class="healthState">Seizure disorder</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 180px;"><p id="hearing_1" class="healthState">Hearing loss</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 240px;"><p id="motor_1" class="healthState">Motor deficit</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 300px;"><p id="visual_1" class="healthState">Visual disturbance</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 360px;"><p id="behavioural_1" class="healthState">Behavioural problems</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 420px;"><p id="clinical_1" class="healthState">Clinical impairments</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 110px;"><p id="invasive_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 433px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "invasive_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("invasive_1", "fatalInvasive_1", {}, "beta_1", "All", "beta");
      createConnection("invasive_1", "impairments_1", {}, "gamma_1", "All", "gamma");
      createConnection("invasive_1", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("invasive_1", "seizure_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("invasive_1", "hearing_1", {}, "zeta_1", "All", "zeta");
      createConnection("invasive_1", "motor_1", {}, "eta_1", "All", "eta");
      createConnection("invasive_1", "visual_1", {}, "theta_1", "All", "theta");
      createConnection("invasive_1", "behavioural_1", {}, "iota_1", "All", "iota");
      createConnection("invasive_1", "clinical_1", {}, "kappa_1", "All", "kappa");
      createConnection("invasive_1", "invasive_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // InvasiveHInfluenzae (consistent)
  // 10/10/2014 - updated with Review 2
  // 30/10/2014 - updated with Review 3
  InvasiveHInfluenzae: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 470px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 150px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 420px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 150px;"><p id="invasive_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 420px;"><p id="fatalInvasive_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 0px;"><p id="impairments_1" class="healthState">Multiple impairments</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 60px;"><p id="cognitive_1" class="healthState">Cognitive difficulties</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 120px;"><p id="seizure_1" class="healthState">Seizure disorder</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 180px;"><p id="hearing_1" class="healthState">Hearing loss</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 240px;"><p id="motor_1" class="healthState">Motor deficit</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 300px;"><p id="visual_1" class="healthState">Visual disturbance</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 360px;"><p id="behavioural_1" class="healthState">Behavioural problems</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 420px;"><p id="clinical_1" class="healthState">Clinical impairments</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 110px;"><p id="invasive_recovery">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 433px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "invasive_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("invasive_1", "fatalInvasive_1", {}, "beta_1", "All", "beta");
      createConnection("invasive_1", "impairments_1", {}, "gamma_1", "All", "gamma");
      createConnection("invasive_1", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("invasive_1", "seizure_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("invasive_1", "hearing_1", {}, "zeta_1", "All", "zeta");
      createConnection("invasive_1", "motor_1", {}, "eta_1", "All", "eta");
      createConnection("invasive_1", "visual_1", {}, "theta_1", "All", "theta");
      createConnection("invasive_1", "behavioural_1", {}, "iota_1", "All", "iota");
      createConnection("invasive_1", "clinical_1", {}, "kappa_1", "All", "kappa");
      createConnection("invasive_1", "invasive_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Legionnaires' disease (consistent)
  // 27/09/2014 - updated with Review 2
  LegionnairesDisease: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 270px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 67px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 220px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="legionellosis" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="legionellosis_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="legionellosis_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="legionellosis_2" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="legionellosis_3" class="healthState">Complicated (ICU)</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:0px; left: 400px">' +
      '   <div class="tableRow"><p id="fatal_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatal_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="fatal_2" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="fatal_3" class="healthState">Complicated (ICU)</p></div>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 233px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="symptomatic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "legionellosis", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("legionellosis_1", "fatal_1", {}, "beta_1", "All", "beta<sub>1</sub>");
      createConnection("legionellosis_2", "fatal_2", {}, "beta_2", "All", "beta<sub>2</sub>");
      createConnection("legionellosis_3", "fatal_3", {}, "beta_3", "All", "beta<sub>3</sub>");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("legionellosis_title", "symptomatic_recovery_1", {});
    }
  },

  // Gonorrhoea (consistent)
  // 27/09/2014 - updated with Review 2
  // 03/04/2015 - updated with Review 3
  Gonorrhoea: {
    htmlCode:
      '<div id="Congenital" class="outcomeTree" style="margin-bottom: 228px"><div class="outcomeTreeTitle">Congenital</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 70px;"><p id="c_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 0px;"><p id="c_symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 130px;"><p id="c_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 143px"><p id="c_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 13px"><p id="c_symptomatic_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Women" class="outcomeTree" style="margin-bottom: 338px"><div class="outcomeTreeTitle">Women</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 130px;"><p id="w_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 240px;"><p id="w_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="w_symptomatic" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="w_symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="w_symptomatic_1" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="w_symptomatic_2" class="healthState">Uncomplicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 130px;"><p id="w_pelvic_1" class="healthState">Pelvic Inflammatory Disease</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 240px;"><p id="w_tubal_1" class="healthState">Tubal infertility</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 0px;"><p id="w_chronic_1" class="healthState">Chronic pelvic pain syndrome</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 80px;"><p id="w_tubo_1" class="healthState">Tubo-ovarian abscess</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 160px;"><p id="w_ectopic_1" class="healthState">Ectopic pregnancy</p></div>' +
      '<div class="healthOutcome death" style="left: 800px; top: 160px;"><p id="w_fatalEctopic_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 203px"><p id="w_ectopic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 93px"><p id="w_tubo_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 253px"><p id="w_tubal_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 13px"><p id="w_chronic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="w_symptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 43px"><p id="w_pelvic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 253px"><p id="w_asymptomatic_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Men" class="outcomeTree" style="margin-bottom: 208px"><div class="outcomeTreeTitle">Men</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 100px;"><p id="m_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="m_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="m_symptomatic" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="m_symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="m_symptomatic_1" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="m_symptomatic_2" class="healthState">Uncomplicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 43px;"><p id="m_epididymitis_1" class="healthState">Epididymitis</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="m_symptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="m_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 56px"><p id="m_epididymitis_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("c_infection_1", "c_symptomatic_1", {}, "c_alpha_1", "Congenital", "alpha");
      createConnection("c_infection_1", "c_asymptomatic_1", {}, "c_alpha_2", "Congenital", "1 - alpha");
      createConnection("c_asymptomatic_1", "c_asymptomatic_recovery_1", {});
      createConnection("c_symptomatic_1", "c_symptomatic_recovery_1", {});

      createConnection("w_infection_1", "w_symptomatic", {}, "w_alpha_1", "Women", "alpha");
      createConnection("w_infection_1", "w_asymptomatic_1", {}, "w_alpha_2", "Women", "1 - alpha");
      createConnection("w_symptomatic", "w_pelvic_1", {}, "w_gamma_1", "Women", "gamma<sub>1</sub>");
      createConnection("w_asymptomatic_1", "w_pelvic_1", {}, "w_gamma_2", "Women", "gamma<sub>2</sub>");
      createConnection("w_pelvic_1", "w_ectopic_1", {}, "w_delta_1", "Women", "delta");
      createConnection("w_pelvic_1", "w_tubo_1", {}, "w_epsilon_1", "Women", "epsilon");
      createConnection("w_pelvic_1", "w_chronic_1", {}, "w_zeta_1", "Women", "zeta");
      createConnection("w_pelvic_1", "w_tubal_1", {}, "w_eta_1", "Women", "eta");
      createConnection("w_ectopic_1", "w_fatalEctopic_1", {}, "w_lambda_1", "Women", "lambda");
      createConnection("w_tubal_1", "w_tubal_recovery_1", {});
      createConnection("w_chronic_1", "w_chronic_recovery_1", {});
      createConnection("w_tubo_1", "w_tubo_recovery_1", {});
      createConnection("w_ectopic_1", "w_ectopic_recovery_1", {});
      createConnection("w_symptomatic_title", "w_symptomatic_recovery_1", {});
      createConnection("w_pelvic_1", "w_pelvic_recovery_1", {});
      createConnection("w_asymptomatic_1", "w_asymptomatic_recovery_1", {});

      createConnection("m_infection_1", "m_symptomatic", {}, "m_alpha_1", "Men", "alpha");
      createConnection("m_infection_1", "m_asymptomatic_1", {}, "m_alpha_2", "Men", "1 - alpha");
      createConnection("m_symptomatic", "m_epididymitis_1", {}, "m_gamma_1", "Men", "gamma");
      createConnection("m_asymptomatic_1", "m_asymptomatic_recovery_1", {});
      createConnection("m_epididymitis_1", "m_epididymitis_recovery_1", {});
      createConnection("m_symptomatic_title", "m_symptomatic_recovery_1", {});
    }
  },

  // Syphilis (consistent)
  // 26/09/2014 - updated with Review 2
  // 29/10/2014 - updated with Review 3
  Syphilis: {
    htmlCode:
      '<div id="Congenital" class="outcomeTree" style="margin-bottom: 230px"><div class="outcomeTreeTitle">Congenital</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 30px;"><p id="c_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 130px;"><p id="c_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:30px; left: 200px"><p id="c_symptomatic_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="top:30px; left: 400px"><p id="c_fatalSymptomatic_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 143px"><p id="c_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 10px"><p id="c_symptomatic_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Adults" class="outcomeTree" style="margin-bottom: 434px"><div class="outcomeTreeTitle">Adults</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 80px;"><p id="a_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 160px;"><p id="a_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:0px; left: 200px"><p id="a_symptomatic_1" class="healthState">Symptomatic infection</p></div>' +

      '<div class="healthOutcome sequela" style="top:20px; left: 400px">' +
      '   <div class="tableRow"><p id="a_latency_1_title" class="healthOutcomeTitle">Latency period</p></div>' +
      '   <div class="tableRow"><p id="a_latency_1_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="a_latency_1_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:320px; left: 30px">' +
      '   <div class="tableRow"><p id="a_secondary_title" class="healthOutcomeTitle">Secondary Syphilis</p></div>' +
      '   <div class="tableRow"><p id="a_secondary_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="a_secondary_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:320px; left: 200px">' +
      '   <div class="tableRow"><p id="a_latency_2_title" class="healthOutcomeTitle">Latency period</p></div>' +
      '   <div class="tableRow"><p id="a_latency_2_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="a_latency_2_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +

      '<div class="healthOutcome sequela" style="top:320px; left: 400px">' +
      '   <div class="tableRow"><p id="a_neurosyphilis_title" class="healthOutcomeTitle">Neuro-Syphilis</p></div>' +
      '   <div class="tableRow"><p id="a_neurosyphilis_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="a_neurosyphilis_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="top:320px; left: 600px">' +
      '   <div class="tableRow"><p id="a_fatalSymptomatic_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="a_fatalSymptomatic_1" class="healthState">following Symptomatic</p></div>' +
      '   <div class="tableRow"><p id="a_fatalSymptomatic_2" class="healthState">following Asymptomatic</p></div>' +
      '</div>' +

      '<div class="healthOutcome recovery" style="left: 330px; top: 173px"><p id="a_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 329px"><p id="a_neurosyphilis_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 160px; top: 329px"><p id="a_secondary_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 3px"><p id="a_symptomatic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("c_infection_1", "c_symptomatic_1", {}, "c_alpha_1", "Congenital", "alpha");
      createConnection("c_infection_1", "c_asymptomatic_1", {}, "c_alpha_2", "Congenital", "1 - alpha");
      createConnection("c_symptomatic_1", "c_fatalSymptomatic_1", {}, "c_beta_1", "Congenital", "beta");
      createConnection("c_asymptomatic_1", "c_asymptomatic_recovery_1", {});
      createConnection("c_symptomatic_1", "c_symptomatic_recovery_1", {});

      createConnection("a_infection_1", "a_symptomatic_1", {}, "a_alpha_1", "Adults", "alpha");
      createConnection("a_infection_1", "a_asymptomatic_1", {}, "a_alpha_2", "Adults", "1 - alpha");
      createConnection("a_symptomatic_1", "a_latency_1_1", {}, "a_gamma_1", "Adults", "gamma");
      createConnection("a_asymptomatic_1", "a_latency_1_2", {}, "a_delta_1", "Adults", "delta");
      createConnection("a_latency_1_1", "a_secondary_1", { connector: ["Flowchart"], paintStyle: { lineWidth: 1, strokeStyle: 'red' } }, "a_lambda_1", "Adults", "lambda<sub>1</sub>");
      createConnection("a_latency_1_2", "a_secondary_2", { connector: ["Flowchart"], paintStyle: { lineWidth: 1, strokeStyle: 'blue' } }, "a_lambda_2", "Adults", "lambda<sub>2</sub>");
      createConnection("a_secondary_1", "a_latency_2_1", {}, "a_eta_1", "Adults", "eta<sub>1</sub>");
      createConnection("a_secondary_2", "a_latency_2_2", {}, "a_eta_2", "Adults", "eta<sub>2</sub>");
      createConnection("a_latency_2_1", "a_neurosyphilis_1", {}, "a_iota_1", "Adults", "iota<sub>1</sub>");
      createConnection("a_latency_2_2", "a_neurosyphilis_2", {}, "a_iota_2", "Adults", "iota<sub>2</sub>");
      createConnection("a_neurosyphilis_1", "a_fatalSymptomatic_1", {}, "a_beta_1", "Adults", "beta<sub>1</sub>");
      createConnection("a_neurosyphilis_2", "a_fatalSymptomatic_2", {}, "a_beta_2", "Adults", "beta<sub>2</sub>");
      createConnection("a_asymptomatic_1", "a_asymptomatic_recovery_1", {});
      createConnection("a_symptomatic_1", "a_symptomatic_recovery_1", {});
      createConnection("a_secondary_title", "a_secondary_recovery_1", {});
      createConnection("a_neurosyphilis_title", "a_neurosyphilis_recovery_1", {});
    }
  },

  // Mumps (consistent)
  // 30/10/2014 - updated with Review 2
  Mumps: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 230px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="infection_1" class="healthState">Infections</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 90px;"><p id="fatalEncephalitis_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 0px;"><p id="hearingLoss_1" class="healthState">Hearing loss</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 0px"><p id="acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute", "hearingLoss_1", {}, "gamma_1", "All", "gamma");
      createConnection("acute", "fatalEncephalitis_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("acute", "acute_recovery_1", {});
    }
  },

  // Pertussis (consistent)
  // 30/10/2014 - updated with Review 2
  // 04/11/2014 - updated with Review 3
  Pertussis: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 230px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="infection_1" class="healthState">Infections</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Complicated</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="left: 400px; top: 84px;"><p id="fatal_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_2", "fatal_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("acute_title", "acute_recovery_1", {});
    }
  },

  // Diphtheria (consistent)
  // 06/10/2014 - updated with Review 3
  Diphtheria: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 234px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 180px;"><p id="arrhythmias_1" class="healthState">Permanent disability due to myocarditis (arrhythmias)</p></div>' +
      '<div class="healthOutcome death" style="top:0px; left: 400px">' +
      '   <div class="tableRow"><p id="fatalAcute_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatalAcute_1" class="healthState">following Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="fatalAcute_2" class="healthState">following Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_1", "fatalAcute_1", {}, "beta_1", "All", "beta<sub>1</sub>");
      createConnection("acute_2", "fatalAcute_2", {}, "beta_2", "All", "beta<sub>2</sub>");
      createConnection("acute_2", "arrhythmias_1", {}, "zeta_1", "All", "zeta");
      createConnection("acute_title", "acute_recovery_1", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
    }
  },

  // Rabies (consistent)
  // 29/10/2014 - updated with Review 2
  Rabies: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 136px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Furious form</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Paralytic form</p></div>' +
      '</div>' +

      '<div class="healthOutcome death" style="top:0px; left: 400px">' +
      '   <div class="tableRow"><p id="fatalAcute_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="fatalAcute_1" class="healthState">following<br />Furious form</p></div>' +
      '   <div class="tableRow"><p id="fatalAcute_2" class="healthState">following<br />Paralytic form</p></div>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("acute_1", "fatalAcute_1", {}, "beta_1", "All", "beta<sub>1</sub>");
      createConnection("acute_2", "fatalAcute_2", {}, "beta_2", "All", "beta<sub>2</sub>");
    }
  },

  // Rubella (consistent)
  // 30/10/2014 - updated with Review 2
  Rubella: {
    htmlCode:
      '<div id="Congenital" class="outcomeTree" style="margin-bottom: 500px"><div class="outcomeTreeTitle">Congenital</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 140px;"><p id="c_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 250px;"><p id="c_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 140px;"><p id="c_acute_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 0px;"><p id="c_hear_1" class="healthState">Permanent disability due to hearing impairment</p></div>' +
      '<div class="healthOutcome sequela" style="left: 530px; top: 30px;"><p id="c_heart_1" class="healthState">Permanent disability due to congenital heart defects</p></div>' +
      '<div class="healthOutcome sequela" style="left: 660px; top: 60px;"><p id="c_microephaly_1" class="healthState">Permanent disability due to microephaly</p></div>' +
      '<div class="healthOutcome sequela" style="left: 790px; top: 100px;"><p id="c_cataracts_1" class="healthState">Permanent disability due to cataracts</p></div>' +
      '<div class="healthOutcome sequela" style="left: 790px; top: 170px;"><p id="c_mental_1" class="healthState">Permanent disability due to mental retardation</p></div>' +
      '<div class="healthOutcome sequela" style="left: 660px; top: 210px;"><p id="c_retinopathy_1" class="healthState">Permanent disability due to retinopathy</p></div>' +
      '<div class="healthOutcome sequela" style="left: 460px; top: 270px;"><p id="c_latency_1_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome sequela" style="left: 430px; top: 330px;"><p id="c_latency_2_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome sequela" style="left: 630px; top: 270px;"><p id="c_insulin_1" class="healthState">Permanent disability due to diabetes</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 328px;"><p id="c_thyroid_1" class="healthState">Permanent disability due to thyroid gland dysfunction</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 390px;"><p id="c_fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 48px"><p id="c_acute_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 310px; top: 263px"><p id="c_asymptomatic_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Acquired" class="outcomeTree" style="margin-bottom: 288px"><div class="outcomeTreeTitle">Acquired</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 90px;"><p id="a_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 260px;"><p id="a_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="a_acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="a_acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="a_acute_1" class="healthState">Arthritis/ Arthralgia</p></div>' +
      '   <div class="tableRow"><p id="a_acute_2" class="healthState">Thrombocyto- <br />penic purpura</p></div>' +
      '   <div class="tableRow"><p id="a_acute_3" class="healthState">Encephalitis</p></div>' +
      '   <div class="tableRow"><p id="a_acute_4" class="healthState">Uncomplicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 47px;">' +
      '   <div class="tableRow"><p id="a_fatalAcute_title" class="healthOutcomeTitle">Fatal cases</p></div>' +
      '   <div class="tableRow"><p id="a_fatalAcute_1" class="healthState">following<br />Thrombocyto- <br />penic purpura</p></div>' +
      '   <div class="tableRow"><p id="a_fatalAcute_2" class="healthState">following<br />Encephalitis</p></div>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top:9px"><p id="a_acute_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 273px"><p id="a_asymptomatic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("c_infection_1", "c_acute_1", {}, "c_alpha_1", "Congenital", "alpha");
      createConnection("c_infection_1", "c_asymptomatic_1", {}, "c_alpha_2", "Congenital", "1 - alpha");
      createConnection("c_acute_1", "c_hear_1", {}, "c_delta_1", "Congenital", "delta");
      createConnection("c_acute_1", "c_heart_1", {}, "c_gamma_1", "Congenital", "gamma");
      createConnection("c_acute_1", "c_microephaly_1", {}, "c_theta_1", "Congenital", "theta");
      createConnection("c_acute_1", "c_cataracts_1", {}, "c_eta_1", "Congenital", "eta");
      createConnection("c_acute_1", "c_mental_1", {}, "c_epsilon_1", "Congenital", "epsilon");
      createConnection("c_acute_1", "c_retinopathy_1", {}, "c_iota_1", "Congenital", "iota");
      createConnection("c_acute_1", "c_latency_1_1", {}, "c_lambda_1", "Congenital", "lambda");
      createConnection("c_acute_1", "c_latency_2_1", {}, "c_mu_1", "Congenital", "mu");
      createConnection("c_latency_1_1", "c_insulin_1", {}, "c_kappa_1", "Congenital", "kappa");
      createConnection("c_latency_2_1", "c_thyroid_1", {}, "c_zeta_1", "Congenital", "zeta");
      createConnection("c_acute_1", "c_fatalAcute_1", {}, "c_beta_1", "Congenital", "beta");
      createConnection("c_acute_1", "c_acute_recovery_1", {});
      createConnection("c_asymptomatic_1", "c_asymptomatic_recovery_1", {});

      createConnection("a_infection_1", "a_acute", {}, "a_alpha_1", "Acquired", "alpha");
      createConnection("a_infection_1", "a_asymptomatic_1", {}, "a_alpha_2", "Acquired", "1 - alpha");
      createConnection("a_acute_2", "a_fatalAcute_1", {}, "a_beta_1", "Acquired", "beta<sub>1</sub>");
      createConnection("a_acute_3", "a_fatalAcute_2", {}, "a_beta_2", "Acquired", "beta<sub>2</sub>");
      createConnection("a_acute_title", "a_acute_recovery_1", {});
      createConnection("a_asymptomatic_1", "a_asymptomatic_recovery_1", {});
    }
  },

  // Tetanus (consistent)
  // 30/10/2014 - updated with Review 2
  Tetanus: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 270px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 67px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 220px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Cephalic tetanus</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Localised tetanus</p></div>' +
      '   <div class="tableRow"><p id="acute_3" class="healthState">Generalized tetanus</p></div>' +
      '</div>' +
      '<div class="healthOutcome death" style="top:67px; left: 400px"><p id="fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 233px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute", "fatalAcute_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("acute_title", "acute_recovery_1", {});
    }
  },

  // Listeriosis (consistent)
  // 31/10/2014 - updated with Review 2
  // 20/09/2015 - fatal cases linked with complicated cases only (email 11/09/2015)
  Listeriosis: {
    htmlCode:
      '<div id="Perinatal" class="outcomeTree" style="margin-bottom: 240px"><div class="outcomeTreeTitle">Perinatal</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 45px;"><p id="p_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 140px;"><p id="p_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="top:45px; left: 200px"><p id="p_acute_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="top:0px; left: 400px"><p id="p_meningitis_1" class="healthState">Permanent disability due to meningitis</p></div>' +
      '<div class="healthOutcome death" style="top:90px; left: 400px"><p id="p_fatal_acute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 153px"><p id="p_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 0px"><p id="p_acute_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Acquired" class="outcomeTree" style="margin-bottom: 208px"><div class="outcomeTreeTitle">Acquired</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="a_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="a_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="a_acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="a_acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="a_acute_1" class="healthState">Uncomplicated</p></div>' +
      '   <div class="tableRow"><p id="a_acute_2" class="healthState">Complicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="top:130px; left: 400px"><p id="a_meningitis_1" class="healthState">Permanent disability due to meningitis</p></div>' +
      '<div class="healthOutcome death" style="top:40px; left: 400px"><p id="a_fatal_acute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="a_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px"><p id="a_acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("p_infection_1", "p_acute_1", {}, "p_alpha_1", "Perinatal", "alpha");
      createConnection("p_infection_1", "p_asymptomatic_1", {}, "p_alpha_2", "Perinatal", "1 - alpha");
      createConnection("p_acute_1", "p_meningitis_1", {}, "p_gamma_1", "Perinatal", "gamma");
      createConnection("p_acute_1", "p_fatal_acute_1", {}, "p_beta_1", "Perinatal", "beta");
      createConnection("p_asymptomatic_1", "p_asymptomatic_recovery_1", {});
      createConnection("p_acute_1", "p_acute_recovery_1", {});

      createConnection("a_infection_1", "a_acute", {}, "a_alpha_1", "Acquired", "alpha");
      createConnection("a_infection_1", "a_asymptomatic_1", {}, "a_alpha_2", "Acquired", "1 - alpha");
      createConnection("a_acute_2", "a_fatal_acute_1", {}, "a_beta_1", "Acquired", "beta");
      createConnection("a_acute_2", "a_meningitis_1", {}, "a_gamma_1", "Acquired", "gamma");
      createConnection("a_asymptomatic_1", "a_asymptomatic_recovery_1", {});
      createConnection("a_acute_title", "a_acute_recovery_1", {});
    }
  },

  // Q Fever (simplified)
  // 29/10/2014 - updated with Review 2
  QFever: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 240px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="infection_1" class="healthState">Infections</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 180px; top: 190px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 180px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Mild</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Severe</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="top:43px; left: 540px"><p id="chronic_1" class="healthState">Chronic Q fever</p></div>' +
      '<div class="healthOutcome sequela" style="top:43px; left: 360px"><p id="latency_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome death" style="top:140px; left: 360px"><p id="fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome death" style="top:43px; left: 730px"><p id="fatalChronic_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 310px; top: 203px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 310px; top: 23px"><p id="acute_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 670px; top: 23px"><p id="chronic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_2", "fatalAcute_1", {}, "beta_1", "All", "beta");
      createConnection("acute", "latency_1", {}, "gamma_1", "All", "gamma");
      createConnection("latency_1", "chronic_1", {}, "lambda_1", "All", "lambda");
      createConnection("chronic_1", "fatalChronic_1", {}, "delta_1", "All", "delta");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("acute", "acute_recovery_1", {});
      createConnection("chronic_1", "chronic_recovery_1", {});
    }
  },

  // TickBorneEncephalitis (consistent)
  // 04/10/2014 - updated with Review 2
  TickBorneEncephalitis: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 257px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 90px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top:90px"><p id="acute_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 90px;"><p id="meningoencephalitic_1" class="healthState">Meningo encephalitic phase</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 180px;"><p id="postencephalitic_1" class="healthState">Post encephalitic TBE syndrome</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 90px;"><p id="paralysis_1" class="healthState">Paralysis</p></div>' +
      '<div class="healthOutcome sequela" style="left: 800px; top: 90px;"><p id="residual_1" class="healthState">Residual paresis</p></div>' +
      '<div class="healthOutcome sequela" style="left: 800px; top: 180px;"><p id="chronic_1" class="healthState">Chronic post encephalitic TBE syndrome</p></div>' +
      '<div class="healthOutcome death" style="left: 600px; top: 0px;"><p id="fatalAcute_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 63px"><p id="symptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 530px; top: 23px"><p id="meningoencephalitic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 63px"><p id="paralysis_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 730px; top: 233px"><p id="postencephalitic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_1", "meningoencephalitic_1", {}, "gamma_1", "All", "gamma");
      createConnection("meningoencephalitic_1", "fatalAcute_1", {}, "beta_1", "All", "beta");
      createConnection("meningoencephalitic_1", "paralysis_1", {}, "delta_1", "All", "delta");
      createConnection("meningoencephalitic_1", "postencephalitic_1", {}, "eta_1", "All", "eta");
      createConnection("paralysis_1", "residual_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("postencephalitic_1", "chronic_1", {}, "lambda_1", "All", "lambda");
      createConnection("acute_1", "symptomatic_recovery_1", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("paralysis_1", "paralysis_recovery_1", {});
      createConnection("meningoencephalitic_1", "meningoencephalitic_recovery_1", {});
      createConnection("postencephalitic_1", "postencephalitic_recovery_1", {});
    }
  },

  // Toxoplasmosis (consistent)
  // 31/10/2014 - updated with Review 2
  // 11/11/2014 - updated with Review 3
  Toxoplasmosis: {
    htmlCode:
      '<div id="Congenital" class="outcomeTree" style="margin-bottom: 450px"><div class="outcomeTreeTitle">Congenital</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 183px;"><p id="c_infection_1" class="healthState">Infected pregnant</p></div>' +
      '<div id="c_symptomatic" class="healthOutcome symptomatic" style="top:140px; left: 200px">' +
      '   <div class="tableRow"><p id="c_symptomatic_title" class="healthOutcomeTitle">Infection</p></div>' +
      '   <div class="tableRow"><p id="c_symptomatic_1" class="healthState">Asymptomatic</p></div>' +
      '   <div class="tableRow"><p id="c_symptomatic_2" class="healthState">Symptomatic</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top:80px;"><p id="c_chorioretinitis_1" class="healthState">Permanent disability due to chorioretinitis within 1st year</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top:0px;"><p id="c_chorioretinitis_post_1" class="healthState">Permanent disability due to chorioretinitis after 1st year</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 290px;"><p id="c_intracranial_1" class="healthState">Permanent disability due to intracranial calcification</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 150px;"><p id="c_hydrocephalus_1" class="healthState">Permanent disability due to hydrocephalus</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 220px;"><p id="c_cns_1" class="healthState">Permanent disability due to CNS abnormalities</p></div>' +
      '<div class="healthOutcome death" style="left: 600px; top: 360px;"><p id="c_fatalSymptomatic_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 250px"><p id="c_symptomatic_recovery_1">R</p></div>' +
      '</div>' +

      '<div id="Acquired" class="outcomeTree" style="margin-bottom: 248px"><div class="outcomeTreeTitle">Acquired</div>' +
      '<div class="healthOutcome infection" style="left: 0px; top: 43px;"><p id="a_infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 180px;"><p id="a_asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome symptomatic" id="a_symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="a_symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="a_symptomatic_1" class="healthState">Complicated</p></div>' +
      '   <div class="tableRow"><p id="a_symptomatic_2" class="healthState">Uncomplicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 43px;"><p id="a_chorioretinitis_1" class="healthState">Permanent disability due to chorioretinitis</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 193px"><p id="a_asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 20px"><p id="a_symptomatic_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("c_infection_1", "c_symptomatic", {}, "c_alpha_1", "Congenital", "alpha");
      createConnection("c_symptomatic", "c_fatalSymptomatic_1", {}, "c_beta_1", "Congenital", "beta");
      createConnection("c_symptomatic", "c_intracranial_1", {}, "c_lambda_1", "Congenital", "lambda");
      createConnection("c_symptomatic", "c_hydrocephalus_1", {}, "c_kappa_1", "Congenital", "kappa");
      createConnection("c_symptomatic", "c_cns_1", {}, "c_theta_1", "Congenital", "theta");
      createConnection("c_symptomatic", "c_chorioretinitis_1", {}, "c_gamma_1", "Congenital", "gamma");
      createConnection("c_symptomatic_1", "c_chorioretinitis_post_1", {}, "c_eta_1", "Congenital", "eta");
      createConnection("c_symptomatic", "c_symptomatic_recovery_1", {});

      createConnection("a_infection_1", "a_symptomatic", {}, "a_alpha_1", "Acquired", "alpha");
      createConnection("a_infection_1", "a_asymptomatic_1", {}, "a_alpha_2", "Acquired", "1 - alpha");
      createConnection("a_symptomatic", "a_chorioretinitis_1", {}, "a_eta_1", "Acquired", "eta");
      createConnection("a_asymptomatic_1", "a_asymptomatic_recovery_1", {});
      createConnection("a_symptomatic", "a_symptomatic_recovery_1", {});
    }
  },

  // vCJD (consistent)
  // 30/10/2014 - updated with Review 2
  vCJD: {
    htmlCode: '<div id="All" class="outcomeTree" style="margin-bottom: 140px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 0px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome symptomatic" style="left: 200px; top: 0px;"><p id="acute_1" class="healthState">Symptomatic infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 90px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 0px;"><p id="fatal_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 103px;"><p id="asymptomatic_recovery">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_1", "fatal_1", {}, "beta_1", "All", "beta");
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Poliomyelitis (consistent)
  // 30/10/2014 - updated with Review 2
  Poliomyelitis: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 270px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 67px;"><p id="infection_1" class="healthState">Infection</p></div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 220px;"><p id="asymptomatic_1" class="healthState">Asymptomatic infection</p></div>' +
      '<div id="acute" class="healthOutcome symptomatic" style="top:0px; left: 200px">' +
      '   <div class="tableRow"><p id="acute_title" class="healthOutcomeTitle">Symptomatic infection</p></div>' +
      '   <div class="tableRow"><p id="acute_1" class="healthState">Paralytic</p></div>' +
      '   <div class="tableRow"><p id="acute_2" class="healthState">Non-paralytic</p></div>' +
      '   <div class="tableRow"><p id="acute_3" class="healthState">Uncomplicated</p></div>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 0px;"><p id="permanentParalytic_1" class="healthState">Permanent disability due to paralytic poliomyelitis</p></div>' +
      '<div class="healthOutcome sequela" style="left: 400px; top: 160px;"><p id="latency_1" class="healthState">Latency period</p></div>' +
      '<div class="healthOutcome sequela" style="left: 600px; top: 160px;"><p id="postPolio_1" class="healthState">Post-polio syndrome</p></div>' +
      '<div class="healthOutcome death" style="top: 80px; left: 400px"><p id="fatal_1" class="healthState">Fatal cases</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 233px"><p id="asymptomatic_recovery_1">R</p></div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 160px"><p id="acute_recovery_1">R</p></div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "acute", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("acute_1", "fatal_1", {}, "beta_1", "All", "beta");
      createConnection("acute_1", "permanentParalytic_1", {}, "gamma_1", "All", "gamma");
      createConnection("acute", "latency_1", {}, "lambda_1", "All", "lambda");
      createConnection("latency_1", "postPolio_1", {}, "delta_1", "All", "delta");
      createConnection("asymptomatic_1", "asymptomatic_recovery_1", {});
      createConnection("acute", "acute_recovery_1", {});
    }
  },

  // Primary blood-stream infections (P-BSI)
  // 24/07/2015 - initial implementation
  // 31/08/2015 - update 1 (mortality linked to all symptomatic cases, not only complicated)
  PBSI: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 334px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 99px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 280px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 56px; left: 200px;">' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_1" class="healthState">Uncomplicated</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_2" class="healthState">Complicated</p>' +
      '   </div>' +
      '</div>' +
      '<div class="healthOutcome death" style="left: 470px; top: 0px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 470px; top: 70px;">' +
      '   <p id="pstd_1" class="healthState">Post traumatic stress disorder</p>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 470px; top: 140px;">' +
      '   <p id="cognitive_1" class="healthState">Cognitive impairment</p>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 470px; top: 210px;">' +
      '   <p id="physical_1" class="healthState">Physical impairment</p>' +
      '</div>' +
      '<div class="healthOutcome sequela" style="left: 470px; top: 280px;">' +
      '   <p id="renal_1" class="healthState">Renal failure, renal replacement therapy</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 75px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 293px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pstd_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "physical_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "renal_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Neonatal Sepsis
  // 24/07/2015 - initial implementation
  NeonatalSepsis: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 330px">' +
      '<div id="infection" class="healthOutcome infection" style="left: 0px; top: 140px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div id="asymptomatic" class="healthOutcome asymptomatic" style="left: 200px; top: 280px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 140px; left: 200px;">' +
      '   <p id="symptomatic_1" class="healthState">Symptomatic infection</p>' +
      '</div>' +
      '<div id="fatalSymptomatic" class="healthOutcome death" style="left: 470px; top: 0px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div id="neuro" class="healthOutcome sequela" style="left: 470px; top: 70px;">' +
      '   <p id="neuro_1" class="healthState">Impaired neuro development</p>' +
      '</div>' +
      '<div id="permanent" class="healthOutcome sequela" style="left: 650px; top: 70px;">' +
      '   <p id="permanent_1" class="healthState">Permanent impaired neuro development</p>' +
      '</div>' +
      '<div id="palsy" class="healthOutcome sequela" style="left: 470px; top: 140px;">' +
      '   <p id="palsy_1" class="healthState">Cerebral palsy</p>' +
      '</div>' +
      '<div id="vision" class="healthOutcome sequela" style="left: 470px; top: 210px;">' +
      '   <p id="vision_1" class="healthState">Vision impairment</p>' +
      '</div>' +
      '<div id="hearing" class="healthOutcome sequela" style="left: 470px; top: 280px;">' +
      '   <p id="hearing_1" class="healthState">Hearing impairment</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 69px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 293px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "neuro_1", {}, "gamma_1", "All", "gamma");
      createConnection("neuro_1", "permanent_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_1", "palsy_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_1", "vision_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_1", "hearing_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // HAP
  // 26/07/2015 - initial implementation
  HAP: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 270px">' +
      '<div id="infection" class="healthOutcome infection" style="left: 0px; top: 110px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div id="asymptomatic" class="healthOutcome asymptomatic" style="left: 200px; top: 220px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 110px; left: 200px;">' +
      '   <p id="symptomatic_1" class="healthState">Symptomatic infection</p>' +
      '</div>' +
      '<div id="fatalSymptomatic" class="healthOutcome death" style="left: 410px; top: 220px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div id="sepsis" class="healthOutcome sequela" style="left: 410px; top: 110px;">' +
      '   <p id="sepsis_1" class="healthState">Severe sepsis, septic shock</p>' +
      '</div>' +
      '<div id="pstd" class="healthOutcome sequela" style="left: 610px; top: 0px;">' +
      '   <p id="pstd_1" class="healthState">Post-traumatic stress disorder</p>' +
      '</div>' +
      '<div id="cognitive" class="healthOutcome sequela" style="left: 610px; top: 70px;">' +
      '   <p id="cognitive_1" class="healthState">Cognitive impairment</p>' +
      '</div>' +
      '<div id="physical" class="healthOutcome sequela" style="left: 610px; top: 140px;">' +
      '   <p id="physical_1" class="healthState">Physical impairment</p>' +
      '</div>' +
      '<div id="renal" class="healthOutcome sequela" style="left: 610px; top: 210px;">' +
      '   <p id="renal_1" class="healthState">Renal failure and renal replacement therapy</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 79px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 340px; top: 233px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pstd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "physical_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "renal_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // CR-BSI
  // 26/07/2015 - initial implementation
  CRBSI: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 334px">' +
      '<div id="infection" class="healthOutcome infection" style="left: 0px; top: 99px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div id="asymptomatic" class="healthOutcome asymptomatic" style="left: 200px; top: 280px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 56px; left: 200px;">' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_1" class="healthState">Uncomplicated</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_2" class="healthState">Complicated</p>' +
      '   </div>' +
      '</div>' +
      '<div id="fatalSymptomatic" class="healthOutcome death" style="left: 470px; top: 0px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div id="pstd" class="healthOutcome sequela" style="left: 470px; top: 70px;">' +
      '   <p id="pstd_1" class="healthState">Post-traumatic stress disorder</p>' +
      '</div>' +
      '<div id="cognitive" class="healthOutcome sequela" style="left: 470px; top: 140px;">' +
      '   <p id="cognitive_1" class="healthState">Cognitive impairment</p>' +
      '</div>' +
      '<div id="physical" class="healthOutcome sequela" style="left: 470px; top: 210px;">' +
      '   <p id="physical_1" class="healthState">Physical impairment</p>' +
      '</div>' +
      '<div id="renal" class="healthOutcome sequela" style="left: 470px; top: 280px;">' +
      '   <p id="renal_1" class="healthState">Renal failure and renal replacement therapy</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 65px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 293px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_2", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pstd_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "physical_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "renal_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // UTI
  // 27/07/2015 - initial implementation
  UTI: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 330px">' +
      '<div id="infection" class="healthOutcome infection" style="left: 0px; top: 140px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div id="asymptomatic" class="healthOutcome asymptomatic" style="left: 200px; top: 220px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 140px; left: 200px;">' +
      '   <p id="symptomatic_1" class="healthState">Symptomatic infection</p>' +
      '</div>' +
      '<div id="urosepsis" class="healthOutcome sequela" style="left: 410px; top: 140px;">' +
      '   <p id="urosepsis_1" class="healthState">Secondary bacteraemia/ urosepsis</p>' +
      '</div>' +
      '<div id="pstd" class="healthOutcome sequela" style="left: 610px; top: 0px;">' +
      '   <p id="pstd_1" class="healthState">Post-traumatic stress disorder</p>' +
      '</div>' +
      '<div id="cognitive" class="healthOutcome sequela" style="left: 610px; top: 70px;">' +
      '   <p id="cognitive_1" class="healthState">Cognitive impairment</p>' +
      '</div>' +
      '<div id="physical" class="healthOutcome sequela" style="left: 610px; top: 140px;">' +
      '   <p id="physical_1" class="healthState">Physical impairment</p>' +
      '</div>' +
      '<div id="renal" class="healthOutcome sequela" style="left: 610px; top: 210px;">' +
      '   <p id="renal_1" class="healthState">Renal failure and renal replacement therapy</p>' +
      '</div>' +
      '<div id="fatalUrosepsis" class="healthOutcome death" style="left: 610px; top: 280px;">' +
      '   <p id="fatalUrosepsis_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 119px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 340px; top: 233px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "urosepsis_1", {}, "gamma_1", "All", "gamma");
      createConnection("urosepsis_1", "pstd_1", {}, "eta_1", "All", "eta");
      createConnection("urosepsis_1", "cognitive_1", {}, "delta_1", "All", "delta");
      createConnection("urosepsis_1", "physical_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("urosepsis_1", "renal_1", {}, "zeta_1", "All", "zeta");
      createConnection("urosepsis_1", "fatalUrosepsis_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Clostridium Difficile
  // 27/07/2015 - initial implementation
  ClostridiumDifficile: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 220px">' +
      '<div id="infection" class="healthOutcome infection" style="left: 0px; top: 43px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div id="asymptomatic" class="healthOutcome asymptomatic" style="left: 200px; top: 170px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 0px; left: 200px;">' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_title" class="healthOutcomeTitle">Symptomatic infection</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_1" class="healthState">Uncomplicated</p>' +
      '   </div>' +
      '   <div class="tableRow">' +
      '       <p id="symptomatic_2" class="healthState">Complicated</p>' +
      '   </div>' +
      '</div>' +
      '<div id="colectomy" class="healthOutcome sequela" style="left: 400px; top: 0px;">' +
      '   <p id="colectomy_1" class="healthState">Post-colectomy state</p>' +
      '</div>' +
      '<div id="fatalSymptomatic" class="healthOutcome death" style="left: 400px; top: 100px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 9px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 183px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic", "colectomy_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // SSI
  // 31/08/2015 - initial implementation
  // 20/09/2015 - simplified, renamed from DSSI CABG to SSI (email 15/09/2015)
  SSI: {
    htmlCode:
      '<div id="All" class="outcomeTree" style="margin-bottom: 150px">' +
      '<div class="healthOutcome infection" style="left: 0px; top: 0px;">' +
      '   <p id="infection_1" class="healthState">Infection</p>' +
      '</div>' +
      '<div class="healthOutcome asymptomatic" style="left: 200px; top: 100px;">' +
      '   <p id="asymptomatic_1" class="healthState">Asymptomatic infection</p>' +
      '</div>' +
      '<div id="symptomatic" class="healthOutcome symptomatic" style="top: 0px; left: 200px;">' +
      '   <p id="symptomatic_1" class="healthState">Symptomatic infection</p>' +
      '</div>' +
      '<div class="healthOutcome death" style="left: 400px; top: 0px;">' +
      '   <p id="fatalSymptomatic_1" class="healthState">Fatal cases</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 45px;">' +
      '   <p id="symptomatic_recovery">R</p>' +
      '</div>' +
      '<div class="healthOutcome recovery" style="left: 330px; top: 113px;">' +
      '   <p id="asymptomatic_recovery">R</p>' +
      '</div>' +
      '</div>',
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Baseline BSI
  // 09/08/2017 - initial implementation
  BaselineBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Baseline RESP
  // 09/08/2017 - initial implementation
  BaselineRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Baseline UTI
  // 12/08/2017 - initial implementation
  BaselineUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Baseline SSI
  // 12/08/2017 - initial implementation
  BaselineSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Baseline OTH
  // 12/08/2017 - initial implementation
  BaselineOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

















  // Klebsiella pneumoniae 3GCRKP BSI
  // 14/08/2017 - initial implementation
  Klebsiella3GCRKPBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae 3GCRKP RESP
  // 14/08/2017 - initial implementation
  Klebsiella3GCRKPRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae 3GCRKP UTI
  // 14/08/2017 - initial implementation
  Klebsiella3GCRKPUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae 3GCRKP SSI
  // 14/08/2017 - initial implementation
  Klebsiella3GCRKPSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae 3GCRKP OTH
  // 14/08/2017 - initial implementation
  Klebsiella3GCRKPOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae CRKP BSI
  // 17/08/2017 - initial implementation
  KlebsiellaCRKPBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae CRKP RESP
  // 17/08/2017 - initial implementation
  KlebsiellaCRKPRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae CRKP UTI
  // 17/08/2017 - initial implementation
  KlebsiellaCRKPUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae CRKP SSI
  // 17/08/2017 - initial implementation
  KlebsiellaCRKPSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae CRKP OTH
  // 17/08/2017 - initial implementation
  KlebsiellaCRKPOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae ColRKP BSI
  // 17/08/2017 - initial implementation
  KlebsiellaColRKPBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae ColRKP RESP
  // 17/08/2017 - initial implementation
  KlebsiellaColRKPRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae ColRKP UTI
  // 17/08/2017 - initial implementation
  KlebsiellaColRKPUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae ColRKP SSI
  // 17/08/2017 - initial implementation
  KlebsiellaColRKPSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Klebsiella pneumoniae ColRKP OTH
  // 17/08/2017 - initial implementation
  KlebsiellaColRKPOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli 3GCREC BSI
  // 17/08/2017 - initial implementation
  EscherichiaColi3GCRECBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli 3GCREC RESP
  // 17/08/2017 - initial implementation
  EscherichiaColi3GCRECRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli 3GCREC UTI
  // 17/08/2017 - initial implementation
  EscherichiaColi3GCRECUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli 3GCREC SSI
  // 17/08/2017 - initial implementation
  EscherichiaColi3GCRECSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli 3GCREC OTH
  // 17/08/2017 - initial implementation
  EscherichiaColi3GCRECOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli CREC BSI
  // 26/08/2017 - initial implementation
  EscherichiaColiCRECBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli CREC RESP
  // 26/08/2017 - initial implementation
  EscherichiaColiCRECRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli CREC UTI
  // 26/08/2017 - initial implementation
  EscherichiaColiCRECUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli CREC SSI
  // 26/08/2017 - initial implementation
  EscherichiaColiCRECSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli CREC OTH
  // 26/08/2017 - initial implementation
  EscherichiaColiCRECOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli ColREC BSI
  // 26/08/2017 - initial implementation
  EscherichiaColiColRECBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli ColREC RESP
  // 26/08/2017 - initial implementation
  EscherichiaColiColRECRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli ColREC UTI
  // 26/08/2017 - initial implementation
  EscherichiaColiColRECUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli ColREC SSI
  // 26/08/2017 - initial implementation
  EscherichiaColiColRECSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Escherichia coli ColREC OTH
  // 26/08/2017 - initial implementation
  EscherichiaColiColRECOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. MDRACI BSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppMDRACIBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. MDRACI RESP
  // 28/08/2017 - initial implementation
  AcinetobacterSppMDRACIRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. MDRACI UTI
  // 28/08/2017 - initial implementation
  AcinetobacterSppMDRACIUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. MDRACI SSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppMDRACISSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. MDRACI OTH
  // 28/08/2017 - initial implementation
  AcinetobacterSppMDRACIOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. CRACI BSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppCRACIBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. CRACI RESP
  // 28/08/2017 - initial implementation
  AcinetobacterSppCRACIRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. CRACI UTI
  // 28/08/2017 - initial implementation
  AcinetobacterSppCRACIUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. CRACI SSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppCRACISSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. CRACI OTH
  // 28/08/2017 - initial implementation
  AcinetobacterSppCRACIOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. ColRACI BSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppColRACIBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. ColRACI RESP
  // 28/08/2017 - initial implementation
  AcinetobacterSppColRACIRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. ColRACI UTI
  // 28/08/2017 - initial implementation
  AcinetobacterSppColRACIUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. ColRACI SSI
  // 28/08/2017 - initial implementation
  AcinetobacterSppColRACISSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Acinetobacter spp. ColRACI OTH
  // 28/08/2017 - initial implementation
  AcinetobacterSppColRACIOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa MDRPA BSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaMDRPABSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa MDRPA RESP
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaMDRPARESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa MDRPA UTI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaMDRPAUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa MDRPA SSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaMDRPASSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa MDRPA OTH
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaMDRPAOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa CRPA BSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaCRPABSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa CRPA RESP
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaCRPARESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa CRPA UTI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaCRPAUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa CRPA SSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaCRPASSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa CRPA OTH
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaCRPAOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa ColRPA BSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaColRPABSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa ColRPA RESP
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaColRPARESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa ColRPA UTI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaColRPAUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa ColRPA SSI
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaColRPASSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Pseudomonas aeruginosa ColRPA OTH
  // 28/08/2017 - initial implementation
  PseudomonasAeruginosaColRPAOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Staphylococcus aureus MRSA BSI
  // 28/08/2017 - initial implementation
  StaphylococcusAureusMRSABSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Staphylococcus aureus MRSA RESP
  // 28/08/2017 - initial implementation
  StaphylococcusAureusMRSARESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Staphylococcus aureus MRSA UTI
  // 28/08/2017 - initial implementation
  StaphylococcusAureusMRSAUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Staphylococcus aureus MRSA SSI
  // 28/08/2017 - initial implementation
  StaphylococcusAureusMRSASSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Staphylococcus aureus MRSA OTH
  // 28/08/2017 - initial implementation
  StaphylococcusAureusMRSAOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Enterococcus faecalis and E. faecium VRE BSI
  // 28/08/2017 - initial implementation
  EnterococcusFaecalisVREBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Enterococcus faecalis and E. faecium VRE RESP
  // 28/08/2017 - initial implementation
  EnterococcusFaecalisVRERESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Enterococcus faecalis and E. faecium VRE UTI
  // 28/08/2017 - initial implementation
  EnterococcusFaecalisVREUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Enterococcus faecalis and E. faecium VRE SSI
  // 28/08/2017 - initial implementation
  EnterococcusFaecalisVRESSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Enterococcus faecalis and E. faecium VRE OTH
  // 28/08/2017 - initial implementation
  EnterococcusFaecalisVREOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PRSP BSI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePRSPBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PRSP RESP
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePRSPRESP: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PRSP UTI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePRSPUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PRSP SSI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePRSPSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PRSP OTH
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePRSPOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PMRSP BSI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePMRSPBSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 260px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div id=\"symptomatic\" class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 216px;\">" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_title\" class=\"healthOutcomeTitle\">Symptomatic infection</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_1\" class=\"healthState\">Uncomplicated</p>" +
      "   </div>" +
      "   <div class=\"tableRow\">" +
      "       <p id=\"symptomatic_2\" class=\"healthState\">Complicated</p>" +
      "   </div>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 470px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 70px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 140px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 210px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 280px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 350px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 420px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 490px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 470px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 225px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_title", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_2", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("symptomatic_2", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("symptomatic_2", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("symptomatic_2", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("symptomatic_2", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("symptomatic_2", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("symptomatic_2", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("symptomatic_2", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_title", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PMRSP RESP
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePMRSPRES: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 610px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 280px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 560px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 280px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 400px;top: 280px;\">" +
      "   <p id=\"sepsis_1\" class=\"healthState\">Severe sepsis, septic shock</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 80px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 0px;\">" +
      "   <p id=\"pi_1\" class=\"healthState\">PI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 80px;\">" +
      "   <p id=\"pi_rf_1\" class=\"healthState\">PI + RF</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 160px;\">" +
      "   <p id=\"pi_rf_ci_1\" class=\"healthState\">PI + RF + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 240px;\">" +
      "   <p id=\"pi_rf_ci_ptsd_1\" class=\"healthState\">PI + RF + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 320px;\">" +
      "   <p id=\"pi_rf_ptsd_1\" class=\"healthState\">PI + RF + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 400px;\">" +
      "   <p id=\"pi_ci_1\" class=\"healthState\">PI + CI</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 480px;\">" +
      "   <p id=\"pi_ci_ptsd_1\" class=\"healthState\">PI + CI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome sequela\" style=\"left: 600px; top: 560px;\">" +
      "   <p id=\"pi_ptsd_1\" class=\"healthState\">PI + PTSD</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 335px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 573px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "<div style=\"position: absolute; width: 220px;\">" +
      "   <p>PTSD: Post traumatic stress disorder</p>" +
      "   <p>RF: Renal failure, renal replacement therapy</p>" +
      "   <p>CI: Cognitive impairment</p>" +
      "   <p>PI: Physical impairment</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "sepsis_1", {}, "theta_1", "All", "theta");
      createConnection("sepsis_1", "pi_1", {}, "gamma_1", "All", "gamma");
      createConnection("sepsis_1", "pi_rf_1", {}, "delta_1", "All", "delta");
      createConnection("sepsis_1", "pi_rf_ci_1", {}, "epsilon_1", "All", "epsilon");
      createConnection("sepsis_1", "pi_rf_ci_ptsd_1", {}, "zeta_1", "All", "zeta");
      createConnection("sepsis_1", "pi_rf_ptsd_1", {}, "eta_1", "All", "eta");
      createConnection("sepsis_1", "pi_ci_1", {}, "kappa_1", "All", "kappa");
      createConnection("sepsis_1", "pi_ci_ptsd_1", {}, "lambda_1", "All", "lambda");
      createConnection("sepsis_1", "pi_ptsd_1", {}, "iota_1", "All", "iota");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PMRSP UTI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePMRSPUTI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PMRSP SSI
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePMRSPSSI: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome death\" style=\"left: 400px; top: 0px;\">" +
      "   <p id=\"fatalSymptomatic_1\" class=\"healthState\">Fatal cases</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 33px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "fatalSymptomatic_1", {}, "beta_1", "All", "beta");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },

  // Streptococcus pneumoniae PMRSP OTH
  // 28/08/2017 - initial implementation
  StreptococcusPneumoniaePMRSPOTH: {
    htmlCode:
      "<div id=\"All\" class=\"outcomeTree\" style=\"margin-bottom: 150px\">" +
      "<div class=\"healthOutcome infection\" style=\"left: 0px; top: 0px;\">" +
      "   <p id=\"infection_1\" class=\"healthState\">Infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome asymptomatic\" style=\"left: 200px; top: 100px;\">" +
      "   <p id=\"asymptomatic_1\" class=\"healthState\">Asymptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome symptomatic\" style=\"left: 200px;top: 0px;\">" +
      "   <p id=\"symptomatic_1\" class=\"healthState\">Symptomatic infection</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 13px;\">" +
      "   <p id=\"symptomatic_recovery\">R</p>" +
      "</div>" +
      "<div class=\"healthOutcome recovery\" style=\"left: 320px; top: 113px;\">" +
      "   <p id=\"asymptomatic_recovery\">R</p>" +
      "</div>" +
      "</div>",
    jsCode: function () {
      createConnection("infection_1", "symptomatic_1", {}, "alpha_1", "All", "alpha");
      createConnection("infection_1", "asymptomatic_1", {}, "alpha_2", "All", "1 - alpha");
      createConnection("symptomatic_1", "symptomatic_recovery", {});
      createConnection("asymptomatic_1", "asymptomatic_recovery", {});
    }
  },
};
